@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2010 Kamil Ignacak
+ * Copyright (C) 2007 - 2011 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -204,7 +204,7 @@ static cdw_rv_t cdw_cdio_resolve_sectors_to_read(cdw_cdio_disc_t *disc, track_t
 
 static int  cdw_cdio_get_cd_sector_meta_data(cdw_cdio_disc_t *disc, lsn_t sector, unsigned char *buffer);
 static void cdw_cdio_update_processwin(lsn_t current_relative_sector, lsn_t n_sectors_in_track, int sector_size);
-static int  cdw_cdio_copy_sectors_to_file_sub(cdw_cdio_disc_t *disc, track_t t, int output_file);
+static int  cdw_cdio_copy_sectors_to_file_sub(cdw_cdio_disc_t *disc, track_t t, int output_file, long int sectors);
 
 static inline long int cdw_cdio_read_sectors(cdw_cdio_disc_t *disc, track_t t, unsigned char *buffer, lsn_t sector, uint32_t n_sectors);
 static void cdw_cdio_log_handler(cdio_log_level_t level, const char *message);
@@ -258,7 +258,7 @@ void cdw_cdio_clean(void)
 
    \param device_fullpath - full path to device file
 
-   \return CDW_GEN_ERROR if function failed to open the disc
+   \return CDW_ERROR if function failed to open the disc
    \return CDW_OK on success
 */
 cdw_rv_t cdw_cdio_open_disc(void)
@@ -284,7 +284,7 @@ cdw_rv_t cdw_cdio_open_disc(void)
 		g_disc.p_cdio = cdio_open(g_disc.device_fullpath, DRIVER_UNKNOWN);
 		if (g_disc.p_cdio == (CdIo_t *) NULL) {
 			cdw_vdm ("ERROR: failed to open cdio disc using DRIVER_UNKNOWN\n");
-			return CDW_GEN_ERROR;
+			return CDW_ERROR;
 		}
 	}
 
@@ -303,10 +303,10 @@ cdw_rv_t cdw_cdio_open_disc(void)
 			if (crv == CDW_OK) {
 				crv = cdw_cdio_get_iso9660_fs_meta_information(&g_disc);
 			}
-			if (crv == CDW_GEN_ERROR) {
+			if (crv == CDW_ERROR) {
 				cdw_vdm ("ERROR: failed to get iso9660 meta information\n");
 				cdw_cdio_close_disc();
-				return CDW_GEN_ERROR;
+				return CDW_ERROR;
 			}
 		} else {
 			; /* other, unsupported file system */
@@ -319,7 +319,7 @@ cdw_rv_t cdw_cdio_open_disc(void)
 	} else {
 		cdw_vdm ("ERROR: failed to get disc meta information\n");
 		cdw_cdio_close_disc();
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 }
 
@@ -334,7 +334,7 @@ cdw_rv_t cdw_cdio_open_iso9660_fs(cdw_cdio_disc_t *disc)
 	disc->fs.fs.iso9660.p_iso = iso9660_open(disc->device_fullpath);
 	if (disc->fs.fs.iso9660.p_iso == (iso9660_t *) NULL) {
 		cdw_vdm ("ERROR: can't open ISO file system on a disc\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	disc->fs.fs.iso9660.fd = open(disc->device_fullpath, O_RDONLY);
@@ -344,7 +344,7 @@ cdw_rv_t cdw_cdio_open_iso9660_fs(cdw_cdio_disc_t *disc)
 		iso9660_close(disc->fs.fs.iso9660.p_iso);
 		disc->fs.fs.iso9660.p_iso = (iso9660_t *) NULL;
 
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 
@@ -361,7 +361,7 @@ cdw_rv_t cdw_cdio_get_iso9660_fs_meta_information(cdw_cdio_disc_t *disc)
 	if (!read_pvd) {
 		cdw_vdm ("ERROR: failed to read pvd from a disc\n");
 
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	char *id = iso9660_get_volume_id(&(disc->fs.fs.iso9660.pvd));
@@ -397,7 +397,7 @@ cdw_rv_t cdw_cdio_get_iso9660_fs_meta_information(cdw_cdio_disc_t *disc)
    \return CDW_OK if disc meta information was read without problems and disc is not blank
    \return CDW_NO if disc meta information was read without problems but disc is blank
    \return CDW_CANCEL if disc meta information was read only partially (perhaps unknown / unsupported disc type?)
-   \return CDW_GEN_ERROR if function can't get any meta information
+   \return CDW_ERROR if function can't get any meta information
 */
 cdw_rv_t cdw_cdio_get_disc_meta_info(cdw_cdio_disc_t *disc)
 {
@@ -418,7 +418,7 @@ cdw_rv_t cdw_cdio_get_disc_meta_info(cdw_cdio_disc_t *disc)
 		/* we won't work with unknown disc types at all */
 		cdw_vdm ("ERROR: failed to get disc simple type, setting disc simple type to UNKNOWN\n");
 		strcpy(disc->simple_type_label, "UNKNOWN");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	disc->first_track = cdio_get_first_track_num(disc->p_cdio);
@@ -440,13 +440,16 @@ cdw_rv_t cdw_cdio_get_disc_meta_info(cdw_cdio_disc_t *disc)
 		/* inspect every track on disc */
 		for (track_t t = disc->first_track; t <= disc->last_track; t++) {
 			cdw_rv_t crv = cdw_cdio_recognize_track(disc, t);
-			if (crv == CDW_GEN_ERROR) {
+			if (crv == CDW_ERROR) {
 				disc->unknown_tracks++;
+			} else {
+				disc->fs.n_sectors += disc->tracks[t].n_sectors_to_read;
+				cdw_vdm ("INFO: n_sectors = %ld\n", disc->fs.n_sectors);
 			}
 		}
 
 		track_t t = disc->first_track;
-		if (disc->tracks[t].fs.cdio_fs == CDIO_FS_UNKNOWN
+		if (disc->tracks[t].fs.cdio_fs == CDIO_FS_UNKNOWN /* this may trigger compiler warning */
 		    || disc->tracks[t].fs.cdio_fs == 0) {
 			cdw_vdm ("INFO: disc is blank\n");
 			disc->blank = true;
@@ -470,7 +473,7 @@ cdw_rv_t cdw_cdio_get_disc_meta_info(cdw_cdio_disc_t *disc)
 	} else {
 		if (disc->unknown_tracks == disc->n_tracks) {
 			cdw_vdm ("WARNING: all tracks are unknown\n");
-			return CDW_GEN_ERROR;
+			return CDW_ERROR;
 
 		} else if (disc->unknown_tracks > 0
 			   && disc->unknown_tracks < disc->n_tracks) {
@@ -629,7 +632,7 @@ void cdw_cdio_disc_reset(cdw_cdio_disc_t *disc)
 	disc->fs.type = 0;
 	disc->fs.type_label = (char *) NULL;
 	disc->fs.volume_id[0] = '\0';
-	disc->fs.n_sectors = -1;
+	disc->fs.n_sectors = 0;
 
 	disc->fs.fs.iso9660.p_iso = (iso9660_t *) NULL;
 	disc->fs.fs.iso9660.fd = -1;
@@ -679,7 +682,7 @@ void cdw_cdio_disc_reset(cdw_cdio_disc_t *disc)
    \param t - number of track to check
 
    \return CDW_OK if track is fully recognized
-   \return CDW_GEN_ERROR if track is unknown or unsupported
+   \return CDW_ERROR if track is unknown or unsupported
 */
 cdw_rv_t cdw_cdio_recognize_track(cdw_cdio_disc_t *disc, track_t t)
 {
@@ -703,7 +706,7 @@ cdw_rv_t cdw_cdio_recognize_track(cdw_cdio_disc_t *disc, track_t t)
 
 		cdw_vdm ("ERROR: failed to fetch correct first_sector (%d) or last_sector (%d) of track #%d\n",
 			 disc->tracks[t].first_sector, disc->tracks[t].last_sector, (int) t);
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	cdw_rv_t crv = CDW_OK;
@@ -712,21 +715,21 @@ cdw_rv_t cdw_cdio_recognize_track(cdw_cdio_disc_t *disc, track_t t)
 		disc->tracks[t].cdio_track_format = cdio_get_track_format(disc->p_cdio, t);
 		if (disc->tracks[t].cdio_track_format == TRACK_FORMAT_ERROR) {
 			cdw_vdm ("ERROR: failed to resolve cdio track format for track #%d\n", t);
-			return CDW_GEN_ERROR;
+			return CDW_ERROR;
 		}
 
 		/* 3: get cdw format of track (disc->tracks[t].cdw_format) */
 		crv = cdw_cdio_resolve_cd_track_cdw_format(disc, t);
 		if (crv != CDW_OK) {
 			cdw_vdm ("ERROR: failed to resolve cdw track format for track #%d\n", t);
-			return CDW_GEN_ERROR;
+			return CDW_ERROR;
 		}
 	} else if (disc->simple_type == CDW_DISC_SIMPLE_TYPE_DVD) {
 		disc->tracks[t].cdw_track_format = CDW_TRACK_DVD;
 	} else { /* unknown disc type */
 		disc->tracks[t].cdw_track_format = CDW_TRACK_BLACK_BOOK_UNKNOWN;
 		cdw_vdm ("WARNING: track #%d has unknown format\n", t);
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	cdio_iso_analysis_t iso_analysis;
@@ -743,7 +746,7 @@ cdw_rv_t cdw_cdio_recognize_track(cdw_cdio_disc_t *disc, track_t t)
 	crv = cdw_cdio_resolve_sector_size(disc, t);
 	if (crv != CDW_OK) {
 		cdw_vdm ("ERROR: failed to resolve sector size for track #%d\n", t);
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	/* 5: try to calculate value of disc->tracks[t].n_sectors_to_read,
@@ -751,7 +754,13 @@ cdw_rv_t cdw_cdio_recognize_track(cdw_cdio_disc_t *disc, track_t t)
 	crv = cdw_cdio_resolve_sectors_to_read(disc, t);
 	if (crv != CDW_OK) {
 		cdw_vdm ("ERROR: failed to calculate number of sectors to read for track #%d\n", t);
-		return CDW_GEN_ERROR;
+		if (t == 1) {
+			return CDW_ERROR;
+		} else {
+			/* cdw can't read data tracks #2 and following, so
+			   there is no point in raising an error for these
+			   tracks */
+		}
 	}
 
 	return CDW_OK;
@@ -773,13 +782,13 @@ cdw_rv_t cdw_cdio_recognize_track(cdw_cdio_disc_t *disc, track_t t)
    check cdw format of given track.
 
    If mmc call for given track fails or yields unknown results, the function
-   returns CDW_GEN_ERROR.
+   returns CDW_ERROR.
 
    \param disc - disc data structure describing current disc
    \param t - number of track that you want to check
 
    \return CDW_OK if function sets cdw_format correctly
-   \return CDW_GEN_ERROR on errors
+   \return CDW_ERROR on errors
 */
 cdw_rv_t cdw_cdio_resolve_cd_track_cdw_format(cdw_cdio_disc_t *disc, track_t t)
 {
@@ -810,7 +819,7 @@ cdw_rv_t cdw_cdio_resolve_cd_track_cdw_format(cdw_cdio_disc_t *disc, track_t t)
 
 	if (disc->tracks[t].cdw_track_format == CDW_TRACK_BLACK_BOOK_UNKNOWN) {
 		cdw_vdm ("ERROR: failed to get cdw format for track #%d (is now CDW_TRACK_BLACK_BOOK_UNKNOWN)\n", t);
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	} else {
 		return CDW_OK;
 	}
@@ -994,7 +1003,7 @@ cdw_track_format_t cdw_cdio_get_track_format_from_sector_header(cdw_cdio_disc_t
    \param t - index of track, for which you want to check sector size
 
    \return CDW_OK if given track is supported by cdw and sector size was set as non-zero
-   \return CDW_GEN_ERROR if given track is not supported by cdw and sector size was set as zero
+   \return CDW_ERROR if given track is not supported by cdw and sector size was set as zero
 */
 cdw_rv_t cdw_cdio_resolve_sector_size(cdw_cdio_disc_t *disc, track_t t)
 {
@@ -1009,7 +1018,7 @@ cdw_rv_t cdw_cdio_resolve_sector_size(cdw_cdio_disc_t *disc, track_t t)
 
 	if (disc->tracks[t].sector_size == 0) {
 		cdw_vdm ("ERROR: sector size for track #%d is 0\n", t);
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	} else {
 		return CDW_OK;
 	}
@@ -1033,7 +1042,7 @@ cdw_rv_t cdw_cdio_resolve_sector_size(cdw_cdio_disc_t *disc, track_t t)
    \param t - index of track, for which you want to calculate n_sectors_to_read
 
    \return CDW_OK if given track is supported by cdw and n_sectors_to_read was set as non-zero
-   \return CDW_GEN_ERROR if given track is not supported by cdw and n_sectors_to_read was set as zero
+   \return CDW_ERROR if given track is not supported by cdw and n_sectors_to_read was set as zero
 */
 cdw_rv_t cdw_cdio_resolve_sectors_to_read(cdw_cdio_disc_t *disc, track_t t)
 {
@@ -1082,7 +1091,7 @@ cdw_rv_t cdw_cdio_resolve_sectors_to_read(cdw_cdio_disc_t *disc, track_t t)
 
 	if (disc->tracks[t].n_sectors_to_read == 0) {
 		cdw_vdm ("ERROR: failed to calculate sectors to read for track #%d\n", t);
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	} else {
 		return CDW_OK;
 	}
@@ -1092,68 +1101,17 @@ cdw_rv_t cdw_cdio_resolve_sectors_to_read(cdw_cdio_disc_t *disc, track_t t)
 
 
 
+#define cdw_cdio_copy_sectors_to_file_debug_print_state(disc)		\
+	cdw_vdm ("## INFO: ---------------------------------------- \n"); \
+	cdw_vdm ("## INFO:      track number = %8d \n", g_disc.tracks[t].first_sector); \
+	cdw_vdm ("## INFO:      first_sector = %8d \n", g_disc.tracks[t].first_sector); \
+	cdw_vdm ("## INFO:       last_sector = %8d \n", g_disc.tracks[t].last_sector); \
+	cdw_vdm ("## INFO:         n_sectors = %8d \n", g_disc.tracks[t].n_sectors); \
+	cdw_vdm ("## INFO: n_sectors_to_read = %8d \n", g_disc.tracks[t].n_sectors_to_read);	\
+	cdw_vdm ("## INFO:       sector_size = %8d \n", g_disc.tracks[t].sector_size); \
+	cdw_vdm ("## INFO: ---------------------------------------- \n");
 
-/**
-   \brief Read tracks from current disc and write them to given file descriptor
 
-   More detailed description is following:
-       cdw_cdio_copy_tracks_to_file()
-   calls for every tracks
-       cdw_cdio_copy_sectors_to_file()
-   to read all tracks from disc.
-
-   This is just a wrapper for one 'for' loop iterating on tracks, one
-   function call inside loop, and some additional stuff.
-
-   \p output_file must be file descriptior for file that is already
-   open for writing.
-
-   \param fd - file descriptor of output file
-
-   \return CDW_OK when all tracks were copied successfully
-   \return CDW_NO when function copied some tracks (but not all) correctly
-   \return CDW_GEN_ERROR when function failed to read any track correctly
-*/
-cdw_rv_t cdw_cdio_copy_tracks_to_file(int fd)
-{
-	cdw_assert (g_disc.open, "ERROR: trying to read from closed disc\n");
-	cdw_assert (fd != -1 && fd != 0, "ERROR: invalid output file: %d\n", fd);
-
-	int failed_tracks = 0;
-
-	cdw_vdm ("INFO: number of tracks on disc to read: %d\n", g_disc.n_tracks);
-
-	for (track_t t = g_disc.first_track; t <= g_disc.last_track; t++) {
-		if (g_disc.tracks[t].cdw_track_format == CDW_TRACK_BLACK_BOOK_UNKNOWN) {
-			/* 2TRANS: this is message printed to log file;
-			   "Copy" is a name of task of copying track from
-			   CD to hard disc; %d is a track number */
-			cdw_logging_write(_("Copy: skipping track #%d with unknown format\n"), t);
-
-			failed_tracks++;
-			continue;
-		}
-
-		int rv = cdw_cdio_copy_sectors_to_file_sub(&g_disc, t, fd);
-		cdw_sdm ("INFO track #%d finished with rv = %d\n", t, rv);
-
-		if (rv != 0) {
-			failed_tracks++;
-			cdw_vdm ("  ERROR: failed reading track #%d\n", t);
-			cdw_cdio_copy_print_debug_on_error(rv);
-		}
-	}
-
-	if (failed_tracks == g_disc.n_tracks) {
-		cdw_vdm ("ERROR: number of tracks read correctly is zero\n")
-		return CDW_GEN_ERROR;
-	} else if (failed_tracks < g_disc.n_tracks && failed_tracks > 0) {
-		cdw_vdm ("WARNING: failed to read correctly %d tracks\n", failed_tracks);
-		return CDW_CANCEL;
-	} else {
-		return CDW_OK;
-	}
-}
 
 
 
@@ -1162,31 +1120,33 @@ cdw_rv_t cdw_cdio_copy_tracks_to_file(int fd)
    \return -1, -2 or -3 on function's internal errors
    \return 'errno - 5' on write() error, caller can interpret value of 'errno + 5' to check the error
 */
-int cdw_cdio_copy_sectors_to_file(track_t t, int fd)
+int cdw_cdio_copy_sectors_to_file(track_t t, int fd, long int n_sectors)
 {
-	return cdw_cdio_copy_sectors_to_file_sub(&g_disc, t, fd);
+	cdw_assert (g_disc.p_cdio != (CdIo_t *) NULL, "ERROR: trying to get info from NULL disc\n");
+	cdw_assert (g_disc.open, "ERROR: trying to read from closed disc\n");
+	cdw_assert (fd != -1 && fd != 0, "ERROR: invalid output file: %d\n", fd);
+
+	if (g_disc.tracks[t].cdw_track_format == CDW_TRACK_BLACK_BOOK_UNKNOWN) {
+		cdw_vdm ("ERROR: called the function for black book (unknown) track\n");
+		return -1;
+	}
+
+	cdw_cdio_copy_sectors_to_file_debug_print_state(g_disc);
+
+	/* most probably we are re-using the same processwin for
+	   N tracks, so clean leftovers from reading previous track */
+	cdw_processwin_reset_progress();
+
+	long int sectors = n_sectors != -1 ? n_sectors : g_disc.tracks[t].n_sectors_to_read;
+	return cdw_cdio_copy_sectors_to_file_sub(&g_disc, t, fd, sectors);
 }
 
 
 
 
 
-#define cdw_cdio_copy_sectors_to_file_debug_print_state(disc)		\
-	cdw_vdm ("## INFO: ---------------------------------------- \n"); \
-	cdw_vdm ("## INFO:      track number = %8d \n", disc->tracks[t].first_sector); \
-	cdw_vdm ("## INFO:      first_sector = %8d \n", disc->tracks[t].first_sector); \
-	cdw_vdm ("## INFO:       last_sector = %8d \n", disc->tracks[t].last_sector); \
-	cdw_vdm ("## INFO:         n_sectors = %8d \n", disc->tracks[t].n_sectors); \
-	cdw_vdm ("## INFO: n_sectors_to_read = %8d \n", disc->tracks[t].n_sectors_to_read);	\
-	cdw_vdm ("## INFO:       sector_size = %8d \n", disc->tracks[t].sector_size); \
-	cdw_vdm ("## INFO: ---------------------------------------- \n");
-
-
-
-
-
 #define cdw_cdio_copy_sectors_to_file_debug_print_progress(disc, t, sector) \
-	cdw_vdm ("INFO: track #%d: sector size=%d, sector=%d/%d (non-relative: %d/%d)\n", \
+	cdw_sdm ("INFO: track #%d: sector size=%d, sector=%d/%d (non-relative: %d/%d)\n", \
 		 (int) t,						\
 		 (int) disc->tracks[t].sector_size,			\
 		 (int) sector - disc->tracks[t].first_sector + 1, /* +1 to display sector numbers from 1, not 0 */ \
@@ -1261,44 +1221,30 @@ static inline long int cdw_cdio_read_sectors(cdw_cdio_disc_t *disc, track_t t, u
 
    \p fd must be file descriptior for file that is already
    open for writing.
+   \p sectors may not be -1
 
    \param disc - disc to read from
    \param t - track number of track to read sectors from
    \param fd - file descriptor to write the data to (already opened and with proper permissions)
+   \param n_sectors_total - number of sectors to read from track \p t
 
    \return 0 on success
    \return -1, -2 on function's internal errors
    \return -3 if write() didn't write all data passed to it, which may mean not enough available space on hard disc
-   \return 'errno - 5' on otehr write() errors, caller can interpret value of 'errno + 5' to check the error
+   \return 'errno - 5' on other write() errors, caller can interpret value of 'errno + 5' to check the error
 */
-int cdw_cdio_copy_sectors_to_file_sub(cdw_cdio_disc_t *disc, track_t t, int fd)
+int cdw_cdio_copy_sectors_to_file_sub(cdw_cdio_disc_t *disc, track_t t, int fd, long int n_sectors_total)
 {
 #define CDW_CDIO_CALC_N_SECTORS(n_left) \
 		(n_left >= CDW_CDIO_RW_N_SECTORS ? CDW_CDIO_RW_N_SECTORS : n_left % CDW_CDIO_RW_N_SECTORS);
-	/* I'm repeating here these asserts as in cdw_cdio_copy_tracks_to_file()
-	   because this function can be called either by
-	   cdw_cdio_copy_tracks_to_file() or directly by other modules */
-	cdw_assert (disc->p_cdio != (CdIo_t *) NULL, "ERROR: trying to get info from NULL disc\n");
-	cdw_assert (disc->open, "ERROR: trying to read from closed disc\n");
-	cdw_assert (fd != -1 && fd != 0, "ERROR: invalid output file: %d\n", fd);
-
-	if (disc->tracks[t].cdw_track_format == CDW_TRACK_BLACK_BOOK_UNKNOWN) {
-		cdw_vdm ("ERROR: called the function for black book (unknown) track\n");
-		return -1;
-	}
-
-	/* we are re-using the same processwin for all tracks,
-	   clean leftovers from reading previous track */
-	cdw_processwin_reset_progress();
-
-	cdw_cdio_copy_sectors_to_file_debug_print_state(disc);
 
 	/* buffer for data read from disc, CDIO_CD_FRAMESIZE_RAWER
 	   is maximal size of sector defined in cdio header files */
 	unsigned char buffer[CDIO_CD_FRAMESIZE_RAWER * CDW_CDIO_RW_N_SECTORS];
 
 	/* size of data left to read from ISO volume */
-	lsn_t n_sectors_left = disc->tracks[t].n_sectors_to_read;
+	lsn_t n_sectors_left = (lsn_t) n_sectors_total;
+
 	/* first sector of ISO volume to start reading from */
 	lsn_t sector = disc->tracks[t].first_sector;
 	/* number of sectors to read/write at one time;
@@ -1330,14 +1276,12 @@ int cdw_cdio_copy_sectors_to_file_sub(cdw_cdio_disc_t *disc, track_t t, int fd)
 				break;
 			}
 
-			lsn_t current_relative_sector = sector - disc->tracks[t].first_sector;
+			lsn_t relative_sector = sector - disc->tracks[t].first_sector;
 			/* don't update processwin too often */
-			if ((current_relative_sector % (40 * n_sectors)) == 0
+			if ((relative_sector % (40 * n_sectors)) == 0
 			     || n_sectors < CDW_CDIO_RW_N_SECTORS) {
 
-				lsn_t n_sectors_in_track = disc->tracks[t].n_sectors_to_read;
-				int sector_size = disc->tracks[t].sector_size;
-				cdw_cdio_update_processwin(current_relative_sector, n_sectors_in_track, sector_size);
+				cdw_cdio_update_processwin(relative_sector, (lsn_t) n_sectors_total, disc->tracks[t].sector_size);
 			}
 			sector += n_sectors;
 			n_sectors_left -= n_sectors;
@@ -1348,10 +1292,8 @@ int cdw_cdio_copy_sectors_to_file_sub(cdw_cdio_disc_t *disc, track_t t, int fd)
 		/* because this function reads N sectors at once, it may happen
 		   that it reads all data from disc, but last update of processwin
 		   shows "99%" instead of "100%". Let's fake "100%" here */
-		lsn_t current_relative_sector = disc->tracks[t].n_sectors_to_read - 1;
-		lsn_t n_sectors_in_track = disc->tracks[t].n_sectors_to_read;
-		int sector_size = disc->tracks[t].sector_size;
-		cdw_cdio_update_processwin(current_relative_sector, n_sectors_in_track, sector_size);
+		lsn_t relative_sector = (lsn_t) n_sectors_total - 1;
+		cdw_cdio_update_processwin(relative_sector, (lsn_t) n_sectors_total, disc->tracks[t].sector_size);
 
 		return 0;
 	} else {
@@ -1375,14 +1317,13 @@ void cdw_cdio_copy_print_debug_on_error(int error)
 	} else if (error < -3) {
 		/* cdw_cdio_copy_sectors_to_file() may
 		   return 'errno - 5' for some errors */
-		error = error + 5;
-		cdw_vdm ("ERROR: errno = %s\n", strerror(error));
+		cdw_vdm ("ERROR: errno = %s\n", strerror(error + 5));
 
 		/* 2TRANS: this is string written to log file when
 		   reading one track from CD disc is finished with
 		   problems; "%s" is system error message;
 		   it is written after "Track %d: " string */
-		cdw_logging_write(_("Maybe this error message will be helpful: \"%s\"\n"), strerror(error));
+		cdw_logging_write(_("Maybe this error message will be helpful: \"%s\"\n"), strerror(error + 5));
 	} else {
 		/* no more information to print */
 	}
@@ -1482,11 +1423,11 @@ int cdw_cdio_get_cd_sector_meta_data(cdw_cdio_disc_t *disc, lsn_t sector, unsign
 
 
 
-void cdw_cdio_update_processwin(lsn_t current_relative_sector, lsn_t n_sectors_in_track, int sector_size)
+void cdw_cdio_update_processwin(lsn_t relative_sector, lsn_t n_sectors, int sector_size)
 {
 	/* dividing by 4s to avoid overflows */
-	long current = ((current_relative_sector / 4) * (sector_size / 4)) / (256 * 256);
-	long total = ((n_sectors_in_track / 4) * (sector_size / 4)) / (256 * 256);
+	long current = ((relative_sector / 4) * (sector_size / 4)) / (256 * 256);
+	long total = ((n_sectors / 4) * (sector_size / 4)) / (256 * 256);
 	char current_value_string[PROCESSWIN_MAX_RTEXT_LEN + 1];
 
 	/* 2TRANS: this string will be displayed as message in progress window;
@@ -1666,7 +1607,7 @@ void cdw_cdio_print_current_disc(void)
 
 
 
-cdw_id_label_t driver_errors[] = {
+cdw_id_clabel_t driver_errors[] = {
 	{ DRIVER_OP_ERROR,          "DRIVER_OP_ERROR" },
 	{ DRIVER_OP_UNSUPPORTED,    "DRIVER_OP_UNSUPPORTED" },
 	{ DRIVER_OP_UNINIT,         "DRIVER_OP_UNINIT" },
@@ -1696,7 +1637,7 @@ const char *cdw_cdio_get_driver_error_label(driver_return_code_t error_num)
 
 static void cdw_cdio_log_handler(cdio_log_level_t level, const char *message)
 {
-	cdw_id_label_t handler_levels[] = {
+	cdw_id_clabel_t handler_levels[] = {
 		{ CDIO_LOG_DEBUG,  "debug"       },
 		{ CDIO_LOG_INFO,   "info"        },
 		{ CDIO_LOG_WARN,   "warning"     },
@@ -1741,9 +1682,10 @@ bool cdw_cdio_is_fs_iso(cdio_fs_t fs)
 	    || fs == CDIO_FS_ISO_HFS
 	    || fs == CDIO_FS_ISO_9660_INTERACTIVE) {
 
+		cdw_vdm ("INFO: file system %d is ISO9660\n", fs);
 		return true;
 	} else {
-		cdw_vdm ("INFO: file system is not ISO9660\n");
+		cdw_vdm ("INFO: file system %d is not ISO9660\n", fs);
 		return false;
 	}
 }
@@ -1755,3 +1697,84 @@ const char *cdw_cdio_get_volume_id(void)
 {
 	return g_disc.fs.volume_id;
 }
+
+
+
+
+long int cdw_cdio_get_fs_n_sectors(void)
+{
+	return g_disc.fs.n_sectors;
+}
+
+
+
+/* *** unused code below *** */
+
+#if 0
+
+
+/**
+   \brief Read tracks from current disc and write them to given file descriptor
+
+   More detailed description is following:
+       cdw_cdio_copy_tracks_to_file()
+   calls for every tracks
+       cdw_cdio_copy_sectors_to_file()
+   to read all tracks from disc.
+
+   This is just a wrapper for one 'for' loop iterating on tracks, one
+   function call inside loop, and some additional stuff.
+
+   \p output_file must be file descriptior for file that is already
+   open for writing.
+
+   \param fd - file descriptor of output file
+
+   \return CDW_OK when all tracks were copied successfully
+   \return CDW_NO when function copied some tracks (but not all) correctly
+   \return CDW_ERROR when function failed to read any track correctly
+*/
+cdw_rv_t cdw_cdio_copy_tracks_to_file(int fd)
+{
+	cdw_assert (g_disc.open, "ERROR: trying to read from closed disc\n");
+	cdw_assert (fd != -1 && fd != 0, "ERROR: invalid output file: %d\n", fd);
+
+	int failed_tracks = 0;
+
+	cdw_vdm ("INFO: number of tracks on disc to read: %d\n", g_disc.n_tracks);
+
+	for (track_t t = g_disc.first_track; t <= g_disc.last_track; t++) {
+		if (g_disc.tracks[t].cdw_track_format == CDW_TRACK_BLACK_BOOK_UNKNOWN) {
+			/* 2TRANS: this is message printed to log file;
+			   "Copy" is a name of task of copying track from
+			   CD to hard disc; %d is a track number */
+			cdw_logging_write(_("Copy: skipping track #%d with unknown format\n"), t);
+
+			failed_tracks++;
+			continue;
+		}
+
+		int rv = cdw_cdio_copy_sectors_to_file_sub(&g_disc, t, fd, sectors);
+		cdw_sdm ("INFO track #%d finished with rv = %d\n", t, rv);
+
+		if (rv != 0) {
+			failed_tracks++;
+			cdw_vdm ("  ERROR: failed reading track #%d\n", t);
+			cdw_cdio_copy_print_debug_on_error(rv);
+		}
+	}
+
+	if (failed_tracks == g_disc.n_tracks) {
+		cdw_vdm ("ERROR: number of tracks read correctly is zero\n")
+		return CDW_ERROR;
+	} else if (failed_tracks < g_disc.n_tracks && failed_tracks > 0) {
+		cdw_vdm ("WARNING: failed to read correctly %d tracks\n", failed_tracks);
+		return CDW_CANCEL;
+	} else {
+		return CDW_OK;
+	}
+}
+#endif
+
+
+
