@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2010 Kamil Ignacak
+ * Copyright (C) 2007 - 2011 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,6 @@
  */
 
 
-#include <math.h>
 #include <string.h>
 #include <stdlib.h>
 
@@ -29,6 +28,7 @@
 #include "cdw_ext_tools.h"
 #include "cdw_dvd_rw_mediainfo.h"
 #include "cdw_cdrecord.h"
+#include "cdw_xorriso.h"
 #include "gettext.h"
 #include "cdw_debug.h"
 #include "cdw_main_window.h"
@@ -128,9 +128,10 @@ static cdw_rv_t cdw_disc_get_meta_info_with_cdio(cdw_disc_t *disc);
 static cdw_rv_t cdw_disc_get_meta_info_with_external_tools(cdw_disc_t *disc);
 
 static cdw_rv_t cdw_disc_validate(cdw_disc_t *disc);
-static int cdw_disc_get_speed_auto_index(cdw_disc_t *disc);
+static int cdw_disc_get_default_drive_speed_id(cdw_disc_t *disc);
 static void cdw_disc_resolve_type_label(cdw_disc_t *disc);
 static void cdw_disc_resolve_write_speeds(cdw_disc_t *disc);
+static void cdw_disc_resolve_capacities(cdw_disc_t *disc);
 
 static void cdw_disc_debug_print_disc(cdw_disc_t *disc);
 
@@ -164,7 +165,7 @@ enum {
 	CDW_DISC_NOT_READABLE_OTHER
 };
 
-cdw_id_label_t disc_not_readable_reasons[] = {
+cdw_id_clabel_t disc_not_readable_reasons[] = {
 	{ CDW_DISC_NOT_READABLE_EMPTY,          (char *) NULL },
 	{ CDW_DISC_NOT_READABLE_NO_ISO9660,     (char *) NULL },
 	{ CDW_DISC_NOT_READABLE_NO_UDF,         (char *) NULL },
@@ -255,10 +256,13 @@ void cdw_disc_reset(cdw_disc_t *disc)
 	disc->type_erasable = CDW_UNKNOWN;
 	disc->type_writable = CDW_UNKNOWN;
 
+	disc->capacity.sectors_total = 0;
+	disc->capacity.sectors_used = 0;
+
 	disc->fs.type = -1;
 	disc->fs.type_label = (char *) NULL;
 	disc->fs.volume_id[0] = '\0';
-        disc->fs.n_sectors = -1;
+        disc->fs.n_sectors = 0;
 	disc->write_speeds.n_speeds = -1;
 	disc->write_speeds.drive_max_speed = -1;
 	disc->write_speeds.drive_default_speed = -1;
@@ -266,11 +270,16 @@ void cdw_disc_reset(cdw_disc_t *disc)
 		disc->write_speeds.speeds[i] = 0;
 	}
 
+	disc->accepted_disc_modes = CDW_DISC_MODE_INIT;
+
 	disc->media_info_source = CDW_TOOL_NONE;
 
 	disc->cdrecord_info.last_sess_start = 0;
 	disc->cdrecord_info.next_sess_start = 0;
 	disc->cdrecord_info.has_toc = false;
+	disc->cdrecord_info.phys_size = -1;
+	disc->cdrecord_info.start_of_lead_out = -1;
+	disc->cdrecord_info.rzone_size = 0;
 
 	disc->dvd_rw_mediainfo_info.read_capacity = -1;
 	/* 'e' = "error"; dvd+rw-mediainfo aborts reading metainfo of DVD-ROM
@@ -280,6 +289,23 @@ void cdw_disc_reset(cdw_disc_t *disc)
 	   handled correctly */
 	disc->dvd_rw_mediainfo_info.disc_status = 'e';
 
+	disc->dvd_rw_mediainfo_info.end_lba = -1;
+	disc->dvd_rw_mediainfo_info.all_tracks_size = -1;
+	disc->dvd_rw_mediainfo_info.data_tracks_size = -1;
+	disc->dvd_rw_mediainfo_info.last_track_size = -1;
+
+	disc->xorriso_info.is_written = CDW_UNKNOWN;
+	disc->xorriso_info.is_blank = CDW_UNKNOWN;
+	disc->xorriso_info.is_appendable = CDW_UNKNOWN;
+	disc->xorriso_info.is_closed = CDW_UNKNOWN;
+	disc->xorriso_info.sectors_used = -1;
+	disc->xorriso_info.sectors_total = -1;
+
+	disc->libburn_info.unformatted_size = 0;
+	disc->libburn_info.formatting_status = BURN_FORMAT_IS_UNKNOWN;
+	disc->libburn_info.read_capacity = -1;
+
+
 	/* take care of other things that should be recalculated based
 	   on the assignments made in this function */
 	cdw_disc_resolve(disc);
@@ -305,29 +331,15 @@ void cdw_disc_reset(cdw_disc_t *disc)
 
    \return correct index of speed in writing_speeds on success
 */
-int cdw_disc_get_initial_write_speed_index(cdw_disc_t *disc)
+int cdw_disc_get_initial_write_speed_id(cdw_disc_t *disc)
 {
 	cdw_assert (disc != (cdw_disc_t *) NULL, "ERROR: disc pointer is null\n");
 	cdw_assert (disc->write_speeds.n_speeds > 0, "ERROR: you forgot to call cdw_disc_validate()\nand now n_speeds is non-positive\n");
 
 	if (disc->write_speeds.n_speeds == 1) {
-		return 0;
+		return disc->write_speeds.speeds[0];
 	} else { /* n_speeds > 1 */
-		if (global_config.speed_range == SPEED_RANGE_LOWEST) {
-			return 0;
-		} else if (global_config.speed_range == SPEED_RANGE_HIGHEST) {
-			return disc->write_speeds.n_speeds - 1;
-		} else if (global_config.speed_range == SPEED_RANGE_MIDDLE) {
-			double mid = floor(((double) disc->write_speeds.n_speeds - 1)/2);
-			return (int) mid;
-		} else {
-			/* config.speed_range == SPEED_RANGE_AUTO; index of
-			   speed selected when user selects "auto" speed
-			   range in configuration; if drive does not provide
-			   default value, safe lowest value available
-			   (index = 0) will be fine */
-			return cdw_disc_get_speed_auto_index(disc);
-		}
+		return cdw_disc_get_default_drive_speed_id(disc);
 	}
 }
 
@@ -347,7 +359,7 @@ int cdw_disc_get_initial_write_speed_index(cdw_disc_t *disc)
    \param disc - disc variable into which to store the information
 
    \return CDW_OK if media info retrieved successfully
-   \return CDW_GEN_ERROR if function failed to retrieve full info
+   \return CDW_ERROR if function failed to retrieve full info
 */
 cdw_rv_t cdw_disc_get_meta_info(cdw_disc_t *disc)
 {
@@ -356,7 +368,7 @@ cdw_rv_t cdw_disc_get_meta_info(cdw_disc_t *disc)
 	cdw_rv_t crv = cdw_disc_get_meta_info_with_cdio(disc);
 	if (crv != CDW_OK) {
 		cdw_vdm ("ERROR: failed to open disc with cdio calls\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	if (disc->simple_type == CDW_DISC_SIMPLE_TYPE_CD
@@ -385,7 +397,7 @@ cdw_rv_t cdw_disc_get_meta_info(cdw_disc_t *disc)
 				   CDW_BUTTONS_OK, CDW_COLORS_ERROR);
 		cdw_main_ui_main_window_wrefresh();
 
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 
@@ -423,7 +435,7 @@ cdw_rv_t cdw_disc_get_meta_info(cdw_disc_t *disc)
 	crv = cdw_disc_validate(disc);
 	if (crv != CDW_OK) {
 		cdw_vdm ("ERROR: failed to validate disc information\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	} else {
 		cdw_disc_debug_print_disc(disc);
 		return CDW_OK;
@@ -448,7 +460,7 @@ cdw_rv_t cdw_disc_get_meta_info(cdw_disc_t *disc)
    \param disc - disc variable into which to store the information
 
    \return CDW_OK on success
-   \return CDW_GEN_ERROR on error
+   \return CDW_ERROR on error
 */
 cdw_rv_t cdw_disc_get_meta_info_with_cdio(cdw_disc_t *disc)
 {
@@ -457,7 +469,7 @@ cdw_rv_t cdw_disc_get_meta_info_with_cdio(cdw_disc_t *disc)
 	cdw_rv_t crv = cdw_cdio_open_disc();
 	if (crv != CDW_OK) {
 		cdw_vdm ("ERROR: failed to open disc with cdio calls\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	disc->simple_type = cdw_cdio_get_simple_type();
@@ -465,6 +477,7 @@ cdw_rv_t cdw_disc_get_meta_info_with_cdio(cdw_disc_t *disc)
 	disc->state_empty = cdw_cdio_is_blank() ? CDW_TRUE : CDW_FALSE;
 	disc->fs.type = cdw_cdio_get_fs_type();
 	disc->fs.type_label = cdw_cdio_get_fs_type_label();
+	disc->fs.n_sectors = cdw_cdio_get_fs_n_sectors();
 	disc->n_tracks = cdw_cdio_get_number_of_tracks();
 	/* cdw_cdio_get_volume_id() always returns pointer to valid C string
 	   no longer than ISO_MAX_VOLUME_ID + 1 (including ending '\0') */
@@ -485,7 +498,7 @@ cdw_rv_t cdw_disc_get_meta_info_with_cdio(cdw_disc_t *disc)
 				   _("Cannot get basic information about disc."),
 				   CDW_BUTTONS_OK, CDW_COLORS_ERROR);
 		cdw_main_ui_main_window_wrefresh();
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 	cdw_vdm ("INFO: disc simple type is \"%s\"\n", disc->simple_type_label);
 
@@ -506,53 +519,53 @@ cdw_rv_t cdw_disc_get_meta_info_with_cdio(cdw_disc_t *disc)
    \param disc - disc variable in which to store disc meta information
 
    \return CDW_OK on success
-   \return CDW_GEN_ERROR on errors
+   \return CDW_ERROR on errors
 */
 cdw_rv_t cdw_disc_get_meta_info_with_external_tools(cdw_disc_t *disc)
 {
 	cdw_assert (disc != (cdw_disc_t *) NULL, "ERROR: disc pointer is null\n");
 
-	CDW_TASK_CREATE_TASK(mediainfo_task, CDW_TASK_MEDIA_INFO);
-
-	cdw_rv_t crv = cdw_task_select_tools_for_task(&mediainfo_task, disc);
-	if (crv != CDW_OK) {
-		cdw_vdm ("ERROR: failed to find a program to get meta info for a disc\n");
-		/* 2TRANS: this is title of dialog window */
-		cdw_buttons_dialog(_("Error"),
-				   /* 2TRANS: this is message dialog window,
-				      cdw can't find a program like cdrecord or dvd+rw-mediainfo */
-				   _("Cannot find a program which cdw could use to get information about disc. Please check your settings in Configuration -> Tools."),
-				   CDW_BUTTONS_OK, CDW_COLORS_ERROR);
-		cdw_main_ui_main_window_wrefresh();
-		return CDW_GEN_ERROR;
+	cdw_task_t *task = cdw_task_new(CDW_TASK_MEDIA_INFO, disc);
+	if (task == (cdw_task_t *) NULL) {
+		cdw_vdm ("ERROR: failed to create a task\n");
+		return CDW_ERROR;
 	}
-
-	int tool_id = mediainfo_task.media_info.tool.id;
-	cdw_assert (tool_id == CDW_TOOL_CDRECORD || tool_id == CDW_TOOL_DVD_RW_MEDIAINFO,
-		    "ERROR: tool selector returns unsupported tool %d\n", tool_id);
-	cdw_assert (mediainfo_task.media_info.tool.fullpath != (char *) NULL,
+	cdw_id_t tool_id = task->media_info.tool.id;
+	cdw_assert (tool_id == CDW_TOOL_CDRECORD
+		    || tool_id == CDW_TOOL_XORRISO
+		    || tool_id == CDW_TOOL_DVD_RW_MEDIAINFO,
+		    "ERROR: tool selector returns unsupported tool %lld\n", tool_id);
+	cdw_assert (task->media_info.tool.label != (char *) NULL,
 		    "ERROR: tool selector returns NULL path to tool\n");
 
 	disc->media_info_source = tool_id;
+	cdw_rv_t crv = CDW_OK;
 	if (tool_id == CDW_TOOL_CDRECORD) {
 		/* 2TRANS: this is message displayed in process window */
 		cdw_processwin_display_main_info(_("Getting disc info with cdrecord"));
-
-		crv = cdw_cdrecord_run_task(&mediainfo_task, disc);
+		crv = cdw_cdrecord_run_task(task, disc);
 
 	} else if (tool_id == CDW_TOOL_DVD_RW_MEDIAINFO) {
 		/* 2TRANS: this is message displayed in process window */
 		cdw_processwin_display_main_info(_("Getting disc info with dvd+rw-mediainfo"));
+		crv = cdw_dvd_rw_mediainfo_run_task(task, disc);
 
-		crv = cdw_dvd_rw_mediainfo_run_task(&mediainfo_task, disc);
+	} else if (tool_id == CDW_TOOL_XORRISO) {
+		/* 2TRANS: this is message displayed in process window */
+		cdw_processwin_display_main_info(_("Getting disc info with xorriso"));
+		crv = cdw_xorriso_run_task(task, disc);
 	} else {
 		; /* covered by assert above */
 	}
+	cdw_rv_t tool_status = cdw_task_check_tool_status(task);
+	cdw_task_delete(&task);
 
-	if (!mediainfo_task.tool_status.ok || crv != CDW_OK) {
+	if (tool_status != CDW_OK ||  crv != CDW_OK) {
 		cdw_vdm ("ERROR: failed to get media info with %s\n",
-			 tool_id == CDW_TOOL_CDRECORD ? "cdrecord" : "dvd+rw-mediainfo");
-		return CDW_GEN_ERROR;
+			 tool_id == CDW_TOOL_CDRECORD ? "cdrecord" :
+			 tool_id == CDW_TOOL_XORRISO ? "xorriso" :
+			 tool_id == CDW_TOOL_DVD_RW_MEDIAINFO ? "dvd+rw-mediainfo" : "unknown tool");
+		return CDW_ERROR;
 	} else {
 		return CDW_OK;
 	}
@@ -562,48 +575,16 @@ cdw_rv_t cdw_disc_get_meta_info_with_external_tools(cdw_disc_t *disc)
 
 
 
-/**
-   \brief Return index to speed that is default writing speed of drive
-
-   In some situations cdw can read default speed of writing of a drive,
-   and this speed is considered a "default speed" - speed that would be
-   used when user (or cdw) didn't specify writing speed. User can decide
-   to use this default speed by selecting "auto" speed range in cdw
-   configuration. This function returns index of default write speed in
-   table of writing speeds.
-
-   If default drive speed is unknown the function returns zero.
-
-   TODO: allow the function to return -1, which would mean that
-   cdw shouldn't pass speed argument to cdrecord or growisofs - that would
-   always work as intended - cdrecord / growisofs or drive firmware would
-   use default speed itself. But that would require us to put "auto" label
-   into "writing speed" dropdown in write wizard.
-
-   \param disc - disc for which to return default speed index
-
-   \return index to table of writing speeds that points to default speed, or zero
-*/
-int cdw_disc_get_speed_auto_index(cdw_disc_t *disc)
+int cdw_disc_get_default_drive_speed_id(cdw_disc_t *disc)
 {
 	cdw_assert (disc != (cdw_disc_t *) NULL, "ERROR: disc pointer is null\n");
 
-	int auto_index = 0;
-	int def = disc->write_speeds.drive_default_speed;
-
-	if (def != 0) {
-		for (int i = 0; i < disc->write_speeds.n_speeds; i++) {
-			if (def == disc->write_speeds.speeds[i]) {
-				auto_index = i;
-				break;
-			}
-		}
+	if (disc->write_speeds.drive_default_speed == -1) {
+		/* default drive speed is unknown */
+		return disc->write_speeds.speeds[0];
+	} else {
+		return disc->write_speeds.drive_default_speed;
 	}
-
-	cdw_vdm ("INFO: auto speed index is %d, speed is %d\n",
-		 auto_index, disc->write_speeds.speeds[auto_index]);
-
-	return auto_index;
 }
 
 
@@ -623,6 +604,8 @@ int cdw_disc_get_speed_auto_index(cdw_disc_t *disc)
    order from highest to lowest. This function reverses the table to have
    values from lowest to highest.
 
+   Similar problem occurs when getting write speeds table with libburn.
+
    The function works on disc->write_speeds.speeds[].
 
    Caller has to make sure that number of (non-sorted) speeds is greater
@@ -632,52 +615,50 @@ int cdw_disc_get_speed_auto_index(cdw_disc_t *disc)
 */
 void cdw_disc_resolve_write_speeds(cdw_disc_t *disc)
 {
+#ifndef NDEBUG
 	cdw_assert (disc != (cdw_disc_t *) NULL, "ERROR: disc pointer is null\n");
-
 	cdw_vdm ("INFO: table of write speeds before resolving:\n");
 	for (int i = 0; i < CDW_DISC_N_SPEEDS_MAX; i++) {
-		if (disc->write_speeds.speeds[i] == 0) {
-			continue;
-		}
-		cdw_vdm ("INFO:   write_speeds[%d] = '%d'\n", i, disc->write_speeds.speeds[i]);
-	}
-
-	/* 1. compress speeds table */
-	disc->write_speeds.speeds[0] = disc->write_speeds.speeds[0];
-	int j = 1;
-	for (int i = 1; i < CDW_DISC_N_SPEEDS_MAX; i++) {
-		if (disc->write_speeds.speeds[i] == 0) {
-			continue;
-		}
-
-		if (disc->write_speeds.speeds[i] == disc->write_speeds.speeds[j - 1]) {
-			;
-		} else {
-			disc->write_speeds.speeds[j] = disc->write_speeds.speeds[i];
-			j++;
+		if (disc->write_speeds.speeds[i] != 0) {
+			cdw_vdm ("INFO:   write_speeds[%d] = '%d'\n", i, disc->write_speeds.speeds[i]);
 		}
 	}
+#endif
 
-	disc->write_speeds.n_speeds = j;
-
-	/* 2. sort speeds table */
 	/* number of speeds in table has been limited at the level of parsing
 	   cdrecord / dvd+rw-mediainfo output, so at this point we are
 	   sure that there is no more than CDW_DISC_N_SPEEDS_MAX
-	   speed values in write_speeds[] table */
+	   speed values in write_speeds[] table
 
-	cdw_assert (disc->write_speeds.n_speeds > 0,
-		    "ERROR: number of speed values in table is %d for disc type \"%s\"\n",
-		    disc->write_speeds.n_speeds, disc->simple_type_label);
-	cdw_vdm ("INFO: there are %d write speed value(s)\n", disc->write_speeds.n_speeds);
+	   output from libburn or dvd+rw-mediainfo may contain duplicates:
+	   example list of speeds from dvd+rw-mediainfo: 16, 16, 12, 10, 8, 4
+	   example list of speeds from libburn: 16, 12, 10, 8, 4, 16, 16, 12, 10, 8, 4, 16 */
 
+	/* algorithm of resolving speeds table:
+	   1. sort from highest to lowest (with possible zeros at the end of result table);
+	   2. remove duplicates (compress);
+	   3. count non-zero speeds to get number of valid speeds: n_speeds;
+	   4. sort from lowest to highest, but only first n_speeds items; */
+
+	qsort(disc->write_speeds.speeds, CDW_DISC_N_SPEEDS_MAX, sizeof(disc->write_speeds.speeds[0]), cdw_utils_compare_ints_reverse);
+
+	cdw_utils_compress_table_of_ints(disc->write_speeds.speeds, CDW_DISC_N_SPEEDS_MAX);
+
+	for (int i = 0; i < CDW_DISC_N_SPEEDS_MAX; i++) {
+		if (disc->write_speeds.speeds[i] == 0) {
+			disc->write_speeds.n_speeds = i;
+			break;
+		}
+	}
 
 	qsort(disc->write_speeds.speeds, (size_t) disc->write_speeds.n_speeds, sizeof(disc->write_speeds.speeds[0]), cdw_utils_compare_ints);
 
-	cdw_vdm ("INFO: table of write speeds after resolving:\n");
+#ifndef NDEBUG
+	cdw_vdm ("INFO: table of write speeds after resolving (%d speeds total):\n", disc->write_speeds.n_speeds);
 	for (int i = 0; i < disc->write_speeds.n_speeds; i++) {
 		cdw_vdm ("INFO:   write_speeds[%d] = '%d'\n", i, disc->write_speeds.speeds[i]);
 	}
+#endif
 
 	return;
 }
@@ -697,7 +678,7 @@ void cdw_disc_resolve_write_speeds(cdw_disc_t *disc)
 
    \param disc - disc to be checked
 
-   \return CDW_GEN_ERROR on errors in disc
+   \return CDW_ERROR on errors in disc
    \return CDW_OK if disc seems to be ok
 */
 cdw_rv_t cdw_disc_validate(cdw_disc_t *disc)
@@ -709,7 +690,7 @@ cdw_rv_t cdw_disc_validate(cdw_disc_t *disc)
 
 		/* simple_type _has to_ be known to do anything useful */
 		cdw_vdm ("ERROR: disc is neither CD nor DVD\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	if (disc->type == CDW_CD_AUDIO
@@ -723,11 +704,11 @@ cdw_rv_t cdw_disc_validate(cdw_disc_t *disc)
 	if (disc->type_writable == CDW_TRUE) {
 		if (disc->write_speeds.n_speeds == 0) {
 			cdw_vdm ("ERROR: no writing speeds for writable disc!\n");
-			return CDW_GEN_ERROR;
+			return CDW_ERROR;
 		} else if (disc->write_speeds.n_speeds == 1) {
 			if (disc->write_speeds.speeds[0] == 0) {
 				cdw_vdm ("ERROR: first and only writing speed is zero!\n");
-				return CDW_GEN_ERROR;
+				return CDW_ERROR;
 			}
 		} else {
 			; /* speeds table is non-empty, so it seems to be ok */
@@ -770,7 +751,7 @@ cdw_rv_t cdw_disc_validate(cdw_disc_t *disc)
 
    \return CDW_OK when there were no problems
    \return CDW_NO if disc type is NONE or UNKNOWN
-   \return CDW_GEN_ERROR on errors
+   \return CDW_ERROR on errors
 */
 cdw_rv_t cdw_disc_resolve(cdw_disc_t *disc)
 {
@@ -836,7 +817,7 @@ cdw_rv_t cdw_disc_resolve(cdw_disc_t *disc)
 		disc->type_writable = CDW_TRUE;
 	} else {
 		cdw_vdm ("ERROR: unknown disc type %d\n", t);
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 
@@ -849,11 +830,13 @@ cdw_rv_t cdw_disc_resolve(cdw_disc_t *disc)
 		cdw_cdrecord_set_disc_states(disc);
 	} else if (disc->media_info_source == CDW_TOOL_DVD_RW_MEDIAINFO) {
 		cdw_dvd_rw_mediainfo_set_disc_states(disc);
+	} else if (disc->media_info_source == CDW_TOOL_XORRISO) {
+		cdw_xorriso_set_disc_states(disc);
 	} else {
-		cdw_vdm ("ERROR: unknown media info tool in disc struct: %d\n",
+		cdw_vdm ("ERROR: unknown media info tool in disc struct: %lld\n",
 			 disc->media_info_source);
 
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 	if (empty != disc->state_empty) {
 		cdw_vdm ("WARNING: external tool changes \"state empty\"\n");
@@ -905,6 +888,10 @@ cdw_rv_t cdw_disc_resolve(cdw_disc_t *disc)
 		/* writable disc has some write speeds that
 		   need to be taken care of */
 		cdw_disc_resolve_write_speeds(disc);
+
+		/* calculating capacity makes sense only for
+		   writable discs */
+		cdw_disc_resolve_capacities(disc);
 	}
 
 	return CDW_OK;
@@ -938,11 +925,10 @@ void cdw_disc_resolve_type_label(cdw_disc_t *disc)
 		strncpy(l, cdw_disc_type_labels[CDW_DISC_TYPE_UNKNOWN], CDW_DISC_TYPE_LABEL_LEN);
 	} else {
 		strncpy(l, cdw_disc_type_labels[disc->type], CDW_DISC_TYPE_LABEL_LEN);
-
 	}
 	l[CDW_DISC_TYPE_LABEL_LEN] = '\0';
 
-	cdw_vdm ("INFO: disc type label resolved as \"%s\"\n", l);
+	cdw_vdm ("INFO: disc type label %d resolved as \"%s\"\n", disc->type, l);
 	return;
 }
 
@@ -1015,7 +1001,7 @@ const char *cdw_disc_check_if_is_readable(cdw_disc_t *disc)
 				return (char *) NULL;
 			} else if (crv == CDW_NO) {
 				return disc_not_readable_reasons[CDW_DISC_NOT_READABLE_NO_ISO9660].label;
-			} else { /* CDW_CANCEL || CDW_GEN_ERROR */
+			} else { /* CDW_CANCEL || CDW_ERROR */
 				return disc_not_readable_reasons[CDW_DISC_NOT_READABLE_OTHER].label;
 			}
 #endif
@@ -1049,6 +1035,10 @@ void cdw_disc_debug_print_disc(cdw_disc_t *disc)
 	cdw_vdm ("CURRENT DISC:  type writable = \"%s\"\n", cdw_utils_get_cdw_bool_type_label(disc->type_writable));
 	cdw_vdm ("CURRENT DISC:  type erasable = \"%s\"\n", cdw_utils_get_cdw_bool_type_label(disc->type_erasable));
 
+	float coeff = 512.0; /* (n_sectors * 2048) / (1024.0 * 1024.0) = n_sectors / 512.0 */
+	cdw_vdm ("CURRENT DISC CAPACITY:  used = %lld (%.3f MB)\n", disc->capacity.sectors_used, (double) disc->capacity.sectors_used / coeff);
+	cdw_vdm ("CURRENT DISC CAPACITY: total = %lld (%.3f MB)\n", disc->capacity.sectors_total, (double) disc->capacity.sectors_total / coeff);
+
 	if (disc->state_empty == CDW_TRUE) {
 		return;
 	}
@@ -1066,3 +1056,63 @@ void cdw_disc_debug_print_disc(cdw_disc_t *disc)
 	return;
 }
 
+
+
+
+
+void cdw_disc_resolve_capacities(cdw_disc_t *disc)
+{
+	if (disc->cdrecord_info.rzone_size > 0
+	    && disc->cdrecord_info.rzone_size > disc->cdrecord_info.phys_size) {
+
+		/* sometimes rzone size is a great indicator of total
+		   disc size, but, alas, only sometimes */
+
+		cdw_vdm ("INFO: using cdrecord info (DVD rzone size)\n");
+		disc->capacity.sectors_total = disc->cdrecord_info.rzone_size;
+		disc->capacity.sectors_used = disc->fs.n_sectors;
+	} else if (disc->cdrecord_info.phys_size != -1) {
+		cdw_vdm ("INFO: using cdrecord info (DVD phys size)\n");
+		disc->capacity.sectors_total = disc->cdrecord_info.phys_size;
+		disc->capacity.sectors_used = disc->fs.n_sectors;
+	} else if (disc->cdrecord_info.start_of_lead_out != -1) {
+		cdw_vdm ("INFO: using cdrecord info (CD)\n");
+		disc->capacity.sectors_total = disc->cdrecord_info.start_of_lead_out;
+		disc->capacity.sectors_used = disc->fs.n_sectors;
+	} else if (disc->dvd_rw_mediainfo_info.end_lba != -1) {
+		cdw_vdm ("INFO: using dvd+rw-mediainfo info\n");
+		disc->capacity.sectors_total = disc->dvd_rw_mediainfo_info.end_lba;
+		disc->capacity.sectors_used = disc->fs.n_sectors;
+	} else if (disc->xorriso_info.sectors_total != -1) {
+		cdw_vdm ("INFO: using xorriso info\n");
+		disc->capacity.sectors_total = disc->xorriso_info.sectors_total;
+		disc->capacity.sectors_used = disc->xorriso_info.sectors_used >= 0 ? disc->xorriso_info.sectors_used : disc->fs.n_sectors;
+	} else if (disc->libburn_info.unformatted_size != 0) {
+		cdw_vdm ("INFO: using libburn info\n");
+		disc->capacity.sectors_total = disc->libburn_info.unformatted_size / 2048;
+		disc->capacity.sectors_used = 0;
+		if (disc->fs.n_sectors > 0) {
+			cdw_vdm ("WARNING: assuming that disc is empty while fs size != 0 (%ld)\n", disc->fs.n_sectors);
+		}
+	} else {
+		cdw_vdm ("WARNING: no info available\n");
+	}
+	return;
+}
+
+
+
+
+
+bool cdw_disc_is_disc_type_writable(cdw_disc_t *disc)
+{
+	if (disc->type == CDW_DISC_TYPE_UNKNOWN
+	    || disc->type == CDW_CD_AUDIO
+	    || disc->type == CDW_CD_ROM
+	    || disc->type == CDW_DVD_ROM) {
+
+		return false;
+	} else {
+		return true;
+	}
+}
