@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2010 Kamil Ignacak
+ * Copyright (C) 2007 - 2011 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -62,11 +62,11 @@ static struct {
 	   0 to N as regular indexes; because of using "-1" the name of
 	   variable is "id"; for almost all cases id is a valid index;
 	   except for -1; */
-	int current_drive_id;
+	cdw_id_t current_drive_id;
 } drives;
 
 
-static cdw_id_label_t drive_requests[] = {
+static cdw_id_clabel_t drive_requests[] = {
 	{ CDROMRESET,         "CDROMRESET" },
 	{ CDROMEJECT,         "CDROMEJECT" },
 	{ CDROMCLOSETRAY,     "CDROMCLOSETRAY" },
@@ -78,7 +78,6 @@ static cdw_id_label_t drive_requests[] = {
 
 void cdw_drive_init(void)
 {
-	drives.drives_dropdown = (CDW_DROPDOWN *) NULL;
 	drives.n_drives = cdw_cdio_drives_get_n_drives();
 	if (drives.n_drives == 0) {
 		drives.current_drive_id = CDW_DRIVE_ID_CUSTOM;
@@ -87,14 +86,24 @@ void cdw_drive_init(void)
 
 	drives.cdio_default_drive_ind = cdw_cdio_drives_get_cdio_default_drive_ind();
 
+	/* global_config.selected_drive is a string describing currently
+	   selected drive; it can have three values:
+	    - "default": use drive that is indicated as "default" by cdio
+	       module; in practice this value happens only when there is
+	       no cdw config file, and cdw runs with default settings;
+	       there is no "select default drive" option;
+	    - "custom": this value of "selected_drive" indicates that
+	      the path to "real" drive is in global_config.custom_drive
+	    - '/dev/xyz': a path to device file in form of "/dev/xyz";
+	      the value is selected from a dropdown, and since the dropdown
+	      is built from values provided by cdio module, the value
+	      should be present in cdw_cdio_drives[] */
+
+	/* find correct index to table of drives, regardless of value
+	   of global_config.selected_drive */
 	if (! strcmp(global_config.selected_drive, "default")) {
-		/* use cdio default; in practice this happens only when
-		   there is no cdio config file, and cdw runs with default
-		   settings; there is no "select default drive" option */
 		drives.current_drive_id = drives.cdio_default_drive_ind;
 	} else if (! strcmp(global_config.selected_drive, "custom")) {
-		/* use custom value entered by user in a field
-		   in configuration window */
 		drives.current_drive_id = CDW_DRIVE_ID_CUSTOM;
 	} else {
 		/* we need to search for device path from cdw_drive
@@ -130,9 +139,6 @@ void cdw_drive_init(void)
 
 void cdw_drive_clean(void)
 {
-	if (drives.drives_dropdown != (CDW_DROPDOWN *) NULL) {
-		cdw_dropdown_delete(&drives.drives_dropdown);
-	}
 	return;
 }
 
@@ -202,7 +208,7 @@ cdw_rv_t cdw_drive_eject_tray(const char *device_fullpath)
 		}
 	}
 	cdw_vdm ("ERROR: failed to eject a drive\n");
-	return CDW_GEN_ERROR;
+	return CDW_ERROR;
 }
 
 
@@ -220,14 +226,14 @@ cdw_rv_t cdw_drive_eject_tray(const char *device_fullpath)
    \param device_fullpath - full path to '/dev' file corresponding to device
 
    \return CDW_OK on success
-   \return CDW_SYS_ERROR if function cannot open() device or if ioctl call for this device fails
+   \return CDW_ERROR if function cannot open() device or if ioctl call for this device fails
  */
 cdw_rv_t cdw_drive_close_tray(const char *device_fullpath)
 {
 	int rv = cdw_drive_execute_request(device_fullpath, CDROMCLOSETRAY);
 	if (rv == -1) {
 		cdw_vdm ("ERROR: ioctl(%s, CDROMCLOSETRAY, ...) returns -1\n", device_fullpath);
-		return CDW_SYS_ERROR;
+		return CDW_ERROR;
 	} else {
 		/* give your drive some time to scan disc after closing
 		   tray; the drive really may need 30 seconds; on my
@@ -238,7 +244,7 @@ cdw_rv_t cdw_drive_close_tray(const char *device_fullpath)
 			rv = cdw_drive_status(device_fullpath);
 			if (rv == -1) {
 				cdw_vdm ("ERROR: can't get drive status\n");
-				return CDW_SYS_ERROR;
+				return CDW_ERROR;
 			} else if (rv == CDS_DRIVE_NOT_READY) {
 				cdw_vdm ("INFO: waiting for drive, second %d\n", i);
 				sleep(1);
@@ -260,9 +266,9 @@ int cdw_drive_execute_request(const char *device_fullpath, long unsigned int req
 	/* yes, I know about sloppy casting of long unsigned int to
 	   long signed int, but this is "only" debug code;
 	   FIXME: fix this somehow; but how?
-	   1. make int field in cdw_id_label_t of unsigned type, but
+	   1. make int field in cdw_id_clabel_t of unsigned type, but
 	      IDs can be negative
-	   2. make two cdw_id_label_t types - for singed and unsigned int
+	   2. make two cdw_id_clabel_t types - for singed and unsigned int
 	      field - maintenance nightmare */
 
 	int cddev = open(device_fullpath, O_RDONLY | O_NONBLOCK);
@@ -409,15 +415,7 @@ bool cdw_drive_is_disc_in_drive(void)
  */
 void cdw_drive_reload_tray_with_ui_update(const char *device_fullpath, bool final_eject)
 {
-	/* 2TRANS: this is message displayed in process window */
-	cdw_processwin_display_sub_info(_("Ejecting tray"));
-	cdw_processwin_wrefresh();
-	cdw_drive_eject_tray(device_fullpath);
-
-	cdw_disc_t *current_disc = cdw_disc_get();
-	cdw_disc_reset(current_disc);
-	cdw_main_ui_disc_info_view_display_data(current_disc);
-
+	cdw_drive_eject_tray_with_ui_update(device_fullpath);
 	if (final_eject) {
 		if (global_config.eject) { /* leave tray open */
 			return;
@@ -428,7 +426,37 @@ void cdw_drive_reload_tray_with_ui_update(const char *device_fullpath, bool fina
 	   performing writing operation (this is when config.eject == false)
 	   or cdw will be doing something more on the disc (this is when
 	   final_eject == false) */
+	cdw_drive_close_tray_with_ui_update(device_fullpath);
 
+	return;
+}
+
+
+
+
+
+void cdw_drive_eject_tray_with_ui_update(const char *device_fullpath)
+{
+	/* 2TRANS: this is message displayed in process window */
+	cdw_processwin_display_sub_info(_("Ejecting tray"));
+	cdw_processwin_wrefresh();
+	cdw_drive_eject_tray(device_fullpath);
+
+	cdw_disc_t *current_disc = cdw_disc_get();
+	cdw_disc_reset(current_disc);
+	cdw_main_ui_disc_info_view_display_data(current_disc);
+	/* update information about disc size and usage */
+	cdw_main_window_volume_info_view_update(false, global_config.follow_symlinks);
+
+	return;
+}
+
+
+
+
+
+void cdw_drive_close_tray_with_ui_update(const char *device_fullpath)
+{
 	/* 2TRANS: this is message displayed in process window;
 	   "tray" is an optical drive tray */
 	cdw_processwin_display_sub_info(_("Closing tray, waiting for drive"));
@@ -438,12 +466,16 @@ void cdw_drive_reload_tray_with_ui_update(const char *device_fullpath, bool fina
 	usleep(500000);
 	cdw_drive_close_tray(device_fullpath);
 
+	cdw_disc_t *current_disc = cdw_disc_get();
+	cdw_disc_reset(current_disc);
 	cdw_rv_t mi = cdw_disc_get_meta_info(current_disc);
 	if (mi == CDW_OK) {
 		/* 'current_disc' still points to one and only 'current_disc'
 		   variable, which only gets updated, not reallocated, after
 		   cdw_disc_get_meta_info() call */
 		cdw_main_ui_disc_info_view_display_data(current_disc);
+		/* update information about disc size and usage */
+		cdw_main_window_volume_info_view_update(false, global_config.follow_symlinks);
 	} else {
 		; /* disc info view is not updated, but user can press
 		     'R' key to try to refresh it */
@@ -459,7 +491,7 @@ void cdw_drive_reload_tray_with_ui_update(const char *device_fullpath, bool fina
 int cdw_drive_toggle_tray2(const char *device_fullpath, bool with_processwin)
 {
 	cdw_rv_t m = cdw_fs_check_device_mounted(device_fullpath);
-	if (m == CDW_SYS_ERROR) {
+	if (m == CDW_ERROR) {
 		; /* let's try anyway */
 	} else if (m == CDW_OK) { /* device is mounted */
 		/* 2TRANS: this is title of dialog window */
@@ -467,7 +499,7 @@ int cdw_drive_toggle_tray2(const char *device_fullpath, bool with_processwin)
 				   /* 2TRANS: this is message in dialog window */
 				   _("Cannot eject tray. Please unmount your drive."),
 				   CDW_BUTTONS_OK, CDW_COLORS_WARNING);
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	} else { /* crv == CDW_NO, not mounted */
 		;
 	}
@@ -581,6 +613,8 @@ void cdw_drive_toggle_tray_with_ui_update2(void)
 	}
 
 	cdw_main_ui_disc_info_view_display_data(current_disc);
+	/* update information about disc size and usage */
+	cdw_main_window_volume_info_view_update(false, global_config.follow_symlinks);
 	/* 2TRANS: this is message in process window,
 	    "drive" is "optical drive" - a burner */
 	cdw_processwin_destroy(_("Drive state changed"), false);
@@ -596,7 +630,7 @@ CDW_DROPDOWN *cdw_drive_make_drives_dropdown(WINDOW *window, int row, int col, i
 {
 #define LABEL_LEN 50
 
-	drives.drives_dropdown = cdw_dropdown_new(window, row, col, width,
+	CDW_DROPDOWN *dropdown = cdw_dropdown_new(window, row, col, width,
 						  drives.n_drives + 1, CDW_COLORS_DIALOG);
 
 
@@ -606,10 +640,10 @@ CDW_DROPDOWN *cdw_drive_make_drives_dropdown(WINDOW *window, int row, int col, i
 	   "path to a device file, specified by user" */
 	strncpy (label, _("Use custom path to drive"), max_len);
 	label[LABEL_LEN] = '\0';
-	cdw_rv_t crv = cdw_dropdown_add_item(drives.drives_dropdown, CDW_DRIVE_ID_CUSTOM, label);
+	cdw_rv_t crv = cdw_dropdown_add_item(dropdown, CDW_DRIVE_ID_CUSTOM, label);
 	if (crv != CDW_OK) {
 		cdw_vdm ("ERROR: failed to add first label \"%s\" to dropdown\n", label);
-		cdw_dropdown_delete(&drives.drives_dropdown);
+		cdw_dropdown_delete(&dropdown);
 		return (CDW_DROPDOWN *) NULL;
 	}
 
@@ -621,23 +655,23 @@ CDW_DROPDOWN *cdw_drive_make_drives_dropdown(WINDOW *window, int row, int col, i
 			 cdw_cdio_drives[i].hw_info.psz_model);
 
 		cdw_vdm ("INFO: adding dropdown label #%d = \"%s\"\n", i, label);
-		crv = cdw_dropdown_add_item(drives.drives_dropdown, i, label);
+		crv = cdw_dropdown_add_item(dropdown, i, label);
 		if (crv != CDW_OK) {
 			cdw_vdm ("ERROR: failed to add first label \"%s\" to dropdown\n", label);
-			cdw_dropdown_delete(&drives.drives_dropdown);
+			cdw_dropdown_delete(&dropdown);
 			return (CDW_DROPDOWN *) NULL;
 		}
 	}
-	cdw_dropdown_finalize(drives.drives_dropdown);
+	cdw_dropdown_finalize(dropdown);
 
-	crv = cdw_dropdown_set_current_item_by_id(drives.drives_dropdown, drives.current_drive_id);
+	crv = cdw_dropdown_set_current_item_by_id(dropdown, drives.current_drive_id);
 	if (crv != CDW_OK) {
 		cdw_vdm ("ERROR: failed to set current item\n");
 		/* what now? */
 	}
-	cdw_dropdown_display_current_item(drives.drives_dropdown);
+	cdw_dropdown_display_current_item(dropdown);
 
-	return drives.drives_dropdown;
+	return dropdown;
 }
 
 
@@ -666,7 +700,7 @@ const char *cdw_drive_get_drive_fullpath(void)
 		return path;
 	} else {
 		const char *path = cdw_cdio_drives[drives.current_drive_id].fullpath;
-		cdw_vdm ("INFO: selecting cdio drive path #%d = \"%s\"\n", drives.current_drive_id, path);
+		cdw_vdm ("INFO: selecting cdio drive path %lld = \"%s\"\n", drives.current_drive_id, path);
 		return path;
 	}
 }
@@ -674,7 +708,7 @@ const char *cdw_drive_get_drive_fullpath(void)
 
 
 
-
+#if 0
 cdw_rv_t cdw_drive_print_help_message(WINDOW *window)
 {
 #if 0
@@ -698,20 +732,80 @@ cdw_rv_t cdw_drive_print_help_message(WINDOW *window)
 		 (char *) NULL);
 
 	cdw_textarea_print_message(window, message, CDW_ALIGN_LEFT);
+	wrefresh(window);
 	free(message);
 	message = (char *) NULL;
 #endif
 	return CDW_OK;
 }
 
+#endif
 
 
 
+char *cdw_drive_get_help_message(void)
+{
+#if 1
+	char *message = cdw_string_concat
+		/* 2TRANS: this is a help message */
+		(drives.n_drives == 0 ? _("cdw can't detect any drives, you may want to enter and use custom value.\n") : "",
+		 /* here -1 is only an initial value */
+		 drives.n_drives != 0 && drives.cdio_default_drive_ind != -1 ?
+		 /* 2TRANS: this is a help message, describing content
+		    of a dropdown */
+		 _("'*' in dropdown denotes default drive.\n") : "",
+		 /* 2TRANS: this is a help message, keep \"/dev/xxx\" unchanged */
+		 _("Custom path to device should have form \"/dev/xxx\", without ending slash.\n\"SCSI device\" field is for experts only, and if filled, cdw may not work correctly.\n"),
+		 (char *) NULL);
+#else
+	/* test message */
+	char *message = strdup("dddddddddddwwwwwwwwwwwwiiiiiiiiiissssssllllllllllllbbbbbbbbboooooooooooeeeeeejjjjjjjd");
+#endif
+	return message;
+}
 
-void cdw_drive_save_config(cdw_config_t *config)
+
+
+
+
+/**
+   \brief Save changes made in configuration window into "cdw drive" data structures
+
+   Function saves some options of current configuration into
+   variables that are private to "cdw drive" module.
+
+   There are also other options in configuration window, but their values
+   should be saved to config variable - use cdw_drive_set_config_fields
+   to do this.
+
+   \p id may be either a "real" ID (CDW_DRIVE_ID_CUSTOM), or non-negative
+   index of one of dropdown items. CDW_DRIVE_ID_CUSTOM is an ID of first
+   item in the dropdown, following IDs are indexed starting from zero.
+
+   \param id - id of current item in drive dropdown in config window;
+*/
+void cdw_drive_save_configuration(cdw_id_t id)
 {
-	int id = cdw_dropdown_get_current_item_id(drives.drives_dropdown);
 	drives.current_drive_id = id;
+	return;
+
+}
+
+
+
+
+
+/**
+   \brief Save values of selected configuration fields in config variable
+
+   Function saves value of drive dropdown in appropriate field of config
+   variable.
+
+   \param config - config variable in which to save state of "drive" dropdown
+   \param id - id of current item of "drive" dropdown
+*/
+void cdw_drive_set_config_fields(cdw_config_t *config, cdw_id_t id)
+{
 	if (id == CDW_DRIVE_ID_CUSTOM) {
 		/* current selection is "use custom drive path" */
 		strncpy(config->selected_drive, "custom", OPTION_FIELD_LEN_MAX);
@@ -724,15 +818,14 @@ void cdw_drive_save_config(cdw_config_t *config)
 		cdw_assert (!strcmp(config->selected_drive, cdw_cdio_drives[id].fullpath), "ERROR: failed to correctly save non-custom value \"%s\"\n", cdw_cdio_drives[id].fullpath);
 	}
 
-	cdw_vdm ("INFO: cdw drive saved as \"%s\"\n", config->selected_drive);
-
 	return;
 }
 
 
 
 
-int cdw_drive_get_current_drive_id(void)
+
+cdw_id_t cdw_drive_get_current_drive_id(void)
 {
 	return drives.current_drive_id;
 }
