@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2010 Kamil Ignacak
+ * Copyright (C) 2007 - 2011 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,18 +31,20 @@
 #include "cdw_thread.h" /* run_command() */
 #include "cdw_widgets.h"
 #include "cdw_config.h"
-#include "cdw_config_ui.h"
+#include "cdw_config_window.h"
 #include "cdw_string.h"
 #include "cdw_utils.h"
 #include "cdw_processwin.h"
 #include "cdw_mkisofs.h"
 #include "cdw_cdrecord.h"
+#include "cdw_cdrecord_options.h"
 #include "cdw_iso9660.h"
 #include "cdw_logging.h"
 #include "cdw_debug.h"
 #include "cdw_ext_tools.h"
 #include "cdw_erase_disc.h"
 #include "cdw_drive.h"
+#include "cdw_fs.h"
 
 /**
   you may want to take a look at this page:
@@ -145,7 +147,6 @@ track:lout lba:    114124 (   456496) 25:23:49 adr: 1 control: 4 mode: -1
 \endverbatim
 */
 
-extern cdw_config_t global_config;      /* variable holding cdw configuration */
 
 static cdw_rv_t cdw_cdrecord_write_from_image(cdw_task_t *task, cdw_disc_t *disc);
 static cdw_rv_t cdw_cdrecord_write_from_files(cdw_task_t *task, cdw_disc_t *disc);
@@ -156,7 +157,7 @@ static char *cdw_cdrecord_create_command_for_writing(cdw_task_t *task, cdw_disc_
 static void cdw_cdrecord_set_disc_mode_arg(cdw_task_t *task, char *disc_mode);
 static bool cdw_cdrecord_add_multisession_info(cdw_task_t *task);
 
-static void cdw_cdrecord_internal_error_dialog(int id);
+static void cdw_cdrecord_internal_error_dialog(cdw_id_t id);
 
 #define TSIZE_LEN 20
 
@@ -185,19 +186,19 @@ cdw_rv_t cdw_cdrecord_run_task(cdw_task_t *task, cdw_disc_t *disc)
 {
 	cdw_rv_t crv = CDW_OK;
 	if (task->id == CDW_TASK_BURN_FROM_FILES) {
-		cdw_assert (task->burn.tool.id == CDW_TOOL_CDRECORD, "ERROR: incorrect tool id %d for task \"burn from files\"\n", task->burn.tool.id);
+		cdw_assert (task->burn.tool.id == CDW_TOOL_CDRECORD, "ERROR: incorrect tool id %lld for task \"burn from files\"\n", task->burn.tool.id);
 		crv = cdw_cdrecord_write_from_files(task, disc);
 	} else if (task->id == CDW_TASK_BURN_FROM_IMAGE) {
-		cdw_assert (task->burn.tool.id == CDW_TOOL_CDRECORD, "ERROR: incorrect tool id %d for task \"burn from image\"\n", task->burn.tool.id);
+		cdw_assert (task->burn.tool.id == CDW_TOOL_CDRECORD, "ERROR: incorrect tool id %lld for task \"burn from image\"\n", task->burn.tool.id);
 		crv = cdw_cdrecord_write_from_image(task, disc);
 	} else if (task->id == CDW_TASK_MEDIA_INFO) {
-		cdw_assert (task->media_info.tool.id == CDW_TOOL_CDRECORD, "ERROR: incorrect tool id %d for task \"media info\"\n", task->media_info.tool.id);
+		cdw_assert (task->media_info.tool.id == CDW_TOOL_CDRECORD, "ERROR: incorrect tool id %lld for task \"media info\"\n", task->media_info.tool.id);
 		crv = cdw_cdrecord_get_meta(task, disc);
 	} else if (task->id == CDW_TASK_ERASE_DISC) {
-		cdw_assert (task->erase.tool.id == CDW_TOOL_CDRECORD, "ERROR: incorrect tool id %d for task \"erase\"\n", task->erase.tool.id);
+		cdw_assert (task->erase.tool.id == CDW_TOOL_CDRECORD, "ERROR: incorrect tool id %lld for task \"erase\"\n", task->erase.tool.id);
 		crv = cdw_cdrecord_erase_disc(task, disc);
 	} else {
-		cdw_assert (0, "ERROR: incorrect task->id %d\n", task->id);
+		cdw_assert (0, "ERROR: incorrect task->id %lld\n", task->id);
 	}
 	return crv;
 }
@@ -213,7 +214,7 @@ cdw_rv_t cdw_cdrecord_run_task(cdw_task_t *task, cdw_disc_t *disc)
    image to optical disc. Run the command, show a window with process
    information to the user.
 
-   Valid file image path should be set in preferences (config.tempdir).
+   Valid file image path should be set in options.
    Function does not check validity of the path.
 
    \pre \p task->source is set to OBJ_ISO_IMAGE.
@@ -231,7 +232,7 @@ cdw_rv_t cdw_cdrecord_write_from_image(cdw_task_t *task, cdw_disc_t *disc)
 	cdw_assert (task->burn.disc_mode != CDW_DISC_MODE_INIT,
 		    "ERROR: disc mode is not set properly\n");
 	cdw_assert (task->id == CDW_TASK_BURN_FROM_IMAGE,
-		    "ERROR: incorrect task id %d\n", task->id);
+		    "ERROR: incorrect task id %lld\n", task->id);
 
 	/* the function will examine 'taks' to check if data
 	   should be read from ISO file */
@@ -244,18 +245,21 @@ cdw_rv_t cdw_cdrecord_write_from_image(cdw_task_t *task, cdw_disc_t *disc)
 				      'image file' refers to 'ISO image file' */
 				   _("Failed to prepare burning image file to disc"),
 				   CDW_BUTTONS_OK, CDW_COLORS_ERROR);
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	cdw_vdm ("INFO: final command for writing image to optical disc:\n");
 	cdw_vdm ("INFO: \"%s\"\n", command);
 
-	run_command(command, task);
-
+	int rv = run_command(command, task);
 	free(command);
 	command = (char *) NULL;
 
-	return CDW_OK;
+	if (rv == 0) {
+		return CDW_OK;
+	} else {
+		return CDW_ERROR;
+	}
 }
 
 
@@ -284,7 +288,7 @@ cdw_rv_t cdw_cdrecord_write_from_image(cdw_task_t *task, cdw_disc_t *disc)
    \param task - variable describing current task
 
    \return CDW_OK on success
-   \return CDW_GEN_ERROR on failure
+   \return CDW_ERROR on failure
  */
 cdw_rv_t cdw_cdrecord_write_from_files(cdw_task_t *task, cdw_disc_t *disc)
 {
@@ -293,7 +297,7 @@ cdw_rv_t cdw_cdrecord_write_from_files(cdw_task_t *task, cdw_disc_t *disc)
 	cdw_assert (task->burn.disc_mode != CDW_DISC_MODE_INIT,
 		    "ERROR: disc mode is not set properly\n");
 	cdw_assert (task->id == CDW_TASK_BURN_FROM_FILES,
-		    "ERROR: incorrect task id %d\n", task->id);
+		    "ERROR: incorrect task id %lld\n", task->id);
 
 	char *mkisofs_command = (char *) NULL;
 	char *tsize_command = (char *) NULL;
@@ -308,7 +312,7 @@ cdw_rv_t cdw_cdrecord_write_from_files(cdw_task_t *task, cdw_disc_t *disc)
 		cdw_vdm ("ERROR: failed to prepare mkisofs command string\n");
 
 		cdw_cdrecord_internal_error_dialog(1);
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 	cdw_vdm ("INFO: mkisofs command for creating ISO file system:\n");
 	cdw_vdm ("INFO: \"%s\"\n", mkisofs_command);
@@ -321,7 +325,7 @@ cdw_rv_t cdw_cdrecord_write_from_files(cdw_task_t *task, cdw_disc_t *disc)
 		cdw_cdrecord_internal_error_dialog(2);
 		CDRECORD_WRITE_DATA_FREE_RESOURCES;
 		cdw_vdm ("ERROR: failed to prepare tsize command string\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 	cdw_vdm ("INFO: mkisofs command for calculating tsize value:\n");
 	cdw_vdm ("INFO: \"%s\"\n", tsize_command);
@@ -337,7 +341,7 @@ cdw_rv_t cdw_cdrecord_write_from_files(cdw_task_t *task, cdw_disc_t *disc)
 		cdw_cdrecord_internal_error_dialog(3);
 		CDRECORD_WRITE_DATA_FREE_RESOURCES;
 
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 	cdw_vdm ("INFO: cdrecord command for burning data incoming from stdin:\n");
 	cdw_vdm ("INFO: \"%s\"\n", cdrecord_command);
@@ -350,7 +354,7 @@ cdw_rv_t cdw_cdrecord_write_from_files(cdw_task_t *task, cdw_disc_t *disc)
 		cdw_cdrecord_internal_error_dialog(4);
 		CDRECORD_WRITE_DATA_FREE_RESOURCES;
 
-		return CDW_MEM_ERROR;
+		return CDW_ERROR;
 	}
 	cdw_vdm ("INFO: final command for writing files to optical disc:\n");
 	cdw_vdm ("INFO: \"%s\"\n", final_command);
@@ -369,11 +373,14 @@ cdw_rv_t cdw_cdrecord_write_from_files(cdw_task_t *task, cdw_disc_t *disc)
 	cdw_logging_write("\n\n");
 	/* no need to print "final_command" to log - this is done in run_command() */
 
-	run_command(final_command, task);
+	int rv = run_command(final_command, task);
 
 	CDRECORD_WRITE_DATA_FREE_RESOURCES;
-
-	return CDW_OK;
+	if (rv == 0) {
+		return CDW_OK;
+	} else {
+		return CDW_ERROR;
+	}
 }
 
 
@@ -397,7 +404,7 @@ cdw_rv_t cdw_cdrecord_write_from_files(cdw_task_t *task, cdw_disc_t *disc)
 
    \return CDW_OK on success (probably)
    \return CDW_NO on failure
-   \return CDW_GEN_ERROR on errors
+   \return CDW_ERROR on errors
  */
 cdw_rv_t cdw_cdrecord_erase_disc(cdw_task_t *task, cdw_disc_t *disc)
 {
@@ -405,15 +412,15 @@ cdw_rv_t cdw_cdrecord_erase_disc(cdw_task_t *task, cdw_disc_t *disc)
 	   config.cdrw_device (/dev/xxx) does not work; if /dev/xxx works fine,
 	   then config.scsi should be left empty */
 	bool use_scsi = cdw_config_has_scsi_device();
-	const char *device = use_scsi ? global_config.scsi : cdw_drive_get_drive_fullpath();
+	const char *device = use_scsi ? cdw_config_get_scsi_drive() : cdw_drive_get_drive_fullpath();
 
 	task->erase.erase_time = 0;
 
-	char speed_string[10];
-	snprintf(speed_string, 10, "%d", task->erase.erase_speed);
+	char speed_string[4 + 1];
+	snprintf(speed_string, 4 + 1, "%lld", task->erase.speed_id);
 
-	cdw_assert (task->erase.tool.fullpath != (char *) NULL, "ERROR: tool fullpath is NULL\n");
-	char *command = cdw_string_concat(task->erase.tool.fullpath, " -v ",
+	cdw_assert (task->erase.tool.label != (char *) NULL, "ERROR: tool fullpath is NULL\n");
+	char *command = cdw_string_concat(task->erase.tool.label, " -v ",
 					  " dev=", device,
 					  " speed=", speed_string,
 					  " blank=", task->erase.erase_mode == CDW_ERASE_MODE_FAST ? "fast " : "all ",
@@ -426,7 +433,7 @@ cdw_rv_t cdw_cdrecord_erase_disc(cdw_task_t *task, cdw_disc_t *disc)
 				   /* 2TRANS: this is message in dialog window */
 				   _("An error occurred when attempting to erase disc."),
 				   CDW_BUTTONS_OK, CDW_COLORS_ERROR);
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	cdw_vdm ("INFO: cdrecord command for erasing disc:\n");
@@ -434,11 +441,15 @@ cdw_rv_t cdw_cdrecord_erase_disc(cdw_task_t *task, cdw_disc_t *disc)
 
 	cdw_vdm ("command = \"%s\"\n", command);
 
-	run_command(command, task);
+	int rv = run_command(command, task);
 
 	free(command);
 	command = (char *) NULL;
 
+	if (rv != 0) {
+		return CDW_ERROR;
+	}
+
 	/* it might occur during blanking that 'this media does not
 	   support blanking, ignoring', and disc->type_erasable was
 	   set to 'MEDIA_ERASABLE_NO' in run_command(); let's inform user
@@ -501,7 +512,7 @@ cdw_rv_t cdw_cdrecord_erase_disc(cdw_task_t *task, cdw_disc_t *disc)
    \param task - variable describing current task
    \param disc - variable describing current disc
 
-   \return CDW_GEN_ERROR on errors
+   \return CDW_ERROR on errors
    \return CDW_OK on success
 */
 cdw_rv_t cdw_cdrecord_get_meta(cdw_task_t *task, cdw_disc_t *disc)
@@ -510,9 +521,9 @@ cdw_rv_t cdw_cdrecord_get_meta(cdw_task_t *task, cdw_disc_t *disc)
 	   config.cdrw_device (/dev/xxx) does not work; if /dev/xxx works fine,
 	   then config.scsi should be left empty */
 	bool use_scsi = cdw_config_has_scsi_device();
-	const char *device = use_scsi ? global_config.scsi : cdw_drive_get_drive_fullpath();
+	const char *device = use_scsi ? cdw_config_get_scsi_drive() : cdw_drive_get_drive_fullpath();
 
-	cdw_assert (task->media_info.tool.fullpath != (char *) NULL, "ERROR: tool fullpath is NULL\n");
+	cdw_assert (task->media_info.tool.label != (char *) NULL, "ERROR: tool fullpath is NULL\n");
 	char *command = (char *) NULL;
 	bool success = true;
 
@@ -522,19 +533,20 @@ cdw_rv_t cdw_cdrecord_get_meta(cdw_task_t *task, cdw_disc_t *disc)
 	char *error_dialog_message = _("An error occurred when attempting to read some meta information from disc.");
 
 
+// #define CDW_CDRECORD_DISABLE_ATIP
+
+#ifdef CDW_CDRECORD_DISABLE_ATIP
 	bool w = cdw_ext_tools_is_cdrecord_wodim();
-	if (disc->simple_type == CDW_DISC_SIMPLE_TYPE_CD
-	    || (disc->simple_type == CDW_DISC_SIMPLE_TYPE_DVD && w) ) {
-
-		/* my version of cdrecord (Cdrecord-ProDVD-ProBD-Clone 2.01.01a37)
-		   had biiiig problems reading DVD-R atip data (it took very
-		   much time), so I decided not to get atip info with cdrecord;
-		   cdrecord prints disc type in prcap output, but I still have to
-		   keep atip, because this is the only place where wodim prints
-		   disc type info: cdrecord prints disc type in both prcap and
-		   atip output, but wodim prints it only in prcap */
-
-		command = cdw_string_concat(task->media_info.tool.fullpath, " -vv -atip "
+	if (disc->type == CDW_DVD_R && !w) {
+		/* sometimes cdrecord may have biiiig problems reading DVD-R
+		   atip data (when it happened on my machine it took very
+		   much time), so it may be wise to sometimes disable getting
+		   atip data for DVD-R */
+	} else {
+#else
+	{
+#endif
+		command = cdw_string_concat(task->media_info.tool.label, " -vv -atip "
 					    " dev=", device, (char *) NULL);
 		if (command == (char *) NULL) {
 			cdw_buttons_dialog(error_dialog_title, error_dialog_message, CDW_BUTTONS_OK, CDW_COLORS_ERROR);
@@ -548,11 +560,11 @@ cdw_rv_t cdw_cdrecord_get_meta(cdw_task_t *task, cdw_disc_t *disc)
 			   it is about getting some meta information from
 			   optical disc */
 			cdw_processwin_display_sub_info(_("Getting ATIP information"));
-			run_command(command, task);
+			int rv = run_command(command, task);
 			free(command);
 			command = (char *) NULL;
 			cdw_task_check_tool_status(task);
-			if (!task->tool_status.ok) {
+			if (!task->tool_status.ok || rv != 0) {
 				cdw_vdm ("ERROR: failed to get ATIP information\n");
 				success = false;
 			}
@@ -560,7 +572,7 @@ cdw_rv_t cdw_cdrecord_get_meta(cdw_task_t *task, cdw_disc_t *disc)
 	}
 
 	if (success) {
-		command = cdw_string_concat(task->media_info.tool.fullpath, " -vvvv -prcap ",
+		command = cdw_string_concat(task->media_info.tool.label, " -vvvv -prcap ",
 					    " dev=", device, (char *) NULL);
 		if (command == (char *) NULL) {
 			cdw_buttons_dialog(error_dialog_title, error_dialog_message, CDW_BUTTONS_OK, CDW_COLORS_ERROR);
@@ -574,19 +586,20 @@ cdw_rv_t cdw_cdrecord_get_meta(cdw_task_t *task, cdw_disc_t *disc)
 			   it is about getting some meta information from
 			   optical disc */
 			cdw_processwin_display_sub_info(_("Getting PRCAP information"));
-			run_command(command, task);
+			int rv = run_command(command, task);
 			free(command);
 			command = (char *) NULL;
 			cdw_task_check_tool_status(task);
-			if (!task->tool_status.ok) {
+			if (!task->tool_status.ok || rv != 0) {
 				cdw_vdm ("ERROR: failed to get PRCAP information\n");
 				success = false;
 			}
 		}
 	}
 
-	if (success) {
-		command = cdw_string_concat(task->media_info.tool.fullpath, " -toc ",
+	/* check state_empty: getting TOC of empty DVD may take looooong time */
+	if (success && (disc->state_empty != CDW_TRUE)) {
+		command = cdw_string_concat(task->media_info.tool.label, " -toc -vv ",
 					    " dev=", device, (char *) NULL);
 		if (command == (char *) NULL) {
 			cdw_buttons_dialog(error_dialog_title, error_dialog_message, CDW_BUTTONS_OK, CDW_COLORS_ERROR);
@@ -600,11 +613,11 @@ cdw_rv_t cdw_cdrecord_get_meta(cdw_task_t *task, cdw_disc_t *disc)
 			   it is about getting some meta information from
 			   optical disc */
 			cdw_processwin_display_sub_info(_("Getting TOC information"));
-			run_command(command, task);
+			int rv = run_command(command, task);
 			free(command);
 			command = (char *) NULL;
 			cdw_task_check_tool_status(task);
-			if (!task->tool_status.ok) {
+			if (!task->tool_status.ok || rv != 0) {
 				cdw_vdm ("ERROR: failed to get TOC information\n");
 				success = false;
 			}
@@ -612,7 +625,7 @@ cdw_rv_t cdw_cdrecord_get_meta(cdw_task_t *task, cdw_disc_t *disc)
 	}
 
 	if (success) {
-		command = cdw_string_concat(task->media_info.tool.fullpath, " -msinfo ",
+		command = cdw_string_concat(task->media_info.tool.label, " -msinfo ",
 					    " dev=", device, (char *) NULL);
 		if (command == (char *) NULL) {
 			cdw_buttons_dialog(error_dialog_title, error_dialog_message, CDW_BUTTONS_OK, CDW_COLORS_ERROR);
@@ -626,11 +639,11 @@ cdw_rv_t cdw_cdrecord_get_meta(cdw_task_t *task, cdw_disc_t *disc)
 			   it is about getting some meta information from
 			   optical disc */
 			cdw_processwin_display_sub_info(_("Getting MSINFO information"));
-			run_command(command, task);
+			int rv = run_command(command, task);
 			free(command);
 			command = (char *) NULL;
 			cdw_task_check_tool_status(task);
-			if (!task->tool_status.ok) {
+			if (!task->tool_status.ok || rv != 0) {
 				cdw_vdm ("ERROR: failed to get MSINFO information\n");
 				success = false;
 			}
@@ -648,7 +661,7 @@ cdw_rv_t cdw_cdrecord_get_meta(cdw_task_t *task, cdw_disc_t *disc)
 		}
 
 		cdw_vdm ("ERROR: failed to correctly get meta information with cdrecord; consult error informations above\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 }
 
@@ -673,6 +686,7 @@ cdw_rv_t cdw_cdrecord_get_meta(cdw_task_t *task, cdw_disc_t *disc)
    be freed() by caller
 
    \param task - variable describing current task
+   \param disc - variable describing current disc
 
    \return command string on success
    \return NULL on error
@@ -684,8 +698,8 @@ char *cdw_cdrecord_create_command_for_writing(cdw_task_t *task, cdw_disc_t *disc
 	cdw_assert (task->burn.disc_mode != CDW_DISC_MODE_INIT,
 		    "you forgot to set disc mode\n");
 	cdw_assert (task->id == CDW_TASK_BURN_FROM_FILES || task->id == CDW_TASK_BURN_FROM_IMAGE,
-		    "ERROR: incorrect task->id %d\n", task->id);
-	cdw_assert (task->burn.tool.id == CDW_TOOL_CDRECORD, "ERROR: incorrect tool id %d\n", task->burn.tool.id);
+		    "ERROR: incorrect task->id %lld\n", task->id);
+	cdw_assert (task->burn.tool.id == CDW_TOOL_CDRECORD, "ERROR: incorrect tool id %lld\n", task->burn.tool.id);
 
 	/* cdrecord needs to know size of files in two cases: when writing
 	   in dao mode and when writing image that is piped to cdrecord (writing
@@ -697,28 +711,22 @@ char *cdw_cdrecord_create_command_for_writing(cdw_task_t *task, cdw_disc_t *disc
 	if (task->id == CDW_TASK_BURN_FROM_FILES) {
 		strcpy(tsize_value, "${tsize}");
 	} else { /* task->id == CDW_TASK_BURN_FROM_IMAGE */
-		long sectors = cdw_iso_image_get_n_sectors();
-		snprintf(tsize_value, TSIZE_LEN + 1, "%ld", sectors);
-		cdw_vdm ("tsize_value string = \"%s\", strlen(tsize_value) = %zd\n", tsize_value, strlen(tsize_value));
+		long long int fs_size = cdw_fs_get_file_size(task->burn.iso9660_file_fullpath);
+		snprintf(tsize_value, TSIZE_LEN + 1, "%lld", fs_size / 2048);
+		cdw_vdm ("INFO: tsize_value string = \"%s\", strlen(tsize_value) = %zd\n", tsize_value, strlen(tsize_value));
 	}
 
-
 	bool add_multi = cdw_cdrecord_add_multisession_info(task);
 
 	/* cdrecord will read data to burn either from its stdin (streamed
 	   from mkisofs) or from files from hard disc */
 	bool read_from_stdin = task->id == CDW_TASK_BURN_FROM_FILES ? true : false;
 
-	bool single_session = false;
-	if (task->burn.session_mode == CDW_SESSION_MODE_CREATE_SINGLE) {
-		single_session = true;
-	}
-
 	/* using config.scsi (X:Y:Z) is optional and advised only if
 	   config.cdrw_device (/dev/xxx) does not work; if /dev/xxx works fine,
 	   then config.scsi should be left empty */
 	bool use_scsi = cdw_config_has_scsi_device();
-	const char *device = use_scsi ? global_config.scsi : cdw_drive_get_drive_fullpath();
+	const char *device = use_scsi ? cdw_config_get_scsi_drive() : cdw_drive_get_drive_fullpath();
 
 	char disc_mode[10];
 	cdw_cdrecord_set_disc_mode_arg(task, disc_mode);
@@ -728,33 +736,35 @@ char *cdw_cdrecord_create_command_for_writing(cdw_task_t *task, cdw_disc_t *disc
 	/* e.g. " padsize=63s -pad ", note spaces a beginning and end;
 	   value of padsize is no longer than PADSIZE_FIELD_LEN_MAX
 	   chars (not including ending \0); last +1 below is for ending \0 */
-	size_t pad_len_max = 9 + PADSIZE_FIELD_LEN_MAX + 7;
+	size_t pad_len_max = 9 + CDW_CDRECORD_OPTIONS_PADSIZE_FIELD_LEN_MAX + 7;
 	char pad_string[pad_len_max + 1];
-	if (global_config.pad_size > 0) {
+	if (task->burn.cdrecord.pad_size > 0) {
 		snprintf(pad_string, pad_len_max + 1,
-			 " padsize=%ds %s ", global_config.pad_size, global_config.pad ? "-pad" : " ");
+			 " padsize=%ds %s ", task->burn.cdrecord.pad_size, task->burn.cdrecord.pad ? "-pad" : " ");
 	} else {
-		snprintf(pad_string, pad_len_max + 1, " %s ", global_config.pad ? "-pad" : " ");
+		snprintf(pad_string, pad_len_max + 1, " %s ", task->burn.cdrecord.pad ? "-pad" : " ");
 	}
 	cdw_vdm ("pad string is \"%s\", %zd / %zd chars\n", pad_string, strlen(pad_string), pad_len_max);
 
 
 	size_t speed_len_max = 7 + 3 + 1; /* " speed=" + "999" + " " */
 	char speed_string[speed_len_max + 1];
-	snprintf(speed_string, speed_len_max + 1, " speed=%d ", task->burn.speed);
+	snprintf(speed_string, speed_len_max + 1, " speed=%lld ", task->burn.speed_id);
 	cdw_vdm ("speed string is \"%s\", %zd / %zd chars\n", speed_string, strlen(speed_string), speed_len_max);
 
 	char track_format[10];
 	if (disc->simple_type == CDW_DISC_SIMPLE_TYPE_CD) {
 		/* "-data": CD-ROM Mode 1 sector format, 2048 bytes per sector */
 		/* "-xa": CD-ROM Mode 2 form 1 sector format, 2048 bytes per sector */
-		strcpy(track_format, single_session ? " -data " : " -xa ");
+		strcpy(track_format,
+		       task->burn.session_mode == CDW_SESSION_MODE_CREATE_SINGLE
+		       ? " -data " : " -xa ");
 	} else { /* dvd, deliberately not using any track format specifier */
 		strcpy(track_format, " ");
 	}
 
-	cdw_assert (task->burn.tool.fullpath != (char *) NULL, "ERROR: tool fullpath is NULL\n");
-	char *command = cdw_string_concat(task->burn.tool.fullpath,
+	cdw_assert (task->burn.tool.label != (char *) NULL, "ERROR: tool fullpath is NULL\n");
+	char *command = cdw_string_concat(task->burn.tool.label,
 					  " -v ",
 					  " dev=", device,
 					  speed_string, /* surrounded by spaces */
@@ -762,8 +772,8 @@ char *cdw_cdrecord_create_command_for_writing(cdw_task_t *task, cdw_disc_t *disc
 					  /* dao, tao, etc. */
 					  strlen(disc_mode) ? disc_mode : " ",
 
-					  global_config.dummy ? " -dummy " : " ",
-					  global_config.burnproof ? " driveropts=burnfree " : " ",
+					  task->burn.dummy ? " -dummy " : " ",
+					  task->burn.cdrecord.burnproof ? " driveropts=burnfree " : " ",
 
 					  /* cdrecord man page says: "Values below
 					     2 seconds are not recommended to give
@@ -773,7 +783,7 @@ char *cdw_cdrecord_create_command_for_writing(cdw_task_t *task, cdw_disc_t *disc
 
 					  add_multi ? " -multi " : " ",
 
-					  " ", global_config.other_cdrecord_options, " ",
+					  " ", task->burn.cdrecord.other_cdrecord_options, " ",
 
 					  pad_string,
 
@@ -783,13 +793,13 @@ char *cdw_cdrecord_create_command_for_writing(cdw_task_t *task, cdw_disc_t *disc
 					  track_format,
 
 					  /* tsize_value is just a string with number, so we have to add 's' */
-					  "tsize=", tsize_value, "s ",
+					  " tsize=", tsize_value, "s ",
 
-					  /* either get data from stdin, or from iso image (tempdir);
+					  /* either get data from stdin, or from iso image;
 					     path to iso file may contain spaces, so put
 					     the path in quotes */
 					  read_from_stdin ? " - " : "\"",
-					  read_from_stdin ? "  " : global_config.iso_image_full_path,
+					  read_from_stdin ? "  " : task->burn.iso9660_file_fullpath,
 					  read_from_stdin ? "  " : "\"",
 
 					  /* last argument passed to concat() must be NULL */
@@ -853,6 +863,9 @@ void cdw_cdrecord_set_disc_mode_arg(cdw_task_t *task, char *disc_mode)
 		strcpy(disc_mode, " -tao ");
 	} else if (task->burn.disc_mode == CDW_DISC_MODE_DAO) {
 		strcpy(disc_mode, " -dao ");
+	} else if (task->burn.disc_mode == CDW_DISC_MODE_SAO) {
+		strcpy(disc_mode, " -sao ");
+#if 0 /* currently unsupported modes */
 	} else if (task->burn.disc_mode == CDW_DISC_MODE_RAW) {
 		strcpy(disc_mode, " -raw ");
 	} else if (task->burn.disc_mode == CDW_DISC_MODE_RAW96R) {
@@ -861,12 +874,13 @@ void cdw_cdrecord_set_disc_mode_arg(cdw_task_t *task, char *disc_mode)
 		strcpy(disc_mode, " -raw96p ");
 	} else if (task->burn.disc_mode == CDW_DISC_MODE_RAW16) {
 		strcpy(disc_mode, " -raw16 ");
+#endif
 	} else {
 		/* cdrecord will use its default mode */
 		disc_mode[0] = '\0';
 	}
 
-	cdw_vdm ("disc mode = \"%s\"\n", disc_mode);
+	cdw_vdm ("INFO: disc mode = \"%s\"\n", disc_mode);
 
 	return;
 
@@ -886,7 +900,7 @@ void cdw_cdrecord_set_disc_mode_arg(cdw_task_t *task, char *disc_mode)
 
    \p id - small number identifying caller of this function
  */
-void cdw_cdrecord_internal_error_dialog(int id)
+void cdw_cdrecord_internal_error_dialog(cdw_id_t id)
 {
 	/* 2TRANS: this is title of dialog window */
 	cdw_buttons_dialog(_("Error message"),
@@ -894,7 +908,7 @@ void cdw_cdrecord_internal_error_dialog(int id)
 			   _("An error occurred. Try to write disc again."),
 			   CDW_BUTTONS_OK, CDW_COLORS_ERROR);
 
-	cdw_vdm ("id is %d\n", id);
+	cdw_vdm ("INFO: caller id is %lld\n", id);
 
 	return;
 }
@@ -953,7 +967,7 @@ cdw_rv_t cdw_cdrecord_set_disc_states(cdw_disc_t *disc)
 		cdw_vdm ("ERROR: last_sess_start = %ld, next_sess_start = %ld:\n",
 			 disc->cdrecord_info.last_sess_start, disc->cdrecord_info.next_sess_start);
 
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 
@@ -971,7 +985,7 @@ cdw_rv_t cdw_cdrecord_set_disc_states(cdw_disc_t *disc)
 		    || disc->type == CDW_DVD_R_RES
 		    || disc->type == CDW_DVD_R_SEQ
 
-		    || (disc->type == CDW_DVD_RP_DL && global_config.support_dvd_rp_dl)) {
+		    || (disc->type == CDW_DVD_RP_DL && cdw_config_support_dvd_rp_dl())) {
 
 			if (disc->cdrecord_info.has_toc) {
 				/* we know now that it is non-empty, but
@@ -989,7 +1003,7 @@ cdw_rv_t cdw_cdrecord_set_disc_states(cdw_disc_t *disc)
 
 	/* sometimes cdrecord / wodim can't recognize (using "cdrecord -toc" command) if
 	   DVD+R DL is empty or not, so in case of DVD+R DL I can't rely on "has_toc" */
-	if (disc->type == CDW_DVD_RP_DL && global_config.support_dvd_rp_dl) {
+	if (disc->type == CDW_DVD_RP_DL && cdw_config_support_dvd_rp_dl()) {
 		if (disc->state_writable == CDW_UNKNOWN
 		    || disc->state_writable == CDW_TRUE) {
 
@@ -1011,6 +1025,35 @@ cdw_rv_t cdw_cdrecord_set_disc_states(cdw_disc_t *disc)
 
 
 
+
+
+/**
+   \brief Check if growisofs supports "dummy" option for given
+   type of current disc
+
+*/
+bool cdw_cdrecord_allow_dummy(cdw_disc_t *disc)
+{
+	/* "dummy" option is not supported for some disc types;
+	   to be on the safe side, I limit the subset of disc
+	   types to only a few */
+	if (disc->type == CDW_CD_R
+	    || disc->type == CDW_CD_RW
+	    || disc->type == CDW_DVD_R
+	    || disc->type == CDW_DVD_R_SEQ
+	    || disc->type == CDW_DVD_RW_SEQ
+	    || disc->type == CDW_DVD_RW_RES) {
+
+		return true;
+	} else {
+		return false;
+	}
+}
+
+
+
+
+
 /* *** unused code below *** */
 
 #if 0
