@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2010 Kamil Ignacak
+ * Copyright (C) 2007 - 2011 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -186,6 +186,10 @@ static int cdw_cdrecord_handle_prcap_write_speed_table(regex_t *regex, regmatch_
 static int cdw_cdrecord_handle_reload_disk_and_hit_cr(regex_t *regex, regmatch_t *matches);
 static int cdw_cdrecord_handle_toc_beginning(regex_t *regex, regmatch_t *matches);
 static int cdw_cdrecord_handle_write_error_after(regex_t *regex, regmatch_t *matches);
+static int cdw_cdrecord_handle_supported_disc_modes(regex_t *regex, regmatch_t *matches);
+static int cdw_cdrecord_handle_phys_size(regex_t *regex, regmatch_t *matches);
+static int cdw_cdrecord_handle_atip_start_of_lead_out(regex_t *regex, regmatch_t *matches);
+static int cdw_cdrecord_handle_rzone_size(regex_t *regex, regmatch_t *matches);
 
 /* functions used for stderr output handling */
 static int cdw_cdrecord_handle_msinfo_cannot_get_next_writable(regex_t *regex, regmatch_t *matches);
@@ -306,6 +310,40 @@ static cdw_regex_t stdout_regex[] = {
 	  (regex_t *) NULL, (regmatch_t *) NULL, REG_EXTENDED,
 	  cdw_cdrecord_handle_reload_disk_and_hit_cr },
 
+	{ "supported modes", 1016,
+	  /* "Supported modes: TAO PACKET SAO SAO/R96P SAO/R96R RAW/R16 RAW/R96P RAW/R96R" */
+	  /* information about disc modes supported by given drive;
+	     may be useful when selecting drive mode for burning */
+	  "Supported modes: (.+)",
+	  (regex_t *) NULL, (regmatch_t *) NULL, REG_EXTENDED,
+	  cdw_cdrecord_handle_supported_disc_modes },
+
+	{ "phys size", 1017,
+	  /* "phys size:...    2295104"
+	     Only for DVDs;
+	     Capacity of a disc in sectors - total numbers of sectors that can
+	     be stored on a disc; parameter not displayed by wodim 1.1.11 */
+	  "phys size:... +([0-9]+)",
+	  (regex_t *) NULL, (regmatch_t *) NULL, REG_EXTENDED,
+	  cdw_cdrecord_handle_phys_size },
+
+	{ "atip start of lead out", 1018,
+	  /* "  ATIP start of lead out: 359849 (79:59/74)"
+	     let's consider start of lead out to be total disc capacity */
+	  "ATIP start of lead out: ([0-9]+) ",
+	  (regex_t *) NULL, (regmatch_t *) NULL, REG_EXTENDED,
+	  cdw_cdrecord_handle_atip_start_of_lead_out },
+
+	{ "rzone size", 1019,
+	  /* "rzone size:...    2295104"
+
+	     Only for DVDs;
+	     RZone is something similar to a track on a CD. I didn't
+	     investigate it too much, so we are in a gray area here. */
+	  "rzone size: +([0-9]+)",
+	  (regex_t *) NULL, (regmatch_t *) NULL, REG_EXTENDED,
+	  cdw_cdrecord_handle_rzone_size },
+
 	{ "table guard", -1,
 	  /* guard: debug_id = -1 */
 	  "",
@@ -1328,7 +1366,7 @@ int cdw_cdrecord_handle_reload_disk_and_hit_cr(__attribute__((unused)) regex_t *
 	cdw_processwin_display_main_info(_("You need to close drive tray and press ENTER"));
 	cdw_processwin_wgetch();
 
-	cdw_rv_t crv = cdw_thread_send_key_to_child_process(CDW_ENTER);
+	cdw_rv_t crv = cdw_thread_send_key_to_child_process(CDW_KEY_ENTER);
 	if (crv == CDW_OK) {
 		;
 	} else {
@@ -1406,3 +1444,116 @@ int cdw_cdrecord_handle_data_may_not_fit(__attribute__((unused)) regex_t *regex,
 
 
 
+int cdw_cdrecord_handle_supported_disc_modes(regex_t *regex, regmatch_t *matches)
+{
+	/* regex for capturing bad option:
+	                       1
+	   "Supported modes: (.+)" */
+	cdw_regex_assert_subex_number(regex->re_nsub, 1);
+
+	char submatch[PIPE_BUFFER_SIZE + 1];
+	unsigned int sub = 1;
+	int len = cdw_regex_get_submatch(matches, sub, stdout_pipe_buffer, submatch);
+	if (len == -1) {
+		cdw_vdm ("ERROR: len of subexpr %d is negative: %d\n", sub, len);
+		return -1;
+	}
+
+	cdw_vdm ("INFO: supported modes: \"%s\"\n", submatch);
+	const char *delim = " ";
+	char *mode = strtok(submatch, delim);
+	while (mode != (char *) NULL) {
+		if (!strcmp(mode, "TAO")) {
+			cdw_vdm ("INFO: drive supports TAO\n");
+			current_disc.accepted_disc_modes |= CDW_DISC_MODE_TAO;
+		} else if (!strcmp(mode, "DAO")) {
+			cdw_vdm ("INFO: drive supports DAO\n");
+			current_disc.accepted_disc_modes |= CDW_DISC_MODE_DAO;
+		} else if (!strcmp(mode, "SAO")) {
+			cdw_vdm ("INFO: drive supports SAO\n");
+			current_disc.accepted_disc_modes |= CDW_DISC_MODE_SAO;
+		} else {
+			;
+		}
+		mode = strtok((char *) NULL, delim);
+	}
+
+	return 0;
+}
+
+
+
+
+int cdw_cdrecord_handle_phys_size(regex_t *regex, regmatch_t *matches)
+{
+	/* regex for capturing physical size:
+	                      1
+	   "phys size:... +([0-9]+)" */
+	cdw_regex_assert_subex_number(regex->re_nsub, 1);
+
+	char submatch[PIPE_BUFFER_SIZE + 1];
+	unsigned int sub = 1;
+	int len = cdw_regex_get_submatch(matches, sub, stdout_pipe_buffer, submatch);
+	if (len == -1) {
+		cdw_vdm ("ERROR: len of subexpr %d is negative: %d\n", sub, len);
+		return -1;
+	}
+
+	current_disc.cdrecord_info.phys_size = strtoll(submatch, (char **) NULL, 10);
+	cdw_vdm ("INFO: physical size: %lld\n", current_disc.cdrecord_info.phys_size);
+
+	return 0;
+}
+
+
+
+
+
+int cdw_cdrecord_handle_atip_start_of_lead_out(regex_t *regex, regmatch_t *matches)
+{
+	/* regex for capturing start of lead out:
+	                              1
+	  "ATIP start of lead out: ([0-9]+) " */
+	cdw_regex_assert_subex_number(regex->re_nsub, 1);
+
+	char submatch[PIPE_BUFFER_SIZE + 1];
+	unsigned int sub = 1;
+	int len = cdw_regex_get_submatch(matches, sub, stdout_pipe_buffer, submatch);
+	if (len == -1) {
+		cdw_vdm ("ERROR: len of subexpr %d is negative: %d\n", sub, len);
+		return -1;
+	}
+
+	current_disc.cdrecord_info.start_of_lead_out = strtoll(submatch, (char **) NULL, 10);
+	cdw_vdm ("INFO: start of lead out: %lld\n", current_disc.cdrecord_info.start_of_lead_out);
+
+	return 0;
+}
+
+
+
+
+
+int cdw_cdrecord_handle_rzone_size(regex_t *regex, regmatch_t *matches)
+{
+	/* regex for capturing physical size:
+	                      1
+	   "rzone size:... +([0-9]+)" */
+	cdw_regex_assert_subex_number(regex->re_nsub, 1);
+
+	char submatch[PIPE_BUFFER_SIZE + 1];
+	unsigned int sub = 1;
+	int len = cdw_regex_get_submatch(matches, sub, stdout_pipe_buffer, submatch);
+	if (len == -1) {
+		cdw_vdm ("ERROR: len of subexpr %d is negative: %d\n", sub, len);
+		return -1;
+	}
+
+	long long int rzone_size = strtoll(submatch, (char **) NULL, 10);
+	if (rzone_size > current_disc.cdrecord_info.rzone_size) {
+		current_disc.cdrecord_info.rzone_size = rzone_size;
+	}
+	cdw_vdm ("INFO: rzone size: %lld\n", current_disc.cdrecord_info.rzone_size);
+
+	return 0;
+}
