@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2010 Kamil Ignacak
+ * Copyright (C) 2007 - 2011 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,109 +45,129 @@ extern char stdout_pipe_buffer[PIPE_BUFFER_SIZE + 1];
 /* extern char stderr_pipe_buffer[PIPE_BUFFER_SIZE + 1]; */
 
 
-
 extern cdw_config_t global_config;
 extern cdw_disc_t current_disc;
 
-
-/* there is a special way of checking if DVD+RW is empty, even if
-   "Disc status" line in dvd+rw-mediainfo says otherwise; this is a flag that
-   is set to true if DVD+RW is REALLY empty, and its state should override
-   value of "Disc status" */
-
-static regex_t regex1, regex2, regex3;
-/* static regex_t regex5, regex4,  regex6; */
-static regex_t regex7, regex8, regex9;
-static regmatch_t *matches1 = NULL, *matches2 = NULL, *matches3 = NULL;
-/* static regmatch_t *matches4 = NULL, *matches5 = NULL, *matches6 = NULL; */
-static regmatch_t *matches7 = NULL, *matches8 = NULL, *matches9 = NULL;
-
-
-
-static void cdw_dvd_rw_mediainfo_pr_handle_disc_status(regex_t *regex, regmatch_t *matches);
-static void cdw_dvd_rw_mediainfo_pr_handle_dvd_write_speed(regex_t *regex, regmatch_t *matches);
-static void cdw_dvd_rw_mediainfo_pr_handle_mounted_media(regex_t *regex, regmatch_t *matches);
-static void cdw_dvd_rw_mediainfo_pr_handle_current_write_speed(regex_t *regex, regmatch_t *matches);
-static void cdw_dvd_rw_mediainfo_pr_handle_read_capacity(regex_t *regex, regmatch_t *matches);
+static int cdw_dvd_rw_mediainfo_pr_handle_disc_status(regex_t *regex, regmatch_t *matches);
+static int cdw_dvd_rw_mediainfo_pr_handle_dvd_write_speed(regex_t *regex, regmatch_t *matches);
+static int cdw_dvd_rw_mediainfo_pr_handle_mounted_media(regex_t *regex, regmatch_t *matches);
+static int cdw_dvd_rw_mediainfo_pr_handle_current_write_speed(regex_t *regex, regmatch_t *matches);
+static int cdw_dvd_rw_mediainfo_pr_handle_read_capacity(regex_t *regex, regmatch_t *matches);
+static int cdw_dvd_rw_mediainfo_pr_handle_end_lba(regex_t *regex, regmatch_t *matches);
+static int cdw_dvd_rw_mediainfo_pr_handle_track_size(regex_t *regex, regmatch_t *matches);
+static int cdw_dvd_rw_mediainfo_pr_handle_dvd_rom_detected(regex_t *regex, regmatch_t *matches);
 
 /* valid but unused */
 /*
-static void handle_dvd_toc(regex_t *regex, regmatch_t *matches);
-static void handle_track_start_address(regex_t *regex, regmatch_t *matches);
-static void handle_next_writable_address(regex_t *regex, regmatch_t *matches); */
+static int cdw_dvd_rw_mediainfo_pr_handle_dvd_toc(regex_t *regex, regmatch_t *matches);
+static int cdw_dvd_rw_mediainfo_pr_handle_track_start_address(regex_t *regex, regmatch_t *matches);
+static int cdw_dvd_rw_mediainfo_pr_handle_next_writable_address(regex_t *regex, regmatch_t *matches);
+*/
+
+static cdw_regex_t stdout_regex[] = {
+	{ "write speed", 1001,
+	  /*  "Write Speed #0:        4.0x1385=5540KB/s" */
+	  "Write Speed #([0-9]+):([\t ]+)([0-9]+).([0-9]+)x",
+	  (regex_t *) NULL, (regmatch_t *) NULL, REG_EXTENDED,
+	  cdw_dvd_rw_mediainfo_pr_handle_dvd_write_speed },
+
+	{ "disc status", 1002,
+	  /* "Disc status:           complete" (appendable/blank/other) */
+	  "Disc status:([\t ]+)(complete|appendable|blank|other)",
+	  (regex_t *) NULL, (regmatch_t *) NULL, REG_EXTENDED,
+	  cdw_dvd_rw_mediainfo_pr_handle_disc_status },
+
+	{ "mounted media", 1003,
+	  /* "Mounted Media:         1Ah, DVD-RW Sequential" */
+	  "Mounted Media:([ ]+)([a-zA-Z0-9]+),([ ]+)([a-zA-Z +-]*)",
+	  (regex_t *) NULL, (regmatch_t *) NULL, REG_EXTENDED,
+	  cdw_dvd_rw_mediainfo_pr_handle_mounted_media },
+
+	{ "current write speed", 1004,
+	  /* "Current Write Speed:   16.0x1385=22160KB/s" */
+	  "Current Write Speed:([\t ]*)([0-9]+)",
+	  (regex_t *) NULL, (regmatch_t *) NULL, REG_EXTENDED,
+	  cdw_dvd_rw_mediainfo_pr_handle_current_write_speed },
+
+	{ "read capacity", 1005,
+	  /* "READ CAPACITY:          0*2048=0" */
+	  /* read capacity for empty disc is 0 */
+	  "READ CAPACITY:([\t ]+)([0-9]+)",
+	  (regex_t *) NULL, (regmatch_t *) NULL, REG_EXTENDED,
+	  cdw_dvd_rw_mediainfo_pr_handle_read_capacity },
+
+	{ "DVD-ROM detected", 1006,
+	  /* "DVD-ROM media detected, exiting..." */
+	  "DVD-ROM media detected",
+	  (regex_t *) NULL, (regmatch_t *) NULL, REG_EXTENDED,
+	  cdw_dvd_rw_mediainfo_pr_handle_dvd_rom_detected },
+
+	{ "end LBA", 1007,
+	  /*
+	   Speed Descriptor#0:    00/2295103 R@4.0x1385=5540KB/s W@16.0x1385=22160KB/s
+	   Speed Descriptor#1:    00/2295103 R@4.0x1385=5540KB/s W@12.0x1385=16620KB/s
+	   Speed Descriptor#2:    00/2295103 R@4.0x1385=5540KB/s W@10.0x1385=13850KB/s
+	   Speed Descriptor#3:    00/2295103 R@4.0x1385=5540KB/s W@8.0x1385=11080KB/s
+	   Speed Descriptor#4:    00/2295103 R@4.0x1385=5540KB/s W@4.0x1385=5540KB/s
+
+	   The most interesting thing is after 00/ - this is end LBA;
+	   check comment for end_lba field in src/disc_and_drive/cdw_disc.h
+	   for more information */
+	  "Speed Descriptor#([0-9]+):( +)([0-9a-fA-F]+)/([0-9]+) ",
+	  (regex_t *) NULL, (regmatch_t *) NULL, REG_EXTENDED,
+	  cdw_dvd_rw_mediainfo_pr_handle_end_lba },
+
+	{ "track size", 1008,
+	  /* data printed for every track size, including the last one (empty)
+	   " Track Size:            1957744*2KB" */
+	  "Track Size:( +)([0-9]+)([*])2KB",
+	  (regex_t *) NULL, (regmatch_t *) NULL, REG_EXTENDED,
+	  cdw_dvd_rw_mediainfo_pr_handle_track_size },
+#if 0
+	/* unused regexes */
+
+	{ "DVD toc", 1009,
+	  /* " Track#2  :             14@145056"
+	   information about DVD TOC, last number is start of track;
+	   track number may be also AA, whatever it means in DVD context */
+	  "Track#([0-9A]+)([ ]*):([ ]*)([0-9]*)@([0-9]+)",
+	  (regex_t *) NULL, (regmatch_t *) NULL, REG_EXTENDED,
+	  cdw_dvd_rw_mediainfo_pr_handle_dvd_toc },
+
+	{ "track start address", 1010,
+	  /* "Track Start Address:   666512" */
+	  "Track Start Address:([ ]*)([0-9]+)",
+	  (regex_t *) NULL, (regmatch_t *) NULL, REG_EXTENDED,
+	  cdw_dvd_rw_mediainfo_pr_handle_track_start_address },
+
+	{ "next writable address", 1011,
+	  /* "Next Writable Address: 470672*2KB"
+	     start address for next session */
+	  "Next Writable Address:([ ]*)([0-9]+)",
+	  (regex_t *) NULL, (regmatch_t *) NULL, REG_EXTENDED,
+	  cdw_dvd_rw_mediainfo_pr_handle_next_writable_address },
+#endif
+
+	{ "table guard", -1,
+	  /* guard: debug_id = -1 */
+	  "",
+	  (regex_t *) NULL, (regmatch_t *) NULL, 0,
+	  (cdw_regex_handler_t) NULL }
+};
 
 
+
+
+
+/**
+   \brief Do some initialization and call cdw_regex_prepare_regexes_table() for stdout regexes table
+*/
 void dvd_rw_mediainfo_stdout_regexp_prepare(void)
 {
-	int rv;
-
-	/*  "Write Speed #0:        4.0x1385=5540KB/s" */
-	rv = regcomp(&regex1, "Write Speed #([0-9]+):([\t ]+)([0-9]+).([0-9]+)x", REG_EXTENDED);
-	matches1 = (regmatch_t *) calloc(1, (regex1.re_nsub + 1) * sizeof(regmatch_t));
-	if (rv) {
-		cdw_regex_regcomp_error_handler(__func__, rv, &regex1, 1);
+	cdw_rv_t crv = cdw_regex_prepare_regexes_table(stdout_regex);
+	if (crv != CDW_OK) {
+		cdw_vdm("ERROR: failed to prepare regexes table for stdout\n");
 	}
-
-	/* "Disc status:           complete" (appendable/blank/other) */
-	rv = regcomp(&regex2, "Disc status:([\t ]+)(complete|appendable|blank|other)", REG_EXTENDED);
-	matches2 = (regmatch_t *) calloc(1, (regex2.re_nsub + 1) * sizeof(regmatch_t));
-	if (rv) {
-		cdw_regex_regcomp_error_handler(__func__, rv, &regex2, 2);
-	}
-
-	/* "Mounted Media:         1Ah, DVD+RW" */
-	/* "Mounted Media:         1Ah, DVD-RW Sequential" */
-	rv = regcomp(&regex3, "Mounted Media:([ ]+)([a-zA-Z0-9]+),([ ]+)([a-zA-Z +-]*)", REG_EXTENDED);
-	matches3 = (regmatch_t *) calloc(1, (regex3.re_nsub + 1) * sizeof(regmatch_t));
-	if (rv) {
-		cdw_regex_regcomp_error_handler(__func__, rv, &regex3, 3);
-	}
-#if 0
-	/* " Track#2  :             14@145056" - information about DVD TOC,
-	   last number is start of track; track number may be also AA, whatever
-	   it means in DVD context */
-	rv = regcomp(&regex4, "Track#([0-9A]+)([ ]*):([ ]*)([0-9]*)@([0-9]+)", REG_EXTENDED);
-	matches4 = (regmatch_t *) calloc(1, (regex4.re_nsub + 1) * sizeof(regmatch_t));
-	if (rv) {
-		cdw_regex_regcomp_error_handler(__func__, rv, &regex4, 4);
-	}
-
-	/* "Track Start Address:   666512" - capture start addresses of DVD tracks*/
-	rv = regcomp(&regex5, "Track Start Address:([ ]*)([0-9]+)", REG_EXTENDED);
-	matches5 = (regmatch_t *) calloc(1, (regex5.re_nsub + 1) * sizeof(regmatch_t));
-	if (rv) {
-		cdw_regex_regcomp_error_handler(__func__, rv, &regex5, 5);
-	}
-
-	/* "Next Writable Address: 470672*2KB" start address for next session */
-	rv = regcomp(&regex6, "Next Writable Address:([ ]*)([0-9]+)", REG_EXTENDED);
-	matches6 = (regmatch_t *) calloc(1, (regex6.re_nsub + 1) * sizeof(regmatch_t));
-	if (rv) {
-		cdw_regex_regcomp_error_handler(__func__, rv, &regex6, 6);
-	}
-#endif
-	/* "Current Write Speed:   16.0x1385=22160KB/s" */
-	rv = regcomp(&regex7, "Current Write Speed:([\t ]*)([0-9]+)", REG_EXTENDED);
-	matches7 = (regmatch_t *) calloc(1, (regex7.re_nsub + 1) * sizeof(regmatch_t));
-	if (rv) {
-		cdw_regex_regcomp_error_handler(__func__, rv, &regex7, 7);
-	}
-
-	/* "READ CAPACITY:          0*2048=0" */
-	/* read capacity for empty disc is 0 */
-	rv = regcomp(&regex8, "READ CAPACITY:([\t ]+)([0-9]+)", REG_EXTENDED);
-	matches8 = (regmatch_t *) calloc(1, (regex8.re_nsub + 1) * sizeof(regmatch_t));
-	if (rv) {
-		cdw_regex_regcomp_error_handler(__func__, rv, &regex8, 8);
-	}
-
-	/* "DVD-ROM media detected, exiting..." */
-	rv = regcomp(&regex9, "DVD-ROM media detected", REG_EXTENDED);
-	matches9 = (regmatch_t *) calloc(1, (regex9.re_nsub + 1) * sizeof(regmatch_t));
-	if (rv) {
-		cdw_regex_regcomp_error_handler(__func__, rv, &regex9, 9);
-	}
-
 	return;
 }
 
@@ -155,69 +175,14 @@ void dvd_rw_mediainfo_stdout_regexp_prepare(void)
 
 
 
+/**
+   \brief Do some initialization and call cdw_regex_execute_regexes_table()
+   for stdout regexes table and stdout pipe buffer.
+*/
 void dvd_rw_mediainfo_stdout_regexp_execute(void)
 {
-	int rv = 0;
-
-	/*  "Write Speed #0:        4.0x1385=5540KB/s" */
-	rv = regexec(&regex1, stdout_pipe_buffer, regex1.re_nsub + 1, matches1, 0);
-	if (rv == 0) {
-		cdw_dvd_rw_mediainfo_pr_handle_dvd_write_speed(&regex1, matches1);
-	}
-
-	/* "Disc status:           xxxxxxx" */
-	rv = regexec(&regex2, stdout_pipe_buffer, regex2.re_nsub + 1, matches2, 0);
-	if (rv == 0) {
-		cdw_dvd_rw_mediainfo_pr_handle_disc_status(&regex2, matches2);
-	}
-
-	/* "Mounted Media:         1Ah, DVD+RW" */
-	rv = regexec(&regex3, stdout_pipe_buffer, regex3.re_nsub + 1, matches3, 0);
-	if (rv == 0) {
-		cdw_dvd_rw_mediainfo_pr_handle_mounted_media(&regex3, matches3);
-	}
-#if 0
-	/* " Track#2  :             14@145056" */
-	rv = regexec(&regex4, stdout_pipe_buffer, regex4.re_nsub + 1, matches4, 0);
-	if (rv == 0) {
-		/* valid but unused */
-		/* handle_dvd_toc(&regex4, matches4); */
-	}
-
-	/* " Track Start Address:   666512" */
-	rv = regexec(&regex5, stdout_pipe_buffer, regex5.re_nsub + 1, matches5, 0);
-	if (rv == 0) {
-		handle_track_start_address(&regex5, matches5);
-	}
-
-	/* "Next Writable Address: 470672*2KB" */
-	rv = regexec(&regex6, stdout_pipe_buffer, regex6.re_nsub + 1, matches6, 0);
-	if (rv == 0) {
-		handle_next_writable_address(&regex6, matches6);
-	}
-#endif
-	/* "Current Write Speed:   16.0x1385=22160KB/s" */
-	rv = regexec(&regex7, stdout_pipe_buffer, regex7.re_nsub + 1, matches7, 0);
-	if (rv == 0) {
-		cdw_dvd_rw_mediainfo_pr_handle_current_write_speed(&regex7, matches7);
-	}
-
-	/* "READ CAPACITY:          0*2048=0" */
-	rv = regexec(&regex8, stdout_pipe_buffer, regex8.re_nsub + 1, matches8, 0);
-	if (rv == 0) {
-		cdw_sdm ("dispatching to cdw_dvd_rw_mediainfo_pr_handle_read_capacity()\n");
-		cdw_dvd_rw_mediainfo_pr_handle_read_capacity(&regex8, matches8);
-	}
-
-	/* "DVD-ROM media detected, exiting..." */
-	rv = regexec(&regex9, stdout_pipe_buffer, regex9.re_nsub + 1, matches9, 0);
-	if (rv == 0) {
-		cdw_vdm ("handling \"DVD-ROM media detected, exiting...\"\n");
-		/* this isn't necessary, because
-		   cdw_dvd_rw_mediainfo_pr_handle_mounted_media()
-		   can recognize DVD-ROM, but it doesn't hurt either */
-		current_disc.type = CDW_DVD_ROM;
-	}
+	stdout_pipe_buffer[PIPE_BUFFER_SIZE] = '\0';
+	cdw_regex_execute_regexes_table(stdout_regex, stdout_pipe_buffer);
 
 	return;
 }
@@ -226,28 +191,12 @@ void dvd_rw_mediainfo_stdout_regexp_execute(void)
 
 
 
+/**
+   \brief Call cdw_regex_clean_up_regexes_table() for stdout regexes table
+*/
 void dvd_rw_mediainfo_stdout_regexp_destroy(void)
 {
-	regfree(&regex1);
-	regfree(&regex2);
-	regfree(&regex3);
-	/* regfree(&regex4);
-	   regfree(&regex5);
-	   regfree(&regex6); */
-	regfree(&regex7);
-	regfree(&regex8);
-	regfree(&regex9);
-
-	free(matches1);
-	free(matches2);
-	free(matches3);
-	/* free(matches4);
-	   free(matches5);
-	   free(matches6); */
-	free(matches7);
-	free(matches8);
-	free(matches9);
-
+	cdw_regex_clean_up_regexes_table(stdout_regex);
 	return;
 }
 
@@ -271,7 +220,7 @@ Write Speed #4:        4.0x1385=5540KB/s
    use the number to index current_disc.write_speeds.speeds[] table,
    and store writing speed in the table.
 */
-void cdw_dvd_rw_mediainfo_pr_handle_dvd_write_speed(regex_t *regex, regmatch_t *matches)
+int cdw_dvd_rw_mediainfo_pr_handle_dvd_write_speed(regex_t *regex, regmatch_t *matches)
 {
 	/* regexp for capturing write speeds is
                             1        2       3        4
@@ -318,7 +267,7 @@ void cdw_dvd_rw_mediainfo_pr_handle_dvd_write_speed(regex_t *regex, regmatch_t *
 		}
 	}
 
-	return;
+	return 0;
 }
 
 
@@ -338,7 +287,7 @@ void cdw_dvd_rw_mediainfo_pr_handle_dvd_write_speed(regex_t *regex, regmatch_t *
  * Disc status for DVD can be one of following:
  * "blank", "appendable", "complete", "other"
  */
-void cdw_dvd_rw_mediainfo_pr_handle_disc_status(regex_t *regex, regmatch_t *matches)
+int cdw_dvd_rw_mediainfo_pr_handle_disc_status(regex_t *regex, regmatch_t *matches)
 {
 	/* regexp for capturing disc status:
                            1                   2
@@ -350,7 +299,7 @@ void cdw_dvd_rw_mediainfo_pr_handle_disc_status(regex_t *regex, regmatch_t *matc
 	int len = cdw_regex_get_submatch(matches, sub, stdout_pipe_buffer, submatch);
 	if (len == -1) {
 		cdw_vdm ("ERROR: len of subexpr %d is negative: %d\n", sub, len);
-		return;
+		return -1;
 	} else {
 		/* yes, using only first letter as information about
 		   status reported by dvd+rw-mediainfo - all statuses
@@ -361,7 +310,7 @@ void cdw_dvd_rw_mediainfo_pr_handle_disc_status(regex_t *regex, regmatch_t *matc
 		cdw_vdm ("INFO: disc status \"%s\" saved as '%c'\n",
 			 submatch,
 			 current_disc.dvd_rw_mediainfo_info.disc_status);
-		return;
+		return 0;
 	}
 }
 
@@ -384,7 +333,7 @@ void cdw_dvd_rw_mediainfo_pr_handle_disc_status(regex_t *regex, regmatch_t *matc
 
    This function sets current_disc.type
 */
-void cdw_dvd_rw_mediainfo_pr_handle_mounted_media(regex_t *regex, regmatch_t *matches)
+int cdw_dvd_rw_mediainfo_pr_handle_mounted_media(regex_t *regex, regmatch_t *matches)
 {
 	/* regexp for capturing media type:
                             1        2           3         4
@@ -398,7 +347,7 @@ void cdw_dvd_rw_mediainfo_pr_handle_mounted_media(regex_t *regex, regmatch_t *ma
 	int len = cdw_regex_get_submatch(matches, sub, stdout_pipe_buffer, submatch);
 	if (len == -1) {
 		cdw_vdm ("ERROR: len of subexpr %d is negative: %d\n", sub, len);
-		return;
+		return -1;
 	}
 
 	cdw_vdm ("INFO submatch %d, media type = \"%s\"\n", sub, submatch);
@@ -441,14 +390,14 @@ void cdw_dvd_rw_mediainfo_pr_handle_mounted_media(regex_t *regex, regmatch_t *ma
 		cdw_vdm ("WARNING: unknown disc type, submatch was \"%s\"\n", submatch);
 	}
 
-	return;
+	return 0;
 }
 
 
 
 
 
-void cdw_dvd_rw_mediainfo_pr_handle_current_write_speed(regex_t *regex, regmatch_t *matches)
+int cdw_dvd_rw_mediainfo_pr_handle_current_write_speed(regex_t *regex, regmatch_t *matches)
 {
 	/* regexp for capturing current write speed is
                                    1       2
@@ -460,7 +409,7 @@ void cdw_dvd_rw_mediainfo_pr_handle_current_write_speed(regex_t *regex, regmatch
 	int len = cdw_regex_get_submatch(matches, sub, stdout_pipe_buffer, submatch);
 	if (len == -1) {
 		cdw_vdm ("ERROR: len of subexpr %d is negative: %d\n", sub, len);
-		return;
+		return -1;
 	}
 
 	/* current write speed */
@@ -468,7 +417,7 @@ void cdw_dvd_rw_mediainfo_pr_handle_current_write_speed(regex_t *regex, regmatch
 	cdw_vdm ("INFO: submatch %d, current write speed = \"%s\" -> %d\n",
 		 sub, submatch, current_disc.write_speeds.drive_default_speed);
 
-	return;
+	return 0;
 }
 
 
@@ -480,7 +429,7 @@ void cdw_dvd_rw_mediainfo_pr_handle_current_write_speed(regex_t *regex, regmatch
 "READ CAPACITY:          0*2048=0"
 \endverbatim
 */
-void cdw_dvd_rw_mediainfo_pr_handle_read_capacity(regex_t *regex, regmatch_t *matches)
+int cdw_dvd_rw_mediainfo_pr_handle_read_capacity(regex_t *regex, regmatch_t *matches)
 {
 	/* regexp for capturing read capacity of DVD disc is
                              1       2
@@ -492,7 +441,7 @@ void cdw_dvd_rw_mediainfo_pr_handle_read_capacity(regex_t *regex, regmatch_t *ma
 	int len = cdw_regex_get_submatch(matches, sub, stdout_pipe_buffer, submatch);
 	if (len == -1) {
 		cdw_vdm ("ERROR: len of subexpr %d is negative: %d\n", sub, len);
-		return;
+		return -1;
 	}
 	current_disc.dvd_rw_mediainfo_info.read_capacity = (ssize_t) atoll(submatch);
 
@@ -504,7 +453,83 @@ void cdw_dvd_rw_mediainfo_pr_handle_read_capacity(regex_t *regex, regmatch_t *ma
 		 sub, submatch, current_disc.dvd_rw_mediainfo_info.read_capacity);
 	cdw_vdm ("INFO: read capacity = %zd\n", current_disc.dvd_rw_mediainfo_info.read_capacity);
 
-	return;
+	return 0;
+}
+
+
+
+
+
+int cdw_dvd_rw_mediainfo_pr_handle_end_lba(regex_t *regex, regmatch_t *matches)
+{
+	/* regexp for capturing current write speed is
+                                1      2        3          4
+	   "Speed Descriptor#([0-9]+):( +)([0-9a-fA-F]+)/([0-9]+) " */
+	cdw_regex_assert_subex_number(regex->re_nsub, 4);
+
+	unsigned int sub = 4;
+	char submatch[PIPE_BUFFER_SIZE];
+	int len = cdw_regex_get_submatch(matches, sub, stdout_pipe_buffer, submatch);
+	if (len == -1) {
+		cdw_vdm ("ERROR: len of subexpr %d is negative: %d\n", sub, len);
+		return -1;
+	}
+
+	long long int end_lba = strtoll(submatch, (char **) NULL, 10);
+	if (end_lba > current_disc.dvd_rw_mediainfo_info.end_lba) {
+		cdw_vdm ("INFO: end LBA = %lld\n", end_lba);
+		current_disc.dvd_rw_mediainfo_info.end_lba = end_lba;
+	}
+
+	return 0;
+
+}
+
+
+
+
+
+int cdw_dvd_rw_mediainfo_pr_handle_track_size(regex_t *regex, regmatch_t *matches)
+{
+	/* regexp for capturing track size information is
+                        1      2    3
+	   "Track Size:( +)([0-9]+)(*)2KB" */
+	cdw_regex_assert_subex_number(regex->re_nsub, 3);
+
+	unsigned int sub = 2;
+	char submatch[PIPE_BUFFER_SIZE];
+	int len = cdw_regex_get_submatch(matches, sub, stdout_pipe_buffer, submatch);
+	if (len == -1) {
+		cdw_vdm ("ERROR: len of subexpr %d is negative: %d\n", sub, len);
+		return -1;
+	}
+
+	long int track_size = strtoll(submatch, (char **) NULL, 10);
+	if (current_disc.dvd_rw_mediainfo_info.all_tracks_size == -1) {
+		/* init */
+		current_disc.dvd_rw_mediainfo_info.all_tracks_size = 0;
+	}
+	current_disc.dvd_rw_mediainfo_info.all_tracks_size += track_size;
+	/* current track may be the last one */
+	current_disc.dvd_rw_mediainfo_info.last_track_size = track_size;
+	cdw_vdm ("INFO: all tracks size = %lld\n", current_disc.dvd_rw_mediainfo_info.all_tracks_size);
+	cdw_vdm ("INFO: last track size = %lld\n", current_disc.dvd_rw_mediainfo_info.last_track_size);
+
+	return 0;
+}
+
+
+
+
+
+int cdw_dvd_rw_mediainfo_pr_handle_dvd_rom_detected(__attribute__((unused)) regex_t *regex, __attribute__((unused)) regmatch_t *matches)
+{
+	/* this isn't necessary, because
+	   cdw_dvd_rw_mediainfo_pr_handle_mounted_media()
+	   can recognize DVD-ROM, but it doesn't hurt either */
+	current_disc.type = CDW_DVD_ROM;
+
+	return 0;
 }
 
 
@@ -533,7 +558,7 @@ FABRICATED TOC:
  * The function has similar effect to handle_track_start_address() but it does
  * not capture start of blank area. However it does capture start of AA area.
  */
-void handle_dvd_toc(regex_t *regex, regmatch_t *matches)
+int cdw_dvd_rw_mediainfo_pr_handle_dvd_toc(regex_t *regex, regmatch_t *matches)
 {
 	char track_index[3];
 	char track_start_address[10];
@@ -576,7 +601,7 @@ void handle_dvd_toc(regex_t *regex, regmatch_t *matches)
 		}
 	}
 
-	return;
+	return 0;
 }
 
 
@@ -601,7 +626,7 @@ void handle_dvd_toc(regex_t *regex, regmatch_t *matches)
    handle_track_start_address() does not capture AA track.
    handle_dvd_toc() does it.
 */
-void handle_track_start_address(regex_t *regex, regmatch_t *matches)
+int cdw_dvd_rw_mediainfo_pr_handle_track_start_address(regex_t *regex, regmatch_t *matches)
 {
 	unsigned int i;
 	for (i = 0; i < (regex->re_nsub + 1); ++i) {
@@ -622,7 +647,7 @@ void handle_track_start_address(regex_t *regex, regmatch_t *matches)
 		}
 	}
 
-	return;
+	return 0;
 }
 
 
@@ -664,7 +689,7 @@ STDOUT READ TRACK INFORMATION[#4]:
  * "Next Writable Address" (nwa) instead of '"Track Start Address" of
  * blank track' as value of next_sess_start.
  */
-void handle_next_writable_address(regex_t *regex, regmatch_t *matches)
+int cdw_dvd_rw_mediainfo_pr_handle_next_writable_address(regex_t *regex, regmatch_t *matches)
 {
 	long int nwa = 0;
 
@@ -708,7 +733,7 @@ void handle_next_writable_address(regex_t *regex, regmatch_t *matches)
 	cdw_sdm ("current_disc.last_sess_start = %ld\n", current_disc.last_sess_start);
 	cdw_sdm ("current_disc.next_sess_start = %ld\n", current_disc.next_sess_start);
 
-	return;
+	return 0;
 }
 
 #endif
