@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2010 Kamil Ignacak
+ * Copyright (C) 2007 - 2011 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -198,13 +198,19 @@ Notes:
 5. no extensive tests of "-M -dvd-compat" done yet, no external information seen yet
 6. initial tests suggest no possibility of closing multisession disc
 
+
+Dummy write:
+                DVD-R   DVD-RW Seq   DVD-R DL Seq   DVD-R DL Jump   Other
+growisofs 7.0:   y          y            y               y            n
+growisofs 7.1:   y          y            y               y            n
+
+
 \endverbatim
 */
 
 #include <stdio.h>
 #include <stdlib.h>
 
-#include "cdw_config.h"
 #include "cdw_string.h"
 #include "gettext.h"
 #include "cdw_processwin.h"
@@ -216,8 +222,6 @@ Notes:
 #include "cdw_drive.h"
 
 
-extern cdw_config_t global_config;
-
 
 static cdw_rv_t cdw_growisofs_write_from_image(cdw_task_t *task, cdw_disc_t *disc);
 static cdw_rv_t cdw_growisofs_write_from_files(cdw_task_t *task, cdw_disc_t *disc);
@@ -230,7 +234,7 @@ static char *cdw_growisofs_create_command_write_from_files(cdw_task_t *task, cdw
 cdw_rv_t cdw_growisofs_run_task(cdw_task_t *task, cdw_disc_t *disc)
 {
 	cdw_assert (task->id == CDW_TASK_BURN_FROM_FILES || task->id == CDW_TASK_BURN_FROM_IMAGE,
-		    "ERROR: incorrect task id: %d\n", task->id);
+		    "ERROR: incorrect task id: %lld\n", task->id);
 
 	cdw_rv_t crv = CDW_OK;
 	if (task->id == CDW_TASK_BURN_FROM_IMAGE) {
@@ -238,7 +242,7 @@ cdw_rv_t cdw_growisofs_run_task(cdw_task_t *task, cdw_disc_t *disc)
 	} else if (task->id == CDW_TASK_BURN_FROM_FILES) {
 		crv = cdw_growisofs_write_from_files(task, disc);
 	} else {
-		cdw_assert (0, "ERROR: incorrect task id %d\n", task->id);
+		cdw_assert (0, "ERROR: incorrect task id %lld\n", task->id);
 	}
 
 	return crv;
@@ -265,7 +269,7 @@ cdw_rv_t cdw_growisofs_write_from_image(cdw_task_t *task, cdw_disc_t *disc)
 	char *command = cdw_growisofs_create_command_write_from_image(task, disc);
 	if (command == (char *) NULL) {
 		cdw_vdm ("ERROR: failed to prepare command for writing from image\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 	cdw_vdm ("INFO: command for writing ISO image to DVD with growisofs is:\n\"%s\"\n", command);
 
@@ -273,12 +277,16 @@ cdw_rv_t cdw_growisofs_write_from_image(cdw_task_t *task, cdw_disc_t *disc)
 	   preparation to writing, but the writing hasn't started yet */
 	cdw_processwin_display_sub_info(_("Preparing to write. Please wait."));
 
-	run_command(command, task);
+	int rv = run_command(command, task);
 
 	free(command);
 	command = (char *) NULL;
 
-	return CDW_OK;
+	if (rv == 0) {
+		return CDW_OK;
+	} else {
+		return CDW_ERROR;
+	}
 }
 
 
@@ -303,7 +311,7 @@ cdw_rv_t cdw_growisofs_write_from_files(cdw_task_t *task, cdw_disc_t *disc)
 	char *command = cdw_growisofs_create_command_write_from_files(task, disc);
 	if (command == (char *) NULL) {
 		cdw_vdm ("ERROR: failed to prepare command for writing from files\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 	cdw_vdm ("INFO: command for writing files to DVD with growisofs is:\n\"%s\"\n", command);
 
@@ -311,12 +319,16 @@ cdw_rv_t cdw_growisofs_write_from_files(cdw_task_t *task, cdw_disc_t *disc)
 	   preparation to writing, but the writing hasn't started yet */
 	cdw_processwin_display_sub_info(_("Preparing to write. Please wait."));
 
-	run_command(command, task);
+	int rv = run_command(command, task);
 
 	free(command);
 	command = (char *) NULL;
 
-	return CDW_OK;
+	if (rv == 0) {
+		return CDW_OK;
+	} else {
+		return CDW_ERROR;
+	}
 }
 
 
@@ -379,24 +391,24 @@ char *cdw_growisofs_create_command_write_from_image(cdw_task_t *task, cdw_disc_t
 	/* remember that when writing image to dvd no mkisofs options are
 	   allowed; this includes "-use-the-force-luke=moi" */
 
-	char speed_string[10];
-	snprintf(speed_string, 9 + 1, "%d", task->burn.speed);
+	char speed_string[9 + 1];
+	snprintf(speed_string, 9 + 1, "%lld", task->burn.speed_id);
 
 	const char *drive = cdw_drive_get_drive_fullpath();
 
-	cdw_assert (task->burn.tool.fullpath != (char *) NULL, "ERROR: tool fullpath is NULL\n");
-	char *command = cdw_string_concat(task->burn.tool.fullpath,
+	cdw_assert (task->burn.tool.label != (char *) NULL, "ERROR: tool fullpath is NULL\n");
+	char *command = cdw_string_concat(task->burn.tool.label,
 				     " -speed=", speed_string, " ",
 				     dao ? " -use-the-force-luke=dao " : " ",
-				     global_config.dummy ? " -use-the-force-luke=dummy " : " ",
-				     " ", global_config.other_growisofs_options, " ",
+				     task->burn.dummy ? " -use-the-force-luke=dummy " : " ",
+				     " ", task->burn.growisofs.other_growisofs_options, " ",
 
 				     dvd_compat ? " -dvd-compat " : " ",
 
 				     " -Z ",
 				     /* path to ISO file may contain spaces, so
 					surround the path with quotes */
-				     drive, "=", "\"", global_config.iso_image_full_path, "\"",
+				     drive, "=", "\"", task->burn.iso9660_file_fullpath, "\"",
 
 				     (char *) NULL);
 
@@ -554,25 +566,25 @@ char *cdw_growisofs_create_command_write_from_files(cdw_task_t *task, cdw_disc_t
 	cdw_logging_write(_("mkisofs options used by growisofs for writing files to DVD:\n"));
 	cdw_logging_write("\"%s\"\n", mkisofs_options);
 
-	char speed_string[10];
-	snprintf(speed_string, 9 + 1, "%d", task->burn.speed);
+	char speed_string[9 + 1];
+	snprintf(speed_string, 9 + 1, "%lld", task->burn.speed_id);
 
 	const char *drive = cdw_drive_get_drive_fullpath();
 
-	cdw_assert (task->burn.tool.fullpath != (char *) NULL, "ERROR: tool fullpath is NULL\n");
-	char *command = cdw_string_concat(task->burn.tool.fullpath,
+	cdw_assert (task->burn.tool.label != (char *) NULL, "ERROR: tool fullpath is NULL\n");
+	char *command = cdw_string_concat(task->burn.tool.label,
 					  " -speed=", speed_string, " ",
 					  use_dao ? " -use-the-force-luke=dao " : " ",
-					  global_config.dummy ? " -use-the-force-luke=dummy " : " ",
+					  task->burn.dummy ? " -use-the-force-luke=dummy " : " ",
 					  " -use-the-force-luke=moi ",
 
-					  " ", global_config.other_growisofs_options, " ",
+					  " ", task->burn.growisofs.other_growisofs_options, " ",
 
 					  dvd_compat ? " -dvd-compat " : " ",
 
 					  write_initial ? " -Z " : " -M ", drive , " ",
 
-					  /* this includes config.other_mkisofs_options */
+					  /* this includes other_mkisofs_options */
 					  mkisofs_options, " ",
 
 					  (char *) NULL);
@@ -597,6 +609,31 @@ char *cdw_growisofs_create_command_write_from_files(cdw_task_t *task, cdw_disc_t
 
 
 
+/**
+   \brief Check if growisofs supports "dummy" option for given
+   type of current disc
+
+*/
+bool cdw_growisofs_allow_dummy(cdw_disc_t *disc)
+{
+	/* "dummy" option is supported for these
+	   disc types: DVD-R, DVD-RW Seq, DVD-R DL Seq, DVD-R DL Jump
+
+	   Currently exact types of DVD DL discs aren't recognized
+	   fully, so the function returns true only for single layer discs */
+	if (disc->type == CDW_DVD_R_SEQ
+	    || disc->type == CDW_DVD_R) {
+		return true;
+	} else {
+		return false;
+	}
+}
+
+
+
+
+
+
 /* *** unused code follows *** */
 
 #if 0
@@ -670,16 +707,22 @@ cdw_rv_t run_command_growisofs_blank_dvd(cdw_task_t *task)
 	   preparation to erasing, but the erasing hasn't started yet */
 	cdw_processwin_display_sub_info(_("Preparing to erase. Please wait."));
 
-	run_command(command, task);
+	int rv = run_command(command, task);
 
 	free(command);
 	command = (char *) NULL;
 
-	/* 2TRANS: this is message in process window - program finished
-	   blanking disc, status unknown */
-	cdw_processwin_destroy(_("Erasing of DVD finished"), true);
-
-	return CDW_OK;
+	if (rv == 0) {
+		/* 2TRANS: this is message in process window - program finished
+		   blanking disc, status unknown */
+		cdw_processwin_destroy(_("Erasing of DVD finished"), true);
+		return CDW_OK;
+	} else {
+		/* 2TRANS: this is message in process window - program didn't
+		   erase blanking because of errors */
+		cdw_processwin_destroy(_("Erasing of DVD failed, please see log file"), true);
+		return CDW_ERROR;
+	}
 }
 
 
