@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2010 Kamil Ignacak
+ * Copyright (C) 2007 - 2011 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +28,8 @@
 #include "cdw_thread.h" /* PIPE_BUFFER_SIZE */
 #include "cdw_debug.h"
 #include "cdw_utils.h"
-
+#include "cdw_logging.h"
+#include "gettext.h"
 
 extern char stdout_pipe_buffer[PIPE_BUFFER_SIZE + 1];
 extern char stderr_pipe_buffer[PIPE_BUFFER_SIZE + 1];
@@ -44,38 +45,63 @@ extern cdw_task_t *thread_task;
 
 
 /* functions used for stderr output handling */
-static void handle_image_creation(regex_t *regex, regmatch_t *matches);
-static void handle_image_creation_100percent(void);
 static void handle_bad_option(regex_t *regex, regmatch_t *matches);
+static int cdw_mkisofs_handle_image_creation(regex_t *regex, regmatch_t *matches);
+static int cdw_mkisofs_handle_image_creation_100percent(regex_t *regex, regmatch_t *matches);
+static int cdw_mkisofs_handle_implementation_botch(regex_t *regex, regmatch_t *matches);
 
-static regex_t eregex1, eregex2, eregex3, eregex4, eregex5, eregex6;
+static regex_t eregex3, eregex4, eregex5, eregex6;
 static regex_t eregex7, eregex8, eregex9, eregex10, eregex11, eregex12;
 static regex_t eregex13, eregex14, eregex15, eregex16;
-static regmatch_t *ematches1 = NULL, *ematches2 = NULL, *ematches3 = NULL, *ematches4 = NULL;
+static regmatch_t *ematches3 = NULL, *ematches4 = NULL;
 static regmatch_t *ematches5 = NULL, *ematches6 = NULL, *ematches7 = NULL, *ematches8 = NULL;
 static regmatch_t *ematches9 = NULL, *ematches10 = NULL, *ematches11 = NULL, *ematches12 = NULL;
 static regmatch_t *ematches13 = NULL, *ematches14 = NULL, *ematches15 = NULL, *ematches16 = NULL;
 
 
+static cdw_regex_t stderr_regex[] = {
+	{ "creating 0-99", 301,
+	  /* Mkisofs process during image creation (0 - 99,9%) */
+	  "([ ]*)([0-9]+).([0-9]+)% done,",
+	  (regex_t *) NULL, (regmatch_t *) NULL, REG_EXTENDED,
+	  cdw_mkisofs_handle_image_creation },
+
+	{ "creating 100", 302,
+	  /* Mkisofs 100% (image created) */
+	  "Total translation table size",
+	  (regex_t *) NULL, (regmatch_t *) NULL, REG_EXTENDED,
+	  cdw_mkisofs_handle_image_creation_100percent },
+
+	{ "Implementation botch", 317,
+	  /* "/home/acerion/bin/mkisofs: Implementation botch: Hashed file '/my_link_VLT' has illegal inode 5B520.
+	      :-( /home/acerion/bin/mkisofs has failed: 255 "
+
+	      This happened when I tried to burn files with growisofs+mkisofs
+	      to open DVD disc to which I have already burned some files with xorriso */
+	  "Implementation botch: (.+)",
+	  (regex_t *) NULL, (regmatch_t *) NULL, REG_EXTENDED,
+	  cdw_mkisofs_handle_implementation_botch },
+
+	{ "table guard", -1,
+	  /* guard: debug_id = -1 */
+	  "",
+	  (regex_t *) NULL, (regmatch_t *) NULL, 0,
+	  (cdw_regex_handler_t) NULL }
+};
+
+
+
+
 
 void mkisofs_stderr_regexp_prepare(void)
 {
+	cdw_rv_t crv = cdw_regex_prepare_regexes_table(stderr_regex);
+	if (crv != CDW_OK) {
+		cdw_vdm("ERROR: failed to prepare regexes table for stderr\n");
+	}
+
 	int rv = 0;
 
-	/* Mkisofs process during image creation (0 - 99,9%) */
-	rv = regcomp(&eregex1, "([ ]*)([0-9]+).([0-9]+)% done,", REG_EXTENDED);
-	ematches1 = (regmatch_t *) calloc(1, (eregex1.re_nsub + 1) * sizeof(regmatch_t));
-	if (rv) {
-		cdw_regex_regcomp_error_handler(__func__, rv, &eregex1, 31);
-	}
-
-	/* Mkisofs 100% (image created) */
-	rv = regcomp(&eregex2, "Total translation table size", REG_EXTENDED);
-	ematches2 = (regmatch_t *) calloc(1, (eregex2.re_nsub + 1) * sizeof(regmatch_t));
-	if (rv) {
-		cdw_regex_regcomp_error_handler(__func__, rv, &eregex2, 32);
-	}
-
 	/* "/usr/bin/mkisofs: Error: <filename1> and <filename2> have the same Joliet name" */
 	rv = regcomp(&eregex3, "(have the same Joliet name)", REG_EXTENDED);
 	ematches3 = (regmatch_t *) calloc(1, (eregex3.re_nsub + 1) * sizeof(regmatch_t));
@@ -213,21 +239,10 @@ void mkisofs_stderr_regexp_prepare(void)
 
 void mkisofs_stderr_regexp_execute(void)
 {
+	stderr_pipe_buffer[PIPE_BUFFER_SIZE] = '\0';
+	cdw_regex_execute_regexes_table(stderr_regex, stderr_pipe_buffer);
+
 	int rv = 0;
-	rv = regexec(&eregex1, stderr_pipe_buffer, eregex1.re_nsub + 1, ematches1, 0);
-	if (rv == 0 && thread_task->id == CDW_TASK_CREATE_IMAGE) {
-		/* additional check is to avoid mistake with direct writing */
-
-		cdw_vdm ("INFO: calling handle_image_creation()\n");
-		handle_image_creation(&eregex1, ematches1);
-	}
-
-	rv = regexec(&eregex2, stderr_pipe_buffer, eregex2.re_nsub + 1, ematches2, 0);
-	if (rv == 0 && thread_task->id == CDW_TASK_CREATE_IMAGE) {
-		/* additional check is to avoid mistake with direct writing */
-		cdw_vdm ("INFO: calling handle_image_creation_100percent()\n");
-		handle_image_creation_100percent();
-	}
 
 	rv = regexec(&eregex3, stderr_pipe_buffer, eregex3.re_nsub + 1, ematches3, 0);
 	if (rv == 0) {
@@ -329,8 +344,8 @@ void mkisofs_stderr_regexp_execute(void)
 
 void mkisofs_stderr_regexp_destroy(void)
 {
-	regfree(&eregex1);
-	regfree(&eregex2);
+	cdw_regex_clean_up_regexes_table(stderr_regex);
+
 	regfree(&eregex3);
 	regfree(&eregex4);
 	regfree(&eregex5);
@@ -346,8 +361,6 @@ void mkisofs_stderr_regexp_destroy(void)
 	regfree(&eregex15);
 	regfree(&eregex16);
 
-	free(ematches1);
-	free(ematches2);
 	free(ematches3);
 	free(ematches4);
 	free(ematches5);
@@ -363,8 +376,6 @@ void mkisofs_stderr_regexp_destroy(void)
 	free(ematches15);
 	free(ematches16);
 
-	ematches1 = (regmatch_t *) NULL;
-	ematches2 = (regmatch_t *) NULL;
 	ematches3 = (regmatch_t *) NULL;
 	ematches4 = (regmatch_t *) NULL;
 	ematches5 = (regmatch_t *) NULL;
@@ -422,8 +433,13 @@ void mkisofs_stdout_regexp_destroy(void)
    \param regex
    \param matches
 */
-void handle_image_creation(regex_t *regex, regmatch_t *matches)
+int cdw_mkisofs_handle_image_creation(regex_t *regex, regmatch_t *matches)
 {
+	/* additional check is to avoid mistake with direct writing */
+	if (thread_task->id != CDW_TASK_CREATE_IMAGE) {
+		return 0;
+	}
+
 	/* Regexp for capturing information on creating image looks like this:
 	     1      2        3
 	  "([ ]*)([0-9]+).([0-9]+)% done,"
@@ -437,7 +453,7 @@ void handle_image_creation(regex_t *regex, regmatch_t *matches)
 	int len = cdw_regex_get_submatch(matches, sub, stderr_pipe_buffer, submatch);
 	if (len == -1) {
 		cdw_vdm ("ERROR: len of subexpr %d is negative: %d\n", sub, len);
-		return;
+		return -1;
 	}
 
 	/* amount of data written to image */
@@ -456,7 +472,7 @@ void handle_image_creation(regex_t *regex, regmatch_t *matches)
 	cdw_processwin_display_progress_conditional(done_size, 100, (char *) NULL);
 	cdw_processwin_wrefresh();
 
-	return;
+	return 0;
 }
 
 
@@ -483,8 +499,13 @@ void handle_image_creation(regex_t *regex, regmatch_t *matches)
 
    This function is called after capturing "Total translation table size"
 */
-void handle_image_creation_100percent(void)
+int cdw_mkisofs_handle_image_creation_100percent(__attribute__((unused)) regex_t *regex, __attribute__((unused)) regmatch_t *matches)
 {
+	/* additional check is to avoid mistake with direct writing */
+	if (thread_task->id != CDW_TASK_CREATE_IMAGE) {
+		return 0;
+	}
+
 	/* fake some data to show that creating image has finished */
 	char eta_string[PROCESSWIN_MAX_RTEXT_LEN + 1];
 	memset(eta_string, '\0', PROCESSWIN_MAX_RTEXT_LEN + 1);
@@ -494,7 +515,7 @@ void handle_image_creation_100percent(void)
 	cdw_processwin_display_progress(100, 100, (char *) NULL);
 	cdw_processwin_wrefresh();
 
-	return;
+	return 0;
 }
 
 
@@ -538,3 +559,33 @@ void handle_bad_option(regex_t *regex, regmatch_t *matches)
 	return;
 }
 
+
+
+
+
+int cdw_mkisofs_handle_implementation_botch(regex_t *regex, regmatch_t *matches)
+{
+	/* regex for capturing error:
+                                    1
+	   "Implementation botch: (.+)" */
+	cdw_regex_assert_subex_number(regex->re_nsub, 1);
+
+	unsigned int sub = 1;
+	char submatch[PIPE_BUFFER_SIZE + 1];
+	int len = cdw_regex_get_submatch(matches, sub, stderr_pipe_buffer, submatch);
+	if (len == -1) {
+		cdw_vdm ("ERROR: len of subexpr %d is negative: %d\n", sub, len);
+
+		return -1;
+	} else {
+		thread_task->tool_status.mkisofs |= CDW_TOOL_STATUS_MKISOFS_IMPLEMENTATION_BOTCH;
+		cdw_vdm ("ERROR: thread_task->tool_status.mkisofs |= CDW_TOOL_STATUS_MKISOFS_IMPLEMENTATION_BOTCH\n");
+		cdw_logging_write(_("ERROR: problem with creating file system on optical disc.\n"));
+		cdw_logging_write(_("ERROR: Perhaps this information will help resolve the problem:\n"));
+		cdw_logging_write("ERROR: %s\n", submatch);
+
+		return 0;
+	}
+}
+
+
