@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2010 Kamil Ignacak
+ * Copyright (C) 2007 - 2011 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 
 
 /**
- * \file thread.c
+ * \file cdw_thread.c
  * \brief Code creating threads and processes used in communication
  *        with external tools
  *
@@ -36,7 +36,7 @@
  * run_command() hides threads and exec() from rest of project code.
  *
  * Thread functions are read_child_stdout() and read_child_stderr() (and
- * write_to_md5sum_stdin() - see below). They are executed in parent
+ * calculate_digest.read_and_write() - see below). They are executed in parent
  * process, reading data from stdout and stderr of child process, using
  * stdout_pipe[PARENTS_END] and stderr_pipe[PARENTS_END] file descriptors.
  * Child process is an external tool that creates iso image, burns data to
@@ -53,7 +53,7 @@
  *
  * File defines two generic thread functions that can be used for almost
  * any external command: read_child_stdout() and read_child_stderr().
- * There is also one special thread function: write_to_md5sum_stdin():
+ * There is also one special thread function: calculate_digest.read_and_write():
  * its task is to _write_ to stdin of child process, and it calls functions
  * related to one specific tool (md5sum). Description of read_child_*()
  * thread functions can be applied to this function if you replace 'read
@@ -77,7 +77,7 @@
 #include "cdw_regex_dispatch.h"
 #include "cdw_thread.h" /* PIPE_BUFFER_SIZE */
 #include "cdw_utils.h"
-#include "cdw_md5sum.h"
+#include "cdw_digest.h"
 #include "cdw_debug.h"
 #include "cdw_ext_tools.h"
 
@@ -125,11 +125,11 @@ static void cdw_thread_set_child_env(void);
 
 #define CDW_THREAD_EVAL_READ(m_char_buf, m_i, m_pipe_buf, m_new_lines, m_f) \
 	cdw_assert (m_f == cdw_stdout || m_f == cdw_stderr, "ERROR: incorrect file %d\n", m_f);	\
-	if (m_char_buf[0] == 0x00) {					\
+	if (m_char_buf == 0x00) {					\
 		;							\
-	} else if (m_char_buf[0] == '\n'				\
-		   || m_char_buf[0] == 0x0D				\
-		   || m_char_buf[0] == '\b' /* ^H */			\
+	} else if (m_char_buf == '\n'					\
+		   || m_char_buf == 0x0D				\
+		   || m_char_buf == '\b' /* ^H */			\
 		   || m_i == PIPE_BUFFER_SIZE) {			\
 									\
 		if (m_i != 0) { /* non-empty line */			\
@@ -154,15 +154,23 @@ static void cdw_thread_set_child_env(void);
 			}						\
 		}							\
 	} else {							\
-		m_pipe_buf[m_i] = m_char_buf[0];			\
+		m_pipe_buf[m_i] = m_char_buf;				\
 		m_i++;							\
 	}
 
 
 
-/* original developer comment:
- * I see CDrecorder version 0.1 source, and I write this code after this!!!
- * THX to Sharad Mittal */
+
+/**
+   \brief Read stdout output of child process
+
+   Function runs in thread of parent process, reading stdout
+   output of child process;
+
+   original developer's comment:
+   "I see CDrecorder version 0.1 source, and I write this code after this!!!
+   THX to Sharad Mittal;"
+*/
 void *read_child_stdout(__attribute__((unused)) void *dummy)
 {
 	cdw_sdm ("INFO: thread %s() awaits for data\n", __func__);
@@ -179,9 +187,8 @@ void *read_child_stdout(__attribute__((unused)) void *dummy)
 
 		/* get some data from pipe */
 		while (r > 0) {
-			char buf[1];
-			buf[0] = '\0';
-			r = read(stdout_pipe[0], buf, 1);
+			char buf = '\0';
+			r = read(stdout_pipe[PARENTS_END], &buf, 1);
 			int e = errno;
 			pthread_mutex_lock(&mutex);
 
@@ -227,9 +234,8 @@ void *read_child_stderr(__attribute__((unused)) void *dummy)
 
 		/* get some data from pipe */
 		while (er > 0) {
-			char ebuf[1];
-			ebuf[0] = '\0';
-			er = read(stderr_pipe[0], ebuf, 1);
+			char ebuf = '\0';
+			er = read(stderr_pipe[PARENTS_END], &ebuf, 1);
 			int e = errno;
 			pthread_mutex_lock(&mutex);
 
@@ -255,20 +261,21 @@ void *read_child_stderr(__attribute__((unused)) void *dummy)
 
 
 /**
- * \brief Run external command
- *
- * Create pairs of sockets to new process (using socketpair()) and launch
- * external program/tool (using provided 'command' and call to exec()) as
- * new child process. The process will inherit created sockets.
- *
- * Function makes 'task' available as 'thread_task' so that thread functions
- * (working in parent code) and regexp functions can set task->tool_status.
- *
- * \param command - name of program (tool) that you want to run + it's arguments
- * \param task - variable describing current task
- *
- * \return 1 on success
- */
+   \brief Run external command
+
+   Create pairs of sockets to new process (using socketpair()) and launch
+   external program/tool (using provided 'command' and call to exec()) as
+   new child process. The process will inherit created sockets.
+
+   Function makes 'task' available as 'thread_task' so that thread functions
+   (working in parent code) and regexp functions can set task->tool_status.
+
+   \param command - name of program (tool) that you want to run + it's arguments
+   \param task - variable describing current task
+
+   \return 0 on success
+   \return -1 on failure
+*/
 int run_command(const char *command, cdw_task_t *task)
 {
 	thread_task = task;
@@ -311,120 +318,71 @@ int run_command(const char *command, cdw_task_t *task)
 		fork_result = fork();
 
 		if (fork_result == -1) {
-			/* 2TRANS: this is string displayed in console when
+			/* 2TRANS: this is string displayed in log file when
 			   something went very badly and cdw cannot create child
-			   process - this is serious error and cdw will exit */
-			fprintf(stderr, _("Fork failure\n"));
-			exit(-1);
+			   process - this is serious error */
+			cdw_logging_write(_("Error: fork failure\n"));
+			return -1;
 		} else if (fork_result == 0) {
-			/* we are child process - don't write to stdout and stderr;
-			   instead we should write to socked inherited from parent process,
-			   so parent process can know what we (child process) are doing
+			/* we are child process; child process still should/will
+			   write to stdout and stderr, but the stdout and
+			   stderr file descriptors shouldn't be connected to
+			   console any longer - they should be connected to
+			   pipes; the pipes are inherited from parent process,
+			   and are supposed to be used to communicate with
+			   parent process, so parent process can know what
+			   child process is doing;
 
-			   dup() will copy it's argument descriptors to lowest available
-			   descriptors; let's make stdin, stdout and stderr
-			   'lowest available' by closing them before calling dup();
+			   child process inherits pipes, which includes
+			   parent's ends of the pipes; they are useless in
+			   child process, so it will be safe to close them
+			   in child process; parent will keep them open to
+			   read data from them - the data sent by child
+			   process; */
 
-			   let's also close original descriptor after copying it:
-			   we have copy, original is no longer needed; we don't need
-			   the other end of pipe here either (that is why we close both
-			   *_pipe[0] and *_pipe[1]), but parent will keep the other ends
-			   open to read from it as we write to our new stdout and stderr
-			*/
+			/* replace initial stdin/stdout/stderr file descriptors
+			   of child process with ends of pipes */
+			dup2(stdin_pipe[CHILDS_END], 0); /* WARNING: closing '0' may have strange influence on mkisofs */
+			dup2(stdout_pipe[CHILDS_END], 1);
+			dup2(stderr_pipe[CHILDS_END], 2);
 
-
-
-			/* now some work on switchboard */
-
-			/* close our original stdin so we can replace it
-			   with stdin descriptor from pipe */
-			close(0); /* WARNING: closing it may have strange influence on mkisofs */
-			/* this makes copy of pipe stdout descriptor
-			   and assigns '0' to this copy */
-			dup(stdin_pipe[CHILDS_END]);
-			/* close unused _copy_ of descriptor from pipe */
+			/* close unused instances of descriptors from pipe */
 			close(stdin_pipe[CHILDS_END]);
-
-			/* close our original stdout so we can replace it
-			   with stdout descriptor from pipe */
-			close(1);
-			/* this makes copy of pipe stdout descriptor
-			   and assigns '1' to this copy */
-			dup(stdout_pipe[CHILDS_END]);
-			/* close unused _copy_ of descriptor from pipe */
 			close(stdout_pipe[CHILDS_END]);
-
-			/* close our original stderr so we can replace
-			   it with stderr descriptor from pipe */
-			close(2);
-			/* this makes copy of pipe stderr descriptor and
-			   assigns '2' to this copy */
-			dup(stderr_pipe[CHILDS_END]);
-			/* close unused _copy_ of descriptor from pipe */
 			close(stderr_pipe[CHILDS_END]);
 
+			/* close parent's ends of pipes */
+			close(stdin_pipe[PARENTS_END]);
+			close(stdout_pipe[PARENTS_END]);
+			close(stderr_pipe[PARENTS_END]);
+
+			/* fcntl(stdin_pipe[CHILDS_END], F_SETFL, O_ASYNC); */
+			/* fcntl(stdin_pipe[PARENTS_END], F_SETFL, O_ASYNC); */
+
 			/* at this point child process still has descriptors
 			   '0', '1' and '2' (stdin, stdout and stderr) open,
 			   but they aren't connected to terminal (keyboard
 			   and screen) anymore: they are now connected to
-			   pipe, so any data printed by child application
+			   pipes, so any data printed by child application
 			   to stdout or stderr (e.g. using
 			   fprintf(stderr, ...) ) will be sent via pipe to
 			   parent. Parent app will be able to read child
 			   app data sent via pipe, as long as parent keeps
 			   his ends of stdout and stderr pipe open. */
 
-			/* close parent's ends of pipes: they are
-			   to be operated by parent only, we shouldn't
-			   read/write to them so let's close them and
-			   don't bother with them anymore */
-			close(stdin_pipe[PARENTS_END]);
-			close(stdout_pipe[PARENTS_END]);
-			close(stderr_pipe[PARENTS_END]);
-
-			fcntl(stdin_pipe[CHILDS_END], F_SETFL, O_ASYNC);
-			/* fcntl(stdin_pipe[PARENTS_END], F_SETFL, O_ASYNC); */
-
 			/* below a copy of cdw turns into e.g. mkisofs */
 
-
-#if 0
-			/* I'm not using execve anymore because it seems
-			   that it didn't have access to extended PATH
-			   variable, which made me unable to search in
-			   user's bin catalogs (using "which"); I could
-			   pass PATH to new process by env[], but building
-			   PATH for new process would require using concat() */
-
-			char *cmd[] = { "sh" , "-c", command, (char *) 0 };
-			char *env[] = { "LC_ALL=C", "TERM=xterm", (char *) 0 };
-
-			/* I'm using execve, because it allows me to set
-			   LC_ALL variable. After setting LC_ALL to C
-			   I am sure that any command will be running with
-			   English locale and will produce English messages
-			   passed via pipe to regexp code. This way I don't
-			   have to worry that some day cdrecord or mkisofs
-			   developers will localize their software. */
-			int exec_ret = execve ("/bin/sh", cmd, env);
-#endif
 			cdw_thread_set_child_env();
-
 			int exec_ret = execl("/bin/sh", "sh", "-c", command, NULL);
 
 			/* exec*() functions don't return */
 
-			/* 2TRANS: this is message displayed in console when
-			   something went very badly and exec() function
-			   returned after a call */
-			perror(_("Error: return of exec() function\n"));
-			/* 2TRANS: this is second part of message displayed
-			   in console when something went very badly and
-			   exec() function returned after a call; %d is
-			   value returned by exec(); cdw will exit after
-			   displaying this message */
-			fprintf(stderr, _("exec() returned with value %d\n"), exec_ret);
-			exit(0);
+			/* 2TRANS: this is message printed to log file when
+			   something went very badly and  exec() function
+			   returned after a call; %d is value returned by
+			   exec() */
+			cdw_logging_write(_("Error: exec() returned with value %d\n"), exec_ret);
+			return -1;
 		} else { /* we are still in parent process */
 
 			/* now close childs ends of pipes: they are to be
@@ -455,20 +413,20 @@ int run_command(const char *command, cdw_task_t *task)
 
 			cdw_sdm ("INFO: creating threads\n");
 
-			// if (task->id == CDW_TASK_CHECK_MD5SUM && task->verify.from_cd) {
-			if (task->id == CDW_TASK_CHECK_MD5SUM) {
-				/* cdw_verify_write_to_md5sum_stdin is a special thread function
-				   just with purpose of configuring and performing read of data
-				   from data source and writing the data to stdin of md5sum process */
-				thread_task->verify.md5sum_stdin_fd = stdin_pipe[PARENTS_END];
+			if (thread_task->id == CDW_TASK_CALCULATE_DIGEST) {
+				/* read_and_write has been configured to be a pointer to
+				   a special thread function from cdw_calculate_digest module;
+				   the function performs reading of data from data source
+				   (this can be optical disc or regular file, which doesn't matter
+				   at this point) and writing the data to stdin of digest tool process */
+				thread_task->calculate_digest.target_fd = stdin_pipe[PARENTS_END];
 				rv = pthread_create(&thread_1, NULL,
-						    thread_task->verify.read_and_write,
+						    thread_task->calculate_digest.read_and_write,
 						    (void *) thread_task);
-				cdw_sdm ("INFO: created thread with function cdw_verify_write_to_md5sum_stdin\n");
+				cdw_sdm ("INFO: created thread with read/write function from cdw_calculate_digest module\n");
 			} else {
 				/* "regular" thread with function for reading child's stderr */
 				rv = pthread_create(&thread_1, NULL, &read_child_stderr, NULL);
-				cdw_sdm ("INFO: created thread with function read_child_stderr\n");
 			}
 
 			if (rv != 0) {
@@ -476,14 +434,18 @@ int run_command(const char *command, cdw_task_t *task)
 				/* 2TRANS: this is debug message displayed in
 				   console when thread cannot be created */
 				fprintf(stderr, _("Thread 1 creation error"));
+			} else {
+				cdw_sdm ("INFO: created thread with function read_child_stderr\n");
 			}
+
 			rv = pthread_create(&thread_2, NULL, &read_child_stdout, NULL);
-			cdw_sdm ("INFO: created thread read_child_stdout\n");
 			if (rv != 0) {
 				cdw_vdm ("ERROR: failed to create thread 2\n");
 				/* 2TRANS: this is debug message displayed in
 				   console when thread cannot be created */
 				fprintf(stderr, _("Thread 2 creation error"));
+			} else {
+				cdw_sdm ("INFO: created thread read_child_stdout\n");
 			}
 
 			/* threads are created; now we must wait for them to complete
@@ -492,7 +454,14 @@ int run_command(const char *command, cdw_task_t *task)
 			pthread_join(thread_2, NULL);
 			pthread_join(thread_1, NULL);
 
-			/* closing file equals marking end of input (eof) for
+			/* this file descriptor most probably has been
+			   already closed by read/write function (otherwise
+			   child process would still wait for data on open
+			   stdin, so it would still be active, so thread would
+			   be still active and could not have been joined;
+
+			   but just to be sure: close it again;
+			   closing file equals marking end of input (eof) for
 			   external processes (like md5sum) waiting for more data */
 			close(stdin_pipe[PARENTS_END]);
 
@@ -500,13 +469,11 @@ int run_command(const char *command, cdw_task_t *task)
 			close(stdout_pipe[PARENTS_END]);
 		}
 	} else { /* creating socket pairs failed */
-		/* TODO: change this function to return CDW_OK / CDW_GEN_ERROR
-		   and make all callers check its return value */
-		exit(-1);
+		cdw_logging_write(_("Error: creating socket pairs failed\n"));
+		return -1;
 	}
-	cdw_sdm ("non-masked tool_status = 0x%04x\n", task->tool_status);
-
-	return 1;
+	cdw_sdm ("INFO: tool status at exit = \"%s\"\n", thread_task->tool_status.ok ? "ok" : "error");
+	return 0;
 }
 
 
@@ -535,19 +502,24 @@ void cdw_thread_set_child_env(void)
 		if (cdw_ext_tools_is_tool_available(CDW_TOOL_MKISOFS)) {
 			const char *fullpath = cdw_ext_tools_get_tool_fullpath(CDW_TOOL_MKISOFS);
 			cdw_assert (fullpath != (char *) NULL, "ERROR: ext tools module returned NULL pointer\n");
-			rv = setenv("MKISOFS", fullpath, 1);
+			int rv1 = setenv("MKISOFS", fullpath, 1);
 			/* growisofs from Debain repository uses GENISOIMAGE instead of MKISOFS :) */
-			rv = setenv("GENISOIMAGE", fullpath, 1);
-			if (rv != 0) {
+			int rv2 = setenv("GENISOIMAGE", fullpath, 1);
+			if (rv1 != 0 || rv2 != 0) {
 				int e = errno;
-				cdw_vdm ("ERROR: setenv(\"MKISOFS\" ...) returns !0, error = \"%s\"\n", strerror(e));
+				cdw_vdm ("ERROR: setenv(\"MKISOFS\"|\"GROWISOFS\" ...) returns !0, error = \"%s\"\n", strerror(e));
 			} else {
 #ifndef NDEBUG
 				const char *m = getenv("MKISOFS");
 				cdw_assert (!strcmp(m, fullpath),
 					    "ERROR: failed to set MKISOFS env variable to fullpath, env = \"%s\", fullpath = \"%s\"\n",
 					    m, fullpath);
-				cdw_vdm ("INFO: MKISOFS env variable set to \"%s\" - \"%s\"\n", m, fullpath);
+				cdw_vdm ("INFO: MKISOFS env variable set to \"%s\" = \"%s\"\n", m, fullpath);
+				m = getenv("GENISOIMAGE");
+				cdw_assert (!strcmp(m, fullpath),
+					    "ERROR: failed to set GENISOIMAGE env variable to fullpath, env = \"%s\", fullpath = \"%s\"\n",
+					    m, fullpath);
+				cdw_vdm ("INFO: GENISOIMAGE env variable set to \"%s\" = \"%s\"\n", m, fullpath);
 #endif
 			}
 		} else {
@@ -574,16 +546,12 @@ void cdw_thread_set_child_env(void)
 
 cdw_rv_t cdw_thread_send_key_to_child_process(int key)
 {
-	unsigned char buf[1];
-	buf[0] = (unsigned char) key;
-	cdw_vdm ("INFO: sending key %d to child process\n", key);
-	ssize_t w = write(stdin_pipe[PARENTS_END], buf, 1);
-	if (w == -1) {
-		cdw_vdm ("ERROR: write returns -1\n");
-		return CDW_GEN_ERROR;
-	} else if (w == 0) {
-		cdw_vdm ("ERROR: write returns 0\n");
-		return CDW_OK;
+	unsigned char buf = (unsigned char) key;
+	cdw_vdm ("INFO: sending key %d / \"%s\" to child process\n", key, cdw_ncurses_key_label(key));
+	ssize_t w = write(stdin_pipe[PARENTS_END], &buf, 1);
+	if (w != 1) {
+		cdw_vdm ("ERROR: write returns %zd\n", w);
+		return CDW_ERROR;
 	} else {
 		return CDW_OK;
 	}
