@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2010 Kamil Ignacak
+ * Copyright (C) 2007 - 2011 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,16 +46,13 @@
 #include <string.h>
 #include <mcheck.h>
 
-#include <ncursesw/ncurses.h>
-#include <ncursesw/menu.h>
-
 #include <dirent.h> /* PATH_MAX */
 
 #include "main.h"
 #include "cdw_fs.h"
 #include "cdw_help.h"
 #include "cdw_config.h"
-#include "cdw_config_ui.h"
+#include "cdw_config_window.h"
 #include "gettext.h"
 #include "cdw_utils.h"
 #include "cdw_logging.h"
@@ -69,17 +66,20 @@
 #include "cdw_create_image.h"
 #include "cdw_read_disc.h"
 #include "cdw_read_disc_info.h"
+#include "cdw_calculate_digest.h"
 #include "cdw_widgets.h"
 #include "cdw_drive.h"
 #include "cdw_debug.h"
 #include "cdw_file.h"
 #include "cdw_file_picker.h"
-#include "cdw_md5sum.h"
+#include "cdw_digest.h"
 #include "cdw_cdio_drives.h"
 #include "cdw_sys.h"
+#include "cdw_task.h"
+#include "cdw_tabs_window.h"
 
 extern cdw_config_t global_config;
-
+extern cdw_arguments_t cdw_commandline_arguments;
 
 /* wrapper function prototypes */
 static int cdw_main_loop(void);
@@ -96,16 +96,11 @@ int main(int argc, char *argv[])
 	cdw_locale_init();
 	/* there is no cdw_locale_clean() to register with atexit() */
 
-	bool support_dvd_rp_dl = false;
-	if (argc > 1) {
-		int rv = process_commandline_args(argc, argv);
-		if (rv == 1) {
-			support_dvd_rp_dl = true;
-		} else { /* rv == 0 || rv == -1 */
-			/* TODO: call clean_before_cdw_exit() here
-			   and see how many assertions fail */
-			return rv;
-		}
+	int rv = cdw_utils_process_commandline_arguments(argc, argv);
+	/* TODO: call clean_before_cdw_exit() here
+	   and see how many assertions fail */
+	if (rv != 1) {
+		return rv;
 	}
 
 	/* initialize ncurses - do this as soon as possible,
@@ -131,9 +126,6 @@ int main(int argc, char *argv[])
 		exit(EXIT_FAILURE);
 	}
 
-	curses_colors_init_phase2();
-	/* there is no curses_colors_clean() to register with atexit() */
-
 	{
 		/* order of calling these init functions is particularly
 		   important; it goes like this: */
@@ -143,7 +135,11 @@ int main(int argc, char *argv[])
 		   and "cdw drive" fields have some valid values */
 		cdw_config_module_init();
 		atexit(cdw_config_module_clean);
-		global_config.support_dvd_rp_dl = support_dvd_rp_dl;
+		global_config.support_dvd_rp_dl = cdw_commandline_arguments.support_dvd_rp_dl;
+
+		/* now we have a config dir path, we can get colors config file */
+		curses_colors_init_phase2();
+		/* there is no curses_colors_clean() to register with atexit() */
 
 		/* cdw_cdio_drives module detects drives in user's system;
 		   it seems that it doesn't matter which of the two modules
@@ -190,7 +186,7 @@ int main(int argc, char *argv[])
 				   _("Error occurred when configuring external tools: \"which\" tool is not available on your system. Closing."),
 				   CDW_BUTTONS_OK, CDW_COLORS_ERROR);
 		exit(EXIT_FAILURE);
-	} else if (crv == CDW_GEN_ERROR) {
+	} else if (crv == CDW_ERROR) {
 		/* 2TRANS: this is title of dialog window */
 		cdw_buttons_dialog(_("Error"),
 				   /* 2TRANS: this is message in dialog window;
@@ -293,45 +289,13 @@ int cdw_main_loop(void)
 			/* read disc meta information and display basic
 			   information in 'Disc info' area */
 			cdw_read_disc_info();
-		//cdw_main_ui_disc_info_view_update();
+			/* update information about disc size and usage */
+			cdw_main_window_volume_info_view_update(true, global_config.follow_symlinks);
 			break;
-#ifndef NDEBUG /* only for testing purposes */
 		case 'v':
 		case 'V':
-			{
-				CDW_TASK_CREATE_TASK(task, CDW_TASK_CHECK_MD5SUM);
-				task.verify.tool.id = CDW_TOOL_MD5SUM;
-				task.verify.tool.fullpath = "/usr/bin/md5sum";
-				task.verify.iso_fullpath = global_config.iso_image_full_path;
-
-				cdw_verify(&task);
-				break;
-			}
-#endif
-
-#if 0
-		case 'P':
-			{
-				char *fullpath = malloc(PATH_MAX);
-				if (fullpath == (char *) NULL) {
-					cdw_vdm ("ERROR: failed to allocate memory for fullpath\n");
-				} else {
-					strcpy(fullpath, "/home/acerion/");
-					cdw_rv_t crv = cdw_fs_ui_file_picker("file picker", "select a file", &fullpath, R_OK | W_OK, CDW_FS_FILE, CDW_FS_NEW | CDW_FS_EXISTING);
-					if (crv == CDW_OK) {
-						cdw_vdm ("INFO: file picker returns \"%s\"\n", fullpath);
-					} else if (crv == CDW_CANCEL) {
-						cdw_vdm ("INFO: pressed escape in file picker, fullpath is \"%s\"\n", fullpath);
-					} else {
-						cdw_vdm ("INFO: file picker returns CDW_GEN_ERROR, fullpath is \"%s\"\n", fullpath);
-					}
-					free(fullpath);
-					fullpath = (char *) NULL;
-				}
-				break;
-			}
-#endif
-
+			cdw_verify();
+			break;
 		case 'q':
 		case 'Q':
 			/* this function calls exit() only if user confirms closing appication */
@@ -350,6 +314,10 @@ int cdw_main_loop(void)
 			   content of log file */
 			cdw_logging_display_log(_("cdw log file"));
 			break;
+	        case 'c':
+		case 'C':
+			cdw_config_window();
+			break;
 		case 'd':
 	        case 'D':
 			cdw_read_and_display_disc_info();
@@ -426,7 +394,7 @@ int cdw_main_loop(void)
 				display_tooltip(item_index(current_item(menu)));
 			}
 			break;
-		case CDW_ENTER:
+		case CDW_KEY_ENTER:
 			{
 			int ii = item_index(current_item(menu));
 			/* UI_MENU* elements are numbered from 0, so they
@@ -453,6 +421,8 @@ int cdw_main_loop(void)
 				cdw_burn_disc(CDW_TASK_BURN_FROM_IMAGE);
 			} else if (ii == CDW_MENU_ERASE_DISC) {
 				cdw_erase_disc();
+			} else if (ii == CDW_MENU_VERIFY) {
+				cdw_verify();
 			} else if (ii == CDW_MENU_CONFIG) {
 				/* Edit cdw options */
 				/* config_window() reads current
@@ -460,7 +430,7 @@ int cdw_main_loop(void)
 				 * conf config', shows user a window that
 				 * allows him modify the options, save options
 				 * to global variable and to disk file */
-				cdw_config_ui_window(true);
+				cdw_config_window();
 			} else if (ii == CDW_MENU_EXIT_CDW) { /* Quit - Exit cdw */
 				/* this function calls exit() only if user
 				   confirms closing appication */
@@ -528,8 +498,6 @@ void cdw_conditional_exit(void)
 #include <string.h>
 #include <stdlib.h>
 
-#include <ncursesw/ncurses.h>
-
 #include "main.h"
 #include "config.h"
 
@@ -543,6 +511,8 @@ void cdw_conditional_exit(void)
 #include "cdw_debug.h"
 #include "cdw_which.h"
 #include "gettext.h"
+#include "cdw_ncurses.h"
+
 
 static WINDOW *cdw_main_test_window = (WINDOW *) NULL;
 WINDOW *cdw_main_test_subwin = (WINDOW *) NULL;
