@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2010 Kamil Ignacak
+ * Copyright (C) 2007 - 2011 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -231,7 +231,7 @@ cdw_rv_t cdw_file_new_fullpath(cdw_file_t *file, const char *dirpath, const char
 	correct_dirpath = (char *) NULL;
 	if (file->fullpath == (char *) NULL) {
 		cdw_assert (0, "ERROR: failed to concatenate dirpath and name: \"%s\" + \"%s\"\n", dirpath, name);
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	file->name_start = cdw_fs_get_filename_start(file->fullpath);
@@ -240,7 +240,7 @@ cdw_rv_t cdw_file_new_fullpath(cdw_file_t *file, const char *dirpath, const char
 		free(file->fullpath);
 		file->fullpath = (char *) NULL;
 		assert (0);
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	file->printable_fullpath = cdw_string_get_printable_if_needed(file->fullpath);
@@ -352,7 +352,7 @@ cdw_file_t *cdw_file_duplicate(const cdw_file_t *file)
    \p file itself. NULL is assigned to \p file before return, so that caller
    doesn't have to do this himself.
 
-   \return CDW_GEN_ERROR if \p file is a pointer to NULL file
+   \return CDW_ERROR if \p file is a pointer to NULL file
    \return CDW_OK on success
 */
 cdw_rv_t cdw_file_delete(cdw_file_t **file)
@@ -361,7 +361,7 @@ cdw_rv_t cdw_file_delete(cdw_file_t **file)
 
 	if (*file == (cdw_file_t *) NULL) {
 		cdw_vdm ("ERROR: passed pointer to NULL file to function\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	if ((*file)->printable_fullpath != (char *) NULL) {
