@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2010 Kamil Ignacak
+ * Copyright (C) 2007 - 2011 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -53,8 +53,6 @@
 #include <stdbool.h>
 #include <stddef.h>
 
-#include <ncursesw/ncurses.h>
-
 #include "cdw_main_window.h"
 #include "gettext.h"
 #include "cdw_file.h"
@@ -146,7 +144,7 @@ cdw_rv_t cdw_file_selector_init(void)
 	}
 	if (file_selector.initial_fullpath == (char *) NULL) {
 		cdw_vdm ("ERROR: failed to create initial fullpath\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	/* initially, after calling cdw_fs_get_initial_dirpath(),
@@ -162,7 +160,7 @@ cdw_rv_t cdw_file_selector_init(void)
 		free(file_selector.initial_fullpath);
 		file_selector.initial_fullpath = (char *) NULL;
 
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 }
@@ -192,7 +190,7 @@ cdw_rv_t cdw_file_manager_init(void)
 	if (crv != CDW_OK) {
 		cdw_vdm ("ERROR: failed to initialize file selector in file manager module\n");
 		//cdw_file_manager_internal_error_dialog(CDW_FM_E_NO_SELECTOR);
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	} else {
 		return CDW_OK;
 	}
@@ -265,7 +263,7 @@ cdw_dll_item_t *cdw_file_manager_get_list_of_selected_files(void)
 
    \return CDW_OK if some files were selected without problems
    \return CDW_NO if no files were selected (and no problems occurred)
-   \return CDW_GEN_ERROR on errors
+   \return CDW_ERROR on errors
 */
 cdw_rv_t cdw_file_manager_handle_adding_to_selected_files(void)
 {
@@ -277,10 +275,10 @@ cdw_rv_t cdw_file_manager_handle_adding_to_selected_files(void)
 	cdw_rv_t crv = cdw_file_selector();
 	cdw_main_ui_main_window_wrefresh();
 
-	cdw_rv_t retval = CDW_GEN_ERROR;
-	if (crv == CDW_GEN_ERROR) {
-		cdw_vdm ("ERROR: file selector returns CDW_GEN_ERROR\n");
-		retval = CDW_GEN_ERROR;
+	cdw_rv_t retval = CDW_ERROR;
+	if (crv == CDW_ERROR) {
+		cdw_vdm ("ERROR: file selector returns CDW_ERROR\n");
+		retval = CDW_ERROR;
 	} else {
 		if (selected_files.display->n_items == 0) {
 			cdw_sdm ("INFO: selected_files->n_items == 0\n");
@@ -316,7 +314,7 @@ cdw_rv_t cdw_file_manager_handle_adding_to_selected_files(void)
    Function displays error message if some problem occurred.
 
    \return CDW_OK on success
-   \return CDW_GEN_ERROR on problems with deselecting file.
+   \return CDW_ERROR on problems with deselecting file.
 */
 cdw_rv_t cdw_file_manager_handle_deleting_from_selected_files(void)
 {
@@ -330,19 +328,19 @@ cdw_rv_t cdw_file_manager_handle_deleting_from_selected_files(void)
 		if (key == KEY_DC) {
 			crv = cdw_selected_files_remove_file(selected_files.display->current_item_ind);
 			if (crv == CDW_OK) {
-				crv = cdw_main_ui_files_info_view_update(false);
+				crv = cdw_main_window_volume_info_view_update(false, global_config.follow_symlinks);
 				if (crv == CDW_OK) {
 					crv = CDW_OK;
 				} else {
 					cdw_vdm ("ERROR: failed to update files info view with fetch_data=false\n");
-					crv = CDW_GEN_ERROR;
+					crv = CDW_ERROR;
 					break;
 				}
 			} else {
 				cdw_vdm ("ERROR: failed to remove file nr %zd / %zd from 'selected files'\n",
 					 selected_files.display->current_item_ind, selected_files.display->n_items);
 				cdw_file_manager_internal_error_dialog(CDW_FM_E_UNEXPECTED);
-				crv = CDW_GEN_ERROR;
+				crv = CDW_ERROR;
 				break;
 			}
 		} else if (key == CDW_KEY_ESCAPE || key == 'q' || key == 'Q') {
@@ -354,12 +352,12 @@ cdw_rv_t cdw_file_manager_handle_deleting_from_selected_files(void)
 	}
 	/* FIXME: this function does not inform user when some errors occur */
 
-	crv = cdw_main_ui_files_info_view_update(true);
+	crv = cdw_main_window_volume_info_view_update(true, global_config.follow_symlinks);
 	if (crv == CDW_OK) {
 		return CDW_OK;
 	} else {
 		cdw_vdm ("ERROR: failed to update files info view with fetch_data=false\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 }
 
@@ -406,30 +404,29 @@ size_t cdw_file_manager_number_of_selected_files(void)
    it 'remembers' last directory which was visited during last call of the
    function (last time that the user selected files).
 
-   In case of errors function returns with CDW_GEN_ERROR, but does not erase
+   In case of errors function returns with CDW_ERROR, but does not erase
    list of already selected files.
 
    \return CDW_OK when user pressed Escape key and no errors occurred during selection of files
-   \return CDW_GEN_ERROR on errors in process of selecting files
+   \return CDW_ERROR on errors in process of selecting files
 */
 cdw_rv_t cdw_file_selector(void)
 {
 	/* the values are selected so that:
-	   - there is enough space on top so app title is visible
-	   - there is enough space at the bottom so 'selected files size'
-	     area is visible
+	   - there is enough space at the bottom so 'selected files size' line is visible
 	   - the window is big enough to cover whole list of selected files
 	     displayed in main app window, so we don't have to refresh it
 	     every time a file is selected; */
-	int n_lines = LINES - 8;
-	int n_cols = COLS - 4;
-	int begin_y = 1;
-	int begin_x = 2;
+	int n_lines = LINES - 6;
+	int n_cols = COLS;
+	int begin_y = 0;
+	int begin_x = 0;
 	WINDOW *window = newwin(n_lines, n_cols, begin_y, begin_x);
 	if (window == (WINDOW *) NULL) {
 		cdw_vdm ("ERROR: failed to create window for file system browser\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
+	cdw_ncurses_window_add_help_marker(window);
 
 	/* fs browser will occupy whole window, without leaving any margins or borders */
 	cdw_fs_browser_t *fs_browser = cdw_fs_browser_new(window, file_selector.initial_fullpath);
@@ -438,7 +435,7 @@ cdw_rv_t cdw_file_selector(void)
 		window = (WINDOW *) NULL;
 
 		cdw_vdm ("ERROR: failed to create file system browser\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	if (file_selector.first_selection) {
@@ -457,14 +454,19 @@ cdw_rv_t cdw_file_selector(void)
 	cdw_rv_t retval = CDW_OK;
 	while (key != CDW_KEY_ESCAPE) {
 		key = cdw_fs_browser_driver(fs_browser);
-		if (key == CDW_KEY_ENTER) {
+		if (key == CDW_KEY_ENTER || key == KEY_BACKSPACE || key == '~') {
 			cdw_file_selector_display_current_path(fs_browser, window);
 		} else if (key == ' ') {
-			cdw_vdm ("INFO: file manager: adding file \"%s\"\n", cdw_fs_browser_get_current_fullpath(fs_browser));
+#ifndef NDEBUG
+			char *fp = cdw_fs_browser_get_current_fullpath(fs_browser);
+			cdw_vdm ("INFO: file manager: adding file \"%s\"\n", fp);
+			free(fp);
+			fp = (char *) NULL;
+#endif
 			cdw_rv_t crv = cdw_file_selector_add_file_to_selected(fs_browser);
-			if (crv == CDW_GEN_ERROR) {
+			if (crv == CDW_ERROR) {
 				cdw_vdm ("ERROR: failed while trying to add file\n");
-				retval = CDW_GEN_ERROR;
+				retval = CDW_ERROR;
 				break;
 			} else if (crv == CDW_NO) { /* item is not appendable, do nothing */
 				;
@@ -477,15 +479,20 @@ cdw_rv_t cdw_file_selector(void)
 				    situation in which previously visited dir
 				    / file disappears between two calls to fs
 				    browser */
-				const char *fullpath = cdw_fs_browser_get_current_fullpath(fs_browser);
+				char *fullpath = cdw_fs_browser_get_current_fullpath(fs_browser);
 				cdw_assert (fullpath != (char *) NULL, "ERROR: failed to get current dirpath\n");
 				cdw_string_set(&file_selector.initial_fullpath, fullpath);
+				free(fullpath);
+				fullpath = (char *) NULL;
 				cdw_vdm ("INFO: saved last file fullpath as %s\n", file_selector.initial_fullpath);
 				file_selector.first_selection = false;
 			}
+		} else if (key == '?') {
+			cdw_fs_browser_help_window(fs_browser);
+			cdw_list_display_refresh(fs_browser->display);
 		} else if (key == -1) {
 			cdw_vdm ("ERROR: file system browser returned -1\n");
-			retval = CDW_GEN_ERROR;
+			retval = CDW_ERROR;
 			break;
 		} else if (key == 'q' || key == 'Q') {
 			/* 'Q' = quit file selector */
@@ -504,12 +511,12 @@ cdw_rv_t cdw_file_selector(void)
 	if (selected_files.display->n_items > 0) {
 		/* true - upon finishing adding files recount size of
 		   selected files */
-		cdw_rv_t crv = cdw_main_ui_files_info_view_update(true);
+		cdw_rv_t crv = cdw_main_window_volume_info_view_update(true, global_config.follow_symlinks);
 		if (crv == CDW_OK) {
 			return CDW_OK;
 		} else {
 			cdw_vdm ("ERROR: failed to update files info view with fetch_data=true\n");
-			return CDW_GEN_ERROR;
+			return CDW_ERROR;
 		}
 	}
 
@@ -531,14 +538,14 @@ cdw_rv_t cdw_file_selector(void)
    \param parent - window into which new widget will be put
 
    \return CDW_OK on success
-   \return CDW_GEN_ERROR when widget was not created properly
+   \return CDW_ERROR when widget was not created properly
 */
 cdw_rv_t cdw_file_manager_create_selected_files_view(WINDOW *parent)
 {
 	/* these are parameters for embedded selector window */
-	size_t n_lines = (size_t) getmaxy(parent);
+	int n_lines = getmaxy(parent);
 	cdw_assert (n_lines > 2, "ERROR: parent has no more than 2 lines\n");
-	size_t n_cols = (size_t) getmaxx(parent);
+	int n_cols = getmaxx(parent);
 	cdw_assert (n_cols > 2, "ERROR: parent has no more than 2 columns\n");
 
 	cdw_assert (selected_files.display == (CDW_LIST_DISPLAY *) NULL,
@@ -550,7 +557,7 @@ cdw_rv_t cdw_file_manager_create_selected_files_view(WINDOW *parent)
 		cdw_vdm ("ERROR: failed to create 'selected files' display\n");
 		cdw_file_manager_internal_error_dialog(CDW_FM_E_NO_SELECTED);
 
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	} else {
 		selected_files.display->display_item = cdw_file_display_file;
 		cdw_list_display_add_return_char(selected_files.display, KEY_DC);
@@ -710,7 +717,7 @@ double cdw_file_manager_calculate_selected_files_size_mb(bool follow_symlinks)
    \param file - file that you want to append to a display with list of selected files
 
    \return CDW_NO if no action was taken, but no error occurred either
-   \return CDW_GEN_ERROR on errors
+   \return CDW_ERROR on errors
    \return CDW_OK if file was appended correctly
 */
 cdw_rv_t cdw_selected_files_clone_and_append_file(const cdw_file_t *file)
@@ -737,7 +744,7 @@ cdw_rv_t cdw_selected_files_clone_and_append_file(const cdw_file_t *file)
 	cdw_file_t *copy = cdw_file_duplicate(file);
 	if (copy == (cdw_file_t *) NULL) {
 		cdw_vdm ("ERROR: failed to duplicate file %ld / \"%s\"\n", (long int) file, file->fullpath);
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	cdw_rv_t crv = cdw_list_display_add_item(selected_files.display, (void *) copy, cdw_file_equal);
@@ -774,7 +781,7 @@ cdw_rv_t cdw_selected_files_clone_and_append_file(const cdw_file_t *file)
    \param file_i - index of file to be removed
 
    \return CDW_OK if removing was successful
-   \return CDW_GEN_ERROR if it failed
+   \return CDW_ERROR if it failed
 */
 cdw_rv_t cdw_selected_files_remove_file(size_t file_i)
 {
@@ -801,7 +808,7 @@ cdw_rv_t cdw_selected_files_remove_file(size_t file_i)
 		return CDW_OK;
 	} else {
 		cdw_vdm ("ERROR: failed to remove item %zd from list of selected files\n", file_i);
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 }
 
@@ -824,6 +831,7 @@ void cdw_file_selector_display_current_path(cdw_fs_browser_t *browser, WINDOW *w
 	cdw_assert (dirpath != (char *) NULL, "ERROR: failed to get current dirpath\n");
 	/* 2TRANS: this is title of file selector window */
 	cdw_ncurses_nice_box(window, _("Add files"), dirpath);
+	cdw_ncurses_window_add_help_marker(window);
 	wrefresh(window);
 	free(dirpath);
 	dirpath = (char *) NULL;
@@ -850,7 +858,7 @@ cdw_rv_t cdw_file_manager_create_graftpoints_file(void)
 		cdw_rv_t crv = cdw_graftpoints_create_file();
 		if (crv != CDW_OK) {
 			cdw_vdm ("ERROR: failed to create graftpoints file\n");
-			return CDW_GEN_ERROR;
+			return CDW_ERROR;
 		} else {
 			return CDW_OK;
 		}
@@ -893,7 +901,7 @@ long long cdw_selected_files_get_size(void)
 /**
    \return CDW_NO if type of currently highlighted file is non-appendable
    \return CDW_OK on success
-   \return CDW_GEN_ERROR on errors
+   \return CDW_ERROR on errors
 */
 cdw_rv_t cdw_file_selector_add_file_to_selected(cdw_fs_browser_t *fs_browser)
 {
@@ -931,16 +939,19 @@ cdw_rv_t cdw_file_selector_add_file_to_selected(cdw_fs_browser_t *fs_browser)
 			   size of selected files was updated in
 			   cdw_selected_files_clone_and_append_file(), and
 			   will be recounted again on file selector exit */
-			crv = cdw_main_ui_files_info_view_update(false);
+			crv = cdw_main_window_volume_info_view_update(false, global_config.follow_symlinks);
+			/* updating files info view may disrupt part of file
+			   selector window; redraw file selector */
+			cdw_list_display_refresh(fs_browser->display);
 			if (crv == CDW_OK) {
 				return CDW_OK;
 			} else {
 				cdw_vdm ("ERROR: failed to update files info view with fetch_data=false\n");
-				return CDW_GEN_ERROR;
+				return CDW_ERROR;
 			}
-		} else if (crv == CDW_GEN_ERROR) {
+		} else if (crv == CDW_ERROR) {
 			cdw_vdm ("ERROR: failed to append file to selected_files\n");
-			return CDW_GEN_ERROR;
+			return CDW_ERROR;
 		} else {
 			/* item not appendable (i.e. not file dir, or link) */
 			return CDW_NO;
@@ -955,7 +966,7 @@ cdw_rv_t cdw_file_selector_add_file_to_selected(cdw_fs_browser_t *fs_browser)
 		   %s is a full path to a file */
 		cdw_logging_write(_("ERROR: failed to fully scan this path: \"%s\"\n"), file->fullpath);
 
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 }
 
