@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2010 Kamil Ignacak
+ * Copyright (C) 2007 - 2011 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -124,7 +124,7 @@ cdw_rv_t cdw_fs_ui_file_picker(const char *title, const char *message, char **ca
 
 	if (crv != CDW_OK) {
 		cdw_vdm ("ERROR: failed to build picker\n");
-		retval = CDW_GEN_ERROR;
+		retval = CDW_ERROR;
 	} else {
 		crv = cdw_fs_ui_file_picker_driver(&file_picker);
 		if (crv == CDW_OK) {
@@ -133,12 +133,12 @@ cdw_rv_t cdw_fs_ui_file_picker(const char *title, const char *message, char **ca
 				retval = CDW_OK;
 			} else {
 				cdw_vdm ("ERROR: failed to set caller's fullpath at return\n");
-				retval =  CDW_GEN_ERROR;
+				retval =  CDW_ERROR;
 			}
 		} else if (crv == CDW_CANCEL) {
 			retval = CDW_CANCEL;
 		} else {
-			retval = CDW_GEN_ERROR;
+			retval = CDW_ERROR;
 		}
 	}
 
@@ -184,7 +184,7 @@ cdw_rv_t cdw_fs_ui_file_picker_build(struct cdw_file_picker *picker, const char
 {
 	if (initial_fullpath == (char *) NULL) {
 		cdw_vdm ("ERROR: invalid initial fullpath \"%s\"\n", initial_fullpath);
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 	/* *** main window + fs browser window *** */
 
@@ -194,7 +194,7 @@ cdw_rv_t cdw_fs_ui_file_picker_build(struct cdw_file_picker *picker, const char
 							 CDW_COLORS_DIALOG, title, (char *) NULL);
 	if (picker->main_window.win == (WINDOW *) NULL) {
 		cdw_vdm ("ERROR: failed to create main window for file picker\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	picker->browser_window.win = cdw_ncurses_window_new(picker->main_window.win,
@@ -203,14 +203,16 @@ cdw_rv_t cdw_fs_ui_file_picker_build(struct cdw_file_picker *picker, const char
 							    CDW_COLORS_DIALOG, (char *) NULL, (char *) NULL);
 	if (picker->browser_window.win == (WINDOW *) NULL) {
 		cdw_vdm ("ERROR: failed to create fs browser window for file picker\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
+
 	cdw_ncurses_nice_box(picker->browser_window.win, "", "");
+	cdw_ncurses_window_add_help_marker(picker->browser_window.win);
 
 	picker->fullpath = strdup(initial_fullpath);
 	if (picker->fullpath == (char *) NULL) {
 		cdw_vdm ("ERROR: failed to strdup() initial fullpath \"%s\"\n", picker->fullpath);
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	/* *** fs browser *** */
@@ -218,7 +220,7 @@ cdw_rv_t cdw_fs_ui_file_picker_build(struct cdw_file_picker *picker, const char
 	picker->fs_browser = cdw_fs_browser_new(picker->browser_window.win, picker->fullpath);
 	if (picker->fs_browser == (cdw_fs_browser_t *) NULL) {
 		cdw_vdm ("ERROR: failed to create fs browser\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 	cdw_list_display_add_return_char(picker->fs_browser->display, CDW_KEY_TAB);
 	cdw_list_display_add_return_char(picker->fs_browser->display, KEY_DOWN);
@@ -232,13 +234,13 @@ cdw_rv_t cdw_fs_ui_file_picker_build(struct cdw_file_picker *picker, const char
 
 	picker->buttons.ok = cdw_button_new(picker->main_window.win,
 					    /* 2TRANS: this is label of button */
-					    (size_t) picker->buttons.row, 4, _("OK"), CDW_COLORS_DIALOG);
+					    picker->buttons.row, 4, _("OK"), CDW_COLORS_DIALOG);
 	picker->buttons.cancel = cdw_button_new(picker->main_window.win,
 						/* 2TRANS: this is label of button */
-						(size_t) picker->buttons.row, 15, _("Cancel"), CDW_COLORS_DIALOG);
+						picker->buttons.row, 15, _("Cancel"), CDW_COLORS_DIALOG);
 	if (picker->buttons.ok == (CDW_BUTTON *) NULL || picker->buttons.cancel == (CDW_BUTTON *) NULL) {
 		cdw_vdm ("ERROR: failed to create one of buttons\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	/* *** input line *** */
@@ -250,7 +252,7 @@ cdw_rv_t cdw_fs_ui_file_picker_build(struct cdw_file_picker *picker, const char
 						      0, 0); /* input type and length limit */
 	if (picker->input.input_line == (CDW_INPUT_LINE *) NULL) {
 		cdw_vdm ("ERROR: failed to create input line\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 	/* content of input field buffer will be stored to provided data buffer
 	   every time user presses TAB key to move away from input line */
@@ -299,9 +301,18 @@ cdw_rv_t cdw_fs_ui_file_picker_driver(struct cdw_file_picker *picker)
 
 		//key = 'a'; /* dummy value that doesn't trigger any behavior */
 		if (picker->focus == PICKER_BROWSER) {
-			cdw_vdm ("INFO: focus on browser, fs browser fullpath = \"%s\"\n", cdw_fs_browser_get_current_fullpath(picker->fs_browser));
-
+#ifndef NDEBUG
+			char *fp = cdw_fs_browser_get_current_fullpath(picker->fs_browser);
+			cdw_vdm ("INFO: focus on browser, fs browser fullpath = \"%s\"\n", fp);
+			free(fp);
+			fp = (char *) NULL;
+#endif
 			while (1) {
+				/* this driver function receives and processes
+				   keys (e.g. causes fs browser to change a
+				   dir on enter or backspace), but it also
+				   returns a (meaningful) key value, so that
+				   file picker can also act upon the key */
 				key = cdw_fs_browser_driver(picker->fs_browser);
 				cdw_fs_ui_file_picker_driver_print_debug(picker, key);
 
@@ -328,6 +339,17 @@ cdw_rv_t cdw_fs_ui_file_picker_driver(struct cdw_file_picker *picker)
 					if (key == CDW_KEY_TAB) {
 						break;
 					}
+				} else if (key == KEY_BACKSPACE || key == '~') {
+					cdw_file_t *file = cdw_fs_browser_get_current_file(picker->fs_browser);
+
+					cdw_input_line_refresh_with_string(picker->input.input_line, file->fullpath);
+					cdw_string_set(&picker->fullpath, file->fullpath);
+					cdw_vdm ("INFO: SYNCHRONIZATION POINT: non-escape key \"%s\" in fs browser\n", cdw_ncurses_key_label(key));
+					cdw_vdm ("INFO: fs browser fullpath at synchronization point: \"%s\"\n", file->fullpath);
+				} else if (key == '?') {
+					cdw_fs_browser_help_window(picker->fs_browser);
+					redrawwin(picker->main_window.win);
+					wrefresh(picker->main_window.win);
 				} else {
 					; /* some other, alphanumeric key */
 				}
@@ -384,13 +406,13 @@ cdw_rv_t cdw_fs_ui_file_picker_driver(struct cdw_file_picker *picker)
 			return CDW_CANCEL;
 		} else {
 			cdw_vdm ("ERROR: got ENTER in unknown focus %d\n", picker->focus);
-			return CDW_GEN_ERROR;
+			return CDW_ERROR;
 		}
 	} else if (key == CDW_KEY_ESCAPE) {
 		return CDW_CANCEL;
 	} else {
 		cdw_vdm ("ERROR: got unknown key at return: %d / \"%s\"\n", key, cdw_ncurses_key_label(key));
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 }
 
@@ -401,19 +423,22 @@ cdw_rv_t cdw_fs_ui_file_picker_driver(struct cdw_file_picker *picker)
 void cdw_fs_ui_file_picker_driver_print_debug(struct cdw_file_picker *picker, int key)
 {
 	if (picker->focus == PICKER_BROWSER) {
+		char *fp = cdw_fs_browser_get_current_fullpath(picker->fs_browser);
 		cdw_vdm ("INFO: fs browser: file_i = %zd, fullpath = \"%s\"\n",
-			 picker->fs_browser->display->current_item_ind, cdw_fs_browser_get_current_fullpath(picker->fs_browser));
+			 picker->fs_browser->display->current_item_ind, fp);
 		if (key == CDW_KEY_ESCAPE) {
 			cdw_vdm ("INFO: got Escape in fs browser, moving away from browser\n");
 		} else if (key == 'q' || key == 'Q') {
 			cdw_vdm ("INFO: got 'Q' in fs browser, moving away from browser\n");
 		} else if (key == ' ') {
-			cdw_vdm ("INFO: got Space in fs browser, selected file = \"%s\"\n", cdw_fs_browser_get_current_fullpath(picker->fs_browser));
+			cdw_vdm ("INFO: got Space in fs browser, selected file = \"%s\"\n", fp);
 		} else if (key == CDW_KEY_TAB) {
 			cdw_vdm ("INFO: got Tab in fs browser, moving away from browser\n");
 		} else {
 			cdw_sdm ("INFO: got key %d / \"%s\" in fs browser, looping\n", key, cdw_ncurses_key_label(key));
 		}
+		free(fp);
+		fp = (char *) NULL;
 	}
 	return;
 }
@@ -444,11 +469,6 @@ void cdw_fs_ui_file_picker_destroy(struct cdw_file_picker *picker)
 		picker->fullpath = (char *) NULL;
 	}
 
-	if (picker->input.input_line != (CDW_INPUT_LINE *) NULL) {
-		cdw_input_line_delete(&(picker->input.input_line));
-		picker->input.input_line = (CDW_INPUT_LINE *) NULL;
-	}
-
 	return;
 }
 
