@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2010 Kamil Ignacak
+ * Copyright (C) 2007 - 2011 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +26,6 @@
 #include <stdlib.h>
 
 
-#include "cdw_file.h"
 #include "cdw_fs.h"
 #include "cdw_file_picker.h"
 #include "cdw_string.h"
@@ -83,11 +82,11 @@ cdw_rv_t cdw_fs_init(void)
 	cdw_rv_t crv = CDW_OK;
 	crv = cdw_fs_init_home_dir_fullpath();
 	if (crv != CDW_OK) {
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	} else {
 		crv =  cdw_fs_init_tmp_dir_fullpath();
 		if (crv != CDW_OK) {
-			return CDW_GEN_ERROR;
+			return CDW_ERROR;
 		}
 	}
 
@@ -344,10 +343,10 @@ int cdw_fs_check_fullpath(const char *fullpath, int expected_file_type, int expe
    First place to search for user's home dir is HOME shell variable. If this
    fails (or in case of other problems) user is asked to enter path to
    home dir manually. If this fails too, or user cancels entering path,
-   function returns CDW_GEN_ERROR
+   function returns CDW_ERROR
 
    \return CDW_OK on success
-   \return CDW_GEN_ERROR if the variable with path can't be initialized for any reason
+   \return CDW_ERROR if the variable with path can't be initialized for any reason
 */
 cdw_rv_t cdw_fs_init_home_dir_fullpath(void)
 {
@@ -413,7 +412,7 @@ cdw_rv_t cdw_fs_init_home_dir_fullpath(void)
 					      application */
 					   _("Path to Home directory not initialized properly, closing."),
 					   CDW_BUTTONS_OK, CDW_COLORS_ERROR);
-			return CDW_GEN_ERROR;
+			return CDW_ERROR;
 		}
 	}
 }
@@ -450,7 +449,7 @@ cdw_rv_t cdw_fs_init_tmp_dir_fullpath(void)
 	/* some sane initial value */
 	char *tmp = strdup("/tmp/");
 	if (tmp == (char *) NULL) {
-		return CDW_MEM_ERROR;
+		return CDW_ERROR;
 	}
 
 	int rv = cdw_fs_check_existing_path(tmp, W_OK, CDW_FS_DIR);
@@ -556,6 +555,28 @@ char *cdw_fs_get_initial_dirpath(void)
 		cdw_vdm ("ERROR: can't get cwd, error = \"%s\"\n", strerror(errno));
 	}
 
+	dirpath = cdw_fs_get_home_or_tmp_dirpath();
+	if (dirpath != (char *) NULL) {
+		return dirpath;
+	}
+
+	dirpath = strdup("/");
+	if (dirpath != (char *) NULL) {
+		return dirpath;
+	} else {
+		cdw_vdm ("ERROR: can't strdup() root dir path\n");
+	}
+
+	return (char *) NULL;
+}
+
+
+
+
+
+char *cdw_fs_get_home_or_tmp_dirpath(void)
+{
+	char *dirpath = (char *) NULL;
 	const char *h = cdw_fs_get_home_dir_fullpath();
 	if (h != (char *) NULL) {
 		dirpath = strdup(h);
@@ -580,14 +601,7 @@ char *cdw_fs_get_initial_dirpath(void)
 		cdw_vdm ("ERROR: can't get home tmp path\n");
 	}
 
-	dirpath = strdup("/");
-	if (dirpath != (char *) NULL) {
-		return dirpath;
-	} else {
-		cdw_vdm ("ERROR: can't strdup() root dir path\n");
-	}
-
-	return (char *) NULL;
+	return dirpath;
 }
 
 
@@ -612,8 +626,7 @@ char *cdw_fs_get_initial_dirpath(void)
 
    \return CDW_OK if device is mounted
    \return CDW_NO if device is not mounted
-   \return CDW_SYS_ERROR if /etc/mtab API function call fails
-   \return CDW_GEN_ERROR on other errors
+   \return CDW_ERROR on errors
 */
 cdw_rv_t cdw_fs_check_device_mounted(const char *device_fullpath)
 {
@@ -627,7 +640,7 @@ cdw_rv_t cdw_fs_check_device_mounted(const char *device_fullpath)
 				   /* 2TRANS: this is message in dialog window */
 				   _("Path to drive specified in configuration is incorrect. Please check your configuration."),
 				   CDW_BUTTONS_OK, CDW_COLORS_ERROR);
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	char *fullpath = (char *) NULL;
@@ -654,7 +667,7 @@ cdw_rv_t cdw_fs_check_device_mounted(const char *device_fullpath)
 				   /* 2TRANS: this is message in dialog window */
 				   _("cdw has problems with searching for your device. Please restart cdw."),
 				   CDW_BUTTONS_OK, CDW_COLORS_ERROR);
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	FILE *mtab_file = setmntent(_PATH_MOUNTED, "r");
@@ -662,7 +675,7 @@ cdw_rv_t cdw_fs_check_device_mounted(const char *device_fullpath)
 		free(fullpath);
 		fullpath = (char *) NULL;
 		cdw_vdm ("ERROR: call to setmnent() failed\n");
-		return CDW_SYS_ERROR;
+		return CDW_ERROR;
 	}
 
 	bool is_mounted = false;
@@ -723,7 +736,7 @@ cdw_rv_t cdw_fs_check_device_mounted(const char *device_fullpath)
    'path' cannot be NULL or empty string. It can be only "/" - function will
    recognize it as correct string.
 
-   If malloc() fails and function returns CDW_MEM_ERROR, path is modified
+   If malloc() fails and function returns CDW_ERROR, path is modified
    no more than by removing ending white chars.
 
    Function may realloc 'path'!
@@ -731,21 +744,20 @@ cdw_rv_t cdw_fs_check_device_mounted(const char *device_fullpath)
    \param path - directory path that you want to check
 
    \return CDW_OK if success
-   \return CDW_MEM_ERROR if malloc() fails
-   \return CDW_ARG_ERROR if argument is incorrect
+   \return CDW_ERROR on errors
 */
 cdw_rv_t cdw_fs_correct_dir_path_ending(char **path)
 {
 	if (*path == NULL) {
 		cdw_vdm ("ERROR: path is NULL\n");
-		return CDW_ARG_ERROR;
+		return CDW_ERROR;
 	}
 
 	size_t len = strlen(*path);
 
 	if (len == 0) {
 		cdw_vdm ("ERROR: path has zero length\n");
-		return CDW_ARG_ERROR;
+		return CDW_ERROR;
 	}
 
 #if 0	/* an experiment, don't enable */
@@ -776,7 +788,7 @@ cdw_rv_t cdw_fs_correct_dir_path_ending(char **path)
 		char *tmp = (char *) malloc(len + missing_chars + 1);
 		if (tmp == (char *) NULL) {
 			cdw_vdm ("ERROR: failed to allocate memory for new path\n");
-			return CDW_MEM_ERROR;
+			return CDW_ERROR;
 		} else {
 			size_t offset = 0;
 			if (missing_root) {
@@ -817,7 +829,7 @@ cdw_rv_t cdw_fs_correct_dir_path_ending(char **path)
 		char *tmp = realloc(*path, len + 2);
 		if (tmp == NULL) {
 			cdw_vdm ("ERROR: failed to allocate memory for new path\n");
-			return CDW_MEM_ERROR;
+			return CDW_ERROR;
 		}
 		*path = tmp;
 		*(*path + len) = '/';
@@ -874,7 +886,7 @@ int cdw_fs_check_dirpath(const char *dirpath)
 		   link to directory is regarded as directory, so this
 		   condition won't be true for link to dir */
 		cdw_vdm ("ERROR: \"%s\" is not a dir\n", dirpath)
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	/* so this is a dir path - check permissions */
@@ -1003,28 +1015,36 @@ long long cdw_fs_get_file_size(const char *fullpath)
    non-zero, therefore \p eps has to have at least one file name. It can't be
    "." - current dir, but it can be ".." - parent dir.
 
+   \p list won't include hidden files if \p include_hidden is false.
+
    \param list - will be set to list of files in given dir
    \param dirpath - path to directory that you scan
    \param eps - list of file names in given dir
    \param n_files - number of file names on \p eps, or in given dir
+   \param include_hidden - controls if hidden UNIX file should be copied from eps to list
 
-   \return CDW_OK on success
-   \return CDW_OK on error
+   \return number of items copied to the list on success (there should be at least one: ".")
+   \return 0 on errors
 */
-cdw_rv_t cdw_fs_copy_dirent_to_list(cdw_dll_item_t **list, const char *dirpath, struct dirent **eps, size_t n_files)
+unsigned long long cdw_fs_copy_dirent_to_list(cdw_dll_item_t **list, const char *dirpath, struct dirent **eps, unsigned long long n_items, bool include_hidden)
 {
 	cdw_assert (*list == (cdw_dll_item_t *) NULL, "ERROR: can't use non-null list\n");
 	cdw_assert (dirpath != (char *) NULL, "ERROR: can't use null dirpath\n");
-	cdw_assert (n_files > 0, "ERROR: you want to create list of zero files\n");
+	cdw_assert (n_items > 0, "ERROR: you want to create list of zero files\n");
 
 	cdw_dll_item_t *dirs = (cdw_dll_item_t *) NULL;
 	cdw_dll_item_t *files = (cdw_dll_item_t *) NULL;
 
-	size_t n_d = 0;
-	size_t n_f = 0;
+	unsigned long long n_dirs = 0;
+	unsigned long long n_files = 0;
+	unsigned long long n_hidden = 0;
 	bool success = true;
 
-	for (size_t i = 0; i < n_files; i++) {
+	for (unsigned long long i = 0; i < n_items; i++) {
+		if (cdw_fs_is_hidden(eps[i]->d_name) && !include_hidden) {
+			n_hidden++;
+			continue;
+		}
 		cdw_file_t *file = cdw_file_new(dirpath, eps[i]->d_name);
 		if (file == (cdw_file_t *) NULL) {
 			success = false;
@@ -1032,30 +1052,30 @@ cdw_rv_t cdw_fs_copy_dirent_to_list(cdw_dll_item_t **list, const char *dirpath,
 		}
 		if (file->type == CDW_FS_DIR) {
 			cdw_dll_append(&dirs, (void *) file, cdw_file_equal);
-			n_d++;
+			n_dirs++;
 		} else { /* CDW_FS_FILE or CDW_FS_OTHER */
 			cdw_dll_append(&files, (void *) file, cdw_file_equal);
-			n_f++;
+			n_files++;
 		}
 	}
 
 	if (success) {
 		/* connect 'dirs' list and 'files' list into one */
-		cdw_assert (n_d + n_f == n_files, "ERROR: numbers of dirs/files don't add up\n");
-		if (n_d > 0) {
+		cdw_assert (n_dirs + n_files + n_hidden == n_items, "ERROR: numbers of dirs/files doesn't add up\n");
+		if (n_dirs > 0) {
 			*list = dirs;
 
-			if (n_f > 0) {
-				cdw_dll_item_t *last_dir = cdw_dll_ith_item(dirs, n_d - 1);
+			if (n_files > 0) {
+				cdw_dll_item_t *last_dir = cdw_dll_ith_item(dirs, n_dirs - 1);
 				last_dir->next = files;
 				files->prev = dirs;
 			}
 		} else {
-			if (n_f > 0) {
+			if (n_files > 0) {
 				*list = files;
 			}
 		}
-		return CDW_OK;
+		return n_files + n_dirs;
 	} else {
 		if (dirs != (cdw_dll_item_t *) NULL) {
 			cdw_file_dealloc_files_from_list(dirs);
@@ -1067,7 +1087,7 @@ cdw_rv_t cdw_fs_copy_dirent_to_list(cdw_dll_item_t **list, const char *dirpath,
 			cdw_dll_clean(files);
 			dirs = (cdw_dll_item_t *) NULL;
 		}
-		return CDW_GEN_ERROR;
+		return 0;
 	}
 }
 
@@ -1149,7 +1169,7 @@ cdw_rv_t cdw_fs_traverse_path(char *fullpath, cdw_fs_visitor_t visitor, cdw_fs_v
 		   an additional error message string */
 		cdw_logging_write(_("Error: perhaps this information can help resolve problem: \"%s\"\n"), strerror(e));
 		// assert (0);
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	cdw_rv_t retval = CDW_NO;
@@ -1173,7 +1193,7 @@ cdw_rv_t cdw_fs_traverse_path(char *fullpath, cdw_fs_visitor_t visitor, cdw_fs_v
 		retval = CDW_NO;
 	}
 
-	if (retval == CDW_GEN_ERROR) {
+	if (retval == CDW_ERROR) {
 		/* 2TRANS: this is message printed to log file;
 		   %s is a full path to file */
 		cdw_logging_write(_("Error: cdw can't get any information about this file: \"%s\"\n"), fullpath);
@@ -1202,7 +1222,7 @@ cdw_rv_t cdw_fs_traverse_dir(char *dirpath, cdw_fs_visitor_t visitor, cdw_fs_vis
 	cdw_rv_t retval = CDW_OK;
 	if (n == -1) {
 		cdw_vdm ("ERROR: scandir() returns -1 (%s) for dirpath \"%s\"\n", strerror(e), dirpath);
-	       retval = CDW_GEN_ERROR;
+	       retval = CDW_ERROR;
 	} else if (n == 0) {
 		/* TODO: empty dir, is there anything more to do? */
 		retval = CDW_OK;
@@ -1215,7 +1235,7 @@ cdw_rv_t cdw_fs_traverse_dir(char *dirpath, cdw_fs_visitor_t visitor, cdw_fs_vis
 			int rv = asprintf(&newpath, "%s/%s", dirpath, eps[i]->d_name);
 			if (rv == -1) {
 				cdw_vdm ("ERROR: failed to allocate memory for new path\n");
-				retval = CDW_GEN_ERROR;
+				retval = CDW_ERROR;
 				break;
 			} else {
 				cdw_rv_t crv = cdw_fs_traverse_path(newpath, visitor, visitor_data);
@@ -1223,7 +1243,7 @@ cdw_rv_t cdw_fs_traverse_dir(char *dirpath, cdw_fs_visitor_t visitor, cdw_fs_vis
 				newpath = (char *) NULL;
 				if (crv != CDW_OK) {
 					cdw_vdm ("ERROR: function failed to traverse path \"%s\"\n", newpath);
-					retval = CDW_GEN_ERROR;
+					retval = CDW_ERROR;
 					break;
 				}
 			}
@@ -1270,12 +1290,12 @@ cdw_rv_t cdw_fs_traverse_symlink(const char *fullpath, cdw_fs_visitor_t visitor,
 			} else {
 				cdw_vdm ("  errno == ENOENT, but reason is other than broken link\n");
 				/* TODO: some more information for user pls */
-				return CDW_GEN_ERROR;
+				return CDW_ERROR;
 			}
 		}
 #endif
 
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	} else { /* resolved_fullpath != NULL */
 		cdw_rv_t retval = CDW_OK;
 		if (!strcmp(resolved_fullpath, ".")) {
@@ -1497,26 +1517,26 @@ struct {
 	  gettext_noop("System error"),
 	  /* 2TRANS: this is message in dialog window, user can press OK button */
 	  gettext_noop("No system memory. You should close application."),
-	  CDW_BUTTONS_OK, CDW_COLORS_ERROR, CDW_SYS_ERROR },
+	  CDW_BUTTONS_OK, CDW_COLORS_ERROR, CDW_ERROR },
 	{ EMFILE, /* The process already has the maximum number of files open */
 	  /* 2TRANS: this is title of dialog window */
 	  gettext_noop("System error"),
 	  /* 2TRANS: this is message in dialog window, user can press OK button */
 	  gettext_noop("Too many files opened. You should close application."),
-	  CDW_BUTTONS_OK, CDW_COLORS_ERROR, CDW_SYS_ERROR },
+	  CDW_BUTTONS_OK, CDW_COLORS_ERROR, CDW_ERROR },
 	{ ENFILE, /* The system limit on the total number of open files has been reached */
 	  /* 2TRANS: this is title of dialog window */
 	  gettext_noop("System error"),
 	  /* 2TRANS: this is message in dialog window, user can press OK button */
 	  gettext_noop("Too many files opened. You should close application."),
-	  CDW_BUTTONS_OK, CDW_COLORS_ERROR, CDW_SYS_ERROR },
+	  CDW_BUTTONS_OK, CDW_COLORS_ERROR, CDW_ERROR },
 	{ EWOULDBLOCK, /* The O_NONBLOCK flag was specified,and an incompatible lease
 			  was held on the file (see fcntl(2)) */
 	  /* 2TRANS: this is title of dialog window */
 	  gettext_noop("System error"),
 	  /* 2TRANS: this is message in dialog window, user can press OK button */
 	  gettext_noop("Unknown error while accessing the file."),
-	  CDW_BUTTONS_OK, CDW_COLORS_ERROR, CDW_SYS_ERROR },
+	  CDW_BUTTONS_OK, CDW_COLORS_ERROR, CDW_ERROR },
 	{ 0, /* arbitrary value in guard element */
 	  (char *) NULL, /* guard value */
 	  (char *) NULL, /* guard value */
@@ -1581,7 +1601,7 @@ cdw_rv_t cdw_fs_errno_handler(int e)
 				      press OK button */
 				   _("Unknown error while accessing the file."),
 				   CDW_BUTTONS_OK, CDW_COLORS_ERROR);
-		return CDW_SYS_ERROR;
+		return CDW_ERROR;
 	}
 }
 
@@ -1589,6 +1609,76 @@ cdw_rv_t cdw_fs_errno_handler(int e)
 
 
 
+bool cdw_fs_is_hidden(const char *file_name)
+{
+	if (file_name[0] != '.') {
+		return false;
+	}
+
+	if (file_name[0] == '.' && file_name[1] == '.' && file_name[2] == '\0') {
+		/* reference to parent dir */
+		return false;
+	} else if (file_name[0] == '.' && file_name[1] == '\0') {
+		/* reference to current dir */
+		return false;
+	} else {
+		return true;
+	}
+}
+
+
+
+
+
+char *cdw_fs_shorten_fullpath(cdw_file_t *file)
+{
+	ssize_t pos = cdw_fs_get_filename_start(file->fullpath);
+	if (pos == -1) {
+		return (char *) NULL;
+	} else {
+		char *new_fullpath = strndup(file->fullpath, (size_t) pos);
+		cdw_vdm ("INFO: original fullpath  = \"%s\"\n", file->fullpath);
+		cdw_vdm ("INFO: shortened fullpath = \"%s\"\n", new_fullpath);
+		return new_fullpath;
+	}
+}
+
+
+
+
+
+/**
+   \brief Check if given fullpath is a symlink, and return target path for it
+
+   This is a simple version of canonicalize_filename_mode(), only
+   resolving symbolic links. It's basically a wrapper around readlink().
+
+   \para fullpath - fullpath to inspect/resolve
+
+   \return freshly allocated buffer with resolved path if \p fullpath is a symbolic link
+   \return NULL pointer on errors or when \p fullpath is not a symbolic link
+*/
+char *cdw_fs_get_target_path_simple(const char *fullpath)
+{
+	char *target_path = (char *) NULL;
+
+	char link_path[PATH_MAX - 2];
+	ssize_t r = readlink(fullpath, link_path, PATH_MAX - 2);
+	if (r != -1) {
+		link_path[r] = '\0';
+
+		target_path = strdup(link_path);
+		cdw_sdm ("INFO: \"%s\" -> \"%s\"\n", fullpath, target_path);
+	} else if (r == EINVAL) {
+		/* "The named file is not a symbolic link." */
+	} else {
+		; /* TODO: check rest of values of r, some of them might
+		     indicate serious problems */
+	}
+
+	return target_path;
+}
+
 
 
 
@@ -1699,12 +1789,12 @@ void test_cdw_fs_correct_dir_path_ending(void)
 	/* first test basic cases */
 
 	crv = cdw_fs_correct_dir_path_ending(&path);
-	assert(crv == CDW_ARG_ERROR);
+	assert(crv == CDW_ERROR);
 
 	crv = cdw_string_set(&path, "");
 	assert(crv == CDW_OK);
 	crv = cdw_fs_correct_dir_path_ending(&path);
-	assert(crv == CDW_ARG_ERROR);
+	assert(crv == CDW_ERROR);
 
 	struct {
 		const char *input;
