@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2010 Kamil Ignacak
+ * Copyright (C) 2007 - 2011 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,14 +43,29 @@
 #include "cdw_ncurses.h"
 #include "cdw_debug.h"
 #include "canonicalize.h"
+#include "cdw_config.h"
+
+extern cdw_config_t global_config;
 
 
-static cdw_fs_browser_t *cdw_fs_browser_new_base(WINDOW *window);
-static ssize_t           cdw_fs_browser_read_dir_content(cdw_dll_item_t **list, const char *dirpath);
-static void              cdw_fs_browser_internal_error_dialog(cdw_rv_t crv);
-static cdw_rv_t          cdw_fs_browser_change_dir_on_enter(cdw_fs_browser_t *browser);
+static cdw_fs_browser_t   *cdw_fs_browser_new_base(WINDOW *window);
+static unsigned long long  cdw_fs_browser_read_dir_content(cdw_dll_item_t **list, const char *dirpath);
+static void                cdw_fs_browser_internal_error_dialog(int rv);
+static cdw_rv_t            cdw_fs_browser_change_dir_on_enter(cdw_fs_browser_t *browser);
 
 
+#ifndef NDEBUG
+#define cdw_fs_browser_debug_report_current_fullpath(macro_browser, macro_message) \
+	{								\
+		char *macro_f = cdw_fs_browser_get_current_fullpath(macro_browser); \
+		cdw_vdm ("INFO: %s, current fullpath = \"%s\"\n", macro_message, macro_f); \
+		free(macro_f);						\
+		macro_f = (char *) NULL;				\
+	}
+#else
+#define cdw_fs_browser_debug_report_current_fullpath(macro_browser, macro_message)
+#endif
+
 
 
 /**
@@ -115,7 +130,7 @@ cdw_fs_browser_t *cdw_fs_browser_new(WINDOW *window, const char *initial_fullpat
 		cdw_vdm ("ERROR: can't get any safe initial dirpath for fs browser, this is bad\n");
 	}
 
-	cdw_fs_browser_internal_error_dialog(CDW_GEN_ERROR);
+	cdw_fs_browser_internal_error_dialog(CDW_ERROR);
 	cdw_fs_browser_delete(&browser);
 	return (cdw_fs_browser_t *) NULL;
 }
@@ -146,18 +161,18 @@ cdw_fs_browser_t *cdw_fs_browser_new_base(WINDOW *window)
 		return (cdw_fs_browser_t *) NULL;
 	}
 
-	size_t n_lines = (size_t) getmaxy(window);
-	cdw_assert (n_lines > 2, "ERROR: number of lines of window is too small: %zd\n", n_lines);
+	int n_lines = getmaxy(window);
+	cdw_assert (n_lines > 2, "ERROR: number of lines of window is too small: %d\n", n_lines);
 	n_lines -= 2;
-	size_t n_cols = (size_t) getmaxx(window);
-	cdw_assert (n_cols > 2, "ERROR: number of cols of window is too small: %zd\n", n_cols);
+	int n_cols = getmaxx(window);
+	cdw_assert (n_cols > 2, "ERROR: number of cols of window is too small: %d\n", n_cols);
 	n_cols -= 2;
 
 	/* 1, 1 - begin_y, begin_x */
 	browser->display = cdw_list_display_new(window, n_lines, n_cols, 1, 1, CDW_COLORS_DIALOG);
 	if (browser->display == (CDW_LIST_DISPLAY *) NULL) {
 		cdw_vdm ("ERROR: failed to create file_browser display\n");
-		cdw_fs_browser_internal_error_dialog(CDW_GEN_ERROR);
+		cdw_fs_browser_internal_error_dialog(CDW_ERROR);
 
 		free(browser);
 		browser = (cdw_fs_browser_t *) NULL;
@@ -166,19 +181,11 @@ cdw_fs_browser_t *cdw_fs_browser_new_base(WINDOW *window)
 	}
 
 	browser->display->display_item = cdw_file_display_file;
-	cdw_list_display_add_return_char(browser->display, KEY_DC);
-	cdw_list_display_add_return_char(browser->display, ' ');
-	cdw_list_display_add_return_char(browser->display, CDW_KEY_ENTER);
-	cdw_list_display_add_return_char(browser->display, 'q'); /* 'q' = "quit" */
-	cdw_list_display_add_return_char(browser->display, 'Q'); /* 'Q' = "Quit" */
-
+	/* space=select, enter=change dir, q = quit, dot=toggle displaying hidden files, 0=guard */
+	cdw_list_display_add_return_keys(browser->display, KEY_DC, ' ', CDW_KEY_ENTER, 'q', 'Q', '.', KEY_BACKSPACE, '~', '?', 0);
 #ifndef NDEBUG
-	cdw_list_display_add_return_char(browser->display, KEY_DOWN);
-	cdw_list_display_add_return_char(browser->display, KEY_UP);
-	cdw_list_display_add_return_char(browser->display, KEY_NPAGE);
-	cdw_list_display_add_return_char(browser->display, KEY_PPAGE);
-	cdw_list_display_add_return_char(browser->display, KEY_HOME);
-	cdw_list_display_add_return_char(browser->display, KEY_END);
+	cdw_list_display_add_return_keys(browser->display,
+					 KEY_DOWN, KEY_UP, KEY_NPAGE, KEY_PPAGE, KEY_HOME, KEY_END, 0);
 #endif
 
 	return browser;
@@ -228,11 +235,17 @@ void cdw_fs_browser_delete(cdw_fs_browser_t **browser)
    Function scans directory specified by \p dirpath, checks information about
    all files (files and directories)  in the directory, and saves the
    information on \p list - one node of 'cdw_file_t *' per file. Function
-   returns number of files and directories in given directory. Reference to
-   current directory (".") is not put on the list.
+   returns number of items in \p list. Reference to current directory (".")
+   is not put on the list.
+
+   There will be at least one item in the \p list: reference to parent dir.
+   If displaying of hidden files is disabled in configuration, the list won't
+   put them on the \p list, and returned value won't include hidden files.
+   Therefore the function returns number of items on the list - not number
+   of items (files+dirs) in directory.
 
    In case of errors the \p list is cleaned, \p list is set to null, and
-   function returns -1.
+   function returns 0.
 
    You can pass \p list that already carries some information, but it will
    be modified: either set to contain new, correct data, or reset to empty
@@ -241,10 +254,10 @@ void cdw_fs_browser_delete(cdw_fs_browser_t **browser)
    \param list - list to be filled with information about files
    \param dirpath - path to directory which you want to scan
 
-   \return number of files in given directory on success
-   \return -1 on errors
+   \return number of files read from current dir into list
+   \return 0 on errors
 */
-ssize_t cdw_fs_browser_read_dir_content(cdw_dll_item_t **list, const char *dirpath)
+unsigned long long cdw_fs_browser_read_dir_content(cdw_dll_item_t **list, const char *dirpath)
 {
 	if (*list != (cdw_dll_item_t *) NULL) {
 		cdw_file_dealloc_files_from_list(*list);
@@ -263,44 +276,45 @@ ssize_t cdw_fs_browser_read_dir_content(cdw_dll_item_t **list, const char *dirpa
 		   disc - the disc wasn't reconnected (?) after coming from
 		   suspension to RAM */
 		cdw_vdm ("ERROR: scandir() returns -1 for dirpath \"%s\"\n", dirpath);
-		cdw_fs_browser_internal_error_dialog(CDW_SYS_ERROR);
-		return -1;
+		cdw_fs_browser_internal_error_dialog(CDW_ERROR);
+		return 0;
 	} else if (n_files == 0) {
 		/* not going to happen, with scandir() called with one(),
 		   there will be at least '..' item in eps */
 		cdw_assert (n_files != 0, "ERROR: scandir() called with one() returs 0 for dirpath \"%s\"\n", dirpath);;
-		return -1;
+		return 0;
 	} else {
-		cdw_rv_t cp = cdw_fs_copy_dirent_to_list(list, dirpath, eps, (size_t) n_files);
+		unsigned long long n = (unsigned long long) n_files;
+		unsigned long long cp = cdw_fs_copy_dirent_to_list(list, dirpath, eps, n, global_config.fs.display_hidden_files);
 
 		/* eps is no longer needed, all interesting information about
 		   directory content is now in display->list */
-		for (ssize_t i = 0; i < n_files; i++) {
+		for (unsigned long long i = 0; i < n; i++) {
 			free(eps[i]);
 			eps[i] = (struct dirent *) NULL;
 		}
 		free(eps);
 		eps = (struct dirent **) NULL;
 
-		if (cp != CDW_OK) {
+		if (cp == 0) {
 			if (*list != (cdw_dll_item_t *) NULL) {
 				cdw_file_dealloc_files_from_list(*list);
 				cdw_dll_clean(*list);
 				*list = (cdw_dll_item_t *) NULL;
 			}
 			cdw_vdm ("ERROR: failed to copy dirent to list for dirpath \"%s\"\n", dirpath);
-			return -1;
+			return 0;
 		} else {
 #ifndef NDEBUG
 			cdw_vdm ("INFO: Content of current directory \"%s\":\n", dirpath);
-			for (ssize_t i = 0; i < n_files; i++) {
+			for (unsigned long long i = 0; i < cp; i++) {
 				cdw_dll_item_t *item = cdw_dll_ith_item(*list, (size_t) i);
 			        cdw_file_t *file = (cdw_file_t *) item->data;
 				ssize_t f = cdw_fs_get_filename_start(file->fullpath);
-				cdw_vdm ("INFO:       file #%5zd = \"%s\"\n", i, file->fullpath + f);
+				cdw_vdm ("INFO:       file #%6lld = \"%s\"\n", i, file->fullpath + f);
 			}
 #endif
-			return n_files;
+			return cp;
 		}
 	}
 }
@@ -331,7 +345,7 @@ ssize_t cdw_fs_browser_read_dir_content(cdw_dll_item_t **list, const char *dirpa
 
    \return CDW_OK if directory was successfully changed and displayed
    \return CDW_NO if directory was not changed, but not because of error
-   \return CDW_GEN_ERROR if other error occurred and dir was not changed because of that error
+   \return CDW_ERROR if other error occurred and dir was not changed because of that error
 */
 cdw_rv_t cdw_fs_browser_change_dir_on_enter(cdw_fs_browser_t *browser)
 {
@@ -355,11 +369,11 @@ cdw_rv_t cdw_fs_browser_change_dir_on_enter(cdw_fs_browser_t *browser)
 	int rv = 0;
 	char *new_dirpath = (char *) NULL;
 	if (file->is_ref_to_parent_dir) {
-		/* "/some/dir/../" */
-		new_dirpath = strndup(file->fullpath, strlen(file->fullpath) - 3);
+		/* "/some/dir/../", remove last dir from the path */
+		new_dirpath = cdw_fs_shorten_fullpath(file);
 		if (new_dirpath == (char *) NULL) {
-			cdw_vdm ("ERROR: failed to strndup() fullpath \"%s\"\n", file->fullpath);
-			return CDW_GEN_ERROR;
+			cdw_vdm ("ERROR: failed to shorten fullpath \"%s\"\n", file->fullpath);
+			return CDW_ERROR;
 		}
 
 		/* browse_to_file() moves cursor (focus) to directory item from
@@ -372,7 +386,7 @@ cdw_rv_t cdw_fs_browser_change_dir_on_enter(cdw_fs_browser_t *browser)
 		if (new_dirpath == (char *) NULL) {
 			/* canonicalization of dirpath failed */
 			cdw_vdm ("ERROR: failed to canonicalize path \"%s\"\n", file->fullpath);
-			return CDW_GEN_ERROR;
+			return CDW_ERROR;
 		}
 		rv = cdw_fs_browser_browse_into_dir(browser, new_dirpath);
 	}
@@ -394,7 +408,7 @@ cdw_rv_t cdw_fs_browser_change_dir_on_enter(cdw_fs_browser_t *browser)
 
 		} else {
 			cdw_vdm ("ERROR: reason unknown\n");
-			cdw_fs_browser_internal_error_dialog(CDW_GEN_ERROR);
+			cdw_fs_browser_internal_error_dialog(CDW_ERROR);
 		}
 		cdw_list_display_refresh(browser->display);
 	}
@@ -408,7 +422,7 @@ cdw_rv_t cdw_fs_browser_change_dir_on_enter(cdw_fs_browser_t *browser)
 		   a good one not to browse into dir */
 		return CDW_NO;
 	} else {
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 }
 
@@ -453,24 +467,54 @@ int cdw_fs_browser_driver(cdw_fs_browser_t *browser)
 		key = cdw_list_display_driver(browser->display);
 		if (key == CDW_KEY_ENTER) {
 			cdw_rv_t crv = cdw_fs_browser_change_dir_on_enter(browser);
-
 			if (crv == CDW_OK) {
-				cdw_vdm ("INFO: ENTER key pressed, current fs fullpath = \"%s\"\n",
-					 cdw_fs_browser_get_current_fullpath(browser));
-
-			} else if (crv == CDW_GEN_ERROR) {
+				cdw_fs_browser_debug_report_current_fullpath(browser, "INFO: ENTER key pressed");
+			} else if (crv == CDW_ERROR) {
 				cdw_vdm ("ERROR: failed to change dir in fs browser\n");
 				return -1;
 			} else { /* CDW_NO - not a dir or no perms to change dir, continue */
 				;
 			}
 		} else if (key == ' ') {
-			cdw_vdm ("INFO: SPACE key pressed, current fs fullpath = \"%s\"\n",
-				 cdw_fs_browser_get_current_fullpath(browser));
+			cdw_fs_browser_debug_report_current_fullpath(browser, "INFO: SPACE key pressed");
+#ifndef NDEBUG
 			/* SPACE key may be used by higher level code for selecting
 			   files (if SPACE is defined as "return on" char) */
 			cdw_assert (browser->display->n_items > browser->display->current_item_ind,
 				    "ERROR: index of current file is larger than number of files\n");
+#endif
+		} else if (key == '.') {
+			if (global_config.fs.display_hidden_files) {
+				global_config.fs.display_hidden_files = false;
+			} else {
+				global_config.fs.display_hidden_files = true;
+			}
+			char *f = cdw_fs_browser_get_current_fullpath(browser);
+			/* FIXME: check return values */
+			int rv = cdw_fs_browser_browse_to_file(browser, f);
+			free(f);
+			f = (char *) NULL;
+			assert (!rv);
+			key = 'a'; /* dummy value, keep looping */
+		} else if (key == KEY_BACKSPACE) {
+			cdw_file_t *file = cdw_fs_browser_get_current_file(browser);
+			char *new_dir = cdw_fs_shorten_fullpath(file);
+			if (new_dir != (char *) NULL) {
+				cdw_fs_browser_browse_to_file(browser, new_dir);
+				free(new_dir);
+				new_dir = (char *) NULL;
+			} else {
+				cdw_vdm ("ERROR: can't go up from \"%s\"\n", file->fullpath);
+			}
+			return key; /* allow client code to update current path */
+		} else if (key == '~') {
+			const char *dir = cdw_fs_get_home_dir_fullpath();
+			cdw_fs_browser_browse_into_dir(browser, dir);
+			return key; /* allow client code to update current path */
+		} else if (cdw_list_display_is_return_char(browser->display, key)) {
+			/* some other return key that client code
+			   has to act upon */
+			return key;
 		} else {
 			;
 		}
@@ -482,8 +526,7 @@ int cdw_fs_browser_driver(cdw_fs_browser_t *browser)
 		    || key == KEY_HOME
 		    || key == KEY_END) {
 
-			const char *f = cdw_fs_browser_get_current_fullpath(browser);
-			cdw_vdm ("INFO: movement key pressed, current fs fullpath = \"%s\"\n", f);
+			cdw_fs_browser_debug_report_current_fullpath(browser, "INFO: movement key pressed");
 		}
 #endif
 
@@ -504,27 +547,30 @@ int cdw_fs_browser_driver(cdw_fs_browser_t *browser)
 
    Function displays dialog window with error message. Function created to
    avoid repetition of the same dialog in two places.
+
+   FIXME: the function accepts values of regular type int
+   and of type cdw_rv_t, but type of its argument is int.
 */
-void cdw_fs_browser_internal_error_dialog(cdw_rv_t crv)
+void cdw_fs_browser_internal_error_dialog(int rv)
 {
 	char *message = (char *) NULL;
-	if (crv == CDW_GEN_ERROR) {
+	if (rv == CDW_ERROR) {
 		/* 2TRANS: this is message in dialog window */
 		message = _("Unexpected error, perhaps some file has been deleted. Please close this window and open it again.");
-	} else if (crv == CDW_FS_CHECK_E_PERM_NO_X) {
+	} else if (rv == CDW_FS_CHECK_E_PERM_NO_X) {
 		/* 2TRANS: this is message in dialog window;
 		   "change directory" means "enter into directory" */
 		message = _("Cannot change directory: wrong permissions.");
-	} else if (crv == CDW_FS_CHECK_E_PERM_NO_R) {
+	} else if (rv == CDW_FS_CHECK_E_PERM_NO_R) {
 		/* 2TRANS: this is message in dialog window;
 		   "read directory" means "read content of directory" */
 		message = _("Cannot read directory: wrong permissions.");
-	} else if (crv == CDW_FS_CHECK_SYS_ERROR) {
+	} else if (rv == CDW_FS_CHECK_SYS_ERROR) {
 		/* 2TRANS: this is message in dialog window:
 		   user highlighted a file and pressed enter,
 		   but the file probably does not exit */
 		message = _("File does not exists or cannot change directory.");
-	} else if (crv == CDW_NO) {
+	} else if (rv == CDW_NO) {
 		/* 2TRANS: this is message in dialog window. Some
 		   function returned error value. No further action
 		   will be performed, no other explanations provided */
@@ -610,8 +656,11 @@ int cdw_fs_browser_browse_into_dir(cdw_fs_browser_t *browser, const char *dirpat
 		return rv;
 	}
 
-	ssize_t n = cdw_fs_browser_read_dir_content(&(browser->display->list), dirpath);
-	if (n == -1) {
+	/* n is a number of files on list, not number of files in
+	   directory. The two may be different if displaying of hidden
+	   files is turned off. */
+	unsigned long long n = cdw_fs_browser_read_dir_content(&(browser->display->list), dirpath);
+	if (n == 0) {
 		cdw_assert (browser->display->list == (cdw_dll_item_t *) NULL,
 			    "ERROR: dir not read properly, but list of files is not null\n");
 
@@ -620,19 +669,16 @@ int cdw_fs_browser_browse_into_dir(cdw_fs_browser_t *browser, const char *dirpat
 
 		return -1;
 	} else {
-
 #ifndef NDEBUG
 		size_t len = cdw_dll_length(browser->display->list);
-		cdw_assert (len == (size_t) n, "ERROR: lengths mismatch: dir content size (n) = %zd, len of browser display list (len) = %zd\n", n, len);
+		cdw_assert (len == (size_t) n, "ERROR: lengths mismatch: dir content size (n) = %lld, len of browser display list (len) = %zd\n", n, len);
 #endif
-
 		browser->display->n_items = (size_t) n;
 		/* first 0 - position cursor on first item in new dir
 		   second 0 - horizontal offset, true - highlight selected item */
 		cdw_list_display_scroll_to(browser->display, 0, 0, true);
 
-		cdw_vdm ("INFO: successfully read new dir, fullpath = \"%s\"\n", cdw_fs_browser_get_current_fullpath(browser));
-
+		cdw_fs_browser_debug_report_current_fullpath(browser, "INFO: successfully read new dir");
 		return 0;
 	}
 }
@@ -642,7 +688,7 @@ int cdw_fs_browser_browse_into_dir(cdw_fs_browser_t *browser, const char *dirpat
 
 
 /**
-   \return ENOENT if there is no directory specified in fullpath
+   \return ENONET if there is no directory specified in fullpath
                   (or there are no access rights to the directory)
    \return ENOENT if there is a dir specified in fullpath, but there
                   is no file specified by fullpath
@@ -669,7 +715,7 @@ int cdw_fs_browser_browse_to_file(cdw_fs_browser_t *browser, const char *fullpat
 		   system, or never existed */
 		bool found = false;
 		size_t item_ind = 0;
-		cdw_vdm ("INFO: ==== searching for file \"%s\"\n", fullpath + f);
+		cdw_sdm ("INFO: ==== searching for file \"%s\"\n", fullpath + f);
 		for (item_ind = 0; item_ind < browser->display->n_items; item_ind++) {
 
 			/* TODO: optimize this line by traversing list directly */
@@ -690,11 +736,22 @@ int cdw_fs_browser_browse_to_file(cdw_fs_browser_t *browser, const char *fullpat
 
 			return 0;
 		} else {
-			cdw_vdm ("WARNING: file \"%s\" not found in dir \"%s\"\n", fullpath + f, dirpath);
 			free(dirpath);
 			dirpath = (char *) NULL;
 
-			return ENOENT;
+			/* file may be not available for one reason:
+			   it is a hidden file, but displaying of files
+			   has been turned off in the middle of browsing;
+			   this is not an error, handle this gracefully */
+			cdw_vdm ("WARNING: file \"%s\" not found in dir \"%s\"\n", fullpath + f, dirpath);
+			ssize_t start = cdw_fs_get_filename_start(fullpath);
+			if (cdw_fs_is_hidden(fullpath + start) && !global_config.fs.display_hidden_files) {
+				cdw_vdm ("INFO: it may be because displaying hidden files has been disabled\n");
+				cdw_list_display_scroll_to(browser->display, 0, 0, true);
+				return 0;
+			} else {
+				return ENOENT;
+			}
 		}
 	}
 }
@@ -737,12 +794,18 @@ char *cdw_fs_browser_get_current_printable_dirpath(cdw_fs_browser_t *browser)
 
 
 
-const char *cdw_fs_browser_get_current_fullpath(cdw_fs_browser_t *browser)
+char *cdw_fs_browser_get_current_fullpath(cdw_fs_browser_t *browser)
 {
 	cdw_file_t *file = cdw_fs_browser_get_current_file(browser);
 	cdw_assert (file != (cdw_file_t *) NULL, "ERROR: can't get current file\n");
 	cdw_assert (file->fullpath != (char *) NULL, "ERROR: current file has no fullpath\n");
-	return file->fullpath;
+	/* file->fullpath is a valid pointer at this point, but if
+	   you will pass it to function that re-reads (and thus reallocates)
+	   list with current dir listing, then the pointer becomes
+	   invalid an you are in trouble; so let's just allocate
+	   new string */
+	char *fullpath = strdup(file->fullpath);
+	return fullpath;
 }
 
 
@@ -766,3 +829,45 @@ bool cdw_fs_browser_is_navigation_key(int key)
 
 
 
+
+
+/**
+   \brief Display window with information about available hotkeys
+
+   Information is displayed in window that is no larger than
+   given \p browser, and \p browser window is automatically refreshed
+   after closing help window.
+
+   \p browser - fs browser, on top of which to display a help window
+*/
+void cdw_fs_browser_help_window(cdw_fs_browser_t *browser)
+{
+	int n_lines = 10;
+	int n_cols = browser->display->n_cols < 50 ? browser->display->n_cols : 50;
+	int begin_y = browser->display->begin_y + (browser->display->n_lines / 2) - (n_lines / 2);
+	int begin_x = browser->display->begin_x + (browser->display->n_cols / 2) - (n_cols / 2);
+
+	WINDOW *window = cdw_ncurses_window_new((WINDOW *) NULL,
+						n_lines, n_cols,
+						begin_y, begin_x,
+						CDW_COLORS_DIALOG, "Help", "Press any key to close");
+	if (window == (WINDOW *) NULL) {
+		cdw_vdm ("ERROR: failed to create new window\n");
+		return;
+	}
+	mvwprintw(window, 1, 2, "Hot keys available in this window:");
+	mvwprintw(window, 3, 2, " .        - toggle displaying hidden files");
+	mvwprintw(window, 4, 2, " ~        - go to home directory");
+	mvwprintw(window, 5, 2, "backspace - go to parent directory");
+	mvwprintw(window, 6, 2, " ?        - display this help");
+
+	wgetch(window);
+	delwin(window);
+	window = (WINDOW *) NULL;
+
+	wrefresh(browser->display->subwindow);
+
+	return;
+}
+
+
