@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2010 Kamil Ignacak
+ * Copyright (C) 2007 - 2011 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,22 +37,24 @@
 #include "cdw_file_manager.h"
 #include "cdw_cdrecord.h"
 #include "cdw_growisofs.h"
-#include "cdw_md5sum.h"
+#include "cdw_xorriso.h"
+#include "cdw_digest.h"
 #include "cdw_logging.h"
 #include "cdw_main_window.h"
 #include "cdw_iso9660.h"
 #include "cdw_ext_tools.h"
-#include "cdw_config_ui.h"
+#include "cdw_config_window.h"
 #include "cdw_processwin.h"
 #include "cdw_read_disc_info.h"
+#include "cdw_calculate_digest.h"
 
-extern cdw_config_t global_config;      /* variable holding cdw configuration */
 
 static cdw_rv_t cdw_burn_disc_dispatcher(cdw_task_t *parent_task, cdw_disc_t *disc);
 static bool cdw_burn_disc_is_disc_ready(void);
-static bool cdw_burn_disc_get_source(cdw_task_t *burn_task);
+static bool cdw_burn_disc_get_source(cdw_task_t *task);
 static void cdw_burn_disc_release_source(cdw_task_t *task);
-
+static bool cdw_burn_disc_check_wodim_large_image(cdw_task_t *task);
+static bool cdw_burn_disc_check_overwrite_content(cdw_task_t *task, cdw_disc_t *disc);
 
 /**
    \brief Check if everything is ready for writing to optical disc and then perform it
@@ -68,7 +70,7 @@ static void cdw_burn_disc_release_source(cdw_task_t *task);
     - write files to cd
     - write image to cd
 
-   The decision is based on value of burn_task->source and type of
+   The decision is based on value of task->source and type of
    disc. Disc type is checked during this function call, before performing
    task.
 
@@ -80,125 +82,65 @@ static void cdw_burn_disc_release_source(cdw_task_t *task);
    \return CDW_OK on success
    \return CDW_NO if some preconditions were not met
    \return CDW_CANCEL if user cancels writing
-   \return CDW_GEN_ERROR on failure or some error
+   \return CDW_ERROR on failure or some error
 */
 cdw_rv_t cdw_burn_disc(int task_id)
 {
 	cdw_assert (task_id == CDW_TASK_BURN_FROM_FILES || task_id == CDW_TASK_BURN_FROM_IMAGE,
 		    "ERROR: called the function with incorrect task id %d\n", task_id);
 
-	CDW_TASK_CREATE_TASK(burn_task, task_id);
-
-	bool source_available = cdw_burn_disc_get_source(&burn_task);
-	if (! source_available) {
-		return CDW_NO;
-	}
-
 	/* check if there is a disc in the drive and if it is writable */
 	if (! cdw_burn_disc_is_disc_ready()) {
-		cdw_burn_disc_release_source(&burn_task);
 		return CDW_NO;
 	}
 
 	cdw_disc_t *current_disc = cdw_disc_get();
-
-	/* check if there is any tool that can be used for a task;
-	   it is important to call this function before resolving
-	   writing modes, because the resolver needs information
-	   on both disc and tool selected for task */
-	cdw_rv_t ts = cdw_task_select_tools_for_task(&burn_task, current_disc);
-	if (ts != CDW_OK) {
-		cdw_vdm ("ERROR: failed to get tool for task TASK_BURN\n");
-		cdw_burn_disc_release_source(&burn_task);
-		return CDW_GEN_ERROR;
-	} else {
-		if (task_id == CDW_TASK_BURN_FROM_IMAGE
-		    && burn_task.burn.tool.id == CDW_TOOL_CDRECORD
-		    && cdw_ext_tools_is_cdrecord_wodim()) {
-
-			/* sector = 2048 bytes */
-			long sectors = cdw_iso_image_get_n_sectors();
-			double gigabytes = ((float) sectors * 2.0) / (1024.0 * 1024.0);
-			cdw_vdm ("INFO: sectors = %ld -> %fGB\n",
-				 sectors, gigabytes);
-			if (sectors > 2 * 1024 * 1024) {
-				cdw_vdm ("INFO: large image: %ld sectors -> %fGB\n",
-					 sectors, gigabytes);
-				/* 2TRANS: this is the title of dialog window */
-				cdw_rv_t crv = cdw_buttons_dialog(_("Warning"),
-								  /* 2TRANS: this is the message in the dialog window */
-								  _("You are attempting to use wodim to burn large image. This task will probably fail. You should try using another tool (cdrecord or growisofs) for this task. Do you want to continue?"),
-								  CDW_BUTTONS_OK_CANCEL, CDW_COLORS_ERROR);
-				if (crv != CDW_OK) {
-					cdw_burn_disc_release_source(&burn_task);
-					return CDW_CANCEL;
-				} else {
-					/* 2TRANS: this is message printed
-					   in log file; %ld is size of
-					   ISO9660 image (in sectors) */
-					cdw_logging_write(_("Attempting to burn ISO image of size %ld sectors with wodim, this may fail...\n"), sectors);
-				}
-			}
-		}
-	}
-
-	/* check if current state of disc allows us to write what we want
-	   (iso file or files from disc), and specify allowed modes of writing
-	   (start multisession, append and close, etc.) */
-	cdw_rv_t crv = cdw_task_resolve_allowed_writing_modes(&burn_task, current_disc);
-	if (crv != CDW_OK) { /* CDW_GEN_ERROR or CDW_NO */
-		cdw_burn_disc_release_source(&burn_task);
+	cdw_task_t *task = cdw_task_new(task_id, current_disc);
+	if (task == (cdw_task_t *) NULL) {
+		cdw_vdm ("ERROR: failed to create a task\n");
 		return CDW_NO;
 	}
 
-	if (burn_task.id == CDW_TASK_BURN_FROM_FILES) {
-		/* ask user for volume name (depends on checkbox in configuration) */
-		crv = cdw_config_ui_conditional_volume_label_dialog();
-		if (crv != CDW_OK) {
-			cdw_vdm ("INFO: cancelled at creating volume label\n");
-			cdw_burn_disc_release_source(&burn_task);
-			/* e.g. "Escape" key */
-			return CDW_NO;
-		}
+	bool source_available = cdw_burn_disc_get_source(task);
+	if (! source_available) {
+		cdw_task_delete(&task);
+		return CDW_NO;
 	}
 
-	cdw_rv_t decision = write_wizard(&burn_task, current_disc);
+	bool wodim_ok = cdw_burn_disc_check_wodim_large_image(task);
+	if (!wodim_ok) {
+		cdw_burn_disc_release_source(task);
+		cdw_task_delete(&task);
+		return CDW_CANCEL;
+	}
+
+	cdw_rv_t decision = cdw_write_wizard(task, current_disc);
 
 	if (decision != CDW_OK) {
-		cdw_burn_disc_release_source(&burn_task);
-		if (decision == CDW_GEN_ERROR) {
+		cdw_burn_disc_release_source(task);
+		cdw_task_delete(&task);
+		if (decision == CDW_ERROR) {
 			/* 2TRANS: this is the title of dialog window */
 			cdw_buttons_dialog(_("Error"),
 					   /* 2TRANS: this is the message in the dialog window */
-					   _("There were some problems when cdw tried to prepare for writing. Try restarting cdw."),
+					   _("cdw can't create write wizard. Please restart cdw."),
 					   CDW_BUTTONS_OK, CDW_COLORS_ERROR);
-		} else { /* CDW_CANCEL */
-			;
 		}
 		return decision;
 	}
-	if (current_disc->state_empty != CDW_TRUE) {
-		if (burn_task.burn.session_mode == CDW_SESSION_MODE_START_MULTI
-		    || burn_task.burn.session_mode == CDW_SESSION_MODE_CREATE_SINGLE) {
 
-			/* 2TRANS: this is the title of dialog window */
-			crv = cdw_buttons_dialog(_("Warning"),
-						 /* 2TRANS: this is the message in the dialog window */
-						 _("The disc is not blank, you will overwrite its current content. Continue?"),
-						 CDW_BUTTONS_OK_CANCEL, CDW_COLORS_WARNING);
-			if (crv != CDW_OK) {
-				/* user decided not to write to disc with
-				   "start multi" or "create single" session mode
-				   so let's abandon burning completely */
-				cdw_burn_disc_release_source(&burn_task);
-				return CDW_CANCEL;
-			}
-		}
+	bool overwrite_content = cdw_burn_disc_check_overwrite_content(task, current_disc);
+	if (!overwrite_content) {
+		/* there was already some content on the disc, it might have
+		   been overwritten, but user decided not to overwrite it */
+		cdw_burn_disc_release_source(task);
+		cdw_task_delete(&task);
+		return CDW_CANCEL;
 	}
 
 	cdw_main_ui_main_window_wrefresh();
 
-	cdw_assert (burn_task.id == CDW_TASK_BURN_FROM_FILES || burn_task.id == CDW_TASK_BURN_FROM_IMAGE,
+	cdw_assert (task->id == CDW_TASK_BURN_FROM_FILES || task->id == CDW_TASK_BURN_FROM_IMAGE,
 		    "ERROR: task id has changed\n");
 
 	/* 2TRANS: this is a title of process window;
@@ -209,74 +151,132 @@ cdw_rv_t cdw_burn_disc(int task_id)
 			      _("Writing to optical disc"), true);
 
 	/* burn_dispatcher() calls actual function performing burning */
-	cdw_rv_t command_return = cdw_burn_disc_dispatcher(&burn_task, current_disc);
+	cdw_rv_t command_return = cdw_burn_disc_dispatcher(task, current_disc);
 	/* writing has been finished, erase information that was updated
 	   (and up to date) only during actual writing; displaying it
 	   after this point may be misleading - user might think that
 	   it is somehow still valid */
 	cdw_processwin_erase_fifo_and_speed();
 	cdw_processwin_erase_eta();
+	cdw_burn_disc_release_source(task);
+	const char *drive_fullpath = cdw_drive_get_drive_fullpath();
+	cdw_drive_eject_tray_with_ui_update(drive_fullpath);
 
-	cdw_burn_disc_release_source(&burn_task);
+	cdw_rv_t tool_status = cdw_task_check_tool_status(task);
 
-	/* this function sets task.success according to task->tool_status,
-	   and resets task->tool_status */
-	cdw_task_check_tool_status(&burn_task);
-	cdw_vdm ("INFO: burn dispatcher returns %s\n", cdw_utils_get_crv_label(crv));
-	cdw_vdm ("%s: burn function sets burn_task.tool_status.ok to %s\n",
-		 burn_task.tool_status.ok ? "INFO" : "ERROR",
-		 burn_task.tool_status.ok ? "true" : "false");
-
-	if (command_return == CDW_OK && burn_task.tool_status.ok) {
-		const char *drive = cdw_drive_get_drive_fullpath();
-		if (burn_task.burn.verify) {
+	if (command_return == CDW_OK && tool_status == CDW_OK) {
+		if (task->burn.verify) {
 			cdw_vdm ("INFO: \"verify burn\" is true, attempting verification\n");
+			cdw_drive_close_tray_with_ui_update(drive_fullpath);
 
-			cdw_drive_reload_tray_with_ui_update(drive, false);
-			/* this is necessary because of dispatches in pipe_regexp.c */
-			burn_task.id = CDW_TASK_CHECK_MD5SUM;
-			burn_task.verify.iso_fullpath = global_config.iso_image_full_path;
-			/* we are passing a copy of burn_task here because
-			   tools are already configured in this variable */
-			cdw_rv_t md5 = cdw_verify(&burn_task);
-			if (md5 != CDW_OK) {
-				/* TODO: check burn_task.tool_status.ok */
-				cdw_vdm ("ERROR: md5sum check function returns !CDW_OK\n");
+			cdw_rv_t verification = cdw_calculate_digest(CDW_CALCULATE_DIGEST_MODE_DISC_FILE, task->burn.iso9660_file_fullpath);
+			if (verification != CDW_OK) {
+				cdw_vdm ("ERROR: verification function returns !CDW_OK\n");
+				command_return = CDW_ERROR;
 			}
 		} else {
 			cdw_vdm ("INFO: \"verify burn\" is false, not attempting to verify\n");
 		}
-		cdw_sdm ("INFO: attempting to reload tray with ui update\n");
-		cdw_drive_reload_tray_with_ui_update(drive, true);
-		cdw_sdm ("INFO: reloading tray with ui update - done\n");
 	} else {
 		/* incorrect situation, already covered by debug messages above */
 		command_return = CDW_NO;
 	}
+	int status = cdw_drive_status(drive_fullpath);
+	if (task->burn.eject) {
+		if (status != CDW_CD_TRAY_OPEN) {
+			cdw_drive_eject_tray_with_ui_update(drive_fullpath);
+		}
+	} else {
+		if (status == CDW_CD_TRAY_OPEN) {
+			cdw_drive_close_tray_with_ui_update(drive_fullpath);
+		}
+	}
+
+	cdw_rv_t retval = CDW_OK;
+	if (command_return == CDW_OK && tool_status == CDW_OK) {
+		cdw_task_save(task);
 
-	if (command_return == CDW_OK) {
 		/* 2TRANS: this is message in dialog window:
 		   operation finished with unknown result, probably success */
 		cdw_processwin_destroy(_("Writing finished"), true);
+		retval = CDW_OK;
 	} else {
-		cdw_vdm ("ERROR: command_return != CDW_OK\n");
+		cdw_vdm ("ERROR: returning CDW_ERROR due to errors reported above\n");
 		/* 2TRANS: this is message in dialog window:
 		   operation finished (most probably) with error */
 		cdw_processwin_destroy(_("Writing probably failed"), true);
+		retval = CDW_ERROR;
 	}
 
 	/* 2TRANS: this is the title of the dialog window, displaying messages
 	   from the program writing iso image or selected files to CD disc */
 	cdw_logging_display_log_conditional(_("\"Write\" log"));
 
-	if (command_return == CDW_OK && burn_task.tool_status.ok) {
-		cdw_sdm ("INFO: burning dispatcher returns CDW_OK and burn_task.tool_status.ok == true\n");
-		return CDW_OK;
+	cdw_task_delete(&task);
+	return retval;
+}
+
+
+
+
+
+bool cdw_burn_disc_check_wodim_large_image(cdw_task_t *task)
+{
+	bool ask = task->id == CDW_TASK_BURN_FROM_IMAGE
+		&& task->burn.tool.id == CDW_TOOL_CDRECORD
+		&& cdw_ext_tools_is_cdrecord_wodim();
+
+	if (!ask) {
+		return true;
+	}
+
+	/* sector = 2048 bytes */
+	long sectors = cdw_iso_image_get_n_sectors();
+	double gigabytes = ((float) sectors * 2.0) / (1024.0 * 1024.0);
+	cdw_vdm ("INFO: sectors = %ld -> %fGB\n", sectors, gigabytes);
+	if (sectors > 2 * 1024 * 1024) {
+		cdw_vdm ("INFO: large image: %ld sectors -> %fGB\n", sectors, gigabytes);
+		/* 2TRANS: this is the title of dialog window */
+		cdw_rv_t crv = cdw_buttons_dialog(_("Warning"),
+						  /* 2TRANS: this is the message in the dialog window */
+						  _("You are attempting to use wodim to burn large image. This task will probably fail. You should try using another tool (cdrecord or growisofs) for this task. Do you want to continue?"),
+						  CDW_BUTTONS_OK_CANCEL, CDW_COLORS_ERROR);
+		if (crv == CDW_OK) {
+			/* 2TRANS: this is message printed in log file;
+			   %ld is size of ISO9660 image (in sectors) */
+			cdw_logging_write(_("Attempting to burn ISO image of size %ld sectors with wodim, this may fail...\n"), sectors);
+			return true;
+		} else {
+			return false;
+		}
 	} else {
-		/* both command_return and burn_task.tool_status.ok were
-		   checked and signaled before, so just use a generic message here */
-		cdw_vdm ("ERROR: returning CDW_GEN_ERROR due to errors reported above\n");
-		return CDW_GEN_ERROR;
+		return true;
+	}
+}
+
+
+
+
+
+bool cdw_burn_disc_check_overwrite_content(cdw_task_t *task, cdw_disc_t *disc)
+{
+	bool ask = disc->state_empty != CDW_TRUE
+		&& (task->burn.session_mode == CDW_SESSION_MODE_START_MULTI
+		    || task->burn.session_mode == CDW_SESSION_MODE_CREATE_SINGLE);
+
+	if (!ask) {
+		return true;
+	}
+
+	/* 2TRANS: this is the title of dialog window */
+	cdw_rv_t crv = cdw_buttons_dialog(_("Warning"),
+					  /* 2TRANS: this is the message in the dialog window */
+					  _("The disc is not blank, you will overwrite its current content. Continue?"),
+					  CDW_BUTTONS_OK_CANCEL, CDW_COLORS_WARNING);
+	if (crv == CDW_OK) {
+		return true;
+	} else {
+		return false;
 	}
 }
 
@@ -285,32 +285,32 @@ cdw_rv_t cdw_burn_disc(int task_id)
 
 
 /**
-   \brief Pass control to cdrecord or growisofs code to perform burning
+   \brief Pass control to cdrecord, growisofs or xorriso code to perform burning
 
-   This function is just a dispatcher, and calls cdw_cdrecord_run_task()
-   or cdw_growisofs_run_task(), depending on task->burn.tool.id. You
-   should call cdw_task_select_tools_for_task() first to set the value of
-   that variable.
+   This function is just a dispatcher, and calls cdw_cdrecord_run_task(),
+   cdw_growisofs_run_task() or cdw_xorriso_run_task(), depending on
+   task->burn.tool.id.
 
    The function checks all important assertions before doing actual
    dispatching, but only in debug build.
 
-   The function will be even more useful when you add another tool
-   that does burning, e.g. xorriso.
-
    \param task - variable describing current task
    \param disc - variable describing current disc
 
-   \return return value from cdrecord or growisofs cdw_*_run_task() function
+   \return return value from cdrecord, growisofs or xorriso cdw_*_run_task() function
 */
 cdw_rv_t cdw_burn_disc_dispatcher(cdw_task_t *task, cdw_disc_t *disc)
 {
-	cdw_assert (task->burn.disc_mode != CDW_DISC_MODE_INIT, "ERROR: calling the function with uninitialized disc mode\n");
+	if (task->burn.tool.id != CDW_TOOL_XORRISO) {
+		cdw_assert (task->burn.disc_mode != CDW_DISC_MODE_INIT, "ERROR: calling the function with uninitialized disc mode\n");
+	}
 	cdw_assert (task->burn.session_mode != CDW_SESSION_MODE_INIT, "ERROR: calling the function with uninitialized session mode\n");
 	cdw_assert (task->id == CDW_TASK_BURN_FROM_FILES || task->id == CDW_TASK_BURN_FROM_IMAGE,
-		    "ERROR: incorrect task id %d\n", task->id);
-	cdw_assert (task->burn.tool.id == CDW_TOOL_CDRECORD || task->burn.tool.id == CDW_TOOL_GROWISOFS,
-		    "ERROR: incorrect task->main_tool_id %d\n", task->burn.tool.id);
+		    "ERROR: incorrect task id %lld\n", task->id);
+	cdw_assert (task->burn.tool.id == CDW_TOOL_CDRECORD
+		    || task->burn.tool.id == CDW_TOOL_GROWISOFS
+		    || task->burn.tool.id == CDW_TOOL_XORRISO,
+		    "ERROR: incorrect task->main_tool_id %lld\n", task->burn.tool.id);
 
 	if (task->id == CDW_TASK_BURN_FROM_FILES) {
 		if (disc->simple_type == CDW_DISC_SIMPLE_TYPE_CD) {
@@ -345,41 +345,47 @@ cdw_rv_t cdw_burn_disc_dispatcher(cdw_task_t *task, cdw_disc_t *disc)
 	} else if (task->burn.tool.id == CDW_TOOL_GROWISOFS) {
 		cdw_vdm ("INFO: dispatching \"burn\" task to growisofs\n");
 		crv = cdw_growisofs_run_task(task, disc);
+	} else if (task->burn.tool.id == CDW_TOOL_XORRISO) {
+		cdw_vdm ("INFO: dispatching \"burn\" task to xorriso\n");
+		crv = cdw_xorriso_run_task(task, disc);
 	} else {
-		cdw_assert (0, "ERROR: wrong tool id '%d' for the job (1)\n", task->burn.tool.id);
+		cdw_assert (0, "ERROR: wrong tool id '%lld' for the job\n", task->burn.tool.id);
 	}
 
 	/* it is too early to check task->tool_status.ok */
-
+	cdw_vdm ("INFO: burn dispatcher returns %s\n", cdw_utils_get_crv_label(crv));
 	return crv;
 }
 
 
 
 
-bool cdw_burn_disc_get_source(cdw_task_t *burn_task)
+
+bool cdw_burn_disc_get_source(cdw_task_t *task)
 {
 	bool source_available = false;
 
-	if (burn_task->id == CDW_TASK_BURN_FROM_IMAGE) {
+	if (task->id == CDW_TASK_BURN_FROM_IMAGE) {
 		/* 2TRANS: this is the title of dialog window */
 		cdw_rv_t crv = cdw_fs_ui_file_picker(_("Path to iso image"),
 						     /* 2TRANS: this is the message in the dialog window;
 						     "writing" means writing to optical disc */
 						     _("Please enter FULL path to an existing iso image file for writing:"),
-						     &(global_config.iso_image_full_path),
+						     &(task->burn.iso9660_file_fullpath),
 						     CDW_FS_FILE, R_OK, CDW_FS_EXISTING);
 
 		cdw_main_ui_main_window_wrefresh();
 		if (crv == CDW_OK) {
-			burn_task->burn.data_size_mb = cdw_iso_image_get_size_mb();
+			/* TODO: replace with call to generic
+			   cdw_fs_get_size_(const char *fullpath, int units); */
+			task->burn.data_size_mb = cdw_iso_image_get_size_mb(task->burn.iso9660_file_fullpath);
 			source_available = true;
 		}
-	} else if (burn_task->id == CDW_TASK_BURN_FROM_FILES) {
+	} else if (task->id == CDW_TASK_BURN_FROM_FILES) {
 		cdw_rv_t crv = cdw_file_manager_create_graftpoints_file();
 		if (crv == CDW_OK) {
 			long long size = cdw_selected_files_get_size();
-			burn_task->burn.data_size_mb = (((double) size) / 1024.0) / 1024.0;
+			task->burn.data_size_mb = (((double) size) / 1024.0) / 1024.0;
 
 			source_available = true;
 			if (cdw_selected_files_file_over_4gb_present()) {
@@ -392,7 +398,7 @@ bool cdw_burn_disc_get_source(cdw_task_t *burn_task)
 						   _("One of selected files has a size over 4GB. Choose your tools wisely (Configuration -> Tools), otherwise writing to disc will give incorrect results."),
 						   CDW_BUTTONS_OK, CDW_COLORS_WARNING);
 			} else {
-				cdw_sdm ("INFO: no file > 4 GB\n");
+				cdw_sdm ("INFO: file size <= 4 GB\n");
 			}
 		} else if (crv == CDW_NO) {
 			; /* probably no selected files */
@@ -430,6 +436,9 @@ bool cdw_burn_disc_is_disc_ready(void)
 		return false;
 	}
 
+	/* update information about disc size and usage */
+	cdw_main_window_volume_info_view_update(true, cdw_config_follow_symlinks());
+
 	cdw_disc_t *current_disc = cdw_disc_get();
 	//cdw_main_ui_disc_info_view_display_data(current_disc);
 
@@ -480,7 +489,7 @@ bool cdw_burn_disc_is_disc_ready(void)
 	} else {
 		/* current_disc->state_appendable == CDW_DISC_APPENDABLE_YES */
 		if (current_disc->type == CDW_DVD_RP_DL
-		    && global_config.support_dvd_rp_dl) {
+		    && cdw_config_support_dvd_rp_dl()) {
 
 			/* 2TRANS: this is the title of the dialog window */
 			cdw_rv_t crv2 = cdw_buttons_dialog(_("Information"),
