@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2010 Kamil Ignacak
+ * Copyright (C) 2007 - 2011 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,6 +44,15 @@
 /* variable holding cdw configuration */
 extern cdw_config_t global_config;
 
+/**
+   \brief Maximum lenght of name of file created when copying tracks to hard disc
+
+   30 chars for core of output file name, track number and file name
+   extension not included; 30 is an arbitrary size, but it seems to be ok;
+   the value does not include ending '\0' char */
+#define CDW_READ_DICS_TRACK_CORE_NAME_LEN 30
+
+
 
 static struct strings {
 	const char *log_string;
@@ -104,9 +113,7 @@ cdw_rv_t cdw_read_disc_read_data_track(const char *fullpath);
 */
 cdw_rv_t cdw_read_disc(void)
 {
-	CDW_TASK_CREATE_TASK(read_task, CDW_TASK_READ_DISC);
-
-	cdw_rv_t crv = cdw_read_disc_check_preconditions(&read_task);
+	cdw_rv_t crv = cdw_read_disc_check_preconditions();
 	if (crv != CDW_OK) { /* conditions for reading disc are not met */
 		return CDW_NO;
 	}
@@ -155,7 +162,7 @@ cdw_rv_t cdw_read_disc(void)
 			audio_file_name = (char *) NULL;
 		}
 		cdw_cdio_close_disc();
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 
@@ -276,7 +283,7 @@ cdw_rv_t cdw_read_disc_read_data_disc_prepare(void)
    \param filename - core of name of files for all tracks
 
    \return CDW_OK on success (all tracks were read correctly)
-   \return CDW_GEN_ERROR if at least one track was not read correctly
+   \return CDW_ERROR if at least one track was not read correctly
 */
 cdw_rv_t cdw_read_disc_read_audio_tracks(const char *filename)
 {
@@ -318,8 +325,8 @@ cdw_rv_t cdw_read_disc_read_audio_tracks(const char *filename)
 	if (status == 0) {
 		return CDW_OK;
 	} else {
-		cdw_vdm ("ERROR: returning CDW_GEN_ERROR after reading errors\n");
-		return CDW_GEN_ERROR;
+		cdw_vdm ("ERROR: returning CDW_ERROR after reading errors\n");
+		return CDW_ERROR;
 	}
 }
 
@@ -350,7 +357,7 @@ cdw_rv_t cdw_read_disc_read_data_track(const char *fullpath)
 	cdw_processwin_wrefresh();
 
 	if (retval != CDW_OK) {
-		cdw_vdm ("ERROR: returning CDW_GEN_ERROR after reading errors\n");
+		cdw_vdm ("ERROR: returning CDW_ERROR after reading errors\n");
 	}
 	return retval;
 }
@@ -407,7 +414,7 @@ cdw_rv_t cdw_read_disc_actual_reading(track_t t, const char *fullpath)
 		   program starts to read one track from CD disc;
 		   "%d" is number of track */
 		cdw_logging_write(_("Track %d: "), t);
-		int rv = cdw_cdio_copy_sectors_to_file(t, fd);
+		int rv = cdw_cdio_copy_sectors_to_file(t, fd, -1);
 
 		/* can't write "success"/"failure" in these branches
 		   of conditions, since success of cdw_cdio_copy_sectors_to_file()
@@ -432,7 +439,7 @@ cdw_rv_t cdw_read_disc_actual_reading(track_t t, const char *fullpath)
 		} else {
 			/* if any error will occur for any track,
 			   then mark whole operation as unsuccessful */
-			retval = CDW_GEN_ERROR;
+			retval = CDW_ERROR;
 			cdw_vdm ("ERROR: track read error for track %d\n", t);
 
 			/* 2TRANS: this is string written to log file when
@@ -464,16 +471,11 @@ cdw_rv_t cdw_read_disc_actual_reading(track_t t, const char *fullpath)
    task.id must be set to CDW_TASK_READ_DISC or CDW_TASK_CHECK_MD5SUM.
    task.id can't be set to CDW_TASK_CHECK_MD5SUM
 
-   \param task - variable describing current task
-
    \return CDW_OK if all necessary preconditions are met
    \return CDW_NO if any of preconditions is not met
 */
-cdw_rv_t cdw_read_disc_check_preconditions(cdw_task_t *task)
+cdw_rv_t cdw_read_disc_check_preconditions(void)
 {
-	cdw_assert (task->id == CDW_TASK_READ_DISC || task->id == CDW_TASK_CHECK_MD5SUM,
-		    "ERROR: incorrect task id\n");
-
 	cdw_rv_t crv = cdw_read_disc_info();
 	if (crv != CDW_OK) {
 		/* 2TRANS: this is title of dialog window */
@@ -525,11 +527,6 @@ cdw_rv_t cdw_read_disc_check_preconditions(cdw_task_t *task)
 		return CDW_NO;
 	}
 
-	if (current_disc->type == CDW_CD_AUDIO) {
-		cdw_assert (task->id != CDW_TASK_CHECK_MD5SUM,
-			    "ERROR: incorrect task id: can't check md5sum of audio track\n");
-	}
-
 	return CDW_OK;
 }
 
@@ -540,7 +537,7 @@ char *cdw_read_disc_get_audio_file_name(void)
 {
 	/* 2TRANS: this is main part of name of audio tracks ripped from
 	   cd, no more than TRACK_CORE_NAME_LEN = 30 chars */
-	char *filename = strndup(_("track"), TRACK_CORE_NAME_LEN);
+	char *filename = strndup(_("track"), CDW_READ_DICS_TRACK_CORE_NAME_LEN);
 	if (filename == (char *) NULL) {
 		return (char *) NULL;
 	}
@@ -554,7 +551,7 @@ char *cdw_read_disc_get_audio_file_name(void)
 	/* 2TRANS: this is label in dialog window, below it is an input field */
 	data.prompt_message = _("Enter core of track file names (no more than 30 characters).");  /* message in input window */
 	data.input_type = CDW_NCURSES_INPUT_NONE;
-	data.chars_max = TRACK_CORE_NAME_LEN;    /* maximal length of string */
+	data.chars_max = CDW_READ_DICS_TRACK_CORE_NAME_LEN;    /* maximal length of string */
 	data.buffer = &filename;
 	/* error message displayed at the end in case of failure */
 	/* 2TRANS: this is error message displayed in dialog window
