@@ -2,11 +2,10 @@
 #define H_CDW_READ_DISC
 
 #include "main.h"
-#include "cdw_task.h"
 
 cdw_rv_t cdw_read_disc(void);
 
 /* used by md5sum code */
-cdw_rv_t cdw_read_disc_check_preconditions(cdw_task_t *task);
+cdw_rv_t cdw_read_disc_check_preconditions(void);
 
 #endif /* H_CDW_READ_DISC */
