@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2010 Kamil Ignacak
+ * Copyright (C) 2007 - 2011 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,12 +22,9 @@
 #include <stdlib.h>
 #include <stdbool.h>
 
-#include <ncursesw/ncurses.h>
-
 #include "cdw_widgets.h"
 #include "cdw_ncurses.h"
 #include "cdw_main_window.h"
-#include "cdw_config.h"
 #include "cdw_string.h"
 #include "cdw_erase_wizard.h"
 #include "cdw_debug.h"
@@ -45,48 +42,57 @@
 */
 
 
-extern cdw_config_t global_config;
 
 
 static void     cdw_erase_wizard_init(void);
 static cdw_rv_t cdw_erase_wizard_build(cdw_task_t *task, cdw_disc_t *disc);
 static cdw_rv_t cdw_erase_wizard_driver(void);
-static void     cdw_erase_wizard_driver_handle_enter(cdw_form_t *cdw_form, int fi, __attribute__((unused)) void *data);
 static void     cdw_erase_wizard_destroy(void);
+static cdw_rv_t cdw_erase_wizard_build_fields(cdw_task_t *task);
 
-static CDW_DROPDOWN *cdw_erase_wizard_make_erase_mode_dropdown(cdw_disc_type_t disc_type);
-static CDW_DROPDOWN *cdw_erase_wizard_make_speed_dropdown(cdw_disc_t *disc);
-static CDW_DROPDOWN *cdw_erase_wizard_make_format_mode_dropdown(cdw_disc_t *disc);
+static CDW_DROPDOWN *cdw_erase_wizard_make_erase_mode_dropdown(WINDOW *window, int begin_y, int begin_x, int width);
+static CDW_DROPDOWN *cdw_erase_wizard_make_speed_dropdown(WINDOW *window, int begin_y, int begin_x, int width);
+static CDW_DROPDOWN *cdw_erase_wizard_make_format_mode_dropdown(WINDOW *window, int begin_y, int begin_x, int width);
 
 static cdw_rv_t cdw_erase_wizard_initial_warning(cdw_disc_type_t disc_type);
 static void     cdw_erase_wizard_print_debug_info(cdw_task_t *task, bool format_mode_visible);
 
+static int cdw_erase_wizard_on_enter_on_cancel(cdw_form_t *cdw_form, void *dummy);
+static int cdw_erase_wizard_on_enter_on_erase(cdw_form_t *cdw_form, void *dummy);
 
 
 static struct {
 	cdw_form_t *cdw_form;
 
-	CDW_DROPDOWN *erase_speed_dropdown; /* fast / all (slow) */
-	CDW_DROPDOWN *erase_mode_dropdown;
 	/* dropdown showing possible formatting modes for DVD-RW:
 	   'don't change format' (default),
 	   'format to sequential',
 	   'format to restricted' */
-	CDW_DROPDOWN *format_mode_dropdown;
 	bool format_mode_dropdown_visible;
-
-	CDW_BUTTON *erase_button;
-	CDW_BUTTON *cancel_button;
 } wizard;
 
 
 
+static cdw_form_dropdown_maker_t dropdown_makers[] = {
+	cdw_erase_wizard_make_erase_mode_dropdown,
+	cdw_erase_wizard_make_speed_dropdown,
+	cdw_erase_wizard_make_format_mode_dropdown
+};
+
+
+
+static cdw_disc_t *local_disc;
+static cdw_task_t *local_task;
+
+
 /* constants for layout of elements in wizard window */
 #define first_col                  2 /* main message and some labels start in leftmost column */
+#define second_col                40
 #define top_label_row              2 /* main message is displayed on top of window */
 #define erase_mode_row             2
 #define erase_speed_row            5
 #define format_mode_row            8 /* label above of button invoking configuration window */
+#define eject_row                 11
 #define buttons_row               13 /* Erase and Cancel buttons are at the bottom of window */
 
 #define window_n_cols             52
@@ -100,16 +106,19 @@ static struct {
 
 
 
-#define CDW_ERASE_WIZARD_N_FIELDS 8 /* does not include ending NULL element */
+#define CDW_ERASE_WIZARD_N_FIELDS 10 /* does not include ending NULL element */
 static FIELD *wizard_fields[CDW_ERASE_WIZARD_N_FIELDS + 1];
 
 enum {
 	f_erase_mode_l = 0, /* all/fast */
-	f_erase_mode_i,
+	f_erase_mode_dd,
 	f_erase_speed_l,
-	f_erase_speed_i,
+	f_erase_speed_dd,
 	f_format_mode_l,
-	f_format_mode_i,
+	f_format_mode_dd,
+
+	f_eject_cb,
+	f_eject_l,
 
 	f_erase_button_i,
 	f_cancel_button_i
@@ -140,12 +149,15 @@ enum {
 */
 cdw_rv_t cdw_erase_wizard(cdw_task_t *task, cdw_disc_t *disc)
 {
-	cdw_assert (task->id == CDW_TASK_ERASE_DISC, "ERROR: incorrect task id, should be %d, is %d\n", CDW_TASK_ERASE_DISC, task->id);
+	cdw_assert (task->id == CDW_TASK_ERASE_DISC, "ERROR: incorrect task id, should be %d, is %lld\n", CDW_TASK_ERASE_DISC, task->id);
 	cdw_assert (disc->type_erasable == CDW_TRUE, "ERROR: disc is not erasable\n");
 
+	local_disc = disc;
+	local_task = task;
+
 	if (disc->type_erasable != CDW_TRUE) {
 		cdw_vdm ("ERROR: disc is not erasable\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	cdw_rv_t crv = cdw_erase_wizard_initial_warning(disc->type);
@@ -172,7 +184,7 @@ cdw_rv_t cdw_erase_wizard(cdw_task_t *task, cdw_disc_t *disc)
 				   /* 2TRANS: this is message in dialog window */
 				   _("Failed to create erasing wizard. Please restart cdw and try again."),
 				   CDW_BUTTONS_OK, CDW_COLORS_ERROR);
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	crv = cdw_erase_wizard_driver();
@@ -180,12 +192,16 @@ cdw_rv_t cdw_erase_wizard(cdw_task_t *task, cdw_disc_t *disc)
 		/* save values set in wizard; this works both when uses
 		   made any changes in wizard, or when only accepts current
 		   settings */
+		CDW_DROPDOWN *dd = (CDW_DROPDOWN *) NULL;
 		if (wizard.format_mode_dropdown_visible) {
-			task->erase.format_mode = cdw_dropdown_get_current_item_id(wizard.format_mode_dropdown);
+			dd = cdw_form_get_dropdown(wizard.cdw_form, f_format_mode_dd);
+			task->erase.format_mode = cdw_dropdown_get_current_item_id(dd);
 		}
-		task->erase.erase_speed = cdw_dropdown_get_current_item_id(wizard.erase_speed_dropdown);
-		task->erase.erase_mode = cdw_dropdown_get_current_item_id(wizard.erase_mode_dropdown);
-		global_config.erase_mode = task->erase.erase_mode;
+		dd = cdw_form_get_dropdown(wizard.cdw_form, f_erase_speed_dd);
+		task->erase.speed_id = cdw_dropdown_get_current_item_id(dd);
+		dd = cdw_form_get_dropdown(wizard.cdw_form, f_erase_mode_dd);
+		task->erase.erase_mode = cdw_dropdown_get_current_item_id(dd);
+		task->erase.eject = cdw_form_get_checkbox_state(wizard.cdw_form, f_eject_cb);
 
 		cdw_erase_wizard_print_debug_info(task, wizard.format_mode_dropdown_visible);
 
@@ -210,8 +226,8 @@ cdw_rv_t cdw_erase_wizard_driver(void)
 	int fi = f_cancel_button_i; /* initial focus on "cancel" button */
 	int key = 'a'; /* safe initial value */
 
-	while (key != CDW_KEY_ESCAPE) {
-		key = cdw_form_driver(wizard.cdw_form, fi, &global_config);
+	while (key != CDW_KEY_ESCAPE && key != 'q' && key != 'Q') {
+		key = cdw_form_driver(wizard.cdw_form, fi);
 
 		/* in this wizard driver we are interested only in
 		   ENTER being pressed on either "erase" or "cancel"
@@ -243,23 +259,13 @@ cdw_rv_t cdw_erase_wizard_driver(void)
 
 /**
    \brief Initialize wizard data structures
-
-   \param task
-   \param disc
 */
 void cdw_erase_wizard_init(void)
 {
 	cdw_assert (LINES > window_n_lines, "ERROR: LINES is too small: %d\n", LINES);
 	cdw_assert (COLS > window_n_cols, "ERROR: COLS is too small: %d\n", COLS);
 
-	wizard.erase_mode_dropdown = (CDW_DROPDOWN *) NULL;
-	wizard.erase_speed_dropdown = (CDW_DROPDOWN *) NULL;
-	wizard.format_mode_dropdown = (CDW_DROPDOWN *) NULL;
 	wizard.format_mode_dropdown_visible = false;
-
-	wizard.erase_button = (CDW_BUTTON *) NULL;
-	wizard.cancel_button = (CDW_BUTTON *) NULL;
-
 	wizard.format_mode_dropdown_visible = false;
 
 	return;
@@ -269,8 +275,6 @@ void cdw_erase_wizard_init(void)
 
 
 
-
-
 /**
    \brief Cleanup function for erase wizard
 
@@ -280,36 +284,16 @@ void cdw_erase_wizard_init(void)
 */
 void cdw_erase_wizard_destroy(void)
 {
-	if (wizard.format_mode_dropdown_visible) {
-		cdw_dropdown_delete(&wizard.format_mode_dropdown);
-		cdw_assert (wizard.format_mode_dropdown == (CDW_DROPDOWN *) NULL,
-			    "ERROR: delete() didn't set pointer to NULL\n");
+	if (wizard.cdw_form->subwindow != (WINDOW *) NULL) {
+		delwin(wizard.cdw_form->subwindow);
+		wizard.cdw_form->subwindow = (WINDOW *) NULL;
 	}
 
-	cdw_dropdown_delete(&wizard.erase_mode_dropdown);
-	cdw_dropdown_delete(&wizard.erase_speed_dropdown);
-	cdw_assert (wizard.erase_mode_dropdown == (CDW_DROPDOWN *) NULL,
-		    "ERROR: delete() didn't set pointer to NULL\n");
-	cdw_assert (wizard.erase_speed_dropdown == (CDW_DROPDOWN *) NULL,
-		    "ERROR: delete() didn't set pointer to NULL\n");
-
-	cdw_button_delete(&wizard.erase_button);
-	cdw_button_delete(&wizard.cancel_button);
-	cdw_assert (wizard.erase_button == (CDW_BUTTON *) NULL,
-		    "ERROR: delete() didn't set button pointer to NULL\n");
-	cdw_assert (wizard.cancel_button == (CDW_BUTTON *) NULL,
-		    "ERROR: delete() didn't set button pointer to NULL\n");
-
 	if (wizard.cdw_form->window != (WINDOW *) NULL) {
 		delwin(wizard.cdw_form->window);
 		wizard.cdw_form->window = (WINDOW *) NULL;
 	}
 
-	if (wizard.cdw_form->subwindow != (WINDOW *) NULL) {
-		delwin(wizard.cdw_form->subwindow);
-		wizard.cdw_form->subwindow = (WINDOW *) NULL;
-	}
-
 	if (wizard.cdw_form != (cdw_form_t *) NULL) {
 		cdw_form_delete_form_objects(wizard.cdw_form);
 		cdw_form_delete(&(wizard.cdw_form));
@@ -378,7 +362,7 @@ cdw_rv_t cdw_erase_wizard_initial_warning(cdw_disc_type_t disc_type)
    \param disc - variable describing disc currently in drive
 
    \return CDW_OK on success
-   \return CDW_GEN_ERROR on failure
+   \return CDW_ERROR on failure
 */
 cdw_rv_t cdw_erase_wizard_build(cdw_task_t *task, cdw_disc_t *disc)
 {
@@ -394,11 +378,9 @@ cdw_rv_t cdw_erase_wizard_build(cdw_task_t *task, cdw_disc_t *disc)
 	wizard.cdw_form = cdw_form_new(CDW_ERASE_WIZARD_N_FIELDS);
 	if (wizard.cdw_form == (cdw_form_t *) NULL) {
 		cdw_vdm ("ERROR: failed to create cdw form\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
-	wizard.cdw_form->fields = wizard_fields;
-
 	int begin_y = ((LINES - window_n_lines) / 2) - 2;
 	int begin_x = (COLS - window_n_cols) / 2;
 	wizard.cdw_form->window = cdw_ncurses_window_new((WINDOW *) NULL,
@@ -413,7 +395,7 @@ cdw_rv_t cdw_erase_wizard_build(cdw_task_t *task, cdw_disc_t *disc)
 							 _("Use 'Tab' key to move"));
 	if (wizard.cdw_form->window == (WINDOW *) NULL) {
 		cdw_vdm ("ERROR: failed to create window\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	wizard.cdw_form->subwindow = cdw_ncurses_window_new(wizard.cdw_form->window,
@@ -423,7 +405,7 @@ cdw_rv_t cdw_erase_wizard_build(cdw_task_t *task, cdw_disc_t *disc)
 
 	if (wizard.cdw_form->subwindow == (WINDOW *) NULL) {
 		cdw_vdm ("ERROR: failed to create subwindow\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	/* *** main message in wizard window *** */
@@ -432,105 +414,32 @@ cdw_rv_t cdw_erase_wizard_build(cdw_task_t *task, cdw_disc_t *disc)
 		     be erasing "%s" disc, where "%s" is DVD or CD */
 		  _("Select parameters for erasing %s disc"), disc->simple_type_label);
 
+	cdw_erase_wizard_build_fields(task);
 
-	cdw_form_descr_t descr[] = {
-		/*    widget type       begin_y               begin_x         n_cols                   n_lines    field enum         data1                        data2 */
-
-		/* 2TRANS: this is label in erase wizard */
-		{ CDW_WIDGET_LABEL,     erase_mode_row,       first_col,      label_n_cols,                1,    f_erase_mode_l,     _("Select mode of erasing:"),   0 },
-		{ CDW_WIDGET_DROPDOWN,  erase_mode_row + 1,   first_col + 1,  wide_dropdown_n_cols - 2,    1,    f_erase_mode_i,     (void *) NULL,                  0 },
-
-		/* 2TRANS: this is label in erase wizard */
-		{ CDW_WIDGET_LABEL,     erase_speed_row,      first_col,      label_n_cols,                1,    f_erase_speed_l,    _("Select speed of erasing:"),  0 },
-		{ CDW_WIDGET_DROPDOWN,  erase_speed_row + 1,  first_col + 1,  narrow_dropdown_n_cols - 2,  1,    f_erase_speed_i,    (void *) NULL,                  0 },
-
-		/* 2TRANS: this is label in erase wizard, it's about target
-		   format for a DVD-RW disc */
-		{ CDW_WIDGET_LABEL,     format_mode_row,      first_col,      label_n_cols,                1,    f_format_mode_l,    _("Select DVD-RW format:"),     0 },
-		{ CDW_WIDGET_DROPDOWN,  format_mode_row + 1,  first_col + 1,  wide_dropdown_n_cols - 2,    1,    f_format_mode_i,    (void *) NULL,                  0 },
-
-		{ CDW_WIDGET_BUTTON,    buttons_row,          3,              2,                           1,    f_erase_button_i,   (void *) NULL,                  0 },
-		{ CDW_WIDGET_BUTTON,    buttons_row,          15,             2,                           1,    f_cancel_button_i,  (void *) NULL,                  0 },
-
-		/* guard */
-		{ -1,                   0,                    0,              0,                           0,    0,                  (void *) NULL,                 0 }};
-
-	/* this function also sets which dropdown item is selected at the beginning */
-	wizard.erase_mode_dropdown = cdw_erase_wizard_make_erase_mode_dropdown(disc->type);
-	if (wizard.erase_mode_dropdown == (CDW_DROPDOWN *) NULL) {
-		cdw_vdm ("ERROR: failed to create session mode dropdown\n");
-		return CDW_GEN_ERROR;
-	} else {
-		descr[f_erase_mode_i].data1 = (void *) wizard.erase_mode_dropdown;
-	}
-
-	/* this function also sets which dropdown item is selected at the beginning */
-	wizard.erase_speed_dropdown = cdw_erase_wizard_make_speed_dropdown(disc);
-	if (wizard.erase_speed_dropdown == (CDW_DROPDOWN *) NULL) {
-		cdw_vdm ("ERROR: failed to create speed dropdown\n");
-		return CDW_GEN_ERROR;
-	} else {
-		descr[f_erase_speed_i].data1 = (void *) wizard.erase_speed_dropdown;
-	}
-
-	if (wizard.format_mode_dropdown_visible) {
-		/* this function also sets which dropdown item is selected at the beginning */
-		wizard.format_mode_dropdown = cdw_erase_wizard_make_format_mode_dropdown(disc);
-		if (wizard.format_mode_dropdown == (CDW_DROPDOWN *) NULL) {
-			cdw_vdm ("ERROR: failed to create session mode dropdown\n");
-			return CDW_GEN_ERROR;
-		} else {
-			descr[f_format_mode_i].data1 = (void *) wizard.format_mode_dropdown;
-		}
-	}
-
-
-	/* *** buttons at the bottom *** */
-	wizard.erase_button = cdw_button_new(wizard.cdw_form->subwindow, buttons_row, 3,
-					     /* 2TRANS: button label, it refers to erasing optical disc */
-					     _("Erase"), CDW_COLORS_DIALOG);
-	descr[f_erase_button_i].data1 = (void *) wizard.erase_button;
-
-	wizard.cancel_button = cdw_button_new(wizard.cdw_form->subwindow, buttons_row, 15,
-					      /* 2TRANS: button label */
-					      _("Cancel"), CDW_COLORS_DIALOG);
-	descr[f_cancel_button_i].data1 = (void *) wizard.cancel_button;
-
-	wizard.cdw_form->n_fields = CDW_ERASE_WIZARD_N_FIELDS;
-	/* the function adds guard at the end of fields */
-	cdw_rv_t crv = cdw_form_description_to_fields(descr, wizard.cdw_form);
-	if (crv != CDW_OK) {
-		cdw_vdm ("ERROR: failed to convert form description to form\n");
-		return CDW_GEN_ERROR;
-	}
-
-	wizard.cdw_form->form = cdw_form_new_form(wizard.cdw_form->window,
-						  wizard.cdw_form->subwindow,
-						  wizard.cdw_form->fields);
+	wizard.cdw_form->form = cdw_ncurses_new_form(wizard.cdw_form->window,
+						     wizard.cdw_form->subwindow,
+						     wizard.cdw_form->fields);
 	if (wizard.cdw_form->form == (FORM *) NULL) {
 		cdw_vdm ("ERROR: failed to create form\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 
 	/* current items are set in functions creating respective
 	   dropdowns; now just display them */
-	cdw_dropdown_display_current_item(wizard.erase_speed_dropdown);
-	cdw_dropdown_display_current_item(wizard.erase_mode_dropdown);
-	if (wizard.format_mode_dropdown_visible) {
-		cdw_dropdown_display_current_item(wizard.format_mode_dropdown);
-	} else {
-		field_opts_off(wizard.cdw_form->fields[f_format_mode_i], O_VISIBLE);
+	if (!wizard.format_mode_dropdown_visible) {
+		CDW_DROPDOWN *dd = cdw_form_get_dropdown(wizard.cdw_form, f_format_mode_dd);
+		cdw_dropdown_make_invisible(dd);
+		field_opts_off(wizard.cdw_form->fields[f_format_mode_dd], O_VISIBLE);
 		field_opts_off(wizard.cdw_form->fields[f_format_mode_l], O_VISIBLE);
 	}
 
-	cdw_button_unfocus(wizard.erase_button);
-	cdw_button_unfocus(wizard.cancel_button);
+	cdw_form_redraw_widgets(wizard.cdw_form);
 
-	cdw_form_add_return_char(wizard.cdw_form, CDW_KEY_ENTER);
-	cdw_form_add_return_char(wizard.cdw_form, CDW_KEY_ESCAPE);
+	cdw_form_add_return_chars(wizard.cdw_form, CDW_KEY_ENTER, CDW_KEY_ESCAPE, 'q', 'Q', 0);
 
-	wizard.cdw_form->handle_enter = cdw_erase_wizard_driver_handle_enter;
+	cdw_form_set_function(wizard.cdw_form, f_erase_button_i, cdw_erase_wizard_on_enter_on_erase);
+	cdw_form_set_function(wizard.cdw_form, f_cancel_button_i, cdw_erase_wizard_on_enter_on_cancel);
 
 	wrefresh(wizard.cdw_form->subwindow);
 	wrefresh(wizard.cdw_form->window);
@@ -542,55 +451,56 @@ cdw_rv_t cdw_erase_wizard_build(cdw_task_t *task, cdw_disc_t *disc)
 
 
 
-/**
-   \brief Check which element has currently keyboard focus, and handle Enter key for this element
-
-   Function checks which element in wizard window (identified by field
-   index \p fi) has currently keyboard focus and executes related code that
-   performs action appropriate for given element.
-
-   "elements" are buttons and dropdowns. Checkboxes don't "recognize" Enter key.
-
-   \param cdw_form - form in which elements are placed
-   \param fi - field index, index of form field that has keyboard focus
-   \param data - pointer to additional data that may be used by some elements
-*/
-void cdw_erase_wizard_driver_handle_enter(cdw_form_t *cdw_form, int fi, __attribute__((unused)) void *data)
+cdw_rv_t cdw_erase_wizard_build_fields(cdw_task_t *task)
 {
-	if (cdw_form->is_dropdown_field_index(cdw_form, fi)) {
-		CDW_DROPDOWN *dropdown = cdw_form->get_dropdown_from_form(cdw_form, fi);
-		if (dropdown != (CDW_DROPDOWN *) NULL) {
-			cdw_rv_t crv = cdw_dropdown_driver(dropdown);
-			if (crv == CDW_OK) {
-				const char *label = cdw_dropdown_get_current_item_label(dropdown);
-				FIELD **fields = form_fields(cdw_form->form);
-				/* setting buffer makes field to display
-				   string, but slightly moved to left */
-				set_field_buffer(*(fields + fi), 0, label);
+	cdw_form_descr_t descr[] = {
+		/*    widget type       begin_y               begin_x         n_cols                   n_lines    field enum         data1                        data2 */
 
-				/* call to cdw_dropdown_focus() fixes it */
-				cdw_dropdown_focus(dropdown);
+		/* 2TRANS: this is label in erase wizard */
+		{ CDW_WIDGET_LABEL,      erase_mode_row,       first_col,      label_n_cols,                1,    f_erase_mode_l,     _("Mode of erasing:"),   0 },
+		{ CDW_WIDGET_DROPDOWN,   erase_mode_row + 1,   first_col + 1,  wide_dropdown_n_cols - 2,    1,    f_erase_mode_dd,    dropdown_makers,                0 },
 
-				form_driver(cdw_form->form, REQ_VALIDATION);
-			} /* else field buffer is not updated and nothing changes */
-		}
+		/* 2TRANS: this is label in erase wizard */
+		{ CDW_WIDGET_LABEL,      erase_speed_row,      first_col,      label_n_cols,                1,    f_erase_speed_l,    _("Speed of erasing:") ,  0 },
+		{ CDW_WIDGET_DROPDOWN,   erase_speed_row + 1,  first_col + 1,  narrow_dropdown_n_cols - 2,  1,    f_erase_speed_dd,    dropdown_makers,                1 },
+
+		/* 2TRANS: this is label in erase wizard, it's about target
+		   format for a DVD-RW disc */
+		{ CDW_WIDGET_LABEL,      format_mode_row,      first_col,      label_n_cols,                1,    f_format_mode_l,    _("Format of DVD-RW:"),      0 },
+		{ CDW_WIDGET_DROPDOWN,   format_mode_row + 1,  first_col + 1,  wide_dropdown_n_cols - 2,    1,    f_format_mode_dd,    dropdown_makers,                2 },
+
+		{ CDW_WIDGET_CHECKBOX,   eject_row,            first_col + 2,  1,                           1,    f_eject_cb,         (void *) NULL,  task->erase.eject ? 1 : 0 },
+		{ CDW_WIDGET_LABEL,      eject_row,            first_col + 5,  label_n_cols - 5,            1,    f_eject_l,          _("Eject tray after erasing") ,  0 },
+
+		/* 2TRANS: button label, it refers to erasing optical disc */
+		{ CDW_WIDGET_BUTTON,     buttons_row,          3,              2,                           1,    f_erase_button_i,   _("Erase"),      CDW_COLORS_DIALOG },
+		/* 2TRANS: button label */
+		{ CDW_WIDGET_BUTTON,     buttons_row,          15,             2,                           1,    f_cancel_button_i,  _("Cancel"),     CDW_COLORS_DIALOG },
+
+		/* guard */
+		{ -1,                    0,                    0,              0,                           0,    0,                  (void *) NULL,                   0 }};
+
+
+	wizard.cdw_form->n_fields = CDW_ERASE_WIZARD_N_FIELDS;
+	wizard.cdw_form->fields = wizard_fields;
+	/* the function adds guard at the end of fields */
+	cdw_rv_t crv = cdw_form_description_to_fields(descr, wizard.cdw_form);
+	if (crv != CDW_OK) {
+		cdw_vdm ("ERROR: failed to convert form description to form\n");
+		return CDW_ERROR;
 	} else {
-		;
+		return CDW_OK;
 	}
-
-	return;
 }
 
 
 
 
 
-CDW_DROPDOWN *cdw_erase_wizard_make_erase_mode_dropdown(cdw_disc_type_t disc_type)
+CDW_DROPDOWN *cdw_erase_wizard_make_erase_mode_dropdown(WINDOW *window, int begin_y, int begin_x, int width)
 {
-	CDW_DROPDOWN *dropdown = cdw_dropdown_new(wizard.cdw_form->subwindow,
-						  erase_mode_row + 1, first_col,
-						  wide_dropdown_n_cols, /* width */
-						  2,                    /* n_items_max */
+	CDW_DROPDOWN *dropdown = cdw_dropdown_new(window, begin_y, begin_x, width,
+						  2, /* n_items_max */
 						  CDW_COLORS_DIALOG);
 	if (dropdown == (CDW_DROPDOWN *) NULL) {
 		cdw_vdm ("ERROR: failed to create new dropdown\n");
@@ -598,7 +508,7 @@ CDW_DROPDOWN *cdw_erase_wizard_make_erase_mode_dropdown(cdw_disc_type_t disc_typ
 	}
 
 
-	if (disc_type == CDW_DVD_RWP) {
+	if (local_disc->type == CDW_DVD_RWP) {
 		/* 2TRANS: this string means "perform erasing in fast mode" */
 		cdw_rv_t crv = cdw_dropdown_add_item(dropdown, CDW_ERASE_MODE_FAST, _("Erase in fast mode"));
 		if (crv != CDW_OK) {
@@ -626,7 +536,7 @@ CDW_DROPDOWN *cdw_erase_wizard_make_erase_mode_dropdown(cdw_disc_type_t disc_typ
 		cdw_dropdown_delete(&dropdown);
 		return (CDW_DROPDOWN *) NULL;
 	} else {
-		cdw_dropdown_set_current_item_by_id(dropdown, global_config.erase_mode);
+		cdw_dropdown_set_current_item_by_id(dropdown, local_task->erase.erase_mode);
 		return dropdown;
 	}
 }
@@ -635,12 +545,10 @@ CDW_DROPDOWN *cdw_erase_wizard_make_erase_mode_dropdown(cdw_disc_type_t disc_typ
 
 
 
-CDW_DROPDOWN *cdw_erase_wizard_make_speed_dropdown(cdw_disc_t *disc)
+CDW_DROPDOWN *cdw_erase_wizard_make_speed_dropdown(WINDOW *window, int begin_y, int begin_x, int width)
 {
-	CDW_DROPDOWN *dropdown = cdw_dropdown_new(wizard.cdw_form->subwindow,
-						  erase_speed_row + 1, first_col,
-						  narrow_dropdown_n_cols, /* width */
-						  disc->write_speeds.n_speeds,   /* n_items_max */
+	CDW_DROPDOWN *dropdown = cdw_dropdown_new(window, begin_y, begin_x, width,
+						  local_disc->write_speeds.n_speeds,   /* n_items_max */
 						  CDW_COLORS_DIALOG);
 	if (dropdown == (CDW_DROPDOWN *) NULL) {
 		cdw_vdm ("ERROR: failed to create new dropdown\n");
@@ -648,11 +556,11 @@ CDW_DROPDOWN *cdw_erase_wizard_make_speed_dropdown(cdw_disc_t *disc)
 	}
 
 	char label[5];
-	for (int i = 0; i < disc->write_speeds.n_speeds; i++) {
-		snprintf(label, 4 + 1, "%d", disc->write_speeds.speeds[i]);
-		cdw_rv_t crv = cdw_dropdown_add_item(dropdown, disc->write_speeds.speeds[i], label);
+	for (int i = 0; i < local_disc->write_speeds.n_speeds; i++) {
+		snprintf(label, 4 + 1, "%d", local_disc->write_speeds.speeds[i]);
+		cdw_rv_t crv = cdw_dropdown_add_item(dropdown, local_disc->write_speeds.speeds[i], label);
 		if (crv != CDW_OK) {
-			cdw_vdm ("ERROR: failed to create dropdown label #%d of value %d\n", i, disc->write_speeds.speeds[i]);
+			cdw_vdm ("ERROR: failed to create dropdown label #%d of value %d\n", i, local_disc->write_speeds.speeds[i]);
 			cdw_dropdown_delete(&dropdown);
 			return (CDW_DROPDOWN *) NULL;
 		}
@@ -665,10 +573,7 @@ CDW_DROPDOWN *cdw_erase_wizard_make_speed_dropdown(cdw_disc_t *disc)
 		cdw_dropdown_delete(&dropdown);
 		return (CDW_DROPDOWN *) NULL;
 	} else {
-		/* the function returns index of speed based on wizard.speed_range;
-		   cdw_disc module guarantees it to be >= 0 */
-		int speed_index = cdw_disc_get_initial_write_speed_index(disc);
-		cdw_dropdown_set_current_item_by_ind(dropdown, speed_index);
+		cdw_dropdown_set_current_item_by_id(dropdown, local_task->erase.speed_id);
 		return dropdown;
 	}
 }
@@ -677,7 +582,7 @@ CDW_DROPDOWN *cdw_erase_wizard_make_speed_dropdown(cdw_disc_t *disc)
 
 
 
-CDW_DROPDOWN *cdw_erase_wizard_make_format_mode_dropdown(cdw_disc_t *disc)
+CDW_DROPDOWN *cdw_erase_wizard_make_format_mode_dropdown(WINDOW *window, int begin_y, int begin_x, int width)
 {
 	/* remember that gettext() result strings must not be free()d */
 
@@ -696,9 +601,7 @@ CDW_DROPDOWN *cdw_erase_wizard_make_format_mode_dropdown(cdw_disc_t *disc)
 		return (CDW_DROPDOWN *) NULL;
 	}
 
-	CDW_DROPDOWN *dropdown = cdw_dropdown_new(wizard.cdw_form->subwindow,
-						  format_mode_row + 1, first_col,
-						  wide_dropdown_n_cols, /* width of dropdown */
+	CDW_DROPDOWN *dropdown = cdw_dropdown_new(window, begin_y, begin_x, width,
 						  3,                    /* n_items_max */
 						  CDW_COLORS_DIALOG);
 	if (dropdown == (CDW_DROPDOWN *) NULL) {
@@ -706,14 +609,14 @@ CDW_DROPDOWN *cdw_erase_wizard_make_format_mode_dropdown(cdw_disc_t *disc)
 		return (CDW_DROPDOWN *) NULL;
 	}
 
-	cdw_rv_t crv1 = CDW_GEN_ERROR;
-	cdw_rv_t crv2 = CDW_GEN_ERROR;
-	if (disc->type == CDW_DVD_RW_SEQ) {
+	cdw_rv_t crv1 = CDW_ERROR;
+	cdw_rv_t crv2 = CDW_ERROR;
+	if (local_disc->type == CDW_DVD_RW_SEQ) {
 		crv1 = cdw_dropdown_add_item(dropdown, CDW_ERASE_DVD_RW_FORMAT_SEQ, dont_change);
 		crv2 = cdw_dropdown_add_item(dropdown, CDW_ERASE_DVD_RW_FORMAT_RES, res);
 
 
-	} else if (disc->type == CDW_DVD_RW_RES) {
+	} else if (local_disc->type == CDW_DVD_RW_RES) {
 		crv1 = cdw_dropdown_add_item(dropdown, CDW_ERASE_DVD_RW_FORMAT_RES, dont_change);
 		crv2 = cdw_dropdown_add_item(dropdown, CDW_ERASE_DVD_RW_FORMAT_SEQ, seq);
 
@@ -753,8 +656,9 @@ void cdw_erase_wizard_print_debug_info(cdw_task_t *task, bool format_mode_dropdo
 	/* for printing debug information we use values saved in "task"
 	   variable, not in "wizard" struct, because we want to check what
 	   really was saved in "task" */
-	cdw_vdm ("INFO:              selected speed = \"%d\"\n", task->erase.erase_speed);
+	cdw_vdm ("INFO:              selected speed = \"%lld\"\n", task->erase.speed_id);
 	cdw_vdm ("INFO:         selected erase mode = \"%s\"\n", task->erase.erase_mode == CDW_ERASE_MODE_FAST ? "fast" : "all");
+	cdw_vdm ("INFO:    eject tray after erasing = \"%s\"\n", task->erase.eject ? "true" : "false");
 
 	if (format_mode_dropdown_visible) {
 		if (task->erase.format_mode == CDW_ERASE_DVD_RW_FORMAT_SEQ) {
@@ -762,7 +666,7 @@ void cdw_erase_wizard_print_debug_info(cdw_task_t *task, bool format_mode_dropdo
 		} else if (task->erase.format_mode == CDW_ERASE_DVD_RW_FORMAT_RES) {
 			cdw_vdm ("INFO: selected DVD-RW format mode = \"Restricted overwrite\"\n");
 		} else {
-			cdw_vdm ("ERROR: incorrect format mode = %d\n", task->erase.format_mode);
+			cdw_vdm ("ERROR: incorrect format mode = %lld\n", task->erase.format_mode);
 		}
 	} else {
 		cdw_vdm ("INFO: DVD-RW format mode dropdown is NOT visible\n");
@@ -772,3 +676,17 @@ void cdw_erase_wizard_print_debug_info(cdw_task_t *task, bool format_mode_dropdo
 
 
 
+
+
+int cdw_erase_wizard_on_enter_on_cancel(__attribute__((unused)) cdw_form_t *cdw_form, __attribute__((unused)) void *dummy)
+{
+	return CDW_KEY_ESCAPE;
+}
+
+
+
+int cdw_erase_wizard_on_enter_on_erase(__attribute__((unused)) cdw_form_t *cdw_form, __attribute__((unused)) void *dummy)
+{
+	return CDW_KEY_ENTER;
+}
+
