@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2010 Kamil Ignacak
+ * Copyright (C) 2007 - 2011 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,6 +20,8 @@
 #include <string.h>
 #include <stdlib.h>
 #include <errno.h>
+#include <stdarg.h>
+#include <ctype.h> /* isprint() */
 
 #include "cdw_debug.h"
 #include "cdw_form.h"
@@ -30,20 +32,16 @@ static void cdw_form_driver_focus_off(cdw_form_t *cdw_form, int fi);
 static void cdw_form_driver_focus_on(cdw_form_t *cdw_form, int fi);
 static void cdw_form_driver_toggle_focus(cdw_form_t *cdw_form, int fi, bool off);
 static void cdw_form_driver_toggle_checkbox(cdw_form_t *cdw_form, int fi);
-static CDW_DROPDOWN *cdw_form_driver_get_dropdown_from_form(cdw_form_t *cdw_form, int fi);
-static CDW_BUTTON   *cdw_form_driver_get_button_from_form(cdw_form_t *cdw_form, int fi);
 static bool cdw_form_is_return_key(cdw_form_t *form, int key);
+static int  cdw_form_handle_enter(cdw_form_t *cdw_form, int fi);
+
+static bool cdw_form_is_checkbox_field_index(cdw_form_t *cdw_form, int fi);
+static bool cdw_form_is_dropdown_field_index(cdw_form_t *cdw_form, int fi);
+static bool cdw_form_is_button_field_index(cdw_form_t *cdw_form, int fi);
+static bool cdw_form_is_input_field_index(cdw_form_t *cdw_form, int fi);
 
-static bool is_checkbox_field_index(cdw_form_t *cdw_form, int fi);
-static bool is_dropdown_field_index(cdw_form_t *cdw_form, int fi);
-static bool is_button_field_index(cdw_form_t *cdw_form, int fi);
-static bool is_input_field_index(cdw_form_t *cdw_form, int fi);
-
-static void cdw_form_driver_go_to_field(FORM *form, int fi, int n_fields);
 
 
-/* used by cdw_form_get_field_string() */
-static char empty_string[] = "";
 
 
 /**
@@ -60,7 +58,7 @@ cdw_form_t *cdw_form_new(int n_fields)
 	}
 	cdw_form->n_fields = n_fields;
 
-	cdw_form->field_widget_types = malloc((size_t) cdw_form->n_fields * sizeof (int));
+	cdw_form->field_widget_types = malloc((size_t) cdw_form->n_fields * sizeof (cdw_id_t));
 	if (cdw_form->field_widget_types ==  NULL) {
 		cdw_vdm ("ERROR: failed to allocate fields widget ids table\n");
 		free(cdw_form);
@@ -79,18 +77,8 @@ cdw_form_t *cdw_form_new(int n_fields)
 		cdw_form->return_keys[i] = 0;
 	}
 
-	cdw_form->handle_enter = (void (*)(cdw_form_t *, int, void *)) NULL;
-
-	cdw_form->get_dropdown_from_form = cdw_form_driver_get_dropdown_from_form;
-	cdw_form->get_button_from_form = cdw_form_driver_get_button_from_form;
-	cdw_form->focus_off = cdw_form_driver_focus_off;
-        cdw_form->focus_on = cdw_form_driver_focus_on;
-	cdw_form->toggle_focus = cdw_form_driver_toggle_focus;
-	cdw_form->toggle_checkbox = cdw_form_driver_toggle_checkbox;
-	cdw_form->is_checkbox_field_index = is_checkbox_field_index;
-	cdw_form->is_dropdown_field_index = is_dropdown_field_index;
-	cdw_form->is_button_field_index = is_button_field_index;
-	cdw_form->is_input_field_index = is_input_field_index;
+	/* may be overwritten by form-specific handler */
+	cdw_form->handle_enter = cdw_form_handle_enter;
 
 	return cdw_form;
 }
@@ -109,12 +97,10 @@ void cdw_form_delete(cdw_form_t **cdw_form)
 		cdw_vdm ("WARNING: passing NULL form to function\n");
 		return;
 	}
-	cdw_assert ((*cdw_form)->window == (WINDOW *) NULL, "ERROR: cdw_form->window is not deallocated\n");
-	cdw_assert ((*cdw_form)->subwindow == (WINDOW *) NULL, "ERROR: cdw_form->window is not deallocated\n");
 
-	if ((*cdw_form)->field_widget_types != (int *) NULL) {
+	if ((*cdw_form)->field_widget_types != (cdw_id_t *) NULL) {
 		free((*cdw_form)->field_widget_types);
-		(*cdw_form)->field_widget_types = (int *) NULL;
+		(*cdw_form)->field_widget_types = (cdw_id_t *) NULL;
 	}
 
 	free(*cdw_form);
@@ -153,6 +139,21 @@ void cdw_form_delete_form_objects(cdw_form_t *cdw_form)
 
 	for (int i = 0; i < cdw_form->n_fields; i++) {
 		if (cdw_form->fields[i] != (FIELD *) NULL) {
+			if (cdw_form->field_widget_types[i] == CDW_WIDGET_CHECKBOX) {
+				CDW_CHECKBOX *cb = (CDW_CHECKBOX *) field_userptr(cdw_form->fields[i]);
+				cdw_assert (cb != (CDW_CHECKBOX *) NULL, "ERROR: field #%d is checkbox, but the pointer is null\n", i);
+				cdw_checkbox_free(cb);
+			} else if (cdw_form->field_widget_types[i] == CDW_WIDGET_BUTTON) {
+				CDW_BUTTON *b = (CDW_BUTTON *) field_userptr(cdw_form->fields[i]);
+				cdw_assert (b != (CDW_BUTTON *) NULL, "ERROR: field #%d is button, but the pointer is null\n", i);
+				cdw_button_free(b);
+			} else if (cdw_form->field_widget_types[i] == CDW_WIDGET_DROPDOWN) {
+				CDW_DROPDOWN *d = (CDW_DROPDOWN *) field_userptr(cdw_form->fields[i]);
+				cdw_assert (d != (CDW_DROPDOWN *) NULL, "ERROR: field #%d is dropdown, but the pointer is null\n", i);
+				cdw_dropdown_free(d);
+			} else {
+				;
+			}
 			free_field(cdw_form->fields[i]);
 			cdw_form->fields[i] = (FIELD *) NULL;
 		}
@@ -165,14 +166,14 @@ void cdw_form_delete_form_objects(cdw_form_t *cdw_form)
 
 
 
-int cdw_form_driver(cdw_form_t *cdw_form, int initial_fi, void *data)
+int cdw_form_driver(cdw_form_t *cdw_form, int initial_fi)
 {
-	cdw_assert (cdw_form->handle_enter != (void (*)(cdw_form_t *, int, void *)) NULL,
-		    "ERROR: this form has no \"handle enter\" set up\n");
+	cdw_assert (cdw_form->handle_enter != (int (*)(cdw_form_t *, int)) NULL,
+		    "ERROR: this form has no Enter handler set up\n");
 
-	cdw_form_driver_go_to_field(cdw_form->form, initial_fi, cdw_form->n_fields);
+	cdw_form_driver_go_to_field(cdw_form, initial_fi);
 	initial_fi = field_index(current_field(cdw_form->form));
-	cdw_form->focus_on(cdw_form, initial_fi);
+	cdw_form_driver_focus_on(cdw_form, initial_fi);
 
 	form_driver(cdw_form->form, REQ_END_LINE);
 	wrefresh(cdw_form->subwindow);
@@ -184,7 +185,7 @@ int cdw_form_driver(cdw_form_t *cdw_form, int initial_fi, void *data)
 
 		if (cdw_form_is_movement_key(key)) {
 			int fi = field_index(current_field(cdw_form->form));
-			cdw_form->focus_off(cdw_form, fi);
+			cdw_form_driver_focus_off(cdw_form, fi);
 		}
 
 		if (key == KEY_HOME) {
@@ -207,14 +208,16 @@ int cdw_form_driver(cdw_form_t *cdw_form, int initial_fi, void *data)
 			form_driver(cdw_form->form, REQ_DEL_CHAR);
 		} else if (key == CDW_KEY_ENTER) {
 			int fi = field_index(current_field(cdw_form->form));
-			cdw_form->handle_enter(cdw_form, fi, data);
+			/* the function may return dummy key 'a' which means
+			   to continue in driver loop */
+			key = cdw_form->handle_enter(cdw_form, fi);
 		} else {
 			int fi = field_index(current_field(cdw_form->form));
-			if (cdw_form->is_checkbox_field_index(cdw_form, fi)) {
+			if (cdw_form_is_checkbox_field_index(cdw_form, fi)) {
 				if (key == ' ' || key == 'x' || key == 'X') {
-					cdw_form->toggle_checkbox(cdw_form, fi);
+					cdw_form_driver_toggle_checkbox(cdw_form, fi);
 				}
-			} else if (cdw_form->is_dropdown_field_index(cdw_form, fi)) {
+			} else if (cdw_form_is_dropdown_field_index(cdw_form, fi)) {
 				/* dropdown in its initial state reacts only
 				   to enter, and this has already been handled
 				   elsewhere by cdw_form->handle_enter() */
@@ -226,10 +229,24 @@ int cdw_form_driver(cdw_form_t *cdw_form, int initial_fi, void *data)
 		}
 		if (cdw_form_is_movement_key(key)) {
 			int fi = field_index(current_field(cdw_form->form));
-			cdw_form->focus_on(cdw_form, fi);
+			cdw_form_driver_focus_on(cdw_form, fi);
+		}
+		if (cdw_form_is_return_key(cdw_form, key)) {
+			int fi = field_index(current_field(cdw_form->form));
+			if (isprint(key)
+			    && cdw_form_is_input_field_index(cdw_form, fi)) {
+				/* keys like 'q'/'Q' may be configured
+				   as return keys, but they shouldn't be
+				   considered as such in regular text
+				   input fields */
+				;
+			} else {
+				break;
+			}
 		}
 
-	} while (key != CDW_KEY_ESCAPE && !cdw_form_is_return_key(cdw_form, key));
+
+	} while (key != CDW_KEY_ESCAPE);
 
 	form_driver(cdw_form->form, REQ_VALIDATION);
 	curs_set(0); /* turn cursor off */
@@ -266,14 +283,12 @@ void cdw_form_add_return_char(cdw_form_t *form, int key)
 		    form->n_return_keys, N_RETURN_KEYS_MAX);
 	cdw_assert (key != 0, "ERROR: trying to add key '0', but '0' is an initializer value\n");
 
-#ifndef NDEBUG
 	if (cdw_form_is_return_key(form, key)) {
-		cdw_vdm ("WARNING: adding key %d / \"%s\", but it is already on the list of return keys\n",
+		cdw_vdm ("WARNING: attempting to add key %d / \"%s\", but it is already on the list of return keys\n",
 			 key, cdw_ncurses_key_label(key));
+	} else {
+		form->return_keys[form->n_return_keys++] = key;
 	}
-#endif
-
-	form->return_keys[form->n_return_keys++] = key;
 
 	return;
 }
@@ -282,34 +297,61 @@ void cdw_form_add_return_char(cdw_form_t *form, int key)
 
 
 
-cdw_rv_t cdw_form_description_to_fields(cdw_form_descr_t descr[], void *cdw_form)
+void cdw_form_add_return_chars(cdw_form_t *form, ...)
 {
-	cdw_form_t *form = (cdw_form_t *) cdw_form;
-	FIELD **fields = form->fields;
+	cdw_assert (form != (cdw_form_t *) NULL, "ERROR: \"form\" argument is NULL\n");
+	cdw_assert (form->n_return_keys < N_RETURN_KEYS_MAX,
+		    "ERROR: there are already %d / %d return keys in the form, can't add another one\n",
+		    form->n_return_keys, N_RETURN_KEYS_MAX);
+
+	va_list ap;
+	va_start(ap, form);
+	int key = 'a';
+	while ((key = va_arg(ap, int)) != 0) {
+		cdw_form_add_return_char(form, key);
+	}
+	va_end(ap);
+
+	return;
+}
+
+
+
+
+
+cdw_rv_t cdw_form_description_to_fields(cdw_form_descr_t descr[], cdw_form_t *cdw_form)
+{
+	FIELD **fields = cdw_form->fields;
 	/* "<=" - to set proper value of guard in fields table */
-	for (int i = 0; i <= form->n_fields; i++) {
+	for (int i = 0; i <= cdw_form->n_fields; i++) {
 		fields[i] = (FIELD *) NULL;
 	}
 
-	cdw_assert (form->field_widget_types != (int *) NULL, "ERROR: table of widget ids is NULL\n");
+	cdw_assert (cdw_form->field_widget_types != (cdw_id_t *) NULL, "ERROR: table of widget ids is NULL\n");
 
-	for (int i = 0; i < form->n_fields; i++) {
-		int widget_type = descr[i].widget_type;
+	for (int i = 0; i < cdw_form->n_fields; i++) {
+		cdw_id_t widget_type = descr[i].widget_type;
 		cdw_assert (i == descr[i].field_enum, "ERROR: order of items! field iterator (%d) != field enum (%d)\n", i, descr[i].field_enum);
 		if (widget_type == -1) { /* guard in form description table */
-			cdw_assert (0, "ERROR: reached descr guard, so loop condition is incorrect; i = %d, n_fields = %d\n", i, form->n_fields);
+			cdw_assert (0, "ERROR: reached descr guard, so loop condition is incorrect; i = %d, n_fields = %d\n", i, cdw_form->n_fields);
 			break;
 		} else if (widget_type == CDW_WIDGET_LABEL) {
 			cdw_vdm ("INFO: creating field #%d: label with string = \"%s\"\n", i, (char *) descr[i].data1);
-			fields[i] = cdw_label_field_new(descr[i].n_cols,
-							descr[i].begin_y, descr[i].begin_x,
-							(char *) descr[i].data1);
+			fields[i] = cdw_ncurses_new_label_field(descr[i].n_lines, descr[i].n_cols,
+								descr[i].begin_y, descr[i].begin_x,
+								(char *) descr[i].data1);
 		} else if (widget_type == CDW_WIDGET_CHECKBOX) {
-			cdw_vdm ("INFO: creating field #%d: checkbox\n", i);
-			fields[i] = cdw_checkbox_field_new(descr[i].begin_y,
-							   descr[i].begin_x,
-							   (const char *) ((int) descr[i].data2 == 1 ? "X" : " "));
-
+			cdw_vdm ("INFO: creating field #%d: checkbox (new)\n", i);
+			fields[i] = cdw_ncurses_new_checkbox_field(descr[i].begin_y, descr[i].begin_x,
+								   (char *) NULL);
+			/* I don't know why, but sometimes moving into
+			   a checkbox field erases content displayed
+			   in overlaying window; turning off O_PUBLIC
+			   seems to fix this problem */
+			field_opts_off(fields[i], O_PUBLIC);
+			CDW_CHECKBOX *cb = cdw_checkbox_new(cdw_form->subwindow, descr[i].begin_y, descr[i].begin_x, descr[i].data2);
+			int rv = set_field_userptr(fields[i], (void *) cb);
+			cdw_assert (rv == E_OK, "ERROR: can't set userptr for field #%d\n", i);
 		} else if (widget_type == CDW_WIDGET_INPUT) {
 			cdw_vdm ("INFO: creating field #%d: input with initial string = \"%s\"\n", i, (char *) descr[i].data1);
 			fields[i] = cdw_ncurses_new_input_field((size_t) descr[i].n_lines,
@@ -327,37 +369,61 @@ cdw_rv_t cdw_form_description_to_fields(cdw_form_descr_t descr[], void *cdw_form
 								descr[i].begin_y,
 								descr[i].begin_x,
 								(char *) NULL,
-								descr[i].data2,
+								0, //descr[i].data2,
 								CDW_NCURSES_INPUT_NONE,
 								CDW_COLORS_INPUT);
 			/* to avoid situation when user presses some
 			   alphanumeric key and the key is entered
 			   into field under a button */
 			field_opts_off(fields[i], O_EDIT);
-
-			int rv = set_field_userptr(fields[i], (void *) descr[i].data1);
+			CDW_BUTTON *b = cdw_button_new(cdw_form->subwindow, descr[i].begin_y, descr[i].begin_x, descr[i].data1, descr[i].data2);
+			if (b == (CDW_BUTTON *) NULL) {
+				cdw_vdm ("ERROR: failed to create \"log path\" button for field #%d\n", i);
+				return CDW_ERROR;
+			}
+			int rv = set_field_userptr(fields[i], (void *) b);
 			cdw_assert (rv == E_OK, "ERROR: can't set userptr for field #%d\n", i);
 		} else if (widget_type == CDW_WIDGET_DROPDOWN) {
-			cdw_vdm ("INFO: creating field #%d: dropdown\n", i);
+			cdw_vdm ("INFO: creating field #%d: dropdown (new)\n", i);
 			fields[i] = cdw_ncurses_new_input_field((size_t) descr[i].n_lines,
 								descr[i].n_cols,
 								descr[i].begin_y,
 								descr[i].begin_x,
 								(char *) NULL,
-								descr[i].data2,
+								0,
 								CDW_NCURSES_INPUT_NONE,
 								CDW_COLORS_INPUT);
-			int rv = set_field_userptr(fields[i], (void *) descr[i].data1);
+
+			cdw_form_dropdown_maker_t *makers = (cdw_form_dropdown_maker_t *) descr[i].data1;
+			int ind = (int) descr[i].data2;
+			CDW_DROPDOWN *dd = makers[ind](cdw_form->subwindow, descr[i].begin_y, descr[i].begin_x,
+						       descr[i].n_cols);
+			int rv = set_field_userptr(fields[i], (void *) dd);
 			cdw_assert (rv == E_OK, "ERROR: can't set userptr, i = %d\n", i);
+		} else if (widget_type == CDW_WIDGET_TEXT) {
+			cdw_vdm ("INFO: creating field #%d: text area\n", i);
+
+			cdw_form_text_maker_t *makers = (cdw_form_text_maker_t *) descr[i].data1;
+			int ind = (int) descr[i].data2;
+			char *message = makers[ind]((int) descr[i].n_cols);
+
+			fields[i] = cdw_ncurses_new_label_field(descr[i].n_lines, descr[i].n_cols,
+								descr[i].begin_y, descr[i].begin_x,
+								message);
+			free(message);
+			message = (char *) NULL;
+
+			field_opts_off(fields[i], O_ACTIVE);
+			field_opts_off(fields[i], O_EDIT);
 		} else {
-			cdw_vdm ("ERROR: unknown widget type: %d (field #%d)\n", widget_type, i);
+			cdw_assert (0, "ERROR: unknown widget type: %lld (field #%d)\n", widget_type, i);
 		}
 
-		form->field_widget_types[i] = widget_type;
+		cdw_form->field_widget_types[i] = widget_type;
 
 		if (fields[i] == (FIELD *) NULL) {
 			cdw_vdm ("ERROR: failed to initialize field #%d\n", i);
-			return CDW_GEN_ERROR;
+			return CDW_ERROR;
 		}
 	}
 
@@ -373,7 +439,7 @@ bool cdw_form_is_movement_key(int key)
 	if (key == KEY_DOWN
 	    || key == CDW_KEY_TAB
 	    || key == KEY_UP
-	    || key  == KEY_BTAB
+	    || key == KEY_BTAB
 	    || key == KEY_BACKSPACE) { /* in some configurations backspace
 					  may be a movement key, i.e. when
 					  cursor is at the beginning of input field */
@@ -387,129 +453,13 @@ bool cdw_form_is_movement_key(int key)
 
 
 
-/**
-   \brief Create new form
-
-   Create new form, do this with proper assignment of its window, subwindow
-   and form. Function does full clean up and returns NULL in case of failure.
-   Otherwise pointer to properly created form is returned.
-
-   Since "Every form has an associated pair of curses windows" (quote from man
-   pages for set_form_win() and set_form_sub()), you have to create two
-   windows (main window and subwindow) before calling this function.
-
-   \param window - window in which form will be placed
-   \param subwindow - subwindow of the form
-   \param field - set of fields to be put in the form
-
-   \return form on success
-   \return NULL on failure
-*/
-FORM *cdw_form_new_form(WINDOW *window, WINDOW *subwindow, FIELD *field[])
-{
-	cdw_assert (window != (WINDOW *) NULL, "ERROR: function called with window == NULL\n");
-	cdw_assert (subwindow != (WINDOW *) NULL, "ERROR: function called with subwindow == NULL\n");
-	cdw_assert (field != (FIELD **) NULL, "ERROR: function called with fields array == NULL\n");
-
-	FORM *form = new_form(field);
-	if (form == (FORM *) NULL) {
-#ifndef NDEBUG
-		int e = errno;
-		cdw_vdm ("ERROR: failed to create form with new_form(), error code is \"%s\"\n",
-			 cdw_ncurses_error_string(e));
-#endif
-		return (FORM *) NULL;
-	}
-	int r;
-	r = set_form_win(form, window);
-	if (r != E_OK) {
-		cdw_vdm ("ERROR: failed to set form window with set_form_win(), error code is \"%s\"\n", cdw_ncurses_error_string(r));
-		free_form(form);
-		form = (FORM *) NULL;
-		return (FORM *) NULL;
-	}
-	r = set_form_sub(form, subwindow);
-	if (r != E_OK) {
-		cdw_vdm ("ERROR: failed to set form subwindow with set_form_sub(), error code is \"%s\"\n", cdw_ncurses_error_string(r));
-		free_form(form);
-		form = (FORM *) NULL;
-		return (FORM *) NULL;
-	}
-	r = post_form(form);
-	if (r != E_OK) {
-		cdw_vdm ("ERROR: failed to post form with post_form(), error code is \"%s\"\n", cdw_ncurses_error_string(r));
-		free_form(form);
-		form = (FORM *) NULL;
-		return (FORM *) NULL;
-	}
-
-	return form;
-}
-
-
-
-
-
-/**
-   \brief Get 0/1 state associated with given field that is a checkboxes
-
-   Check content of field's buffer (if it is "X" or not) and return 0 or 1.
-
-   The field must be configured and used as a checkbox (field of sizes 1x1),
-   because only first character of field's buffer is checked.
-
-   \param field - field that you want to inspect
-
-   \return true if given field's buffer contains "X" string
-   \return false if given field's buffer does not contain "X" string
-*/
-bool cdw_form_get_field_bit(FIELD *field)
-{
-	cdw_assert (field != (FIELD *) NULL, "ERROR: field argument passed for examination is NULL\n");
-	return !strncmp(field_buffer(field, 0), "X", 1) ? true : false;
-}
-
-
-
-
-
-/**
-   \brief Get string from given field's buffer
-
-   Function removes trailing white chars from te string before returning it.
-
-   Function returns empty string ("") if given field's buffer is NULL.
-
-   \param field - field that you want to inspect
-
-   \return string from field's buffer, may be empty string ("")
-*/
-char *cdw_form_get_field_string(FIELD *field)
-{
-	assert(field != (FIELD *) NULL);
-
-	char *s = field_buffer(field, 0);
-	if (s == (char *) NULL) {
-		return empty_string;
-	} else {
-		cdw_vdm ("INFO: returning field string \"%s\"\n", s);
-		return cdw_string_rtrim(s);
-	}
-
-}
-
-
-
-
-
 /* this function is called when a movement key has been detected;
    this function checks if current focus is on a widget that
    implements highlighted/non-highlighted states, and if it is,
    then highlight of current widget is turned off */
 void cdw_form_driver_focus_off(cdw_form_t *cdw_form, int fi)
 {
-	cdw_form_t *form = (cdw_form_t *) cdw_form;
-	form->toggle_focus(cdw_form, fi, true);
+	cdw_form_driver_toggle_focus(cdw_form, fi, true);
 	return;
 }
 
@@ -523,8 +473,7 @@ void cdw_form_driver_focus_off(cdw_form_t *cdw_form, int fi)
    then highlight of current widget is turned off */
 void cdw_form_driver_focus_on(cdw_form_t *cdw_form, int fi)
 {
-	cdw_form_t *form = (cdw_form_t *) cdw_form;
-	form->toggle_focus(cdw_form, fi, false);
+	cdw_form_driver_toggle_focus(cdw_form, fi, false);
 	return;
 }
 
@@ -542,8 +491,8 @@ void cdw_form_driver_toggle_focus(cdw_form_t *cdw_form, int fi, bool off)
 		    "ERROR: fi is larger than number of fields: %d / %d\n",
 		    fi, cdw_form->n_fields);
 
-	if (cdw_form->is_button_field_index(cdw_form, fi)) {
-		CDW_BUTTON *button = cdw_form->get_button_from_form(cdw_form, fi);
+	if (cdw_form_is_button_field_index(cdw_form, fi)) {
+		CDW_BUTTON *button = cdw_form_get_button(cdw_form, fi);
 		if (button != (CDW_BUTTON *) NULL) {
 			void (*function)(CDW_BUTTON *) = (void (*)(CDW_BUTTON *)) NULL;
 			if (off) {
@@ -557,8 +506,8 @@ void cdw_form_driver_toggle_focus(cdw_form_t *cdw_form, int fi, bool off)
 			}
 			function(button);
 		}
-	} else if (cdw_form->is_dropdown_field_index(cdw_form, fi)) {
-		CDW_DROPDOWN *dropdown = cdw_form_driver_get_dropdown_from_form(cdw_form, fi);
+	} else if (cdw_form_is_dropdown_field_index(cdw_form, fi)) {
+		CDW_DROPDOWN *dropdown = cdw_form_get_dropdown(cdw_form, fi);
 		if (dropdown != (CDW_DROPDOWN *) NULL) {
 			void (*function)(CDW_DROPDOWN *) = (void (*)(CDW_DROPDOWN *)) NULL;
 			if (off) {
@@ -572,8 +521,8 @@ void cdw_form_driver_toggle_focus(cdw_form_t *cdw_form, int fi, bool off)
 			}
 			function(dropdown);
 		}
-	} else if (cdw_form->is_checkbox_field_index(cdw_form, fi)
-		   || cdw_form->is_input_field_index(cdw_form, fi)) {
+	} else if (cdw_form_is_checkbox_field_index(cdw_form, fi)
+		   || cdw_form_is_input_field_index(cdw_form, fi)) {
 		if (off) {
 			/* moving away from checkbox/input, don't display blinking cursor */
 			curs_set(0);
@@ -595,13 +544,13 @@ void cdw_form_driver_toggle_focus(cdw_form_t *cdw_form, int fi, bool off)
 
 
 
-CDW_DROPDOWN *cdw_form_driver_get_dropdown_from_form(cdw_form_t *cdw_form, int fi)
+CDW_DROPDOWN *cdw_form_get_dropdown(cdw_form_t *cdw_form, int fi)
 {
 	cdw_assert (fi < cdw_form->n_fields,
 		    "ERROR: fi is larger than number of fields: %d / %d\n",
 		    fi, cdw_form->n_fields);
 	FIELD *field = cdw_form->fields[fi];
-	cdw_assert (field != (FIELD *) NULL, "ERROR field %d is NULL\n", fi);
+	cdw_assert (field != (FIELD *) NULL, "ERROR: field %d is NULL\n", fi);
 
 	CDW_DROPDOWN *dropdown = (CDW_DROPDOWN *) NULL;
 	if (cdw_form->field_widget_types[fi] == CDW_WIDGET_DROPDOWN) {
@@ -616,13 +565,35 @@ CDW_DROPDOWN *cdw_form_driver_get_dropdown_from_form(cdw_form_t *cdw_form, int f
 
 
 
-CDW_BUTTON *cdw_form_driver_get_button_from_form(cdw_form_t *cdw_form, int fi)
+const char *cdw_form_get_string(cdw_form_t *cdw_form, int fi)
 {
 	cdw_assert (fi < cdw_form->n_fields,
 		    "ERROR: fi is larger than number of fields: %d / %d\n",
 		    fi, cdw_form->n_fields);
 	FIELD *field = cdw_form->fields[fi];
-	cdw_assert (field != (FIELD *) NULL, "ERROR field %d is NULL\n", fi);
+	cdw_assert (field != (FIELD *) NULL, "ERROR: field %d is NULL\n", fi);
+
+	char *string = (char *) NULL;
+	if (cdw_form->field_widget_types[fi] == CDW_WIDGET_INPUT) {
+		string = field_buffer(cdw_form->fields[fi], 0);
+		cdw_string_rtrim(string);
+		cdw_assert (string != (char *) NULL, "ERROR: field user pointer is NULL char (fi = %d)\n", fi);
+	}
+
+	return string;
+}
+
+
+
+
+
+CDW_BUTTON *cdw_form_get_button(cdw_form_t *cdw_form, int fi)
+{
+	cdw_assert (fi < cdw_form->n_fields,
+		    "ERROR: fi is larger than number of fields: %d / %d\n",
+		    fi, cdw_form->n_fields);
+	FIELD *field = cdw_form->fields[fi];
+	cdw_assert (field != (FIELD *) NULL, "ERROR: field %d is NULL\n", fi);
 
 	CDW_BUTTON *button = (CDW_BUTTON *) NULL;
 	if (cdw_form->field_widget_types[fi] == CDW_WIDGET_BUTTON) {
@@ -637,31 +608,62 @@ CDW_BUTTON *cdw_form_driver_get_button_from_form(cdw_form_t *cdw_form, int fi)
 
 
 
-bool is_checkbox_field_index(cdw_form_t *cdw_form, int fi)
+CDW_CHECKBOX *cdw_form_get_checkbox(cdw_form_t *cdw_form, int fi)
 {
 	cdw_assert (fi < cdw_form->n_fields,
 		    "ERROR: fi is larger than number of fields: %d / %d\n",
 		    fi, cdw_form->n_fields);
-	return cdw_form->field_widget_types[fi] == CDW_WIDGET_CHECKBOX ? true : false;
+	FIELD *field = cdw_form->fields[fi];
+	cdw_assert (field != (FIELD *) NULL, "ERROR: field %d is NULL\n", fi);
+
+	CDW_CHECKBOX *checkbox = (CDW_CHECKBOX *) NULL;
+	if (cdw_form->field_widget_types[fi] == CDW_WIDGET_CHECKBOX) {
+		checkbox = (CDW_CHECKBOX *) field_userptr(field);
+		cdw_assert (checkbox != (CDW_CHECKBOX *) NULL, "ERROR: field user pointer is NULL checkbox (fi = %d)\n", fi);
+	}
+
+	return checkbox;
 }
 
 
 
 
 
-bool is_dropdown_field_index(cdw_form_t *cdw_form, int fi)
+bool cdw_form_is_checkbox_field_index(cdw_form_t *cdw_form, int fi)
 {
 	cdw_assert (fi < cdw_form->n_fields,
 		    "ERROR: fi is larger than number of fields: %d / %d\n",
 		    fi, cdw_form->n_fields);
-	return cdw_form->field_widget_types[fi] == CDW_WIDGET_DROPDOWN ? true : false;
+	cdw_id_t type = cdw_form->field_widget_types[fi];
+
+	if (type == CDW_WIDGET_CHECKBOX) {
+		return true;
+	} else {
+		return false;
+	}
 }
 
 
 
 
 
-bool is_button_field_index(cdw_form_t *cdw_form, int fi)
+bool cdw_form_is_dropdown_field_index(cdw_form_t *cdw_form, int fi)
+{
+	cdw_assert (fi < cdw_form->n_fields,
+		    "ERROR: fi is larger than number of fields: %d / %d\n",
+		    fi, cdw_form->n_fields);
+	if (cdw_form->field_widget_types[fi] == CDW_WIDGET_DROPDOWN) {
+		return true;
+	} else {
+		return false;
+	}
+}
+
+
+
+
+
+bool cdw_form_is_button_field_index(cdw_form_t *cdw_form, int fi)
 {
 	cdw_assert (fi < cdw_form->n_fields,
 		    "ERROR: fi is larger than number of fields: %d / %d\n",
@@ -673,7 +675,7 @@ bool is_button_field_index(cdw_form_t *cdw_form, int fi)
 
 
 
-bool is_input_field_index(cdw_form_t *cdw_form, int fi)
+bool cdw_form_is_input_field_index(cdw_form_t *cdw_form, int fi)
 {
 	cdw_assert (fi < cdw_form->n_fields,
 		    "ERROR: fi is larger than number of fields: %d / %d\n",
@@ -699,14 +701,17 @@ bool is_input_field_index(cdw_form_t *cdw_form, int fi)
 */
 void cdw_form_driver_toggle_checkbox(cdw_form_t *cdw_form, int fi)
 {
-	cdw_assert (cdw_form->field_widget_types[fi] == CDW_WIDGET_CHECKBOX, "ERROR: called the function for non-checkbox\n");
+	cdw_assert (cdw_form->field_widget_types[fi] == CDW_WIDGET_CHECKBOX,
+		    "ERROR: called the function for non-checkbox\n");
 	cdw_assert (fi < cdw_form->n_fields,
 		    "ERROR: fi is larger than number of fields: %d / %d\n",
 		    fi, cdw_form->n_fields);
-	if (cdw_form_get_field_bit(cdw_form->fields[fi])) {
-		set_field_buffer(cdw_form->fields[fi], 0, " ");
-	} else {
-		set_field_buffer(cdw_form->fields[fi], 0, "X");
+
+	CDW_CHECKBOX *checkbox = cdw_form_get_checkbox(cdw_form, fi);
+	cdw_checkbox_toggle(checkbox);
+	if (checkbox->on_toggle != (cdw_form_widget_function_t) NULL) {
+		bool state = cdw_checkbox_get_state(checkbox);
+		checkbox->on_toggle(cdw_form, &state);
 	}
 	return;
 }
@@ -715,10 +720,11 @@ void cdw_form_driver_toggle_checkbox(cdw_form_t *cdw_form, int fi)
 
 
 
-void cdw_form_driver_go_to_field(FORM *form, int fi, int n_fields)
+void cdw_form_driver_go_to_field(cdw_form_t *cdw_form, int fi)
 {
-	if (fi < 0 || fi >= n_fields) {
-		cdw_vdm ("WARNING: fi = %d out of bounds (n_fields = %d)\n", fi, n_fields);
+	FORM *form = cdw_form->form;
+	if (fi < 0 || fi >= cdw_form->n_fields) {
+		cdw_vdm ("WARNING: fi = %d out of bounds (n_fields = %d)\n", fi, cdw_form->n_fields);
 		fi = 0;
 	}
 
@@ -733,9 +739,9 @@ void cdw_form_driver_go_to_field(FORM *form, int fi, int n_fields)
 	   before correct field */
 	int i = field_index(current_field(form));
 	if (i == fi) {
-		return;
+		;
 	} else {
-		while (i < fi && i < n_fields - 1) {
+		while (i < fi && i < cdw_form->n_fields - 1) {
 			rv = form_driver(form, REQ_NEXT_FIELD);
 			if (rv != E_OK) {
 				cdw_vdm ("ERROR: form_driver() returns \"%s\"\n", cdw_ncurses_error_string(rv));
@@ -746,6 +752,101 @@ void cdw_form_driver_go_to_field(FORM *form, int fi, int n_fields)
 		}
 	}
 
+	cdw_form_driver_focus_on(cdw_form, fi);
+
 	return;
 }
 
+
+
+
+
+void cdw_form_redraw_widgets(cdw_form_t *cdw_form)
+{
+	for (int i = 0; i < cdw_form->n_fields; i++) {
+		cdw_id_t id = cdw_form->field_widget_types[i];
+		if (id == CDW_WIDGET_CHECKBOX) {
+			CDW_CHECKBOX *cb = cdw_form_get_checkbox(cdw_form, i);
+			cdw_checkbox_draw(cb);
+		} else if (id == CDW_WIDGET_BUTTON) {
+			CDW_BUTTON *b = cdw_form_get_button(cdw_form, i);
+			cdw_button_unfocus(b);
+		} else if (id == CDW_WIDGET_DROPDOWN) {
+			CDW_DROPDOWN *dd = cdw_form_get_dropdown(cdw_form, i);
+			cdw_dropdown_display_current_item(dd);
+		} else {
+			;
+		}
+
+	}
+	return;
+}
+
+
+
+
+
+bool cdw_form_get_checkbox_state(cdw_form_t *cdw_form, int fi)
+{
+	cdw_assert (cdw_form->field_widget_types[fi] == CDW_WIDGET_CHECKBOX,
+		    "ERROR: trying to get state of field which isn't a CHECKBOX\n");
+
+	CDW_CHECKBOX *cb = cdw_form_get_checkbox(cdw_form, fi);
+	return cdw_checkbox_get_state(cb);
+}
+
+
+
+
+
+void *cdw_form_set_function(cdw_form_t *cdw_form, int fi, cdw_form_widget_function_t function)
+{
+	cdw_id_t id = cdw_form->field_widget_types[fi];
+	if (id == CDW_WIDGET_BUTTON) {
+		CDW_BUTTON *b = field_userptr(cdw_form->fields[fi]);
+		b->on_click = function;
+		set_field_userptr(cdw_form->fields[fi], (void *) b);
+		return (void *) b;
+	} else if (id == CDW_WIDGET_CHECKBOX) {
+		CDW_CHECKBOX *cb = field_userptr(cdw_form->fields[fi]);
+		cb->on_toggle = function;
+		set_field_userptr(cdw_form->fields[fi], (void *) cb);
+		return (void *) cb;
+	} else {
+		cdw_assert(0, "ERROR: called the function for widget, which doesn't have a function to set\n");
+		return (void *) NULL;
+	}
+}
+
+
+
+
+
+int cdw_form_handle_enter(cdw_form_t *cdw_form, int fi)
+{
+	if (cdw_form_is_button_field_index(cdw_form, fi)) {
+		CDW_BUTTON *b = cdw_form_get_button(cdw_form, fi);
+		return b->on_click((void *) cdw_form, (void *) NULL);
+	} else if (cdw_form_is_dropdown_field_index(cdw_form, fi)) {
+		CDW_DROPDOWN *dropdown = cdw_form_get_dropdown(cdw_form, fi);
+		cdw_assert (dropdown != (CDW_DROPDOWN *) NULL, "ERROR: dropdown is NULL\n");
+		cdw_rv_t crv = cdw_dropdown_driver(dropdown);
+		if (crv == CDW_OK) {
+			const char *label = cdw_dropdown_get_current_item_label(dropdown);
+			FIELD **fields = form_fields(cdw_form->form);
+			/* setting buffer makes field to display
+			   string, but slightly moved to left */
+			set_field_buffer(*(fields + fi), 0, label);
+
+			/* call to cdw_dropdown_focus() fixes it */
+			cdw_dropdown_focus(dropdown);
+
+			form_driver(cdw_form->form, REQ_VALIDATION);
+		} else {
+			;
+		}
+		return 'a';
+	} else {
+		return 'a';
+	}
+}
