@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2010 Kamil Ignacak
+ * Copyright (C) 2007 - 2011 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -117,7 +117,7 @@ static void cdw_list_display_scroll_to_current(CDW_LIST_DISPLAY *display, size_t
    \return pointer to new display on success
    \return NULL on errors
 */
-CDW_LIST_DISPLAY *cdw_list_display_new(WINDOW *parent, size_t n_lines, size_t n_cols, size_t begin_y, size_t begin_x, cdw_colors_t colors)
+CDW_LIST_DISPLAY *cdw_list_display_new(WINDOW *parent, int n_lines, int n_cols, int begin_y, int begin_x, cdw_colors_t colors)
 {
 	cdw_assert (parent != (WINDOW *) NULL, "ERROR: called the function with null parent window\n");
 
@@ -271,16 +271,16 @@ int cdw_list_display_driver(CDW_LIST_DISPLAY *display)
 
 	if (display->n_items != 0) {
 		/* initialization: highlight initial item */
-		size_t subwindow_row = display->current_item_ind % display->n_lines;
+		size_t subwindow_row = display->current_item_ind % (size_t) display->n_lines;
 		cdw_dll_item_t *item = cdw_dll_ith_item(display->list, display->current_item_ind);
 		display->display_item(display, item->data, subwindow_row, h_offset, true);
 	}
 	wrefresh(display->subwindow);
 
-	int c = 'a'; /* safe (?) initial value */
-	while (c != CDW_ESCAPE) {
-		c = wgetch(display->window);
-		switch (c) {
+	int key = 'a'; /* safe (?) initial value */
+	while (key != CDW_KEY_ESCAPE) {
+		key = wgetch(display->window);
+		switch (key) {
 		        case KEY_HOME:
 				/* first Home key should move content of
 				   window to beginning of lines, leaving
@@ -341,7 +341,7 @@ int cdw_list_display_driver(CDW_LIST_DISPLAY *display)
 			case KEY_PPAGE:
 				/* Previous Page, Page Up */
 				if (display->current_item_ind > 0) {
-					if (display->current_item_ind < display->n_lines) {
+					if (display->current_item_ind < (size_t) display->n_lines) {
 						/* we are on top of listing,
 						   so there is only one place
 						   to move cursor and item index */
@@ -349,7 +349,7 @@ int cdw_list_display_driver(CDW_LIST_DISPLAY *display)
 					} else {
 						/* move item index exactly one
 						   screen size up */
-						display->current_item_ind -= display->n_lines;
+						display->current_item_ind -= (size_t) display->n_lines;
 					}
 					cdw_list_display_scroll_to_current(display, h_offset, true);
 				}
@@ -358,7 +358,7 @@ int cdw_list_display_driver(CDW_LIST_DISPLAY *display)
 			case KEY_NPAGE:
 				/* Next Page, Page Down */
 				if (display->current_item_ind < display->n_items - 1) {
-					display->current_item_ind += display->n_lines;
+					display->current_item_ind += (size_t) display->n_lines;
 					if (display->current_item_ind > display->n_items - 1) {
 						display->current_item_ind = display->n_items - 1;
 					}
@@ -371,7 +371,7 @@ int cdw_list_display_driver(CDW_LIST_DISPLAY *display)
 
 		} /* switch (c) */
 
-		if (cdw_list_display_is_return_char(display, c)) {
+		if (cdw_list_display_is_return_char(display, key)) {
 			break;
 		}
 	} /* while (c != CDW_ESCAPE) */
@@ -380,13 +380,13 @@ int cdw_list_display_driver(CDW_LIST_DISPLAY *display)
 		/* control (and focus) is leaving display area, and to indicate this,
 		   we un-highlight current item */
 		cdw_dll_item_t *item = cdw_dll_ith_item(display->list, display->current_item_ind);
-		size_t subwindow_row = display->current_item_ind % display->n_lines;
+		size_t subwindow_row = display->current_item_ind % (size_t) display->n_lines;
 		display->display_item(display, item->data, subwindow_row, h_offset, false);
 	}
 
 	wrefresh(display->subwindow);
 
-	return c;
+	return key;
 }
 
 
@@ -407,6 +407,7 @@ void cdw_list_display_refresh(CDW_LIST_DISPLAY *display)
 	cdw_assert (display->subwindow != (WINDOW *) NULL, "ERROR: \"display->subwindow\" is NULL\n");
 
 	wrefresh(display->subwindow);
+	redrawwin(display->window);
 	wrefresh(display->window);
 
 	return;
@@ -462,15 +463,15 @@ void cdw_list_display_scroll_to_current(CDW_LIST_DISPLAY *display, size_t h_offs
 	/* get first item to be displayed in 0-th row of subwindow; get it
 	   with expensive cdw_dll_ith_item() call, we will get rest of items
 	   by traversing list "manually" in loop */
-	size_t item_ind = display->current_item_ind - (display->current_item_ind % display->n_lines);
+	size_t item_ind = display->current_item_ind - (display->current_item_ind % (size_t) display->n_lines);
 	cdw_dll_item_t *item = cdw_dll_ith_item(display->list, item_ind);
 
 	/* increment subwindow row number and traverse list at the same time */
-	for (size_t subwindow_row = 0;
+	for (int subwindow_row = 0;
 	     subwindow_row < display->n_lines && item != (cdw_dll_item_t *) NULL;
 	     subwindow_row++, item = item->next, item_ind++) {
 
-		display->display_item(display, item->data, subwindow_row, h_offset, false);
+		display->display_item(display, item->data, (size_t) subwindow_row, h_offset, false);
 
 		/* since we already traverse list of items, we can update
 		   value of "current_item" field: since this function was called,
@@ -484,7 +485,7 @@ void cdw_list_display_scroll_to_current(CDW_LIST_DISPLAY *display, size_t h_offs
 	if (highlight) {
 		/* highlight current item */
 		item = cdw_dll_ith_item(display->list, display->current_item_ind);
-		size_t subwindow_row = display->current_item_ind % display->n_lines;
+		size_t subwindow_row = display->current_item_ind % (size_t) display->n_lines;
 		display->display_item(display, item->data, subwindow_row, h_offset, true);
 	}
 
@@ -520,15 +521,12 @@ void cdw_list_display_add_return_char(CDW_LIST_DISPLAY *display, int c)
 		    display->n_return_chars, N_RETURN_KEYS_MAX);
 	cdw_assert (c != 0, "ERROR: trying to add char == 0, but 0 is an initializer value\n");
 
-#ifndef NDEBUG
 	if (cdw_list_display_is_return_char(display, c)) {
-		cdw_vdm ("WARNING: adding char %d / \"%s\", but it is already on the list of return chars\n",
+		cdw_vdm ("WARNING: attempting to add char %d / \"%s\", but it is already on the list of return chars\n",
 			 c, cdw_ncurses_key_label(c));
+	} else {
+		display->return_chars[display->n_return_chars++] = c;
 	}
-#endif
-
-	display->return_chars[display->n_return_chars] = c;
-	display->n_return_chars++;
 
 	return;
 }
@@ -536,6 +534,43 @@ void cdw_list_display_add_return_char(CDW_LIST_DISPLAY *display, int c)
 
 
 
+/**
+   \brief Add key(s) that, if captured by display's driver, will cause return of control by driver
+
+   Function registers keys, which, if captured by driver, will cause
+   the driver to return control to it's (driver's) caller.
+
+   You may want to use this function e.g. when you want to perform some
+   action whenever ENTER key is hit. In that case you have to configure
+   a display before using it by calling this function with CDW_KEY_ENTER as
+   second argument. Similarly for other keys: Space, Delete, any digit or
+   letter. Digits should be passed as characters.
+
+   \param display - display that you want to configure
+   \param ... - list of keys to add; use '0' as a last key (guard)
+*/
+void cdw_list_display_add_return_keys(CDW_LIST_DISPLAY *display, ...)
+{
+	cdw_assert (display != (CDW_LIST_DISPLAY *) NULL, "ERROR: \"display\" argument is NULL\n");
+	cdw_assert (display->n_return_chars < N_RETURN_KEYS_MAX,
+		    "ERROR: there are already %d / %d return keys in the form, can't add another one\n",
+		    display->n_return_chars, N_RETURN_KEYS_MAX);
+
+	va_list ap;
+	va_start(ap, display);
+	int key = 'a';
+	while ((key = va_arg(ap, int)) != 0) {
+		cdw_list_display_add_return_char(display, key);
+	}
+	va_end(ap);
+
+	return;
+}
+
+
+
+
+
 
 /**
   \brief Check if given char is one of configured "return" chars
@@ -639,7 +674,7 @@ cdw_rv_t cdw_list_display_remove_item(CDW_LIST_DISPLAY *display, size_t item_i)
 		}
 		return CDW_OK;
 	} else {
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 }
 
