@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2010 Kamil Ignacak
+ * Copyright (C) 2007 - 2011 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -80,8 +80,6 @@
 #include <ctype.h>
 #include <errno.h>
 
-#include <ncursesw/form.h>
-
 #include "cdw_string.h"
 #include "cdw_debug.h"
 #include "main.h"
@@ -90,6 +88,12 @@
 #include "cdw_widgets.h"
 #include "gettext.h"
 
+
+extern cdw_arguments_t cdw_commandline_arguments;
+
+/* used by cdw_ncurses_get_field_string() */
+static char empty_string[] = "";
+
 static void cdw_ncurses_get_field_buffer_check(FIELD *field, size_t limit);
 static void cdw_ncurses_make_custom_key_definitions(void);
 
@@ -111,6 +115,27 @@ cdw_rv_t cdw_ncurses_init(void)
 	noecho();
 	keypad(stdscr, TRUE);
 
+	if (cdw_commandline_arguments.escdelay_ms != -1) {
+		set_escdelay(cdw_commandline_arguments.escdelay_ms);
+	} else {
+		/* we are about to modify global variable ESCDELAY;
+		   this is a library variable, not a shell variable;
+		   I'm not sure what "global" means in this context,
+		   man page for set_escdelay suggests that it is global
+		   in "screen" scope; just to be on the safe side I will
+		   check if the variable isn't already modified by other
+		   apps to "low enough" level;
+
+		   of course this does not prevent cdw from modifying
+		   ESCDELAY if some other app modified it to some "large
+		   enough" value... */
+		if (ESCDELAY <= 150) {
+			/* low enough, don't modify */
+		} else {
+			set_escdelay(50); /* milliseconds */
+		}
+	}
+
 	/* turn cursor off - now cursor will be	turned on only if needed */
 	curs_set(0);
 
@@ -129,7 +154,7 @@ cdw_rv_t cdw_ncurses_init(void)
 		   and exits; please keep '\n' chars */
 		fprintf(stderr, _("\nCannot start cdw: cdw needs at least 80x24 terminal!\n"));
 		cdw_vdm ("ERROR: failed to initialize curses, either lines (%d) or cols (%d) value is too small\n", lines, cols);
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	} else {
 		cdw_sdm ("INFO: lines = %d, cols = %d\n", lines, cols);
 	}
@@ -169,7 +194,7 @@ void cdw_ncurses_clean(void)
    \param limit - maximal number of chars to copy, does not include ending nul
 
    \return CDW_OK on success
-   \return CDW_GEN_ERROR on malloc error
+   \return CDW_ERROR on malloc error
  */
 cdw_rv_t cdw_ncurses_get_field_buffer(FIELD *field, char **buffer, size_t limit)
 {
@@ -197,7 +222,7 @@ cdw_rv_t cdw_ncurses_get_field_buffer(FIELD *field, char **buffer, size_t limit)
 	char *tmp = (char *) realloc(*buffer, len + 1);
 	if (tmp == (char *) NULL) {
 		cdw_vdm ("ERROR: failed to reallocate buffer with new len %zd\n", len);
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	} else {
 		*buffer = tmp;
 		strncpy(*buffer, (char *) cdw_string_rtrim(field_buffer(field, 0)), len);
@@ -251,23 +276,25 @@ void cdw_ncurses_get_field_buffer_check(FIELD *field, size_t limit)
 
 
 
-static cdw_id_label_t cdw_ncurses_error_strings[] = {
-	{ E_BAD_ARGUMENT,      "E_BAD_ARGUMENT" },     /* Routine detected an incorrect or out-of-range argument. */
-	{ E_SYSTEM_ERROR,      "E_SYSTEM_ERROR" },     /* System error occurred, e.g., malloc failure. */
-	{ E_CONNECTED,         "E_CONNECTED" },        /* Field is connected. */
-	{ E_BAD_STATE,         "E_BAD_STATE" },        /* Routine was called from an initialization or termination function. */
-	{ E_NOT_POSTED,        "E_NOT_POSTED" },       /* The form has not been posted. */
-	{ E_INVALID_FIELD,     "E_INVALID_FIELD" },    /* Contents of field is invalid. */
-	{ E_REQUEST_DENIED,    "E_REQUEST_DENIED" },   /* The form driver could not process the request. */
-	{ E_NOT_CONNECTED,     "E_NOT_CONNECTED" },    /* No items are connected to the form. */
-	{ E_NO_ROOM,           "E_NO_ROOM" },          /* Form is too large for its window. */
-	{ E_POSTED,            "E_POSTED" },           /* The form has already been posted. */
+/* names are from /usr/include/ncursesw/eti.h,
+   descriptions come from ncurses man pages */
+static cdw_id_clabel_t cdw_ncurses_error_strings[] = {
+	{ E_OK,                "E_OK"              },  /* The routine succeeded */
+	{ E_SYSTEM_ERROR,      "E_SYSTEM_ERROR"    },  /* System error occurred, e.g., malloc failure. */
+	{ E_BAD_ARGUMENT,      "E_BAD_ARGUMENT"    },  /* Routine detected an incorrect or out-of-range argument. */
+	{ E_POSTED,            "E_POSTED"          },  /* The form has already been posted. */
+	{ E_CONNECTED,         "E_CONNECTED"       },  /* Field is connected. */
+	{ E_BAD_STATE,         "E_BAD_STATE"       },  /* Routine was called from an initialization or termination function. */
+	{ E_NO_ROOM,           "E_NO_ROOM"         },  /* Form is too large for its window. */
+	{ E_NOT_POSTED,        "E_NOT_POSTED"      },  /* The form has not been posted. */
 	{ E_UNKNOWN_COMMAND,   "E_UNKNOWN_COMMAND" },  /* The form driver code saw an unknown request code. */
-        { E_OK,                "E_OK" },               /* The routine succeeded */
-	{ 0,                   (char *) NULL }};
-
-
-
+	{ E_NO_MATCH,          "E_NO_MATCH"        },  /*  */
+	{ E_NOT_SELECTABLE,    "E_NOT_SELECTABLE"  },  /*  */
+	{ E_NOT_CONNECTED,     "E_NOT_CONNECTED"   },  /* No items are connected to the form. */
+	{ E_REQUEST_DENIED,    "E_REQUEST_DENIED"  },  /* The form driver could not process the request. */
+	{ E_INVALID_FIELD,     "E_INVALID_FIELD"   },  /* Contents of field is invalid. */
+	{ E_CURRENT,           "E_CURRENT"         },  /* The field is the current field */
+	{ 0,                   (char *) NULL       }};
 
 
 const char *cdw_ncurses_error_string(int error)
@@ -292,7 +319,7 @@ const char *cdw_ncurses_error_string(int error)
    \param top_string - string that will appear on top of window
    \param bottom_string - string that will appear on bottom of window
 
-   \return CDW_MEM_ERROR on malloc errors
+   \return CDW_ERROR on malloc errors
    \return CDW_OK when no problems occurred
 */
 cdw_rv_t cdw_ncurses_nice_box(WINDOW *window, const char *top_string, const char *bottom_string)
@@ -330,7 +357,7 @@ cdw_rv_t cdw_ncurses_nice_box(WINDOW *window, const char *top_string, const char
 			if (!buf) {
 				cdw_vdm ("ERROR: failed to duplicate bottom string \"%s\" with strdup(bottom_string + offset)\n", bottom_string);
 				cdw_vdm ("ERROR: details: strlen(bottom_string) = %zd, offset = %zd\n", strlen(bottom_string), offset);
-				return CDW_MEM_ERROR;
+				return CDW_ERROR;
 			} else {
 				malloced = true;
 				strncpy(buf, "...", 3);
@@ -359,7 +386,7 @@ cdw_rv_t cdw_ncurses_nice_box(WINDOW *window, const char *top_string, const char
 			if (!buf) {
 				cdw_vdm ("ERROR: failed to duplicate top string \"%s\" with strndup(top_string, usable_n_cols)\n", top_string);
 				cdw_vdm ("ERROR: details: strlen(top_string) = %zd, usable_n_cols = %zd\n", strlen(top_string), usable_n_cols);
-				return CDW_MEM_ERROR;
+				return CDW_ERROR;
 			} else {
 				malloced = true;
 				strncpy(buf + usable_n_cols - 3, "...", 3);
@@ -443,11 +470,9 @@ void cdw_ncurses_delete_menu_and_items(MENU **menu)
 
 
 
-static cdw_id_label_t cdw_ncurses_key_labels[] =
+static cdw_id_clabel_t cdw_ncurses_key_labels[] =
 	{
-		{ CDW_ENTER,      "ENTER" },
 		{ CDW_KEY_ENTER,  "ENTER" },
-		{ CDW_ESCAPE,     "ESCAPE" },
 		{ CDW_KEY_ESCAPE, "ESCAPE" },
 		{ KEY_DOWN,     "KEY_DOWN" },
 		{ KEY_UP,       "KEY_UP" },
@@ -511,7 +536,7 @@ const char *cdw_ncurses_key_label(int key)
    \return pointer to new field on success
    \return NULL on errors
 */
-FIELD *cdw_ncurses_new_input_field(size_t n_lines, size_t n_cols, size_t begin_y, size_t begin_x, const char *initial_string, size_t len_max, int input_type, cdw_colors_t colors)
+FIELD *cdw_ncurses_new_input_field(int n_lines, int n_cols, int begin_y, int begin_x, const char *initial_value, size_t len_max, int input_type, cdw_colors_t colors)
 {
 	int e = 0;
 	/* input field
@@ -567,12 +592,12 @@ FIELD *cdw_ncurses_new_input_field(size_t n_lines, size_t n_cols, size_t begin_y
 			cdw_vdm ("ERROR: failed to turn off O_PUBLIC with field_opts_off(), error = \"%s\"\n", cdw_ncurses_error_string(e));
 		}
 	} else {
-		if (initial_string != (char *) NULL) {
+		if (initial_value != (char *) NULL) {
 			/* this may also set content of buffer to "" */
-			e = set_field_buffer(field, 0, initial_string);
+			e = set_field_buffer(field, 0, initial_value);
 			if (e != E_OK) {
 				const char *tmp = (e == E_SYSTEM_ERROR) ? strerror(errno) : cdw_ncurses_error_string(e);
-				cdw_assert (0, "ERROR: failed to set buffer content with string \"%s\", error = \"%s\"\n", initial_string, tmp);
+				cdw_assert (0, "ERROR: failed to set buffer content with string \"%s\", error = \"%s\"\n", initial_value, tmp);
 			}
 		}
 	}
@@ -593,12 +618,12 @@ FIELD *cdw_ncurses_new_input_field(size_t n_lines, size_t n_cols, size_t begin_y
 	e = set_field_back(field, COLOR_PAIR(colors));
 	if (e != E_OK) {
 		const char *tmp = (e == E_SYSTEM_ERROR) ? strerror(errno) : cdw_ncurses_error_string(e);
-		cdw_assert (0, "ERROR: failed to set buffer content with string \"%s\", error = \"%s\"\n", initial_string, tmp);
+		cdw_assert (0, "ERROR: failed to set buffer content with string \"%s\", error = \"%s\"\n", initial_value, tmp);
 	}
 	e = set_field_fore(field, COLOR_PAIR(colors));
 	if (e != E_OK) {
 		const char *tmp = (e == E_SYSTEM_ERROR) ? strerror(errno) : cdw_ncurses_error_string(e);
-		cdw_assert (0, "ERROR: failed to set buffer content with string \"%s\", error = \"%s\"\n", initial_string, tmp);
+		cdw_assert (0, "ERROR: failed to set buffer content with string \"%s\", error = \"%s\"\n", initial_value, tmp);
 	}
 
 	return field;
@@ -691,18 +716,13 @@ WINDOW *cdw_ncurses_window_new(WINDOW *parent, int n_lines, int n_cols, int begi
 /**
  * \brief Create new non-editable ncurses field that will be used as text label
  *
- * Create 1xX input field that will be used in a page as text label.
- *
- * \param width - width of the label
- * \param y - row in which the field will be put
- * \param x - column in which the field will be put
- * \param buffer_content - text that will be used as value of the label
+ * Create XxY input field that will be used in a page as text label.
  *
  * \return field - newly created field
  */
-FIELD *cdw_label_field_new(size_t width, size_t y, size_t x, const char *buffer_content)
+FIELD *cdw_ncurses_new_label_field(int n_lines, int n_cols, int begin_y, int begin_x, const char *initial_value)
 {
-	FIELD *field = new_field(1, (int) width, (int) y, (int) x, 0, 0);
+	FIELD *field = new_field(n_lines, n_cols, begin_y, begin_x, 0, 0);
 	if (field == (FIELD *) NULL) {
 		cdw_vdm ("ERROR: failed to create a field with new_field()\n");
 		return (FIELD *) NULL;
@@ -712,9 +732,9 @@ FIELD *cdw_label_field_new(size_t width, size_t y, size_t x, const char *buffer_
 	if (rv != E_OK) {
 		cdw_vdm ("ERROR: field_opts_on() returns !E_OK\n");
 	}
-	rv = set_field_buffer(field, 0, buffer_content);
+	rv = set_field_buffer(field, 0, initial_value);
 	if (rv != E_OK) {
-		cdw_vdm ("ERROR: failed to set field buffer with string = \"%s\"\n", buffer_content);
+		cdw_vdm ("ERROR: failed to set field buffer with string = \"%s\"\n", initial_value);
 	}
 	/* supposedly you can't justify if field is inactive, so first justify and then deactivate */
 	rv = set_field_just(field, JUSTIFY_LEFT);
@@ -747,28 +767,28 @@ FIELD *cdw_label_field_new(size_t width, size_t y, size_t x, const char *buffer_
  * Create 1x1 input field that will be used in a page as checkbox. Function
  * does not write '[' ']' characters around the field.
  *
- * \param y - row in which the field will be put
- * \param x - column in which the field will be put
+ * \param begin_y - row in which the field will be put
+ * \param begin_x - column in which the field will be put
  * \param buffer_content - initial state of the checkbox
  *
  * \return field - newly created field
  */
-FIELD *cdw_checkbox_field_new(size_t y, size_t x, const char *buffer_content)
+FIELD *cdw_ncurses_new_checkbox_field(int begin_y, int begin_x, const char *initial_value)
 {
-	/*
-	FIELD *new_field(int height, int width,	int toprow, int leftcol,
-			 int offscreen, int nbuffers);
-	*/
+	/* FIELD *new_field(int height, int width, int toprow, int leftcol,
+	                    int offscreen, int nbuffers); */
 
-	FIELD *field = new_field(1, 1, (int) y, (int) x, 0, 0);
+	FIELD *field = new_field(1, 1, begin_y, begin_x, 0, 0);
 	if (field == (FIELD *) NULL) {
 		cdw_vdm ("ERROR: failed to create a field with new_field\n");
 		return (FIELD *) NULL;
 	}
-
-	int rv = set_field_buffer(field, 0, buffer_content);
-	if (rv != E_OK) {
-		cdw_vdm ("ERROR: set_field_buffer() returns !E_OK\n");
+	int rv = 0;
+	if (initial_value != (char *) NULL) {
+		rv = set_field_buffer(field, 0, initial_value);
+		if (rv != E_OK) {
+			cdw_vdm ("ERROR: set_field_buffer() returns !E_OK\n");
+		}
 	}
 	field_opts_off(field, O_AUTOSKIP);
 	if (rv != E_OK) {
@@ -789,6 +809,7 @@ FIELD *cdw_checkbox_field_new(size_t y, size_t x, const char *buffer_content)
 
 
 
+
 void cdw_ncurses_make_custom_key_definitions(void)
 {
 	/* new definitions of keys: a workaround for problems with
@@ -814,19 +835,189 @@ void cdw_ncurses_make_custom_key_definitions(void)
 
 		{ (const char *) NULL, 0 }};   /* guard */
 
-	for (int i = 0; ; i++) {
-		if (keycodes[i].val == 0) {
-			break;
-		} else {
-			int rv = define_key(keycodes[i].string, keycodes[i].val);
-			if (rv != OK) {
-				cdw_vdm ("WARNING: can't define key #%d / \"%s\"\n", keycodes[i].val, keycodes[i].string);
-				/* not returning, missing key definitions aren't critical error */
-			}
+	int i = 0;
+	while (keycodes[i].val != 0) {
+		int rv = define_key(keycodes[i].string, keycodes[i].val);
+		if (rv != OK) {
+			cdw_vdm ("WARNING: can't define key #%d / \"%s\"\n", keycodes[i].val, keycodes[i].string);
+			/* not returning, missing key definitions aren't critical error */
 		}
+		i++;
 	}
 
 	return;
 }
 
 
+
+
+
+/**
+   \brief Delete ncurses window, set pointer to NULL
+
+   This is such a common piece of code that I had to put it in
+   a separate function.
+   The function accepts pointer to NULL window.
+   The function sets window pointer to NULL.
+
+   \param window - window to delete
+*/
+void cdw_ncurses_window_delete(WINDOW **window)
+{
+	if (*window == (WINDOW *) NULL) {
+		cdw_vdm ("WARNING: passing NULL window to the function\n");
+		return;
+	} else {
+		delwin(*window);
+		*window = (WINDOW *) NULL;
+		return;
+	}
+}
+
+
+
+
+
+/**
+   \brief Create new form
+
+   Create new form, do this with proper assignment of its window, subwindow
+   and form. Function does full clean up and returns NULL in case of failure.
+   Otherwise pointer to properly created form is returned.
+
+   Since "Every form has an associated pair of curses windows" (quote from man
+   pages for set_form_win() and set_form_sub()), you have to create two
+   windows (main window and subwindow) before calling this function.
+
+   \param window - window in which form will be placed
+   \param subwindow - subwindow of the form
+   \param field - set of fields to be put in the form
+
+   \return form on success
+   \return NULL on failure
+*/
+FORM *cdw_ncurses_new_form(WINDOW *window, WINDOW *subwindow, FIELD *field[])
+{
+	cdw_assert (window != (WINDOW *) NULL, "ERROR: function called with window == NULL\n");
+	cdw_assert (subwindow != (WINDOW *) NULL, "ERROR: function called with subwindow == NULL\n");
+	cdw_assert (field != (FIELD **) NULL, "ERROR: function called with fields array == NULL\n");
+
+	FORM *form = new_form(field);
+	if (form == (FORM *) NULL) {
+#ifndef NDEBUG
+		int e = errno;
+		cdw_vdm ("ERROR: failed to create form with new_form(), error code is \"%s\"\n",
+			 cdw_ncurses_error_string(e));
+#endif
+		return (FORM *) NULL;
+	}
+	int r;
+	r = set_form_win(form, window);
+	if (r != E_OK) {
+		cdw_vdm ("ERROR: failed to set form window with set_form_win(), error code is \"%s\"\n", cdw_ncurses_error_string(r));
+		free_form(form);
+		form = (FORM *) NULL;
+		return (FORM *) NULL;
+	}
+	r = set_form_sub(form, subwindow);
+	if (r != E_OK) {
+		cdw_vdm ("ERROR: failed to set form subwindow with set_form_sub(), error code is \"%s\"\n", cdw_ncurses_error_string(r));
+		free_form(form);
+		form = (FORM *) NULL;
+		return (FORM *) NULL;
+	}
+	r = post_form(form);
+	if (r != E_OK) {
+		cdw_vdm ("ERROR: failed to post form with post_form(), error code is \"%s\"\n", cdw_ncurses_error_string(r));
+		free_form(form);
+		form = (FORM *) NULL;
+		return (FORM *) NULL;
+	}
+
+	return form;
+}
+
+
+
+
+
+/**
+   \brief Get string from given field's buffer
+
+   Function removes trailing white chars from te string before returning it.
+
+   Function returns empty string ("") if given field's buffer is NULL.
+
+   \param field - field that you want to inspect
+
+   \return string from field's buffer, may be empty string ("")
+*/
+char *cdw_ncurses_get_field_string(FIELD *field)
+{
+	assert(field != (FIELD *) NULL);
+
+	char *s = field_buffer(field, 0);
+	if (s == (char *) NULL) {
+		return empty_string;
+	} else {
+		cdw_vdm ("INFO: returning field string \"%s\"\n", s);
+		return cdw_string_rtrim(s);
+	}
+
+}
+
+
+
+
+
+/**
+   \brief Add '?' char in upper right corner of given window
+
+   The char indicates that pressing '?' key in this window invokes
+   small help window, displaying some help text.
+
+   \param window - window for which to display the char
+   \param n_cols - number of columns in the window
+*/
+void cdw_ncurses_window_add_help_marker(WINDOW *window)
+{
+	int n_cols = getmaxx(window);
+	mvwprintw(window, 0, n_cols - 1, "?");
+	mvwaddch(window, 0, n_cols - 2, ACS_RTEE);
+	mvwaddch(window, 1, n_cols - 1, ACS_TTEE);
+
+	return;
+}
+
+
+
+/* *** unused code below */
+
+
+
+
+
+# if 0
+
+
+
+/**
+   \brief Get 0/1 state from a checkbox widget associated with given field
+
+   Check content of field's buffer (if it is "X" or not) and return 0 or 1.
+
+   The field must be configured and used as a checkbox (field of size 1x1),
+   because only first character of field's buffer is checked.
+
+   \param field - field that you want to inspect
+
+   \return true if given field's buffer contains "X" string
+   \return false if given field's buffer does not contain "X" string
+*/
+bool cdw_ncurses_get_field_bit(FIELD *field)
+{
+	cdw_assert (field != (FIELD *) NULL, "ERROR: field argument passed for examination is NULL\n");
+	return !strncmp(field_buffer(field, 0), "X", 1) ? true : false;
+}
+
+#endif
