@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2010 Kamil Ignacak
+ * Copyright (C) 2007 - 2011 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -109,7 +109,7 @@ cdw_rv_t cdw_processwin_create(const char *window_title, const char *window_labe
 {
 	if (processwin.active) {
 		cdw_vdm ("ERROR: called the function for active process window\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	memset(processwin_empty_string, ' ', EMPTY_STRING_LEN + 1);
@@ -130,7 +130,7 @@ cdw_rv_t cdw_processwin_create(const char *window_title, const char *window_labe
 
 	if (processwin.window == (WINDOW *) NULL) {
 		cdw_vdm ("ERROR: failed to create process window with newwin()\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	keypad(processwin.window, TRUE);
@@ -146,7 +146,7 @@ cdw_rv_t cdw_processwin_create(const char *window_title, const char *window_labe
 		delwin(processwin.window);
 		processwin.window = (WINDOW *) NULL;
 		cdw_vdm ("ERROR: failed to create subwindow_txt1 with derwin()\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 	processwin.subwindow_txt2 = derwin(processwin.window, 2,
 					   processwin.n_cols - 2,
@@ -157,7 +157,7 @@ cdw_rv_t cdw_processwin_create(const char *window_title, const char *window_labe
 		delwin(processwin.window);
 		processwin.window = (WINDOW *) NULL;
 		cdw_vdm ("ERROR: failed to create subwindow_txt2 with derwin()\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	cdw_ncurses_nice_box(processwin.window, window_title, (char *) NULL);
@@ -225,10 +225,10 @@ void cdw_processwin_destroy(const char *status_string, bool wait)
 	processwin.window = (WINDOW *) NULL;
 	processwin.active = false;
 
-	cdw_main_ui_main_window_wrefresh_part((size_t) processwin.n_rows,
-					      (size_t) processwin.n_cols,
-					      (size_t) processwin.begin_y,
-					      (size_t) processwin.begin_x);
+	cdw_main_ui_main_window_wrefresh_part(processwin.n_rows,
+					      processwin.n_cols,
+					      processwin.begin_y,
+					      processwin.begin_x);
 
 	return;
 }
@@ -408,7 +408,12 @@ void cdw_processwin_display_progress_conditional(long current, long total, const
 		    "ERROR: value of \"current\" or \"total\" is negative: %ld / %ld\n", current, total);
 	/* new_static_percent reflects current amount of work done
 	   at given specific call to this function */
-	float new_static_percent = (float) current / (float) total;
+	float new_static_percent = 0.0;
+	if (total != 0) {
+		/* FIXME: don't allow passing total == 0,
+		   improve interface of the function */
+		new_static_percent = (float) current / (float) total;
+	}
 
 	/* static_percent reflects amount of work done at
 	   previous call to this function */
