@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2010 Kamil Ignacak
+ * Copyright (C) 2007 - 2011 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,8 +34,6 @@
 #include <errno.h>
 #include <unistd.h>
 
-#include <ncursesw/form.h>
-
 #include "gettext.h"
 #include "cdw_widgets.h"
 #include "cdw_main_window.h" /* main_win_refresh_part() */
@@ -53,44 +51,44 @@ typedef struct {
 	cdw_colors_t colors;
 
 	WINDOW *window;
-	size_t n_rows;
-	size_t n_cols;
-	size_t begin_y;
-	size_t begin_x;
+	int n_rows;
+	int n_cols;
+	int begin_y;
+	int begin_x;
 
 	/* space between dialog window borders and content of the window:
 	   the text area and buttons/input field; pad_x may be 0 if
 	   more space is needed to display text without truncations */
-	size_t pad_y;
-	size_t pad_x;
+	int pad_y;
+	int pad_x;
 
 	/* size of content (txt area + (buttons or input field)) of dialog
 	   window, sum this with pad_x and pad_y to get total dialog window size */
-	size_t n_rows_internal;
-	size_t n_cols_internal;
+        int n_rows_internal;
+	int n_cols_internal;
 
 	/* text area in upper part of dialog; it can be one-line prompt, but
 	   it can be multi-line text too */
 	struct {
 		WINDOW *subwindow;
-		size_t n_rows;
-		size_t n_cols;
+		int n_rows;
+		int n_cols;
 	} txt;
 
 	/* area where buttons or input field is displayed */
 	struct {
-		size_t n_cols;
-		size_t n_rows; /* will be always (?) equal one, see comment below for explanation */
+		int n_cols;
+		int n_rows; /* will be always (?) equal one, see comment below for explanation */
 
 		/* buttons dialog:
 		   all buttons are in one row, the row's number is begin_y;
 
 		   input line dialog:
 		   input line occupies one row in dialog window, the row's number is begin_y */
-		size_t begin_y;
+		int begin_y;
 
 		/* empty line between text area and entry (input) area below */
-		size_t spacer_y;
+		int spacer_y;
 	} input;
 
 	/* if dialog->type is CDW_DIALOG_BUTTONS, then this struct
@@ -99,7 +97,7 @@ typedef struct {
 		/* 4 buttons should be enough for everyone */
 		CDW_BUTTON *buttons[4];
 		char *labels[4];
-		size_t label_lens[4];
+		int label_lens[4];
 
 		/* currently selected button, used e.g. in driver */
 		int current;
@@ -137,8 +135,8 @@ static void cdw_dialog_display_buttons(CDW_DIALOG_BASE *dialog);
 
 static void cdw_input_line_safe_driver_message(CDW_INPUT_LINE *input_line, char *insecure);
 
-static size_t cdw_textarea_wrap_text(char *buffer, size_t width);
-static size_t cdw_textarea_calculate_height(const char *message, size_t width);
+static int cdw_textarea_wrap_text(char *buffer, size_t width);
+static int cdw_textarea_calculate_height(const char *message, size_t width);
 
 static cdw_rv_t cdw_dropdown_expanded_driver(CDW_DROPDOWN *dropdown);
 static cdw_rv_t cdw_dropdown_expand(CDW_DROPDOWN *dropdown);
@@ -235,7 +233,7 @@ cdw_rv_t cdw_safe_input_dialog(cdw_safe_input_data_t *data)
 	CDW_DIALOG_BASE *dialog = cdw_dialog_base_new(CDW_DIALOG_INPUT);
 	if (dialog == (CDW_DIALOG_BASE *) NULL) {
 		cdw_vdm ("ERROR: failed to get new dialog base\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	cdw_dialog_base_calculate_sizes(dialog, data->prompt_message);
@@ -254,10 +252,10 @@ cdw_rv_t cdw_safe_input_dialog(cdw_safe_input_data_t *data)
 	CDW_INPUT_LINE *input_line = cdw_input_line_new(dialog->window,
 							(int) dialog->input.begin_y, begin_x,
 							(int) dialog->input.n_cols, *(data->buffer),
-							data->input_type, (size_t) data->chars_max);
+							data->input_type, data->chars_max);
 	if (input_line == (CDW_INPUT_LINE *) NULL) {
 		cdw_vdm ("ERROR: failed to create input line\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	cdw_rv_t retval = CDW_CANCEL;
@@ -273,7 +271,7 @@ cdw_rv_t cdw_safe_input_dialog(cdw_safe_input_data_t *data)
 		cdw_buttons_dialog(data->window_title,
 				   data->error_message,
 				   CDW_BUTTONS_OK, CDW_COLORS_WARNING);
-		retval = CDW_GEN_ERROR;
+		retval = CDW_ERROR;
 	}
 
 	cdw_input_line_delete(&input_line);
@@ -306,7 +304,7 @@ cdw_rv_t cdw_safe_input_dialog(cdw_safe_input_data_t *data)
    \param colors - color scheme of dialog
 
    \return value of type cdw_rv_t, corresponding to selected button
-   \return CDW_MEM_ERROR on malloc error
+   \return CDW_ERROR on malloc error
  */
 cdw_rv_t cdw_buttons_dialog(const char *title, const char *message, int buttons_type, cdw_colors_t colors)
 {
@@ -318,7 +316,7 @@ cdw_rv_t cdw_buttons_dialog(const char *title, const char *message, int buttons_
 
 	CDW_DIALOG_BASE *dialog = cdw_dialog_base_new(CDW_DIALOG_BUTTONS);
 	if (dialog == (CDW_DIALOG_BASE *) NULL) {
-		return CDW_MEM_ERROR;
+		return CDW_ERROR;
 	}
 
 	dialog->colors = colors;
@@ -452,10 +450,10 @@ void cdw_dialog_base_delete(CDW_DIALOG_BASE **dialog)
 
 cdw_rv_t cdw_dialog_base_calculate_input_sizes(CDW_DIALOG_BASE *dialog)
 {
-	const size_t nice_input_width = 50;
+	const int nice_input_width = 50;
 
 	/* maximal width of input field or buttons area, -2 is for window borders */
-	const size_t max_cols = (size_t) COLS - 2 - 2 * dialog->pad_x;
+	const int max_cols = COLS - 2 - 2 * dialog->pad_x;
 
 	if (dialog->dialog_type == CDW_DIALOG_BUTTONS) {
 		if (dialog->buttons.type == CDW_BUTTONS_OK) {
@@ -507,7 +505,7 @@ cdw_rv_t cdw_dialog_base_calculate_input_sizes(CDW_DIALOG_BASE *dialog)
 	cdw_sdm ("INFO: final required width of dialog input area is %zd\n", dialog->input.n_cols);
 
 	if (buttons_crammed) {
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	} else {
 		return CDW_OK;
 	}
@@ -546,7 +544,7 @@ cdw_rv_t cdw_dialog_base_calculate_sizes(CDW_DIALOG_BASE *dialog, const char *me
 		dialog->n_cols_internal = dialog->txt.n_cols;
 	}
 
-	size_t max = (size_t) LINES - 2   /* -2 for window borders */
+	int max = LINES - 2               /* -2 for window borders */
 		- 2 * dialog->pad_y       /* may be already set to 0 by calculate_subwindow_txt_size() */
 		- dialog->input.n_rows    /* this is not to be changed anymore */
 		- dialog->input.spacer_y; /* additional row above and below input area */
@@ -555,7 +553,7 @@ cdw_rv_t cdw_dialog_base_calculate_sizes(CDW_DIALOG_BASE *dialog, const char *me
 
 		dialog->pad_y = 0;
 
-		max = (size_t) LINES - 2          /* -2 for window borders */
+		max = LINES - 2                   /* -2 for window borders */
 			- 2 * dialog->pad_y       /* may be already set to 0 by calculate_subwindow_txt_size() */
 			- dialog->input.n_rows    /* this is not to be changed anymore */
 			- dialog->input.spacer_y; /* additional rows above and below input area */
@@ -564,7 +562,7 @@ cdw_rv_t cdw_dialog_base_calculate_sizes(CDW_DIALOG_BASE *dialog, const char *me
 
 			dialog->input.spacer_y = 0;
 
-			max = (size_t) LINES - 2          /* -2 for window borders */
+			max = LINES - 2                   /* -2 for window borders */
 				- 2 * dialog->pad_y       /* may be already set to 0 by calculate_subwindow_txt_size() */
 				- dialog->input.n_rows    /* this is not to be changed anymore */
 				- dialog->input.spacer_y; /* additional rows above and below input area */
@@ -580,7 +578,7 @@ cdw_rv_t cdw_dialog_base_calculate_sizes(CDW_DIALOG_BASE *dialog, const char *me
 
 	if (crv1 != CDW_OK || crv2 != CDW_OK) {
 		cdw_vdm ("ERROR: something went wrong when calculating sizes in dialogbox, expect layout problems\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	} else {
 		return CDW_OK;
 	}
@@ -594,33 +592,33 @@ cdw_rv_t cdw_dialog_base_calculate_subwindow_txt_sizes(CDW_DIALOG_BASE *dialog,
 {
 	/* change value of height so that window will be high
 	   enough to display whole message */
-	dialog->txt.n_rows = cdw_textarea_calculate_height(message, dialog->txt.n_cols);
+	dialog->txt.n_rows = cdw_textarea_calculate_height(message, (size_t) dialog->txt.n_cols);
 
-	size_t n_rows = dialog->txt.n_rows + 2 * dialog->pad_y + dialog->input.spacer_y + dialog->input.n_rows;
-	if (n_rows > (size_t) LINES) {
+	int n_rows = dialog->txt.n_rows + 2 * dialog->pad_y + dialog->input.spacer_y + dialog->input.n_rows;
+	if (n_rows > LINES) {
 		/* message won't fit into dialog window of current width,
 		   stretch the width .. */
 		/* first -2 is for window borders */
-		dialog->txt.n_cols = (size_t) COLS - 2 - 2 * dialog->pad_x;
+		dialog->txt.n_cols =  COLS - 2 - 2 * dialog->pad_x;
 		/* ... and try again */
-		dialog->txt.n_rows = cdw_textarea_calculate_height(message, dialog->txt.n_cols);
+		dialog->txt.n_rows = cdw_textarea_calculate_height(message, (size_t) dialog->txt.n_cols);
 
 		n_rows = dialog->txt.n_rows + 2 * dialog->pad_y + dialog->input.spacer_y + dialog->input.n_rows;
-		if (n_rows > (size_t) LINES) {
+		if (n_rows > LINES) {
 
 			/* last try, get rid of non-critical space... */
 			dialog->pad_x = 0;
-			dialog->txt.n_cols = (size_t) COLS - 2 - 2 * dialog->pad_x;
+			dialog->txt.n_cols = COLS - 2 - 2 * dialog->pad_x;
 			/* ... and try again */
-			dialog->txt.n_rows = cdw_textarea_calculate_height(message, dialog->txt.n_cols);
+			dialog->txt.n_rows = cdw_textarea_calculate_height(message, (size_t) dialog->txt.n_cols);
 
 			n_rows = dialog->txt.n_rows + 2 * dialog->pad_y + dialog->input.spacer_y + dialog->input.n_rows;
 		}
 	}
 
-	if (n_rows > (size_t) LINES) {
+	if (n_rows > LINES) {
 		cdw_vdm ("ERROR: failed to calculate satisfactory sizes of txt subwindow, text may be clipped\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	} else {
 		return CDW_OK;
 	}
@@ -635,8 +633,8 @@ void cdw_dialog_base_display_basics(CDW_DIALOG_BASE *dialog, const char *title,
 	/* +2 is for borders */
 	dialog->n_rows = dialog->n_rows_internal + 2 * dialog->pad_y + 2;
 	dialog->n_cols = dialog->n_cols_internal + 2 * dialog->pad_x + 2;
-	dialog->begin_y = ((size_t) LINES - dialog->n_rows) / 2;
-	dialog->begin_x = ((size_t) COLS - dialog->n_cols) / 2;
+	dialog->begin_y = (LINES - dialog->n_rows) / 2;
+	dialog->begin_x = (COLS - dialog->n_cols) / 2;
 
 	dialog->window = newwin((int) dialog->n_rows, (int) dialog->n_cols, (int) dialog->begin_y, (int) dialog->begin_x);
 
@@ -672,10 +670,10 @@ void cdw_dialog_create_button_labels(CDW_DIALOG_BASE *dialog)
 	/* 2TRANS: button label, please keep as short as possible */
 	dialog->buttons.labels[BUTTON_CANCEL] = _("Cancel");
 
-	dialog->buttons.label_lens[BUTTON_OK] = strlen(dialog->buttons.labels[BUTTON_OK]);
-	dialog->buttons.label_lens[BUTTON_YES] = strlen(dialog->buttons.labels[BUTTON_YES]);
-	dialog->buttons.label_lens[BUTTON_NO] = strlen(dialog->buttons.labels[BUTTON_NO]);
-	dialog->buttons.label_lens[BUTTON_CANCEL] = strlen(dialog->buttons.labels[BUTTON_CANCEL]);
+	dialog->buttons.label_lens[BUTTON_OK] = (int) strlen(dialog->buttons.labels[BUTTON_OK]);
+	dialog->buttons.label_lens[BUTTON_YES] = (int) strlen(dialog->buttons.labels[BUTTON_YES]);
+	dialog->buttons.label_lens[BUTTON_NO] = (int) strlen(dialog->buttons.labels[BUTTON_NO]);
+	dialog->buttons.label_lens[BUTTON_CANCEL] = (int) strlen(dialog->buttons.labels[BUTTON_CANCEL]);
 
 	return;
 }
@@ -814,11 +812,11 @@ void cdw_dialog_display_buttons(CDW_DIALOG_BASE *dialog)
 	   but dialog window might be too narrow for this (remember about
 	   translations that may be longer than original strings). */
 
-	size_t n_cols = (size_t) getmaxx(dialog->window) - 2;
+	int n_cols = getmaxx(dialog->window) - 2;
 
 	/* draw initial dialog box */
 	if (dialog->buttons.type == CDW_BUTTONS_OK) {
-		size_t ok_col = (n_cols - (dialog->buttons.label_lens[BUTTON_OK] + 2)) / 2;
+		int ok_col = (n_cols - (dialog->buttons.label_lens[BUTTON_OK] + 2)) / 2;
 
 		dialog->buttons.buttons[BUTTON_OK] = cdw_button_new(dialog->window,
 								    dialog->input.begin_y, ok_col,
@@ -828,8 +826,8 @@ void cdw_dialog_display_buttons(CDW_DIALOG_BASE *dialog)
 		dialog->buttons.current = BUTTON_OK;
 
 	} else if (dialog->buttons.type == CDW_BUTTONS_OK_CANCEL) {
-		size_t ok_col = ((n_cols/2) - (dialog->buttons.label_lens[BUTTON_OK] + 2)) / 2;
-		size_t cancel_col = (n_cols/2) + (((n_cols/2) - (dialog->buttons.label_lens[BUTTON_CANCEL] + 2)) / 2);
+		int ok_col = ((n_cols/2) - (dialog->buttons.label_lens[BUTTON_OK] + 2)) / 2;
+		int cancel_col = (n_cols/2) + (((n_cols/2) - (dialog->buttons.label_lens[BUTTON_CANCEL] + 2)) / 2);
 
 		dialog->buttons.buttons[BUTTON_OK] = cdw_button_new(dialog->window,
 								    dialog->input.begin_y, ok_col,
@@ -843,13 +841,13 @@ void cdw_dialog_display_buttons(CDW_DIALOG_BASE *dialog)
 		dialog->buttons.current = BUTTON_OK;
 
 	} else if (dialog->buttons.type == CDW_BUTTONS_YES_NO_CANCEL) {
-		size_t blen = 0;
+		int blen = 0;
 		blen = dialog->buttons.label_lens[BUTTON_YES] + 2;
-		size_t yes_col = ((n_cols/3) - blen) / 2;
+		int yes_col = ((n_cols/3) - blen) / 2;
 		blen = dialog->buttons.label_lens[BUTTON_NO] + 2;
-		size_t no_col = (n_cols/3) + (((n_cols/3) - blen) / 2);
+		int no_col = (n_cols/3) + (((n_cols/3) - blen) / 2);
 		blen = dialog->buttons.label_lens[BUTTON_CANCEL] + 2;
-		size_t cancel_col = ((2 * n_cols)/3) + (((n_cols/3) - blen) / 2);
+		int cancel_col = ((2 * n_cols)/3) + (((n_cols/3) - blen) / 2);
 
 		dialog->buttons.buttons[BUTTON_YES] = cdw_button_new(dialog->window,
 								     dialog->input.begin_y, yes_col,
@@ -898,7 +896,7 @@ void cdw_dialog_display_buttons(CDW_DIALOG_BASE *dialog)
    \return 0 if \p window has just enough or more rows than needed to print a message without truncations
    \return number of lines that weren't displayed because \p window doesn't have enough rows
  */
-size_t cdw_textarea_print_message(WINDOW *window, const char *message, int align)
+int cdw_textarea_print_message(WINDOW *window, const char *message, int align)
 {
 	cdw_assert (window != (WINDOW *) NULL, "ERROR: cannot print in NULL window\n");
 	cdw_assert (message != (char *) NULL, "ERROR: cannot print NULL message\n");
@@ -906,8 +904,8 @@ size_t cdw_textarea_print_message(WINDOW *window, const char *message, int align
 	char *w_message = strdup(message);
 	cdw_assert (w_message != (char *) NULL, "ERROR: failed to strdup() message \"%s\"\n", message);
 
-	size_t n_cols = (size_t) getmaxx(window);
-	size_t n_rows = (size_t) getmaxy(window);
+	int n_cols = getmaxx(window);
+	int n_rows = getmaxy(window);
 
 #if 0   /* debug code */
 	cdw_vdm ("INFO: width = %d, control string is this:\n", (int) n_cols)
@@ -917,17 +915,17 @@ size_t cdw_textarea_print_message(WINDOW *window, const char *message, int align
 	fprintf(stderr, "\n");
 #endif
 
-	size_t n_lines = cdw_textarea_wrap_text(w_message, n_cols);
+	int n_lines = cdw_textarea_wrap_text(w_message, (size_t) n_cols);
 	if (n_rows < n_lines) {
-		cdw_vdm ("ERROR: cdw_textarea_wrap_text() produces wrapped text with more lines (%zd) than rows in window (%zd)\n",
+		cdw_vdm ("ERROR: cdw_textarea_wrap_text() produces wrapped text with more lines (%d) than rows in window (%d)\n",
 			 n_lines, n_rows);
 	}
 	size_t len = strlen(w_message);
 
 	/* now display every line of wrapped message in given window,
 	   row after row, nicely aligned */
-	size_t col = 0;
-	size_t row = 0;
+	int col = 0;
+	int row = 0;
 	char *line = w_message;
 	for (size_t i = 0; i < len; i++) {
 		if (*(w_message + i) == '\n') {
@@ -936,9 +934,9 @@ size_t cdw_textarea_print_message(WINDOW *window, const char *message, int align
 			if (align == CDW_ALIGN_LEFT) {
 				col = 0;
 			} else if (align == CDW_ALIGN_CENTER) {
-				col = (n_cols - strlen(line)) / 2;
+				col = (n_cols - (int) strlen(line)) / 2;
 			} else { /* align == CDW_ALIGN_RIGHT */
-				col = n_cols - strlen(line);
+				col = n_cols - (int) strlen(line);
 			}
 			mvwprintw(window, (int) row, (int) col, line);
 
@@ -948,7 +946,7 @@ size_t cdw_textarea_print_message(WINDOW *window, const char *message, int align
 				/* TODO: this doesn't seem to be right,
 				   inspect whole function and check
 				   what "row == n_rows" means */
-				cdw_vdm ("ERROR: failed to print %zd lines of text\n", n_lines - n_rows);
+				cdw_vdm ("ERROR: failed to print %d lines of text\n", n_lines - n_rows);
 				break;
 			}
 		}
@@ -957,9 +955,9 @@ size_t cdw_textarea_print_message(WINDOW *window, const char *message, int align
 	if (align == CDW_ALIGN_LEFT) {
 		col = 0;
 	} else if (align == CDW_ALIGN_CENTER) {
-		col = (n_cols - strlen(line)) / 2;
+		col = (n_cols - (int) strlen(line)) / 2;
 	} else { /* align == CDW_ALIGN_RIGHT */
-		col = n_cols - strlen(line);
+		col = n_cols - (int) strlen(line);
 	}
 	mvwprintw(window, (int) row, (int) col, line);
 
@@ -993,7 +991,7 @@ size_t cdw_textarea_print_message(WINDOW *window, const char *message, int align
 
    \return height of window that would fit given message
 */
-size_t cdw_textarea_calculate_height(const char *message, size_t width)
+int cdw_textarea_calculate_height(const char *message, size_t width)
 {
 	cdw_assert (message != (char *) NULL, "ERROR: cannot operate on NULL message\n");
 
@@ -1008,7 +1006,7 @@ size_t cdw_textarea_calculate_height(const char *message, size_t width)
 	fprintf(stderr, "\n");
 #endif
 
-	size_t height = cdw_textarea_wrap_text(buffer, width);
+	int height = cdw_textarea_wrap_text(buffer, width);
 
 	free(buffer);
 	buffer = (char *) NULL;
@@ -1033,7 +1031,7 @@ size_t cdw_textarea_calculate_height(const char *message, size_t width)
 
    \return number of lines that the text will occupy in window
 */
-size_t cdw_textarea_wrap_text(char *buffer, size_t width)
+int cdw_textarea_wrap_text(char *buffer, size_t width)
 {
 	cdw_assert (width > 5, "ERROR: textarea is far too narrow\n");
 	cdw_assert (buffer != (char *) NULL, "ERROR: text to wrap is NULL\n");
@@ -1108,7 +1106,7 @@ size_t cdw_textarea_wrap_text(char *buffer, size_t width)
 		}
 	}
 
-	size_t height = 0;
+	int height = 0;
 	size_t i = 0;
 	for (i = 0; i < len; i++) {
 		if (*(buffer + i) == '\n') {
@@ -1168,7 +1166,7 @@ void cdw_put_ckbox(WINDOW *window, int y, int x)
    \return pointer to new button widget on success
    \return NULL pointer if creating widget fails
 */
-CDW_BUTTON *cdw_button_new(WINDOW *parent, size_t begin_y, size_t begin_x, const char *label, cdw_colors_t colors)
+CDW_BUTTON *cdw_button_new(WINDOW *parent, int begin_y, int begin_x, const char *label, cdw_colors_t colors)
 {
 	cdw_assert (parent != (WINDOW *) NULL, "ERROR: parent window is null\n");
 
@@ -1188,9 +1186,8 @@ CDW_BUTTON *cdw_button_new(WINDOW *parent, size_t begin_y, size_t begin_x, const
 
 	button->begin_x = begin_x;
 	button->begin_y = begin_y;
-
 	button->colors = colors;
-
+	button->on_click = (cdw_form_widget_function_t) NULL;
 	button->parent = parent;
 
 	button->strip = derwin(parent, 1,  2 + (int) button->label_len + 2,
@@ -1239,6 +1236,33 @@ void cdw_button_delete(CDW_BUTTON **button)
 
 
 
+/**
+   \brief Free CDW_BUTTON widget
+
+   Deallocate all resources used by CDW_BUTTON widget. The function
+   accepts NULL pointer, but does not set its argument to NULL.
+
+   \param button - button to destroy
+*/
+void cdw_button_free(CDW_BUTTON *button)
+{
+	if (button != (CDW_BUTTON *) NULL) {
+		delwin(button->strip);
+		button->strip = (WINDOW *) NULL;
+
+		free(button->label);
+		button->label = (char *) NULL;
+
+		free(button);
+	}
+
+	return;
+}
+
+
+
+
+
 /**
    \brief Highlight given button
 
@@ -1306,16 +1330,16 @@ CDW_DROPDOWN *cdw_dropdown_new(WINDOW *parent, int begin_y, int begin_x, int wid
 	dropdown->n_items_max = n_items_max;
 	dropdown->n_items = 0;
 
-	dropdown->items = (cdw_dropdown_item_t **) malloc((size_t) (dropdown->n_items_max + 1) * sizeof (cdw_dropdown_item_t *));
+	dropdown->items = (cdw_id_label_t **) malloc((size_t) (dropdown->n_items_max + 1) * sizeof (cdw_id_label_t *));
 
-	if (dropdown->items == (cdw_dropdown_item_t **) NULL) {
+	if (dropdown->items == (cdw_id_label_t **) NULL) {
 		cdw_vdm ("ERROR: failed to allocate memory for dropdown items\n");
 		free(dropdown);
 		dropdown = (CDW_DROPDOWN *) NULL;
 		return (CDW_DROPDOWN *) NULL;
 	} else {
 		for (int i = 0; i < dropdown->n_items_max; i++) {
-			dropdown->items[i] = (cdw_dropdown_item_t *) NULL;
+			dropdown->items[i] = (cdw_id_label_t *) NULL;
 		}
 	}
 
@@ -1346,27 +1370,27 @@ CDW_DROPDOWN *cdw_dropdown_new(WINDOW *parent, int begin_y, int begin_x, int wid
    \param label - "label" field of a new item
 
    \return CDW_OK on success
-   \return CDW_GEN_ERROR on error
+   \return CDW_ERROR on error
 */
-cdw_rv_t cdw_dropdown_add_item(CDW_DROPDOWN *dropdown, int id, const char *label)
+cdw_rv_t cdw_dropdown_add_item(CDW_DROPDOWN *dropdown, cdw_id_t id, const char *label)
 {
 	cdw_assert (dropdown != (CDW_DROPDOWN *) NULL, "ERROR: passing null dropdown to function\n");
-	cdw_assert (dropdown->items != (cdw_dropdown_item_t **) NULL, "ERROR: dropdown items is null\n");
+	cdw_assert (dropdown->items != (cdw_id_label_t **) NULL, "ERROR: dropdown items is null\n");
 	cdw_assert (dropdown->n_items < dropdown->n_items_max, "ERROR: trying to create item #%d while maximum is #%d\n",
 		    dropdown->n_items, dropdown->n_items_max);
 
-	dropdown->items[dropdown->n_items] = (cdw_dropdown_item_t *) malloc(sizeof (cdw_dropdown_item_t));
-	if (dropdown->items[dropdown->n_items] == (cdw_dropdown_item_t *) NULL) {
+	dropdown->items[dropdown->n_items] = (cdw_id_label_t *) malloc(sizeof (cdw_id_label_t));
+	if (dropdown->items[dropdown->n_items] == (cdw_id_label_t *) NULL) {
 		cdw_vdm ("ERROR: failed to malloc memory for item #%d\n", dropdown->n_items);
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	dropdown->items[dropdown->n_items]->label = strndup(label, (size_t) dropdown->width);
 	if (dropdown->items[dropdown->n_items]->label == (char *) NULL) {
 		cdw_vdm ("ERROR: failed to strndup label #%d: \"%s\"\n", dropdown->n_items, label);
 		free(dropdown->items[dropdown->n_items]);
-		dropdown->items[dropdown->n_items] = (cdw_dropdown_item_t *) NULL;
-		return CDW_GEN_ERROR;
+		dropdown->items[dropdown->n_items] = (cdw_id_label_t *) NULL;
+		return CDW_ERROR;
 	}
 	dropdown->items[dropdown->n_items]->id = id;
 	dropdown->n_items++;
@@ -1388,7 +1412,7 @@ cdw_rv_t cdw_dropdown_add_item(CDW_DROPDOWN *dropdown, int id, const char *label
    \param dropdown - dropdown to be finalized
 
    \return CDW_OK on success
-   \return CDW_GEN_ERROR on failure
+   \return CDW_ERROR on failure
 */
 cdw_rv_t cdw_dropdown_finalize(CDW_DROPDOWN *dropdown)
 {
@@ -1397,7 +1421,7 @@ cdw_rv_t cdw_dropdown_finalize(CDW_DROPDOWN *dropdown)
 	/* standard code creating ITEM elements */
 	dropdown->menu_items = (ITEM **) calloc((size_t) dropdown->n_items + 1, sizeof(ITEM *));
 	if (dropdown->menu_items == (ITEM **) NULL) {
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	for (int i = 0; i < dropdown->n_items; i++) {
@@ -1406,7 +1430,7 @@ cdw_rv_t cdw_dropdown_finalize(CDW_DROPDOWN *dropdown)
 			int e = errno;
 			cdw_vdm ("ERROR: failed to create menu item for item #%d with label \"%s\", errno = \"%s\"\n",
 				 i, dropdown->items[i]->label, strerror(e));
-			return CDW_GEN_ERROR;
+			return CDW_ERROR;
 		}
 	}
 	/* obligatory ending element */
@@ -1417,7 +1441,7 @@ cdw_rv_t cdw_dropdown_finalize(CDW_DROPDOWN *dropdown)
 	if (dropdown->menu == (MENU *) NULL) {
 		int e = errno;
 		cdw_vdm ("ERROR: failed to create menu, errno = \"%s\"\n", strerror(e));
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	dropdown->parent_x = getbegx(dropdown->parent);
@@ -1430,22 +1454,22 @@ cdw_rv_t cdw_dropdown_finalize(CDW_DROPDOWN *dropdown)
 	if (dropdown->menu_window == (WINDOW *) NULL) {
 		int e = errno;
 		cdw_vdm ("ERROR: failed to create dropdown window, errno = \"%s\"\n", cdw_ncurses_error_string(e));
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 	int rv = set_menu_win(dropdown->menu, dropdown->menu_window);
 	if (rv != E_OK) {
 		cdw_vdm ("ERROR: failed to set menu window, errno = \"%s\"\n", cdw_ncurses_error_string(rv));
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 	dropdown->menu_window_sub = derwin(dropdown->menu_window, (int) dropdown->n_items, (int)  (dropdown->width - 2), 0, 0);
 	if (dropdown->menu_window_sub == (WINDOW *) NULL) {
 		cdw_vdm ("ERROR: failed to create menu subwindow\n");
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 	rv = set_menu_sub(dropdown->menu, dropdown->menu_window_sub);
 	if (rv != E_OK) {
 		cdw_vdm ("ERROR: failed to set menu subwindow, errno = \"%s\"\n", cdw_ncurses_error_string(rv));
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	/* remember that post_menu() displays menu in its subwindow, not on the
@@ -1453,7 +1477,7 @@ cdw_rv_t cdw_dropdown_finalize(CDW_DROPDOWN *dropdown)
 	rv = post_menu(dropdown->menu);
 	if (rv != E_OK) {
 		cdw_vdm ("ERROR: failed to post menu, errno = \"%s\"\n", cdw_ncurses_error_string(rv));
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	dropdown->visible = true;
@@ -1672,50 +1696,79 @@ cdw_rv_t cdw_dropdown_delete(CDW_DROPDOWN **dropdown)
 		return CDW_OK;
 	}
 
-	unpost_menu((*dropdown)->menu);
+	cdw_dropdown_free(*dropdown);
+	*dropdown = (CDW_DROPDOWN *) NULL;
 
-	if ((*dropdown)->menu != (MENU *) NULL) {
-		unpost_menu((*dropdown)->menu);
-		free_menu((*dropdown)->menu);
-		(*dropdown)->menu = (MENU *) NULL;
+	return CDW_OK;
+}
 
-		for (int j = 0; j < (*dropdown)->n_items; j++){
-			free_item((*dropdown)->menu_items[j]);
-			(*dropdown)->menu_items[j] = (ITEM *) NULL;
-		}
 
-		free((*dropdown)->menu_items);
-		(*dropdown)->menu_items = (ITEM **) NULL;
+
+
+
+/**
+   \brief Destroy given dropdown, deallocate its resources
+
+   Note that item_labels passed to constructor of dropdown is _not_ freed by
+   destructor. The variable is owned by caller of cdw_dropdown_new().
+
+   This destructor does not NULL pointer (non-initialized widget).
+
+   \param dropdown - pointer to dropdown that you want to destroy
+
+   \return CDW_OK
+*/
+void cdw_dropdown_free(CDW_DROPDOWN *dropdown)
+{
+	if (dropdown == (CDW_DROPDOWN *) NULL) {
+		/* probably called the function when other function that
+		   creates dropdown failed to allocate initial memory for
+		   dropdown - hence null pointer */
+		cdw_vdm ("WARNING: you passed NULL dropdown to function\n");
+		return;
 	}
 
-	for (int i = 0; i < (*dropdown)->n_items; i++) {
-		if ((*dropdown)->items[i] != (cdw_dropdown_item_t *) NULL) {
-			if ((*dropdown)->items[i]->label != (char *) NULL) {
-				free((*dropdown)->items[i]->label);
-				(*dropdown)->items[i]->label = (char *) NULL;
+	if (dropdown->menu != (MENU *) NULL) {
+		unpost_menu(dropdown->menu);
+		free_menu(dropdown->menu);
+		dropdown->menu = (MENU *) NULL;
+
+		for (int j = 0; j < dropdown->n_items; j++){
+			free_item(dropdown->menu_items[j]);
+			dropdown->menu_items[j] = (ITEM *) NULL;
+		}
+
+		free(dropdown->menu_items);
+		dropdown->menu_items = (ITEM **) NULL;
+	}
+
+	for (int i = 0; i < dropdown->n_items; i++) {
+		if (dropdown->items[i] != (cdw_id_label_t *) NULL) {
+			if (dropdown->items[i]->label != (char *) NULL) {
+				free(dropdown->items[i]->label);
+				dropdown->items[i]->label = (char *) NULL;
 			} else {
 				cdw_vdm ("WARNING: menu item label #%d is null\n", i);
 			}
 
-			free((*dropdown)->items[i]);
-			(*dropdown)->items[i] = (cdw_dropdown_item_t *) NULL;
+			free(dropdown->items[i]);
+			dropdown->items[i] = (cdw_id_label_t *) NULL;
 		} else {
 			cdw_vdm ("WARNING: menu item #%d is null\n", i);
 		}
 	}
-	free((*dropdown)->items);
-	(*dropdown)->items = (cdw_dropdown_item_t **) NULL;
+	free(dropdown->items);
+	dropdown->items = (cdw_id_label_t **) NULL;
 
-	delwin((*dropdown)->menu_window_sub);
-	(*dropdown)->menu_window_sub = (WINDOW *) NULL;
+	delwin(dropdown->menu_window_sub);
+	dropdown->menu_window_sub = (WINDOW *) NULL;
 
-	delwin((*dropdown)->menu_window);
-	(*dropdown)->menu_window = (WINDOW *) NULL;
+	delwin(dropdown->menu_window);
+	dropdown->menu_window = (WINDOW *) NULL;
 
-	free(*dropdown);
-	*dropdown = (CDW_DROPDOWN *) NULL;
+	free(dropdown);
 
-	return CDW_OK;
+	return;
 }
 
 
@@ -1877,7 +1930,7 @@ cdw_rv_t cdw_dropdown_set_current_item_by_ind(CDW_DROPDOWN *dropdown, int ind)
 		return CDW_OK;
 	} else {
 		cdw_vdm ("ERROR: index (%d) is larger than number of items (%d)\n", ind, dropdown->n_items);
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 }
 
@@ -1885,7 +1938,7 @@ cdw_rv_t cdw_dropdown_set_current_item_by_ind(CDW_DROPDOWN *dropdown, int ind)
 
 
 
-cdw_rv_t cdw_dropdown_set_current_item_by_id(CDW_DROPDOWN *dropdown, int id)
+cdw_rv_t cdw_dropdown_set_current_item_by_id(CDW_DROPDOWN *dropdown, cdw_id_t id)
 {
 	cdw_assert (dropdown != (CDW_DROPDOWN *) NULL, "ERROR: passing null dropdown to function\n");
 
@@ -1895,15 +1948,15 @@ cdw_rv_t cdw_dropdown_set_current_item_by_id(CDW_DROPDOWN *dropdown, int id)
 			return CDW_OK;
 		}
 	}
-	cdw_vdm ("ERROR: failed to set current item index by id, id = %d\n", id);
-	return CDW_GEN_ERROR;
+	cdw_vdm ("ERROR: failed to set current item index by id, id = %lld\n", id);
+	return CDW_ERROR;
 }
 
 
 
 
 
-int cdw_dropdown_get_current_item_id(CDW_DROPDOWN *dropdown)
+cdw_id_t cdw_dropdown_get_current_item_id(CDW_DROPDOWN *dropdown)
 {
 	cdw_assert (dropdown != (CDW_DROPDOWN *) NULL, "ERROR: cannot set current item id from NULL dropdown\n");
 
@@ -1944,19 +1997,19 @@ const char *cdw_dropdown_get_label_by_ind(CDW_DROPDOWN *dropdown, int ind)
 
 
 
-const char *cdw_dropdown_get_label_by_id(CDW_DROPDOWN *dropdown, int id)
+const char *cdw_dropdown_get_label_by_id(CDW_DROPDOWN *dropdown, cdw_id_t id)
 {
 	cdw_assert (dropdown != (CDW_DROPDOWN *) NULL, "ERROR: cannot set current item id from NULL dropdown\n");
 
 	for (int i = 0; i < dropdown->n_items; i++) {
 		if (dropdown->items[i]->id == id) {
 			char *tmp = dropdown->items[i]->label;
-			cdw_assert (tmp != (char *) NULL, "ERROR: item with id %d has null label\n", id);
+			cdw_assert (tmp != (char *) NULL, "ERROR: item with id %lld has null label\n", id);
 			return tmp;
 		}
 	}
 
-	cdw_vdm ("ERROR: can't find item for id %d\n", id);
+	cdw_vdm ("ERROR: can't find item for id %lld\n", id);
 
 	return (char *) NULL;
 }
@@ -1965,7 +2018,38 @@ const char *cdw_dropdown_get_label_by_id(CDW_DROPDOWN *dropdown, int id)
 
 
 
-CDW_INPUT_LINE *cdw_input_line_new(WINDOW *parent, int begin_y, int begin_x, int n_cols, const char *initial_string, int input_type, size_t chars_max)
+CDW_DROPDOWN *cdw_dropdown_maker_wrapper(WINDOW *window, int begin_y, int begin_x, int width, int n_items, cdw_id_clabel_t *items)
+{
+	CDW_DROPDOWN *dropdown = cdw_dropdown_new(window, begin_y, begin_x, width, n_items, CDW_COLORS_DIALOG);
+	if (dropdown == (CDW_DROPDOWN *) NULL) {
+		cdw_vdm ("ERROR: failed to create new dropdown\n");
+		return (CDW_DROPDOWN *) NULL;
+	}
+
+	for (int i = 0; i < n_items; i++) {
+		cdw_rv_t crv = cdw_dropdown_add_item(dropdown, items[i].id, _(items[i].label));
+		if (crv != CDW_OK) {
+			cdw_vdm ("ERROR: failed to add item #%d (id = %lld, label = \"%s\"\n",
+				 i, items[i].id, items[i].label);
+			cdw_dropdown_delete(&dropdown);
+			return (CDW_DROPDOWN *) NULL;
+		}
+	}
+
+	cdw_rv_t crv = cdw_dropdown_finalize(dropdown);
+	if (crv != CDW_OK) {
+		cdw_vdm ("ERROR: failed to finalize dropdown\n");
+		cdw_dropdown_delete(&dropdown);
+		return (CDW_DROPDOWN *) NULL;
+	}
+	return dropdown;
+}
+
+
+
+
+
+CDW_INPUT_LINE *cdw_input_line_new(WINDOW *parent, int begin_y, int begin_x, int n_cols, const char *initial_string, int input_type, int chars_max)
 {
 	CDW_INPUT_LINE *input_line = (CDW_INPUT_LINE *) malloc(sizeof(CDW_INPUT_LINE));
 	if (input_line == (CDW_INPUT_LINE *) NULL) {
@@ -2000,7 +2084,7 @@ CDW_INPUT_LINE *cdw_input_line_new(WINDOW *parent, int begin_y, int begin_x, int
 		cdw_vdm ("ERROR: failed to create form field\n");
 		success = false;
 	} else {
-		input_line->form = cdw_form_new_form(input_line->parent, input_line->win, input_line->field);
+		input_line->form = cdw_ncurses_new_form(input_line->parent, input_line->win, input_line->field);
 		if (input_line->form == (FORM *) NULL) {
 			cdw_vdm ("ERROR: failed to create form with new_form()\n");
 			success = false;
@@ -2111,7 +2195,7 @@ int cdw_input_line_safe_driver(CDW_INPUT_LINE *input_line, char **buffer, int at
 			if (validation_result == E_OK) {
 
 				/* all that can go wrong at this point is malloc()
-				   failure - return value will be CDW_GEN_ERROR */
+				   failure - return value will be CDW_ERROR */
 				cdw_rv_t crv = cdw_ncurses_get_field_buffer(input_line->field[0], &local_buffer, (size_t) input_line->chars_max);
 				if (crv != CDW_OK) {
 					cdw_vdm ("ERROR: failed to get field buffer with cdw_ncurses_get_field_buffer()\n");
@@ -2274,3 +2358,137 @@ void cdw_input_line_refresh_with_string(CDW_INPUT_LINE *input_line, const char *
 
 
 
+
+
+CDW_CHECKBOX *cdw_checkbox_new(WINDOW *parent, int begin_y, int begin_x, bool checked)
+{
+	cdw_assert (parent != (WINDOW *) NULL, "ERROR: parent window can't be NULL\n");
+
+	CDW_CHECKBOX *checkbox = (CDW_CHECKBOX *) malloc(sizeof (CDW_CHECKBOX));
+	if (checkbox == (CDW_CHECKBOX *) NULL) {
+		cdw_vdm ("ERROR: failed to malloc checkbox\n");
+		return (CDW_CHECKBOX *) NULL;
+	}
+
+	checkbox->parent = parent;
+	checkbox->begin_y = begin_y;
+	checkbox->begin_x = begin_x;
+	checkbox->checked = checked;
+
+	checkbox->visible = true;
+	checkbox->on_toggle = (cdw_form_widget_function_t) NULL;
+
+	return checkbox;
+}
+
+
+
+
+
+bool cdw_checkbox_toggle(CDW_CHECKBOX *checkbox)
+{
+	if (checkbox->checked) {
+		mvwprintw(checkbox->parent, checkbox->begin_y, checkbox->begin_x, " ");
+		checkbox->checked = false;
+	} else {
+		mvwprintw(checkbox->parent, checkbox->begin_y, checkbox->begin_x, "X");
+		checkbox->checked = true;
+	}
+	/* why the cursor moves? why? why? */
+	wmove(checkbox->parent, checkbox->begin_y, checkbox->begin_x);
+
+	return checkbox->checked;
+}
+
+
+
+
+
+bool cdw_checkbox_get_state(CDW_CHECKBOX *checkbox)
+{
+	return checkbox->checked;
+}
+
+
+
+
+
+bool cdw_checkbox_set_state(CDW_CHECKBOX *checkbox, bool checked)
+{
+	checkbox->checked = checked;
+	if (checkbox->checked) {
+		mvwprintw(checkbox->parent, checkbox->begin_y, checkbox->begin_x, "X");
+	} else {
+		mvwprintw(checkbox->parent, checkbox->begin_y, checkbox->begin_x, " ");
+	}
+
+	wrefresh(checkbox->parent);
+
+	return checkbox->checked;
+}
+
+
+
+
+
+void cdw_checkbox_set_visibility(CDW_CHECKBOX *checkbox, bool visible)
+{
+	checkbox->visible = visible;
+	return;
+}
+
+
+
+
+
+void cdw_checkbox_delete(CDW_CHECKBOX **checkbox)
+{
+	if (*checkbox == (CDW_CHECKBOX *) NULL) {
+		cdw_vdm ("WARNING: passing NULL checkbox to the function\n");
+		return;
+	}
+
+	free(*checkbox);
+	*checkbox = (CDW_CHECKBOX *) NULL;
+	return;
+}
+
+
+
+
+
+void cdw_checkbox_free(CDW_CHECKBOX *checkbox)
+{
+	if (checkbox == (CDW_CHECKBOX *) NULL) {
+		cdw_vdm ("WARNING: passing NULL checkbox to the function\n");
+		return;
+	}
+
+	free(checkbox);
+	return;
+}
+
+
+
+
+
+void cdw_checkbox_draw(CDW_CHECKBOX *checkbox)
+{
+	if (!checkbox->visible) {
+		return;
+	}
+
+	mvwprintw(checkbox->parent, checkbox->begin_y, checkbox->begin_x - 1, "[");
+	mvwprintw(checkbox->parent, checkbox->begin_y, checkbox->begin_x + 1, "]");
+
+	if (checkbox->checked) {
+		mvwprintw(checkbox->parent, checkbox->begin_y, checkbox->begin_x, "X");
+	} else {
+		mvwprintw(checkbox->parent, checkbox->begin_y, checkbox->begin_x, " ");
+	}
+
+	redrawwin(checkbox->parent);
+	wrefresh(checkbox->parent);
+
+	return;
+}
