@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2010 Kamil Ignacak
+ * Copyright (C) 2007 - 2011 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -54,7 +54,7 @@ static cdw_cdll_t *cdw_cdll_new_element(void);
  *
  * \param head - first element of list
  *
- * \return CDW_MEM_ERROR if some malloc error occurs
+ * \return CDW_ERROR if some malloc error occurs
  * \return CDW_NO if list is already initialized
  * \return CDW_OK if success
  */
@@ -65,7 +65,7 @@ cdw_rv_t cdw_cdll_init(cdw_cdll_t **head)
 	*head = cdw_cdll_new_element();
 
 	if (*head == NULL) {
-		return CDW_MEM_ERROR;
+		return CDW_ERROR;
 	} else {
 		(*head)->prev = *head;
 		(*head)->next = *head;
@@ -138,7 +138,7 @@ cdw_cdll_t *cdw_cdll_new_element(void)
  *
  * \param data - pointer to data that you want to append to list
  *
- * \return CDW_GEN_ERROR if malloc fails
+ * \return CDW_ERROR if malloc fails
  * \return CDW_OK if success
  */
 cdw_rv_t cdw_cdll_append_non_unique(cdw_cdll_t *head, void *data)
@@ -151,11 +151,11 @@ cdw_rv_t cdw_cdll_append_non_unique(cdw_cdll_t *head, void *data)
 	} else {
 		cdw_cdll_t *last = cdw_cdll_last_element(head);
 		if (last == (cdw_cdll_t *) NULL) {
-			return CDW_GEN_ERROR;
+			return CDW_ERROR;
 		} else {
 			cdw_cdll_t *e = cdw_cdll_new_element();
 			if (e == (cdw_cdll_t *) NULL) {
-				return CDW_MEM_ERROR;
+				return CDW_ERROR;
 			} else {
 				head->prev = e;
 				e->next = head;
@@ -182,7 +182,7 @@ cdw_rv_t cdw_cdll_append_non_unique(cdw_cdll_t *head, void *data)
  *
  * \param data - pointer to data that you want to store on list
  *
- * \return CDW_MEM_ERROR if malloc fails
+ * \return CDW_ERROR if malloc fails
  * \return CDW_NO if data is already on list
  * \return CDW_OK if success
  */
@@ -268,19 +268,19 @@ void *cdw_cdll_ith_data(cdw_cdll_t *head, size_t i)
  *
  * \param i - index of element that should be removed (zero-based)
  *
- * \return CDW_GEN_ERROR if list is empty or other error occurred
+ * \return CDW_ERROR if list is empty or other error occurred
  * \return CDW_OK if success
  */
 cdw_rv_t cdw_cdll_remove(cdw_cdll_t **head, size_t i)
 {
 	if (*head == (cdw_cdll_t *) NULL) {
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	cdw_cdll_t *f = cdw_cdll_get_ith_element(*head, i);
 
 	if (f == NULL) {
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	/* works even if j is first or last node, NULLs are preserved */
