@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2010 Kamil Ignacak
+ * Copyright (C) 2007 - 2011 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@
    \param head - pointer to beginning of list
    \param data - initial data to be put into list
 
-   \return CDW_MEM_ERROR if some malloc error occurs
+   \return CDW_ERROR if some malloc error occurs
    \return CDW_NO if list is already initialized
    \return CDW_OK if success
 */
@@ -58,7 +58,7 @@ cdw_rv_t cdw_dll_init(cdw_dll_item_t **head, void *data)
 	*head = (cdw_dll_item_t *) cdw_dll_new_item(data);
 
 	if (*head == (cdw_dll_item_t *) NULL) {
-		return CDW_MEM_ERROR;
+		return CDW_ERROR;
 	} else {
 		return CDW_OK;
 	}
@@ -153,11 +153,11 @@ cdw_rv_t cdw_dll_append_non_unique(cdw_dll_item_t **head, void *data)
 	} else {
 		cdw_dll_item_t *last = cdw_dll_last_item(*head);
 		if (last == (cdw_dll_item_t *) NULL) {
-			return CDW_GEN_ERROR;
+			return CDW_ERROR;
 		} else {
 			cdw_dll_item_t *f = cdw_dll_new_item(data);
 			if (f == (cdw_dll_item_t *) NULL) {
-				return CDW_MEM_ERROR;
+				return CDW_ERROR;
 			} else {
 				last->next = f;
 				f->prev = last;
@@ -180,7 +180,7 @@ cdw_rv_t cdw_dll_append_non_unique(cdw_dll_item_t **head, void *data)
  *
  * \param cdw_file - file that you want to store on list
  *
- * \return CDW_MEM_ERROR if malloc fails
+ * \return CDW_ERROR if malloc fails
  * \return CDW_NO if file is already on list
  * \return CDW_OK if success
  */
@@ -234,19 +234,19 @@ cdw_dll_item_t *cdw_dll_ith_item(cdw_dll_item_t *head, size_t i)
  *
  * \param i - index of file that should be removed (zero-based)
  *
- * \return CDW_GEN_ERROR if list is empty or other error occurred
+ * \return CDW_ERROR if list is empty or other error occurred
  * \return CDW_OK if success
  */
 cdw_rv_t cdw_dll_remove_ith_item(cdw_dll_item_t **head, size_t i)
 {
 	if (*head == (cdw_dll_item_t *) NULL) {
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	cdw_dll_item_t *f = cdw_dll_ith_item(*head, i);
 
 	if (f == (cdw_dll_item_t *) NULL) {
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 
 	/* works even if j is first or last node, NULLs are preserved */
