@@ -19,11 +19,14 @@ cdw_rv_t cdw_string_security_parser(const char *string, char *invalid);
 
 bool cdw_string_starts_with_ci(const char *field, const char *fieldname);
 
+int cdw_string_get_words_end(const char *string, int chars_max);
 
 void cdw_string_free_table_of_strings(char *labels[], int n_max);
 cdw_rv_t cdw_string_ints2labels_3(char **labels, int ints[], size_t n);
 cdw_rv_t cdw_string_get_bool_value(const char *buffer, bool *value);
 
+int cdw_string_wrap_for_form_get_n_lines(const char *message, int n_cols);
+char  *cdw_string_wrap_for_form(const char *message, int n_cols, int n_lines);
 
 char *cdw_string_get_printable_if_needed(const char *string);
 
