@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2010 Kamil Ignacak
+ * Copyright (C) 2007 - 2011 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,12 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-
+#define _GNU_SOURCE /* getopt_long() */
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <locale.h> /* setlocale() */
 #include <sys/ioctl.h>
+#include <getopt.h> /* getopt_long() */
 
 /* open */
 #include <sys/types.h>
@@ -37,6 +38,12 @@
 
 
 
+static void cdw_utils_print_version(void);
+static void cdw_utils_print_help(const char *program_name);
+
+
+
+
 void after_event(const char *title, __attribute__((unused)) int adddic)
 {
 	cdw_logging_display_log_conditional(title);
@@ -64,14 +71,25 @@ void after_event(const char *title, __attribute__((unused)) int adddic)
 
 
 
+cdw_arguments_t cdw_commandline_arguments;
+
+static struct option cdw_getopt_long_options[] = {
+	{ "version",          no_argument,       0, 0},
+	{ "enable-dvd-rp-dl", no_argument,       0, 0},
+	{ "escdelay",         required_argument, 0, 0},
+	{ "help",             no_argument,       0, 0},
+	{ 0,                  0,                 0, 0} };
+
+
+
 /**
    \brief Process main(argc, argv[]) arguments
 
-   \return -1 if at least one argument was incorrect
-   \return 1 if user wants to turn support for DVD+R/DL on
-   \return 0 otherwise
+   \return -1 on errors - cdw has to close returning -1
+   \return 0 on no errors - cdw has to close returning 0
+   \return 1 on no errors - cdw should continue running
 */
-int process_commandline_args(int argc, char *argv[])
+int cdw_utils_process_commandline_arguments(int argc, char *argv[])
 {
 #ifdef HAVE_LIBMYSQLCLIENT
 	if ((strcmp((char *) argv[1], "--catalog") == 0) || (strcmp((char *) argv[1], "-c") == 0)) {
@@ -93,73 +111,120 @@ int process_commandline_args(int argc, char *argv[])
 	}
 #endif
 
-	bool support_dvd_rp_dl = false;
+	cdw_commandline_arguments.support_dvd_rp_dl = false;
+	cdw_commandline_arguments.escdelay_ms = -1;
 
-	/* flags used to prevent displaying the same content when short and long
-	   option are used in the same command line */
-	bool displayed_help = false;
-	bool displayed_version = false;
+	while (1) {
+		int option_index = 0;
+		int c = getopt_long(argc, argv, "hv", cdw_getopt_long_options, &option_index);
+		if (c == -1)
+			break;
 
-	for (int i = 1; i < argc; i++) {
-		cdw_sdm ("INFO: processing option argv[%d] = \"%s\"\n", i, argv[i]);
-		if ((strcmp((char *) argv[i], "--version") == 0) || (strcmp((char *) argv[i], "-v") == 0)) {
-			if (displayed_version) {
-				;
-			} else {
-				/* 2TRANS: this is message printed in terminal,
-				   first %s is program name, second %s is
-				   program version */
-				printf(_("%s-%s Copyright (C) 2002 - 2003 Balazs Varkonyi\n"), PACKAGE, VERSION);
-				/* 2TRANS: this is message printed in terminal,
-				   first %s is program name, second %s is
-				   program version */
-				printf(_("%s-%s Copyright (C) 2007 - 2010 Kamil Ignacak\n\n"), PACKAGE, VERSION);
-
-				displayed_version = true;
+		switch (c) {
+		case 0:
+			cdw_sdm ("INFO: option %s\n", cdw_getopt_long_options[option_index].name);
+			if (optarg) {
+			        cdw_sdm ("INFO:    with arg %s\n", optarg);
 			}
-		} else if ((strcmp((char *) argv[i], "--help") == 0) || (strcmp((char *) argv[i], "-h") == 0)) {
-			if (displayed_help) {
-				;
-			} else {
-				/* 2TRANS: this is help text displayed in console;
-				   %s will be replaced by program name, don't change its
-				   position */
-				printf(_("Usage: %s [options]\n\n"), argv[0]);
-				/* 2TRANS: this is help text displayed in console */
-				printf(_("Options:\n\n"));
-#ifdef HAVE_LIBMYSQLCLIENT
-				/* 2TRANS: this is help text displayed in console;
-				   don't localize anything before colon */
-				printf(_("  --catalog | -c            : start in Disk Catalog mode\n"));
-#endif
-				/* 2TRANS: this is help text displayed in console;
-				   don't localize anything before colon */
-				printf(_("  --help    | -h            : show this screen\n"));
-				/* 2TRANS: this is help text displayed in console;
-				   don't localize anything before colon */
-				printf(_("  --version | -v            : show version\n"));
-				/* 2TRANS: this is help text displayed in console;
-				   don't localize anything before colon */
-				printf(_("  --enable-dvd-rp-dl        : enable support for DVD+R DL (buggy, with dvd+rw-tools only)\n\n"));
-
-				displayed_help = true;
+			if (option_index == 0) {
+				cdw_utils_print_version();
+				return 0;  /* close cdw without signalling errors */
+			} else if (option_index == 1) {
+				cdw_commandline_arguments.support_dvd_rp_dl = true;
+			} else if (option_index == 2) {
+				char *rest = (char *) NULL;
+				cdw_commandline_arguments.escdelay_ms = (int) strtol(optarg, &rest, 10);
+				if (rest != (char *) NULL && strlen(rest)) {
+					/* 2TRANS: this is error message printed to terminal,
+					   first '%s' is name of option expecting a valid value,
+					   second '%s' is invalid value of the option */
+					fprintf(stderr, _("ERROR: incorrect value of option %s: \"%s\"\n"),
+						cdw_getopt_long_options[option_index].name, optarg);
+					return -1;
+				}
+				if (cdw_commandline_arguments.escdelay_ms < 0) {
+					/* 2TRANS: this is error message printed to terminal,
+					   "ESC delay" is number describing time between
+					    pressing ESC key, and reaction to the key */
+					fprintf(stderr, _("ERROR: ESC delay can't be negative\n"));
+					return -1;
+				}
+			} else if (option_index == 3) {
+				cdw_utils_print_help(argv[0]);
+				return 0; /* close cdw without signalling errors */
 			}
-		} else if (strcmp((char *) argv[i], "--enable-dvd-rp-dl") == 0) {
-			support_dvd_rp_dl = true;
-		} else {
-			/* 2TRANS: first %s is program name, second - argument given to the program */
-			printf(_("%s: invalid option: %s\n"), argv[0], argv[i]);
-			/* 2TRANS: %s is program name */
-			printf(_("Try \"%s --help\"\n"), argv[0]);
+			break;
+		case 'h':
+			cdw_utils_print_help(argv[0]);
+			return 0; /* close cdw without signalling errors */
+		case 'v':
+			cdw_utils_print_version();
+			return 0; /* close cdw without signalling errors */
+		case '?':
+			/* 2TRANS: this is error message printed to terminal */
+			fprintf(stderr, _("ERROR: unknown option in command line\n"));
 			return -1;
+		default:
+			break;
 		}
 	}
+	return 1; /* 1 = cdw can continue running */
+}
 
-	if (support_dvd_rp_dl) {
-		return 1;
-	} else {
-		return 0;
-	}
+
+
+
+
+void cdw_utils_print_version(void)
+{
+	/* 2TRANS: this is message printed in terminal,
+	   first %s is program name, second %s is
+	   program version */
+	printf(_("%s-%s Copyright (C) 2002 - 2003 Balazs Varkonyi\n"), PACKAGE, VERSION);
+	/* 2TRANS: this is message printed in terminal,
+	   first %s is program name, second %s is
+	   program version */
+	printf(_("%s-%s Copyright (C) 2007 - 2011 Kamil Ignacak\n\n"), PACKAGE, VERSION);
+
+	return;
+}
+
+
+
+
+
+void cdw_utils_print_help(const char *program_name)
+{
+	/* 2TRANS: this is help text displayed in console;
+	   %s will be replaced by program name, don't change its
+	   position */
+	printf(_("Usage: %s [options]\n\n"), program_name);
+	/* 2TRANS: this is help text displayed in console */
+	printf(_("Options:\n\n"));
+#ifdef HAVE_LIBMYSQLCLIENT
+	/* 2TRANS: this is help text displayed in console;
+	   don't localize anything before colon */
+	printf(_("  --catalog | -c            : start in Disk Catalog mode\n"));
+#endif
+	/* 2TRANS: this is help text displayed in console;
+	   don't localize anything before colon */
+	printf(_("  --help    | -h            : show this screen\n"));
+	/* 2TRANS: this is help text displayed in console;
+	   don't localize anything before colon */
+	printf(_("  --escdelay=X              : modify ESC key delay period;\n"));
+	/* 2TRANS: this is help text displayed in console */
+	printf(_("                              'X' is non-negative time in milliseconds\n"));
+	/* 2TRANS: this is help text displayed in console;
+	   don't localize anything before colon */
+	printf(_("  --version | -v            : show version\n"));
+	/* 2TRANS: this is help text displayed in console;
+	   don't localize anything before colon */
+	printf(_("  --enable-dvd-rp-dl        : enable support for DVD+R DL;\n"));
+	/* 2TRANS: this is help text displayed in console;
+	   "buggy" refers to support of DVD+R DL discs */
+	printf(_("                              (buggy, with dvd+rw-tools only)\n\n"));
+
+	return;
 }
 
 
@@ -373,10 +438,10 @@ cdw_rv_t get_cds_mixed_with_ioctl(void)
 				return CDW_NO;
 			}
 		} else {
-			return CDW_GEN_ERROR;
+			return CDW_ERROR;
 		}
 	} else {
-		return CDW_GEN_ERROR;
+		return CDW_ERROR;
 	}
 }
 
@@ -466,9 +531,9 @@ char *my_readline_10k(FILE *file)
    \return pointer to const string that matches given id,
    \return fall back error string if no id matches
 */
-const char *cdw_utils_id_label_table_get_label(cdw_id_label_t *table, long int id)
+const char *cdw_utils_id_label_table_get_label(cdw_id_clabel_t *table, cdw_id_t id)
 {
-	cdw_assert (table != (cdw_id_label_t *) NULL, "ERROR: input table is NULL\n");
+	cdw_assert (table != (cdw_id_clabel_t *) NULL, "ERROR: input table is NULL\n");
 
 	/* this function uses while() loop for traversing table with id/label
 	   pairs, but since:
@@ -506,17 +571,12 @@ const char *cdw_utils_id_label_table_get_label(cdw_id_label_t *table, long int i
 
 
 
-static cdw_id_label_t cdw_rv_t_table[] = {
-	{ CDW_OK,             "CDW_OK"            },
-	{ CDW_CANCEL,         "CDW_CANCEL"        },
-	{ CDW_NO,             "CDW_NO"            },
-	{ CDW_GEN_ERROR,      "CDW_GEN_ERROR"     },
-	{ CDW_FILE_INVALID,   "CDW_FILE_INVALID"  },
-	{ CDW_MEM_ERROR,      "CDW_MEM_ERROR"     },
-	{ CDW_SYS_ERROR,      "CDW_SYS_ERROR"     },
-	{ CDW_ARG_ERROR,      "CDW_ARG_ERROR"     },
-	{ CDW_FAILSAFE_MODE,  "CDW_FAILSAFE_MODE" },
-	{ -1,                 "ERROR: unimplemented return value" }}; /* -1 = guard */
+static cdw_id_clabel_t cdw_rv_t_table[] = {
+	{ CDW_OK,             "CDW_OK"        },
+	{ CDW_CANCEL,         "CDW_CANCEL"    },
+	{ CDW_NO,             "CDW_NO"        },
+	{ CDW_ERROR,          "CDW_ERROR"     },
+	{ -1,                 (char *) NULL   }}; /* guard */
 
 
 const char *cdw_utils_get_crv_label(cdw_rv_t crv)
@@ -528,16 +588,42 @@ const char *cdw_utils_get_crv_label(cdw_rv_t crv)
 
 
 
-cdw_id_label_t cdw_bool_type_labels[] = {
+cdw_id_clabel_t cdw_bool_type_labels[] = {
 	{ CDW_UNKNOWN, "CDW_UNKNOWN" },
 	{ CDW_TRUE,    "CDW_TRUE" },
 	{ CDW_FALSE,   "CDW_FALSE" },
 	{ 0,           (char *) NULL }};
 
 
-const char *cdw_utils_get_cdw_bool_type_label(int val)
+const char *cdw_utils_get_cdw_bool_type_label(int cdw_bool)
 {
-	return cdw_utils_id_label_table_get_label(cdw_bool_type_labels, val);
+	return cdw_utils_id_label_table_get_label(cdw_bool_type_labels, cdw_bool);
+}
+
+
+
+
+
+/* boolean, but with three values */
+cdw_id_clabel_t cdw_bool_type_chars[] = {
+	/* 2TRANS: this string means "unknown type or state";
+	   no more than 2 chars */
+	{ CDW_UNKNOWN, gettext_noop("?") }, /* also known as "init" */
+	/* 2TRANS: "N" like in "No, the disc doesn't have this attribute";
+	   no more than 2 chars */
+	{ CDW_FALSE,   gettext_noop("N") },
+	/* 2TRANS: "Y" like in "Yes, the disc has this attribute";
+	   no more than 2 chars */
+	{ CDW_TRUE,    gettext_noop("Y") },
+	{ -1,          (char *) NULL     }}; /* guard */
+
+
+const char *cdw_utils_get_cdw_bool_type_char(int cdw_bool)
+{
+	cdw_assert (cdw_bool == CDW_UNKNOWN || cdw_bool == CDW_FALSE || cdw_bool == CDW_TRUE,
+		    "ERROR: invalid value of cdw_bool: %d\n", cdw_bool);
+
+	return cdw_utils_id_label_table_get_label(cdw_bool_type_chars, cdw_bool);
 }
 
 
@@ -577,6 +663,38 @@ int cdw_utils_compare_ints(const void *int1, const void *int2)
 
 
 
+/**
+   \brief Helper function for qsort() - compare two integer values
+
+   Function that is used as compar() argument to qsort() function.
+   It returns -1, 0 or 1  if first argument is less than, equal or
+   larger than second.
+
+   Passing this function to qsort() results in table of ints sorted
+   from lowest to highest.
+
+   \param int1 - first integer to compare
+   \param int2 - second integer to compare
+
+   \return -1, 0 or 1  if first int is less than, equal or larger than second
+ */
+int cdw_utils_compare_ints_reverse(const void *int1, const void *int2)
+{
+	const int *i1 = (const int *) int1;
+	const int *i2 = (const int *) int2;
+
+	if (*i1 > *i2) {
+		return -1;
+	} else if (*i1 < *i2) {
+		return 1;
+	} else {
+		return 0;
+	}
+}
+
+
+
+
 
 /**
    \brief Calculate estimated time of accomplishment for a task, convert this time to string
@@ -625,6 +743,24 @@ void cdw_utils_eta_calculations(char *eta_string, int eta, size_t n)
 
 
 
+int cdw_utils_compress_table_of_ints(int *table, int n_max)
+{
+	table[0] = table[0];
+	int j = 1;
+	for (int i = 1; i < n_max; i++) {
+		if (table[i] == table[j - 1]) {
+			;
+		} else {
+			table[j] = table[i];
+			j++;
+		}
+	}
+	return j;
+}
+
+
+
+
 /* *********************** */
 /* *** unit tests code *** */
 /* *********************** */
@@ -635,6 +771,7 @@ void cdw_utils_eta_calculations(char *eta_string, int eta, size_t n)
 
 
 static void test_cdw_utils_eta_calculations(void);
+static void test_cdw_utils_compress_table_of_ints(void);
 
 
 void cdw_utils_run_tests(void)
@@ -642,6 +779,7 @@ void cdw_utils_run_tests(void)
 	fprintf(stderr, "testing cdw_utils.c\n");
 
 	test_cdw_utils_eta_calculations();
+	test_cdw_utils_compress_table_of_ints();
 
 	fprintf(stderr, "done\n\n");
 
@@ -688,5 +826,56 @@ void test_cdw_utils_eta_calculations(void)
 }
 
 
+
+
+
+void test_cdw_utils_compress_table_of_ints(void)
+{
+	fprintf(stderr, "\ttesting cdw_utils_compress_table_of_ints()... ");
+
+#define TABLE_SIZE 18
+	struct {
+		int table_in[TABLE_SIZE];
+		int table_out[TABLE_SIZE];
+		int expected_n;
+	} test_data[] = {
+		{ { 0, 1, 1, 2, 3, 4, 4, 4, 4,  6, 6, 6, 6, 7, 9, 9, 9, 10 },
+		  { 0, 1, 2, 3, 4, 6, 7, 9, 10, 6, 6, 6, 6, 7, 9, 9, 9, 10 },
+		  9 },
+
+		{ { 0, 1, 2, 3, 3, 3, 4, 5,  5,  7,  8, 8, 8, 10, 10, 10, 11, 12 },
+		  { 0, 1, 2, 3, 4, 5, 7, 8, 10, 11, 12, 8, 8, 10, 10, 10, 11, 12 },
+		  11 },
+
+		{ { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
+		  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
+		  1 },
+
+		{ { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
+		  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
+		  -1 /* guard */ } };
+
+	int i = 0;
+	while (test_data[i].expected_n != -1) {
+		int rv = cdw_utils_compress_table_of_ints(test_data[i].table_in, TABLE_SIZE);
+		cdw_assert (rv == test_data[i].expected_n,
+			    "ERROR: data set #%d: expected number of unique elements = %d, result is %d\n",
+			    i, test_data[i].expected_n, rv);
+		for (int j = 0; j < rv; j++) {
+			cdw_assert (test_data[i].table_in[j] == test_data[i].table_out[j],
+				    "ERROR: data set #%d: elements #%d do not match (%d != %d)\n",
+				    i, j, test_data[i].table_in[j], test_data[i].table_out[j]);
+		}
+		i++;
+	}
+
+	fprintf(stderr, "OK\n");
+
+	return;
+}
+
+
+
+
 #endif /* #ifdef CDW_UNIT_TEST_CODE */
 
