@@ -8,18 +8,28 @@
 
 #include "main.h"
 
-int process_commandline_args(int argc, char *argv[]);
+
+typedef struct {
+	bool support_dvd_rp_dl;
+	int escdelay_ms;
+} cdw_arguments_t;
+
+int cdw_utils_process_commandline_arguments(int argc, char *argv[]);
+
+
 int cdw_locale_init(void);
 cdw_rv_t get_cds_mixed_with_ioctl(void);
 void after_event(const char *title, int adddic);
 char *my_readline_10k(FILE *file);
 
-const char *cdw_utils_id_label_table_get_label(cdw_id_label_t *table, long int id);
+const char *cdw_utils_id_label_table_get_label(cdw_id_clabel_t *table, cdw_id_t id);
 const char *cdw_utils_get_crv_label(cdw_rv_t crv);
-const char *cdw_utils_get_cdw_bool_type_label(int val);
+const char *cdw_utils_get_cdw_bool_type_label(int cdw_bool);
+const char *cdw_utils_get_cdw_bool_type_char(int cdw_bool);
 int         cdw_utils_compare_ints(const void *int1, const void *int2);
+int         cdw_utils_compare_ints_reverse(const void *int1, const void *int2);
 void        cdw_utils_eta_calculations(char *eta_string, int eta, size_t n);
-
+int         cdw_utils_compress_table_of_ints(int *table, int n_max);
 
 /* unit tests */
 void cdw_utils_run_tests(void);
